%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glinitgui.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% handles = glinitgui(varargin, handles)
%
% Set the input args in a task GUI. Inittialize the methods and solver
% selection.
%
% varargin      varargin gui input of pairs, if it contains a 'Task' entry
%               the default GUI values will be set after the entries in 
%               the task following this entry. If it does not contain a
%               task, the default values are set from the config.
% handles       handles structure of a task gui.
%               there are 3 kinds of entries which will be set:
%   .(vname)_CB This is a checkbox, its value will be set after the
%               correspondig field: task.argin.(vname)
%   .(vname)_ET This is a edit text, its value will be set after the
%               correspondig field: task.argin.(vname)
%   .(vname)_PM This is a popup menu, its value will be set after the
%               correspondig field: task.argin.(vname).
%               There are some special cases of popup menus:
%               .method_PM          This popupmenu will be filled up with 
%                                   all available methods called 
%                                   'gl' (taskname) '_' ... '.m'.
%                                   e.g. for the split task, if 
%                                       glsplit_maxvariation.m
%                                       m-file exists, the option 
%                                       'maxvariation' can be selected
%                                       from the 'method_PM' popupmenu. 
%               .method_(snam)_PM   This popupmenu will be filled up with 
%                                   all submethods available to the
%                                   current method: 
%                                  'gl' (taskname) (snam) '_' 
%                                     (current method name) '_' ... '.m'.
%                                   e.g. for the split task if the selected
%                                       method is 'maxvariation' and the
%                                       glsplitdir_maxvariation_maxwidth.m
%                                       m-file exists, the option 
%                                       'maxwidth' can be selected
%                                       from the 'method_dir_PM' popupmenu. 
%               .(mname)_(vnam)_PM  This popupmenu will be filled as 
%                                   follows:
%                                   The mname must a valid fuction name
%                                   begining with 'gl', and when it is
%                                   called without input arguments it must
%                                   return a cell of selections.
%                                   The returned cell is filled in the PM,
%                                   the it will be handled a .(vname)_CB.
%               .locsolver_PM -     This popupmenu will be filled up with 
%                                   all available local solvers.
%               .lsolver_PM -       This popupmenu will be filled up with 
%                                   all available linear solvers.
%               .csolver_PM -       This popupmenu will be filled up with 
%                                   all available conic solvers.
%           
% handles       new handles structure of the task gui.

function handles = glinitgui(varargin, handles)

global glconf;    % global configuration variable

% argcechk
if nargin<2, error('glinitgui: Not enough arguments!');end

% updata the data files by adding possibly new entries
glupdateguidata(handles);

% set the data into the gui
glgetguidata(handles);

% get name taskname = name of the figure
tname=get(handles.figure1, 'Name'); % name of the task && the figure
% change taskname if the task is a Begin-End task
if length(tname)>6 && strcmp(tname(1:6),'Begin '), tname=tname(7:end); end
tname_low=lower(tname);             % name of the task in lowercase letters
tpre=tname_low(1:4);                % task perfix, used in the config
sourcedir=[fileparts(which('Gloptlab.m')) '/Source/'];

% process varargin
task=[];
if ~isempty(varargin)
    for i=1:2:length(varargin)
        if strcmp(varargin{i},'Task')
            task=varargin{i+1};
        end
    end
end 
if isempty(task)
    if isfield(glconf,tpre), task.argin=glconf.(tpre);
    else task.argin=[];
    end
end

% set various fields in the gui
fnams=fieldnames(handles);
if isfield(handles, 'method_PM')
    ind=[];
    for i=1:length(fnams)
        fn=fnams{i};
        vnam=fn(1:end-3);
        if length(vnam)>7 && strcmp(vnam(1:7),'method_')
            % this is a submethod entry and it needs to be moved at the 
            % end of the fnams cell, since a submethod setting needs 
            % the method to be set before.
            ind=[ind i];
        end
    end
    if ~isempty(ind)
        fnams=[fnams ; fnams(ind)];
        fnams(ind)=[];
    end
end

for i=1:length(fnams)
    fn=fnams{i};
    if length(fn)>3 && fn(end-2)=='_'
        vnam=fn(1:end-3);           % full variable name
        type=fn(end-1:end);         % variable type
        gfield=handles.(fn);        % actual gui field
        % handle the different types
        switch type
            case 'CB'
                if isfield(task.argin, vnam)
                    value=task.argin.(vnam);
                    set(gfield, 'Value', value);
                end
            case 'RB'
                ind=find(vnam=='_');
                if ~isempty(ind), vnam=vnam(1:ind(1)-1); end
                if isfield(task.argin, vnam)
                    value=task.argin.(vnam);
                    cvalue=get(gfield, 'String');
                    if strcmp(value,cvalue), set(gfield, 'Value', 1);
                    else set(gfield, 'Value', 0);
                    end
                end
            case 'ET'
                if isfield(task.argin, vnam)
                    value=task.argin.(vnam);
                    set(gfield, 'String', value);
                end
            case 'EV'
                if isfield(task.argin, vnam)
                    value=num2str(task.argin.(vnam));
                    set(gfield, 'String', value);
                end
            case 'PM'
                if strcmp(vnam(1:2),'gl')
                    % external method detected, it must be valid and return
                    % a cell of options, which will be set in the PM.
                    ind=find(vnam=='_');
                    if ~isempty(ind)
                        fnam=vnam(1:ind(1)-1);vnam=vnam(ind(1)+1:end);
                        if exist(fnam,'file')==2
                            opts=feval(fnam);
                            if iscell(opts)
                                set(gfield, 'String', opts);
                            end
                        end
                    end
                elseif strcmp(vnam,'method')
                    % method detected, find available methods
                    methods=glfindfun(['gl' tname_low '_'], sourcedir);
                    % set the available methods in the gui
                    if ~isempty(methods), set(gfield, 'String', methods);
                    else set(gfield, 'String', {'NONE'});
                    end
                elseif length(vnam)>7 && strcmp(vnam(1:7),'method_')
                    % submethod detected, find available submethods
                    subnam=vnam(8:end);
                    methods=get(handles.method_PM, 'String');
                    method=lower(methods{get(handles.method_PM, 'Value')});
                    % find available sub methods
                    dirchoosers=glfindfun(['gl' tname_low subnam '_' ...
                        method '_'], sourcedir);
                    % set the available methods in the gui
                    if ~isempty(dirchoosers), set(gfield, 'String',...
                            dirchoosers);
                    else set(gfield, 'String', 'NONE');
                    end
                elseif strcmp(vnam, 'eqsolver')
                    %set available linear equation solver names
                    eqsols=glfindfun('gllinsol_', sourcedir);
                    if ~isempty(eqsols), set(gfield, 'String', eqsols);
                    else set(gfield, 'String', 'NONE');
                    end
                elseif strcmp(vnam, 'locsolver') && isfield(glconf,'locsolvers')
                    %set available linear solver names
                    if ~isempty(glconf.locsolvers),
                        set(gfield, 'String', glconf.locsolvers);
                    else set(gfield, 'String', 'NONE');
                    end
                elseif strcmp(vnam, 'lsolver') && isfield(glconf,'lsolvers')
                    %set available linear solver names
                    if ~isempty(glconf.lsolvers),
                        set(gfield, 'String', glconf.lsolvers);
                    else set(gfield, 'String', 'NONE');
                    end
                elseif strcmp(vnam, 'csolver') && isfield(glconf,'csolvers')
                    %set available conic solver names
                    if ~isempty(glconf.csolvers),
                        set(gfield, 'String', glconf.csolvers);
                    else set(gfield, 'String', 'NONE');
                    end
                end
                if isfield(task.argin, vnam)
                    value=task.argin.(vnam);
                    if ~isempty(value)
                        if iscell(value)
                            % value is a cell, set the strings of the PM
                            set(gfield, 'String', value, 'Value', 1);
                        else
                            % value is a single string, choose the string
                            % in the PM
                            values=get(gfield, 'String');
                            if iscell(values)
                                ind=0;
                                for j=1:length(values)
                                    if strcmp(values{j},value), 
                                        ind=j;break; 
                                    end
                                end
                                if ind>0, set(gfield, 'Value', ind);end
                            else set(gfield, 'Value', 1);
                            end
                        end
                    end
                end
            otherwise
                continue;
        end
    end
end

handles = glguichk(handles);
