%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% glupdateguidata.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glupdateguidata(handles, lang, ldir)
%
% Get the current strings and tooltips from the gui and save it to the
% corresponding language file.
%
% handles      handles structure of the gui
% lang         2 x 1 language identifier string, e.g. 'EN', 'DE' 
% ldir         the directory containig the language files 

function glupdateguidata(handles, lang, ldir)

global glconf;    % global configuration variable

% argchek
if nargin <1, error('glupdateguidata: Not enough arguments!');end
if nargin <2, lang=[]; end
if nargin <3, ldir=[]; end

% init
[d,fname]=fileparts(get(handles.figure1, 'FileName'));
if isempty(lang)
    if isfield(glconf,'language'), lang=glconf.language; 
    else lang='en';
    end
end
lang=lower(lang);
if isempty(ldir)
    ldir=[d filesep 'GuiData' filesep upper(lang) filesep ];
end

% save the 'Strings' from the gui
fnam=[ldir fname '_str_' lang '.txt'];
glsaveprops(handles, 'String', fnam, 0);

% save the 'Values' from the gui
fnam=[ldir fname '_val_' lang '.txt'];
cond='text,1;_T,end-1;frame,1;_PB,end-2;';
glsaveprops(handles, 'Value', fnam, 1, cond);

% save the 'TooltipStrings' from the gui
fnam=[ldir fname '_tts_' lang '.txt'];
cond='text,1;_T,end-1;frame,1;';
glsaveprops(handles, 'TooltipString', fnam, 1, cond);

% save the help from the gui, by using the 'TooltipStrings' property
% but clearing the values
fnam=[ldir fname '_hlp_' lang '.txt'];
cond='text,1;_T,end-1;frame,1;';
glsaveprops(handles, 'TooltipString', fnam, 1, cond, 1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsaveprops.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [names, values]=glsaveprops(handles,prop,fnam,full,conds,clr)
%
% Get the given property of each handle in the handles structure. 
% Save it to the given file. If the file exist save only the new entries.
% The fields with names meeting one of the given conditions are omitted,
% as well as the fields whith empty values (if full=0)
%
% handles      handles structure
% prop         property string
% fnam         the name of the property file
% full         if full==0 ommit fields with empty value.
% conds        conditions string. If a field name has the condition text at
%              the given index, it will be excluded from the output list. 
%              Format: conds='text,ind;...text,ind;' with 'txt' is the 
%              condition text and 'ind' is the condition begin index.
% clr          if this flag is set, the values are set to [].


function [names, values]=glsaveprops(handles, prop, fnam, full, conds, clr)

% argcheck
if nargin <3, error('glsaveprops: Not enough arguments!');end
if nargin <4, full=1; end
if nargin <5, conds=''; end
if nargin <6, clr=0; end

% parse condition indices and text
ind=[0 find(conds==';')];cn=length(ind)-1;
cbeg=cell(1,cn);cend=cell(1,cn);ctxt=cell(1,cn);
for i=1:cn
    co=conds(ind(i)+1:ind(i+1)-1);
    % split a condition string into its two components
    ind2=find(co==',');
    if length(ind2)~=1, error('glsaveprops: Wrong condition format!'); end
    ctxt{i}=co(1:ind2(1)-1); % the condition text
    % convert the begin and end indices while end-i entries are encoded 
    % into the negative numbers -i
    cb=co(ind2(1)+1:end);
    if length(cb)>3 && strcmp(cb(1:3),'end')
        cbeg{i}=str2num(cb(4:end));
    else cbeg{i}=str2num(cb);
    end
    cend{i}=cbeg{i}+length(ctxt{i})-1;
    if isempty(cbeg) || isempty(cend) || cbeg{i}==0 || ...
            (cbeg{i}<0 && cend{i}>0),
        gldisp('conds',conds,'cb',cbeg{i},'ce',cend{i},'ct',ctxt{i});
        error('glsaveprops: Wrong condition format!');
    end
end
%cbeg,cend,ctxt

% get the fieldname and value list from the handles
names=sort(fieldnames(handles));
fn=length(names);
values=cell(fn,1);
rind=[];
for i=1:fn
    nam=names{i};
    cfound=0;
    % test the conditions
    for j=1:cn
        if (cbeg{j}>0 && length(nam)>=cend{j} && ...
                strcmp(nam(cbeg{j}:cend{j}),ctxt{j})) ||...
            (cbeg{j}<0 && length(nam)>-cbeg{j} && ...
                strcmp(nam(end+cbeg{j}:end+cend{j}),ctxt{j}))
            cfound=1;
            break; 
        end
    end
    struc=get(handles.(nam));
    if cfound || ~isfield(struc,prop) || ...
            (~full && isempty(struc.(prop))) 
        % the condition was found or this structure does not
        % have the required property or the value is empty and full==1:
        % ignore this handle.
        rind=[rind i];
    else
        s=struc.(prop);
        if ~clr
            % create the value string
            if iscell(s)
                txt='';empt=1;
                for j=1:length(s),
                    if isnumeric(s), s=num2str(s); end
                    txt=[txt '{"' s{j} '"};'];
                    if ~isempty(s{j}), empt=0; end
                end
                if ~empt || full, values{i}=['[' txt(1:end-1) ']'];
                else rind=[rind i];
                end
            else
                if isnumeric(s), s=num2str(s); end
                values{i}=['"' s '"'];
            end
        else values{i}='""';
        end
    end
end
names(rind)=[];values(rind)=[];

% check if the file that sould contain the data is already there.
% if yes, set only the new entries in it and leave the old ones unchanged.
if exist(fnam,'file')==2,
    % load the previous text file
    data=glload(fnam);
    % find the new components
    ind=[];
    for i=1:length(names)
        if ~isfield(data,names{i}), ind=[ind i]; end
    end
    if ~isempty(ind)
        % new components have been added
        names2=fieldnames(data);
        fn=length(names2);
        values2=cell(fn,1);
        for i=1:fn,
            s=data.(names2{i});
            if iscell(s) && ~isempty(s)
                txt='';
                for j=1:length(s),txt=[txt '{"' s{j} '"};'];end
                values2{i}=['[' txt(1:end-1) ']'];
            else values2{i}=['"' s '"'];
            end
        end
        % add the new components to the old ones, and sort
        [names,I]=sort([names2;names(ind)]);
        values=[values2;values(ind)];values=values(I);
        % save the file
        fid=fopen(fnam,'w');
        for i=1:length(names)
            fprintf(fid,'%s\n',[names{i} ' = ' values{i} ';']);
        end
        fclose(fid);
    end
else
    % save the file
    fid=fopen(fnam,'w');
    for i=1:length(names)
        fprintf(fid,'%s\n',[names{i} ' = ' values{i} ';']);
    end
    fclose(fid);
end
    