%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glboxplot.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function argout=glboxplot(box, varargin);
%
% Plot the input box 'box' either in a 2D or in a HD box plot.
% For the 2D plot the optional argin can contain a setting for the
% reference box ('Refbox'), the box color ('Color'), or the box filling 
% ('Fill'). Returned is the handle vector of the fill.
% The graphical information for the HD plot is supplied through the global 
% gloutput.grahics structure or the input argument ('Grdat').
% In the HD case a new grdat structure is returned, and set in the 
% gloutput.grahics structure (if it already exists)
%
% box               bound constraints
%  .inf             nx x 1 lower bounds
%  .sup             nx x 1 upper bounds
%
% varargin          1 x 2n cell of the n input parameters, available
%                   options are 'Grdat', 'Refbox', 'Color', 'Fill'
% Refbox            nx x 1 reference box (2D)
%  .inf             lower bounds
%  .sup             upper bounds
% Color             color of the boxplot (2D)
% Fill              fill resolution:0-no fill, 0<fillres<inf shading 
%                   resolution inf-fillin (2D)
% Grdat             grahics information structure (2D/HD)
%  .refbox          nx x 1 reference box 
%    .inf           lower bounds
%    .sup           upper bounds
%  .xind            n x 1 vector of selected variable indices (xind(i) 
%                   in 1...nx)
%  .cmap            colormap structure
%    .hues          the number of color hues
%    .rang1         3 x 2 marix of color range of the unselected boxes 
%                   [rang1(i,1),rang1(i,2)] must be in the interval [0,1]
%    .rang2         3 x 2 marix of color range of the selected boxes
%  .gridsize        the gridsize (resolution) of the images
%  .ahand           1 x ((n-1)*n)/2 vector of axe handles
%  .ihand           1 x ((n-1)*n)/2 vector of image handles of each image
%  .imdat           gridsize x (((nx-1)*nx)/2)*grids matrix of image data
% * fields which may need to be changed before using this method
%  .fillbox         either do not plot the current box (0), plot the boundary 
%                   of the the current box (1) or plot it in full color (2)
%  .plotdata        plot the data contained in the grdat structure
%  .plotc           add this constant to the entries of the grdat.imdat which
%                   are contained in the input box (only if the plotdata==1)
%
% argout            handle vector of the fill (for 2d plot) or a new 
%                   grahics information structure (grdat, for a HD plot) 

function argout=glboxplot(box, varargin)

global gloutput;    % global output structure
global glconf;      % gloptlab configuration file

RRAN=[-15,15];      % default reference box range
MINWID=1e-8;        % minimal wodth of a reference box box

% argcheck
if nargin<1, error('glboxplot: not enough arguments!');end
if nargin<2, varargin=[];end
if isempty(box) || isempty(box.inf),return;end
if any(box.sup-box.inf)<0, error('glboxplot: invalid box found!');end

grdat=[];bcolor=[];fillres=[];refbox=[];
for i=1:2:length(varargin)
    switch varargin{i}
        case 'Grdat'
            grdat=varargin{i+1};
        case 'Color'
            bcolor=varargin{i+1};
        case 'Fill'
            fillres=varargin{i+1};
        case 'Refbox'
            refbox=varargin{i+1};
        otherwise
            error(['glboxplot: unknown input parameter '...
                varargin{i} '!']);
    end
end
if isempty(grdat) && isfield(gloutput, 'graphics'), 
    grdat=gloutput.graphics; 
end

% set refernce box
if isempty(refbox)
    if ~isempty(grdat),
        refbox=grdat.refbox;
    elseif isfield(glconf,'refboxran'),
        nx=length(box.inf);
        refbox.inf=zeros(nx,1)+glconf.refboxran(1);
        refbox.sup=zeros(nx,1)+glconf.refboxran(2);
    else
        nx=length(box.inf);
        refbox.inf=zeros(nx,1)+RRAN;
        refbox.sup=zeros(nx,1)+RRAN;
    end
end

if any(refbox.sup-refbox.inf)<0,
    error('glboxplot: invalid reference box found!');
end
nx=length(refbox.inf);
bw=box.sup-box.inf;         % box width
rw=refbox.sup-refbox.inf;   % reference box width
ind=rw<MINWID;
rw(ind)=MINWID;
refbox.inf(ind)=refbox.inf(ind)-MINWID/2;
refbox.sup(ind)=refbox.sup(ind)+MINWID/2;

if nx==2,
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % two dimensional box plot %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    CCOL=[0.8 1 1];     % color of the box (if no color supplied)
    FRES=0;             % default fill resolution
    SBMULT=10e-3;       % small box multiplyier if box <= SBMULT*refbox
    % it will be plotted as a circle
    MARK1='o';          % marker 1 formarking very small boxes
    MARK2='';           % marker 2 formarking very small boxes (optional)
    MARK3='';           % marker 3 formarking very small boxes (optional)
    MARKS=14;           % marker size for marking very small boxes
    
    if isempty(bcolor)
        if ~isempty(grdat), bcolor=grdat.bcolor;
        elseif isfield(glconf,'cboxcol'), bcolor=glconf.cboxcol;
        else bcolor=CCOL;
        end
    end
    if isempty(fillres) 
        if ~isempty(grdat), fillres=grdat.fillres;
        else fillres=FRES;
        end
    end
    if isfield(glconf,'sbmult'),SBMULT=glconf.sbmult;end
    if isfield(glconf,'sbmark1'),MARK1=glconf.sbmark1;end
    if isfield(glconf,'sbmark2'),MARK2=glconf.sbmark2;end
    if isfield(glconf,'sbmark3'),MARK3=glconf.sbmark3;end
    if isfield(glconf,'sbmarksiz'),MARKS=glconf.sbmarksiz;end
    
    % init
    box.inf(isinf(box.inf))=-1e5;
    box.sup(isinf(box.sup))=1e5;
    
    % plot the box
    if ~any(isinf(rw)) && bw(1)<SBMULT*rw(1) && bw(2)<SBMULT*rw(2)
        % very small box, plot marker
        midp=(box.sup+box.inf)/2;
        h=plot(midp(1),midp(2),MARK1,'Color',bcolor,'MarkerSize',MARKS);
        if ~isempty(MARK2)
            h=[h plot(midp(1),midp(2),MARK2,'Color',bcolor,'MarkerSize',MARKS)];
        end
        if~isempty(MARK3)
            h=[h plot(midp(1),midp(2),MARK3,'Color',bcolor,'MarkerSize',MARKS)];
        end
    else
        % normal box
        if ~isinf(fillres),
            h=zeros(4,1);
            h(1)=line([box.inf(1),box.sup(1)],[box.inf(2),box.inf(2)],...
                'Color',bcolor,'LineWidth',0.5);
            h(2)=line([box.inf(1),box.sup(1)],[box.sup(2),box.sup(2)],...
                'Color',bcolor,'LineWidth',0.5);
            h(3)=line([box.inf(1),box.inf(1)],[box.inf(2),box.sup(2)],...
                'Color',bcolor,'LineWidth',0.5);
            h(4)=line([box.sup(1),box.sup(1)],[box.inf(2),box.sup(2)],...
                'Color',bcolor,'LineWidth',0.5);
        end
        if fillres>0 && ~isinf(fillres)
            % partial filling
            xr=(box.sup(1)-box.inf(1))/fillres;
            yr=(box.sup(2)-box.inf(2))/fillres;
            n=2*fillres+2;h1=zeros(n,1);
            X(1:2:n-1)=box.inf(1);Y(1:2:n-1)=box.inf(2):yr:box.sup(2);
            X(2:2:n)=box.inf(1):xr:box.sup(1);Y(2:2:n)=box.inf(2);
            X(n+1:2:2*n-1)=box.inf(1):xr:box.sup(1);Y(n+1:2:2*n-1)=box.sup(2);
            X(n+2:2:2*n)=box.sup(1);Y(n+2:2:2*n)=box.inf(2):yr:box.sup(2);
            for k=1:n
                i=2*k-1;
                h1(k)=line([X(i),X(i+1)],[Y(i),Y(i+1)],'Color',bcolor,...
                    'LineWidth',0.5);
            end
            h=[h;h1];
        elseif isinf(fillres),
            % full filling
            h=fill([box.inf(1) box.sup(1) box.sup(1) box.inf(1)],...
                [box.inf(2) box.inf(2) box.sup(2) box.sup(2)],'',...
                'FaceColor',bcolor,'EdgeColor',bcolor);
        end
    end
    argout=h;
elseif nx>2
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % high dimensional box plot %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isempty(grdat)
        % no main plot has been set in the global output structure vector
        error(['glboxplot: the global graphics structure has to be'...
            'initialized before using glboxplot!']);
    end
    
    grdat.xind=sort(grdat.xind);
    n=length(grdat.xind);
    
    % correct box bounds, if they are larger than the refence box
    ind=find(refbox.sup<box.sup);box.sup(ind)=refbox.sup(ind);
    ind=find(refbox.inf>box.inf);box.inf(ind)=refbox.inf(ind);
    % scale the box into the raster on the reference box
    q=[1+round((box.inf-refbox.inf)./rw*(grdat.gridsize-1)) ...
        1+round((box.sup-refbox.inf)./rw*(grdat.gridsize-1))];

    count=1;
    for i=1:nx-1,
        ii=find(grdat.xind==i);
        for j=nx:-1:i+1,
            jj=find(grdat.xind==j);
            ind1=q(i,1):q(i,2);ind2=q(j,1):q(j,2);
            if grdat.plotdata
                % get the image data corresponding to x_i,x_j
                dind=((count-1)*grdat.gridsize+1):(count*grdat.gridsize);
                dat=grdat.imdat(:,dind);
                % add the value of the plotc to all entries in the range.
                dat(ind2,ind1)=dat(ind2,ind1)+grdat.plotc;
                dat(dat>grdat.cmap.hues)=grdat.cmap.hues;
                dat(dat<1)=1;
                % save the image data in the grdat structure
                grdat.imdat(:,dind)=dat;
            else
                dat=ones(grdat.gridsize,grdat.gridsize);
            end
            if ~isempty(jj) && ~isempty(ii)
                % the x_i,x_j axes are selected in grdat.xind, and have
                % to be repainted
                if grdat.fillbox == 2,
                    % the filled and shaded box is plotted
                    dat(ind2,ind1)=grdat.cmap.hues+dat(ind2,ind1);
                elseif grdat.fillbox == 1,
                    % only the boundary of the box is plotted
                    dat(ind2,ind1(1))=grdat.cmap.hues+1;
                    dat(ind2,ind1(end))=grdat.cmap.hues+1;
                    dat(ind2(1),ind1)=grdat.cmap.hues+1;
                    dat(ind2(end),ind1)=grdat.cmap.hues+1;
                end
                pos2=((n+jj-1)*(n-jj))/2+ii;
                set(grdat.ihand(pos2),'CData',dat);
            end
            count=count+1;
        end
    end
    if isfield(gloutput, 'graphics'), gloutput.graphics=grdat;end
    argout=grdat;
end
