%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glplot.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function glplot(P, xbox, varargin)
%
% plot a Gloptlab probem and the current box either in a 2D plot,
% or a high dimensional box plot. The graphical information input
% and output is through the global gloutput.grahics structure.
%
% P                 problem definition
%  .Q.A             nc x nx+nx^2 matrix of constraints
%  .Q.Fbox          contraint bounds
%    .inf           nc x 1 lower bounds
%    .sup           nc x 1 upper bounds
%  .Q.nxo           number of original varibles
% xbox              bound constraints
%  .inf             nx x 1 lower bounds
%  .sup             nx x 1 upper bounds
% varargin          1 x 2n cell of the n input parameters, available
%                   options are 'Refbox', 'IndexSet', 'Figure',
%                   'PlotSpace', 'ObjColor','ConColor', 'BoxColor'
% Refbox            no x 1 reference box
%  .inf             lower bounds
%  .sup             upper bounds
% IndexSet          n x 1 vector of selected variable indices (xind(i) 
%                   in 1...nx)
% Figure            the handle of the figure in which the plot axes should
%                   be placed. If empty the current figure is used.
% PlotSpace         1 x 4 position vector [x,y,xw,yw] of the plot space in
%                   the figure. If empty the whole space in the figure
%                   is used.
% ObjColor          color of the objective in the plot (2D)
% ConColor          color of the constraints in the plot (2D)
% BoxColor          color of the box (2D)
% GridSize          resolution of the grid in each image (HD)
% CMapHues          number of hues in the color map (HD)
% PCMapRange        2 x 3 matrix of the RGB color map intervals of the plot 
% CCMapRange        2 x 3 matrix of the RGB color map intervals of the 
%                   current box (both HD)

% gloutput.graphics global grahics information structure
%  .refbox           no x 1 reference box 
%    .inf            lower bounds
%    .sup            upper bounds
%  .xind             n x 1 vector of selected variable indices (xind(i) 
%                    in 1...nx), for 2 dim problems xind is always [1 2]
%  .ahand            1 x n2+(2*n-2)+2*n vector containing the graphics 
%                    handles for the n2:=((n-1)*n)/2 subplots, the (2*n-2) 
%                    labels for the variables, n labels for the range 
%                    variables and n textfields for axis scaling ranges
% * only for HDplots
%  .ihand            1 x ((n-1)*n)/2 vector of image handles of each image
%  .cmap             colormap structure
%    .hues           the number of color hues
%    .rang1          3 x 2 marix of color range of the unselected boxes 
%                    [rang1(i,1),rang1(i,2)] must be in the interval [0,1]
%    .rang2          3 x 2 marix of color range of the selected boxes
%  .gridsize         the gridsize (resolution) of the images
%  .imdat            gridsize x (((no-1)*no)/2)*grids matrix of image data

function glplot(P, xbox, varargin)

global glconf;      % global configuration variable
global gloutput;    % global output structure

HDMAXVARS=5;        % maximum variable number in the HD plot
INF=1e5;            % positive and negative infinity
OBJCOL='m';         % color of the objective in the plot 2D
CONCOL='b';         % color of the constraints in the plot 2D
BCOLOR='r';         % color of the box in the 2D plot
GRIDSIZE=300;       % gridsize of the HDplot
CMAPHUES=20;        % color map constants for the HDplot
PCMAPRANG=[0 1;0 1;0 1];
CCMAPRANG=[0 1;0 0;0 0];

% argcheck
if nargin<1, error('glprobplot: not enough arguments!');end
if nargin<2, xbox=[]; end       
if nargin<3, varargin=[];end
if isempty(P),return;end
nx=length(P.Q.int);             % number of all variables
nxo=P.Q.nxo;                    % number of original variables

refbox=[];xind=[];fh=[];pspac=[];
objcol=[];concol=[];boxcol=[];
gridsize=[];cmaphues=[];cmapra=[];cmaprs=[];
for i=1:2:length(varargin)
    switch varargin{i}
        case 'Refbox'
            refbox=varargin{i+1};
        case 'IndexSet'
            xind=varargin{i+1};
        case 'Figure'
            fh=varargin{i+1};
        case 'PlotSpace'
            pspac=varargin{i+1};
        case 'ObjColor'
            objcol=varargin{i+1};
        case 'ConColor'
            concol=varargin{i+1};
        case 'BoxColor'
            boxcol=varargin{i+1};
        case 'GridSize'
            gridsize=varargin{i+1};
        case 'CMapHues'
            cmaphues=varargin{i+1};
        case 'PCMapRange'
            cmapra=varargin{i+1};
        case 'CCMapRange'
            cmaprs=varargin{i+1};
        otherwise
            error(['glboxplot: unknown input parameter '...
                varargin{i} '!']);
    end
end

% init
if isfield(glconf,'infty'),INF=glconf.infty;end
if isfield(glconf,'hdmaxvars'),HDMAXVARS=glconf.hdmaxvars;end
if isempty(fh)
    % get the plot figure
    if isfield(gloutput, 'graphics') && isfield(gloutput.graphics,'plotfig'),
        fh=gloutput.graphics.plotfig;
    else fh=gcf;
    end
end
if isempty(pspac)
    % get the plot space
    if isfield(gloutput, 'graphics') && isfield(gloutput.graphics,'plotspac'),
        pspac=gloutput.graphics.plotspac;
    else pspac=[0 0 1 1];
    end
end
if isfield(gloutput, 'graphics') && isfield(gloutput.graphics,'refbox') &&...
        length(gloutput.graphics.refbox.inf)~=nx
    % the current plot has the wrong dimension, delete it
    delete(gloutput.graphics.ahand);
    gloutput=rmfield(gloutput,'graphics');
end
if isempty(refbox)
    % get the reference box
    if isfield(gloutput,'graphics') && ...
            isfield(gloutput.graphics,'refbox'),
        refbox=gloutput.graphics.refbox;
    elseif isfield(glconf, 'refboxran')
        refbox.inf=zeros(nx,1)+glconf.refboxran(1);
        refbox.sup=zeros(nx,1)+glconf.refboxran(2);
    else
        refbox.inf=zeros(nx,1)-10;
        refbox.sup=zeros(nx,1)+10;
    end
end
if isempty(xind)
    % get the varaiable indices
    if isfield(gloutput,'graphics') && isfield(gloutput.graphics,'xind'),
        xind=gloutput.graphics.xind;
    else
        xind=1:min(nxo,HDMAXVARS);
    end
end
xind=sort(xind);
xind=xind(1:min(length(xind),HDMAXVARS));

% draw the problem
if nx==2
    %%%%%%%%%%%
    % 2D plot %
    %%%%%%%%%%%
    if isempty(objcol)
        % get the color of the constraints
        if isfield(glconf,'objcol'),objcol=glconf.objcol;
        else objcol=OBJCOL;
        end
    end
    if isempty(concol)
        % get the color of the constraints
        if isfield(glconf,'concol'),concol=glconf.concol;
        else concol=CONCOL;
        end
    end
    if isempty(boxcol)
        % get the box color
        if isfield(glconf,'cboxcol'),boxcol=glconf.cboxcol;
        else boxcol=BCOLOR;
        end
    end
    if ~isfield(gloutput, 'graphics') || ~isfield(gloutput.graphics,'refbox')...
            || ~all(gloutput.graphics.refbox.inf==refbox.inf) || ...
            ~all(gloutput.graphics.refbox.sup==refbox.sup)
        % create new axes for the 2D plot
        if isfield(gloutput, 'graphics') && isfield(gloutput.graphics,'ahand')
            % delete the old handles
            delete(gloutput.graphics.ahand);
        end
        % [MARGIN_LEFT, MARGIN_RIGHT, MARGIN_UP, MARGIN_DOWN, 
        %  PLOTSPAC_WIDTH, PLOTSPAC_HEIGHT, LABEL_WIDTH, LABEL_HEIGHT, 
        %  LABELSPAC_WIDTH, LABELSPAC_HEIGHT]
        params=[ 0.01, 0.01, 0.01, 0.01...
                 0.01, 0.02, 0.02, 0.02,...
                 0.005, 0.005];
        gloutput.graphics=glplot_init(refbox, xind, P.origvarnams, fh,...
            pspac, params);
    end
    
    % hold current figure
    hold on;
    % plot the constraints
    DIV=3;      % diam divisor if the bound indicator function was not plotted
    MAXTRY=3;   % maximum tryes to draw the bound indicator function
    h=[];
    A=P.Q.A.inf;F=P.Q.F;
    for i=1:size(A,1)
        if isfield(P,'opt') && P.opt==i, pcolor=objcol;
        else pcolor=concol;
        end
        diam=F.sup(i)-F.inf(i);
        if diam<INF                         % two sided constraint
            % lower
            u=-F.inf(i);
            ha=glqvecplot([u A(i,:)], 'Color', pcolor);
            if ~isempty(ha),
                h=[ha h];
                ha=[];d=-0.05*diam;j=0;
                while isempty(ha) && j<MAXTRY,
                    ha=glqvecplot([u+d A(i,:)], 'LineSpec', '-.',...
                        'Color', pcolor);
                    d=d/DIV;j=j+1;
                end
                h=[ha h];
            end
            %upper
            u=-F.sup(i);
            ha=glqvecplot([u A(i,:)], 'Color', pcolor);
            if ~isempty(ha),
                h=[ha h];
                ha=[];d=0.05*diam;j=0;
                while isempty(ha) && j<MAXTRY,
                    ha=glqvecplot([u+d A(i,:)], 'LineSpec', '-.',...
                        'Color', pcolor);
                    d=d/DIV;j=j+1;
                end
                h=[ha h];
            end
        else
            if F.sup(i)>INF                 % bounded up
                u=-F.inf(i);
                ha=glqvecplot([u A(i,:)], 'Color', pcolor);
                if ~isempty(ha),
                    h=[ha h];
                    if u>0, d=-0.2;
                    elseif u<0, d=0.2;
                    else u=-1;d=0;
                    end
                    ha=[];j=0;
                    while isempty(ha) && j<MAXTRY,
                        ha=glqvecplot([((1+d))*u A(i,:)], 'LineSpec',...
                            '-.', 'Color', pcolor);
                        d=d/DIV;j=j+1;
                    end
                    h=[ha h];
                end
            end
            if F.inf(i)<-INF                % bounded below
                u=-F.sup(i);
                ha=glqvecplot([u A(i,:)], 'Color', pcolor);
                if ~isempty(ha),
                    h=[ha h];
                    if u<0, d=-0.2;
                    elseif u>0, d=0.2;
                    else u=-1;d=0;
                    end
                    ha=[];j=0;
                    while isempty(ha) && j<MAXTRY,
                        ha=glqvecplot([((1+d)*u) A(i,:)], 'LineSpec',...
                            '-.', 'Color', pcolor);
                        d=d/DIV;j=j+1;
                    end
                    h=[ha h];
                end
            end
        end
    end
    % set axis scaling
    aran=[refbox.inf(1) refbox.sup(1) refbox.inf(2) refbox.sup(2)];
    axis(aran);
    % coordinate system aran
    line([aran(1),aran(2)],[0,0], 'Color','k');
    line([0,0],[aran(3),aran(4)], 'Color','k');
elseif nx>2
    %%%%%%%%%%%
    % HD plot %
    %%%%%%%%%%%
    if isempty(gridsize)
        % get the grid size
        if isfield(glconf,'hdgridsize'),gridsize=glconf.hdgridsize;
        else gridsize=GRIDSIZE;
        end
    end
    if isempty(cmaphues)
        % get color map hues
        if isfield(glconf,'cmaphues'),cmaphues=glconf.cmaphues;
        else cmaphues=CMAPHUES;
        end
    end
    if isempty(cmapra)
        % get the color map range of the plot
        if isfield(glconf,'cmapra'),cmapra=glconf.cmapra;
        else cmapra=PCMAPRANG;
        end
    end
    if isempty(cmaprs)
        % get the color map range of the current box
        if isfield(glconf,'cmaprs'),cmaprs=glconf.cmaprs;
        else cmaprs=CCMAPRANG;
        end
    end
    % check the multiplot
    if ~isfield(gloutput, 'graphics') || ~isfield(gloutput.graphics,...
            'xind') || length(gloutput.graphics.xind)~=length(xind) ||...
            ~all(gloutput.graphics.xind==xind) || ...
            ~all(gloutput.graphics.refbox.inf==refbox.inf) || ...
            ~all(gloutput.graphics.refbox.sup==refbox.inf)
        % create a new HDplot
        if isfield(gloutput, 'graphics') && isfield(gloutput.graphics,'ahand')
            % delete the old handles
            delete(gloutput.graphics.ahand);
        end
        % [MARGIN_LEFT, MARGIN_RIGHT, MARGIN_UP, MARGIN_DOWN, 
        %  PLOTSPAC_WIDTH, PLOTSPAC_HEIGHT, LABEL_WIDTH, LABEL_HEIGHT, 
        %  LABELSPAC_WIDTH, LABELSPAC_HEIGHT]
        params=[ 0.01, 0.01, 0.01, 0.01...
                 0.01, 0.02, 0.02, 0.02,...
                 0.005, 0.005];
        gridsize=round(gridsize/length(xind-1));
        cmap.hues=cmaphues;cmap.rang1=cmapra;cmap.rang2=cmaprs;
        gloutput.graphics=glplot_init(refbox, xind, P.origvarnams, ...
            fh, pspac, params, cmap, gridsize);
        % plot filled box and the plotting of the old boxes is turned off 
        gloutput.graphics.fillbox=2;
        gloutput.graphics.plotdata=0;
        gloutput.graphics.plotc=0;
    end  
end
if ~isempty(xbox), glboxplot(xbox,'Color',boxcol); end
