%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glplot_init.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function grdat = glplot_init(refbox, xind, origvarnams, fh, pspac, 
%               pparams, cmap, gridsize, grdatold)
%
% Initializes the plotting. Creates ((n-1)*n)/2 subplots in the area 
% defined by 'pspac' in the figure with the handle 'fh'.
% There n is the length of the 'xind' vector containing the selected 
% variable indices. Use this method before using glHDboxplot.
%
% refbox    no x 1 reference box 
%  .inf     lower bounds
%  .sup     upper bounds
% xind      n x 1 vector of selected variable indices (xind(i) in 1...no)
% origvarnams no x 1 cell of originalvariable names or empty
% fh        the handle of the figure in which the plot axes should
%           be placed. If empty the current figure is used.
% pspac     1 x 4 position vector [x,y,xw,yw] of the plot space in
%           the figure. If empty the whole space in the figure
%           is used.
% pparams   1 x 10 plot parameters: 
%           [MARGIN_LEFT, MARGIN_RIGHT, MARGIN_UP, MARGIN_DOWN, 
%            PLOTSPAC_WIDTH, PLOTSPAC_HEIGHT, LABEL_WIDTH, 
%            LABEL_HEIGHT, LABELSPAC_WIDTH, LABELSPAC_HEIGHT]
% * inputs only needed HD plots (no>2)
% cmap      colormap structure
%  .hues    the number of color hues
%  .rang1   3 x 2 marix of color range of the unselected boxes 
%             [rang1(i,1),rang1(i,2)] must be in the interval [0,1]
%  .rang2   3 x 2 marix of color range of the selected boxes
% gridsize  the gridsize (resolution) of the images
% grdatold  an old graphics structure returned by this method.
%           this is when due to the change of xind, refbox or gridsize
%           the current image data (grdatold.imdat) must be rescaled
%           or newly drawn. (optional)
%
% grdat     graphics structre which contains all the information 
%           needed for the .... methods
% .refbox   no x 1 reference box 
%   .inf    lower bounds
%   .sup    upper bounds
% .xind     n x 1 vector of selected variable indices (xind(i) 
%           in 1...nx), for 2 dim problems xind is always [1 2]
% .ahand    1 x n2+(2*n-2)+2*n vector containing the graphics 
%           handles for the n2:=((n-1)*n)/2 subplots, the (2*n-2) 
%           labels for the variables, n labels for the range 
%           variables and n textfields for axis scaling ranges
% .I        2 x n2 index matrix: I(:,i) shows that the ith subplot has on 
%           the horizontal axis the variable x_I(1,i) and on the vertical
%           axis the variable x_I(2,i).
% * outputs only for 2D plots
%  .bcolor  color of the boxplot
%  .fillres fill resolution:0-no fill, 0<fillres<inf shading 
%           resolution inf-fillin
% * outputs only for HD plots (no>2)
% .ihand    1 x ((n-1)*n)/2 vector of image handles of each image
% .cmap     colormap structure
%   .hues   the number of color hues
%   .rang1  3 x 2 marix of color range of the unselected boxes 
%           [rang1(i,1),rang1(i,2)] must be in the interval [0,1]
%   .rang2  3 x 2 marix of color range of the selected boxes
% .gridsize the gridsize (resolution) of the images
% .imdat    gridsize x (((no-1)*no)/2)*grids matrix of image data

function grdat=glplot_init(refbox, xind, origvarnams, fh, pspac, pparams,...
                            cmap, gridsize, grdatold)

% constants
MARGIN_LEFT     =0.01;
MARGIN_RIGHT    =0.01;
MARGIN_UP       =0.01;
MARGIN_DOWN     =0.01;
PLOTSPAC_WIDTH  =0.02;
PLOTSPAC_HEIGHT =0.02;
LABEL_WIDTH     =0.02;
LABEL_HEIGHT    =0.02;
LABELSPAC_WIDTH =0.005;
LABELSPAC_HEIGHT=0.005;

MINWID=1e-8;

% argcheck and init
if nargin<1,error('glplot_init: not enough arguments!');end
if any(refbox.sup-refbox.inf)<0,
    error('glplot_init: invalid reference box found!'),
end
no=length(refbox.inf);no2=(no*(no-1))/2;
if nargin<2,xind=1:no;end
xind=sort(xind); % sort the indices
n=length(xind);n2=(n*(n-1))/2;
if nargin<3,origvarnams=[];end
if nargin<4,fh=gcf;end
if nargin<5,pspac=[0 0 1 1];end
if nargin<6,pparams=[];end
if nargin<7,
    cmap.hues=16;
    cmap.rang1=[0 1;0 1;0 1];
    cmap.rang2=[0 1;0 0;0 0];
end
if nargin<8,gridsize=60;end
if nargin<9, imdat=[];
else
    % use previous image data
    % RESCALING; if the refbox is changed IS STILL MISSING
    if isfield(grdatold,'imdat') && size(grdatold.imdat,1)==gridsize
        imdat=grdatold.imdat;
    else imdat=[];
    end
    if isfield(grdatold,'ahand')
        % delete the previous axes
        delete(grdatold.ahand);
    end
end

% save the figure handle and the plot space in the grdat structure
grdat.plotfig=fh;
grdat.plotspac=pspac;

% set plot parameters
if ~isempty(pparams)
    MARGIN_LEFT     =pparams(1);
    MARGIN_RIGHT    =pparams(2);
    MARGIN_UP       =pparams(3);
    MARGIN_DOWN     =pparams(4);
    PLOTSPAC_WIDTH  =pparams(5);
    PLOTSPAC_HEIGHT =pparams(6);
    LABEL_WIDTH     =pparams(7);
    LABEL_HEIGHT    =pparams(8);
    LABELSPAC_WIDTH =pparams(9);
    LABELSPAC_HEIGHT=pparams(10);
end

% set margin
pspac=pspac+[MARGIN_LEFT, MARGIN_DOWN, -MARGIN_RIGHT-MARGIN_LEFT,...
           -MARGIN_UP-MARGIN_DOWN];
pspac2=pspac+[LABEL_WIDTH+LABELSPAC_WIDTH+PLOTSPAC_WIDTH, ...
              LABEL_HEIGHT+LABELSPAC_HEIGHT+PLOTSPAC_HEIGHT, ...
              -LABEL_WIDTH-LABELSPAC_WIDTH-PLOTSPAC_WIDTH,...
              -2*LABEL_HEIGHT-2*LABELSPAC_HEIGHT-PLOTSPAC_HEIGHT];
          
% compute the axes coordinates
if n>1, gxw=(pspac2(3)-(n-2)*PLOTSPAC_WIDTH)/(n-1);
else gxw=0;
end
gx=pspac2(1)+((0:(n-2))*(gxw+PLOTSPAC_WIDTH));
if n>1, gyw=(pspac2(4)-(n-2)*PLOTSPAC_HEIGHT)/(n-1);
else gyw=0;
end
gy=pspac2(2)+((0:(n-2))*(gyw+PLOTSPAC_HEIGHT));
% create the index matrix: I in N^(2 x n2) with the I(:,i) shows that
% the ith subplot has on the horizontal axis the variable x_I(1,i) and on 
% the vertical axis the variable x_I(2,i).
count=1;
I=zeros(2,n2);
for i=1:n-1
    for j=1:n-i
        I(1,count)=xind(j);
        I(2,count)=xind(n-i+1);
        count=count+1;
    end
end

% CREATE THE PLOT
% handle vector for the n2 subplots,(2*n-2)labels for the variable indices,
% n labels for variable indices (for the ranges) and n textfields for
% axis scaling ranges
ahand=zeros(1,n2+(2*n-2)+2*n); 
%str=cell(1,n2);
count=1;count2=n2+1;
for i=1:n-1,
    for j=1:n-i,
        % create subplot
        h = axes('Parent',fh,'Position',[gx(j),gy(n-i),gxw,gyw],'Tag',...
            ['plot_AX_x' num2str(xind(j)) '_x' num2str(xind(n-i+1))]);
        % set the axis ranges
        axis([refbox.inf(xind(j))-MINWID/2 refbox.sup(xind(j))+MINWID/2 ...
              refbox.inf(xind(n-i+1))-MINWID/2 refbox.sup(xind(n-i+1))+MINWID/2]);
        ahand(count)=h;
        % str{count}=['x_' num2str(xind(j)) ', x_' num2str(xind(n-i+1))];
        if i==1,
            % horizontal variable labeling
            hl=uicontrol('Parent',fh,'Style', 'text', 'Foreground', 'blue', ...
                'String', glvar2str(xind(j), 2, origvarnams),'Units', 'Normalized',...
                'Position',[gx(j)+gxw/2-LABEL_WIDTH/2,pspac(2)+pspac(4)-...
                LABEL_HEIGHT,LABEL_WIDTH,LABEL_HEIGHT]);
            ahand(count2)=hl;count2=count2+1;
        end
        if j==1,
            % vertical variable labeling
            hl=uicontrol('Parent',fh,'Style', 'text', 'Foreground', 'blue', ...
                'String',glvar2str(xind(n-i+1), 2, origvarnams),'Units','Normalized',...
                'Position',[pspac(1),gy(n-i)+gyw/2-LABEL_HEIGHT/2,...
                LABEL_WIDTH,LABEL_HEIGHT]);
            ahand(count2)=hl;count2=count2+1;
        end
        count=count+1;
    end
end

% create the n axis range labels and textfields
gxw2=(pspac(3)-LABELSPAC_WIDTH*(n-1))/n;
gx2=pspac(1)+((0:(n-1))*(gxw2+LABELSPAC_WIDTH));
for j=1:n,
    ind=xind(j);
    lx=gx2(j);ly=pspac(2);lw=LABEL_WIDTH*1.5;lh=LABEL_HEIGHT;
    tx=lx+lw ;ty=ly      ;tw=gxw2-lw      ;th=LABEL_HEIGHT;
    % create an 2 x n-1 matrix J indicating that one axis
    % in the subplots with handles A(1,:) corresponds to the
    % varaible x_ind. The second row A(2,:) indicates to
    % which axis (1 for horizontal or 2 for vertical) x_ind
    % corresponds in each plot.
    ind1=find(I(1,:)==ind);ind2=find(I(2,:)==ind);
    J=[ahand(ind1) ahand(ind2);ones(1,length(ind1)) ...
        2*ones(1,length(ind2))];
    % create the variable range label for the variable x_ind
    vran=[refbox.inf(ind) refbox.sup(ind)];
    hl=uicontrol('Parent',fh,'Style', 'text', 'String',...
        [glvar2str(ind, 2, origvarnams) ' in '], 'Units', ...
        'Normalized','Position',[lx,ly,lw,lh]);
    % create the range textfield for the variable x_ind, the callback 
    % function glHDboxplot_callback gets the matrix J, so it can modify the
    % right axes in the right subplots.
    htf=uicontrol('Parent',fh,'Style', 'edit', 'String',...
        [num2str(vran(1)) '  ' num2str(vran(2))], 'Units', 'Normalized',...
        'Position', [tx,ty,tw,th],'ForegroundColor','k',...
        'BackgroundColor','w','Tag',['range_TF_' num2str(ind)],...
        'Callback',{@glplot_callback, ind , J});
    ahand(end-2*n+j)=hl;
    ahand(end-n+j)=htf;
end

% create the first part of the grdat structure
grdat.refbox=refbox;
grdat.xind=xind;
grdat.ahand=ahand;
grdat.I=I;

if no==2
    % additional initialization for the 2d boxplot
    grdat.bcolor='r';
    grdat.fillres=0;
else
    % create the hues*2 x 3 color map, the first hues-rows are used
    % for drawing the unselected part the last hues-rows are used
    % for drawing the part under the current box.
    cm=zeros(cmap.hues,3);
    for i=1:3
        w=cmap.rang1(i,2)-cmap.rang1(i,1);
        if w>0
            cm(1:cmap.hues,i)=cmap.rang1(i,2):-w/(cmap.hues-1):...
                cmap.rang1(i,1);
        else
            cm(1:cmap.hues,i)=cmap.rang1(i,1)*ones(cmap.hues,1);
        end
        w=cmap.rang2(i,2)-cmap.rang2(i,1);
        if w>0
            cm(cmap.hues+1:2*cmap.hues,i)=cmap.rang2(i,2):...
                -w/(cmap.hues-1):cmap.rang2(i,1);
        else
            cm(cmap.hues+1:2*cmap.hues,i)=cmap.rang2(i,1)*ones(cmap.hues,1);
        end
    end
    % create and set the images in the subplot
    ihand=zeros(1,n2);% handle vector of the n2 subplot images.
    for i=1:n2,
        r=axis(ahand(i));
        ihand(i)=image(ones(gridsize,gridsize),'Parent', ahand(i),...
            'XData', r(1:2),'YData', r(3:4));
        colormap(cm);
    end
    if ~isempty(imdat)
        % set the old image data into the plot
        count=1;
        for i=1:no-1,
            ii=find(xind==i);
            for j=no:-1:i+1,
                jj=find(xind==j);
                if ~isempty(jj) && ~isempty(ii)
                    set(ihand(((n+jj-1)*(n-jj))/2+ii),'CData',...
                        imdat(:,((count-1)*gridsize+1):(count*gridsize)));
                    %disp([num2str(i) ', ' num2str(j) ':' str{pos2}]);
                end
                count=count+1;
            end
        end
    else
        imdat=ones(gridsize,no2*gridsize);
    end

    % create the HD part of the grdat structure
    grdat.cmap=cmap;
    grdat.gridsize=gridsize;
    grdat.ihand=ihand;
    grdat.imdat=imdat;
    % additional initialization
    grdat.fillbox=1;
    grdat.plotdata=0;
    grdat.plotc=0;
end