%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqvecplot.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% argout=glqvecplot(u, varargin);
%
% plots the quadratic vector uq(x) with
% q(x)=[c;x(1);x(2);x(1)^2;x(1)x(2);x(2)x(1);x(2)^2].
%
% u                 1 x 7 vector of the quadractic coeffs
% varargin          1 x 2n cell of the n input parameters, available
%                   options are 'Refbox', 'LineSpec'
% Refbox            reference box
%  .inf             2 x 1 lower bounds
%  .sup             2 x 1 upper bounds
% LineSpec          line specification string of the plot (color+line type)
%
% h                 handle vector of plot or [] if nothing was plotted

function h=glqvecplot(u, varargin)

global gloutput;    % global output structure
global glconf;      % global configuration variable

RES=1000;            %resoulution
ASYMPM=10000;        %multiplyier for determining asymptotes

% argcheck
if nargin<1, error('glqvecplot: not enough arguments!');end
if nargin<2, varargin=[];end
if isempty(u),return;end
if length(u)~=7, error('glqvecplot: input vector dimension error!');end
refbox=[];lspec=[];color=[];
for i=1:2:length(varargin)
    switch varargin{i}
        case 'Refbox'
            refbox=varargin{i+1};
        case 'LineSpec'
            lspec=varargin{i+1};
        case 'Color'
            color=varargin{i+1};
        otherwise
            error(['glboxplot: unknown input parameter '...
                varargin{i} '!']);
    end
end
if isempty(refbox) && isfield(gloutput, 'graphics') &&...
        isfield(gloutput.graphics, 'refbox'),
    refbox=gloutput.graphics.refbox;
end
if isempty(refbox),
    xbox=[-15 15];
else
    xbox=[refbox.inf(1) refbox.sup(1)];
end
if isempty(lspec), lspec='-b'; end

% set constants from config
if isfield(glconf,'res'),   RES=glconf.res;end
if isfield(glconf,'asymp'), ASYMPM=glconf.asymp;end
if size(u,1)<size(u,2), u=u';end;

% init all points with nan
X1=NaN+zeros(RES+1,1);X2=X1;Y1=X1;Y2=X2;
% create x values vector
x=xbox(1):(xbox(2)-xbox(1))/RES:xbox(2);
if u(3)==0 && u(5)==0 && u(6)==0 && u(7)==0,
    % all coeffs of y terms are 0
    a=u(4);b=u(2);c=u(1);
    ybox=[-1000 1000];
    if a~=0,
        b=b/2;D=b^2-a*c+1e-10*(b^2+abs(a*c));
        if D>=0,
            Y1=(ybox(1):(ybox(2)-ybox(1))/RES:ybox(2))';
            Y2=(ybox(1):(ybox(2)-ybox(1))/RES:ybox(2))';
            if b<0, X1(:)=(-b+sqrt(D))/a;X2(:)=c/(-b+sqrt(D));
            elseif b>0, X2(:)=c/(-b+sqrt(D));X1(:)=(-b+sqrt(D))/a;
            else X1(:)=sqrt(-c/a);Y2(:)=-sqrt(-c/a);
            end
        end
    elseif b~=0,
        X1=-c/b*ones(RES,1);
        Y1=(ybox(1):(ybox(2)-ybox(1))/RES:ybox(2))';
    else
        %only constant term exists
        X1=-c*ones(RES,1);
        Y1=(ybox(1):(ybox(2)-ybox(1))/RES:ybox(2))';
    end
else
    %coeffs of y not 0
    for i=1:RES,
        %set in x_i in the quadratic eqauation
        %(1) in y_i constant coeff: c=u(0)+u(2)x_i+u(4)x_i^2
        m=sparse([1,1,1],[1,2,4],[1, x(i), x(i)^2],1,7,3);
        c=m*u;
        %(2) in y_i linaer coeff: b=(u(3)+(u(5)+u(6))x_i)y_i
        b=(u(3)+(u(5)+u(6))*x(i));
        %(3) in y_i quadratic coeff: a=u(7)y_i^2
        a=u(7);
        %a,b,c
        %solve ay_i^2+by_i+c=0
        if a~=0,
            %quadratic term exists
            b=b/2;D=b^2-a*c;
            if D>=0,
                X1(i)=x(i);X2(i)=x(i);
                if b<0,s=-b+sqrt(D);Y1(i)=s/a;Y2(i)=c/s;
                elseif b>0,s=-b-sqrt(D);Y1(i)=c/s;Y2(i)=s/a;
                else Y1(i)=sqrt(-c/a);Y2(i)=-Y1(i);
                end
            end
        elseif b~=0
            %only linear term exists
            X1(i)=x(i);Y1(i)=-c/b;
        else
            %only constant term exists
            X1(i)=x(i);Y1(i)=-c;
        end
    end
end
%merge two parts of the solutions
if all(isnan(X1)) && all(isnan(X2))
    %disp('Warning: qvecplot: function not plotted! box too small?');
    h=[];
    return;
elseif all(isnan(X2))
    %line | paraboloid
    X=X1(~isnan(X1));Y=Y1(~isnan(Y1));
else
    mod=0;n=RES+1;
    lb1=[];rb1=[];%left and right bounds for x1
    for i=1:n
        if mod==0 && ~isnan(X1(i))
            lb1=[lb1 i];mod=1;
        elseif mod==1 && isnan(X1(i))
            rb1=[rb1 i-1];mod=0;
        end
    end
    if isempty(rb1),rb1=n;end
    mod=0;lb2=[];rb2=[];%left and right bounds for x2
    for i=1:n
        if mod==0 && ~isnan(X2(i))
            lb2=[lb2 i];mod=1;
        elseif mod==1 && isnan(X1(i))
            rb2=[rb2 i-1];mod=0;
        end
    end
    if isempty(rb2),rb2=n;end
    X=[];Y=[];
    for i=1:length(lb1)
        %determine if connection possible
        %if slope on the left end of the upper curve and slope on
        %the left end of the lower curve have different signs
        %and the right end is not at the bound of the drawbox
        %connect else no, same for right end.
        if lb1(i)>1 && lb1(i)<n && lb2(i)>1 && lb2(i)<n && ...
                sign(Y1(lb1(i)+1)-Y1(lb1(i))) ~= sign(Y2(lb2(i)+1)-Y2(lb2(i)))
            cl=1;
        else cl=0;
        end
        if rb1(i)<n && rb1(i)>1 && rb2(i)<n && rb2(i)>1 && ...
                sign(Y1(rb1(i))-Y1(rb1(i)-1)) ~= sign(Y2(rb2(i))-Y2(rb2(i)-1))
            cr=1;
        else cr=0;
        end
        if cr,
            X=[X;X1(lb1(i):rb1(i));X2(rb2(i):-1:lb2(i))];
            Y=[Y;Y1(lb1(i):rb1(i));Y2(rb2(i):-1:lb2(i))];
            if cl
                X=[X;X1(lb1(i))];Y=[Y;Y1(lb1(i))];
            end
        elseif cl,
            X=[X;X2(rb2(i):-1:lb2(i));X1(lb1(i):rb1(i))];
            Y=[Y;Y2(rb2(i):-1:lb2(i));Y1(lb1(i):rb1(i))];
        else
            X=[X;X1(lb1(i):rb1(i));nan;X2(rb2(i):-1:lb2(i))];
            Y=[Y;Y1(lb1(i):rb1(i));nan;Y2(rb2(i):-1:lb2(i))];
        end
        X=[X;nan];Y=[Y;nan];%nan indictes another part (not connected)
        %must be drawn
    end
    X=X(1:end-1);Y=Y(1:end-1);%remove last nan
end
%split by asymptotes when the func is not a line paralell to y
if ~(u(3)==0 && u(5)==0 && u(6)==0 && u(7)==0),
    asymp=ASYMPM*(xbox(2)-xbox(1))/RES;
    for i=1:length(X)-1
        if abs(Y(i)-Y(i+1))>asymp
            %asymptote found split
            X=[X(1:i);nan;X((i+1):end)];Y=[Y(1:i);nan;Y((i+1):end)];
        end
    end
end

%plot
ind=[0 find(isnan(X))' length(X)+1];
h=zeros(1,length(ind)-1);
for i=1:length(ind)-1
    if isempty(color),
        h(i)=plot(X(ind(i)+1:ind(i+1)-1),Y(ind(i)+1:ind(i+1)-1),lspec,...
            'LineWidth',1);
    else
        h(i)=plot(X(ind(i)+1:ind(i+1)-1),Y(ind(i)+1:ind(i+1)-1),lspec,...
            'LineWidth',1,'Color', color);
    end
end