%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcoef.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [u,A,hess,ier]=glcoef(P);
% calculates rigorous coefficient matrix for the 
% global optimization problem defined in P
%
% P            structure containing the internal problem definition
%   P.CC(i)    interval enclosure of constant i  (Intlab compatible)
%   P.coef     nA x 3 matrix of indices [i,j,k] defining A(i,j)=C(k)
%   P.xref     nx x nref array of reference vectors
%   P.ibounds  nc x 2 array with indices for constraint bounds 
%   P.basis    ne x 2 array defining nonquadratic basis functions
% 
% u            structure with bounds for general constraints
%   .inf,.sup   outer bounds for constraints
%   .iinf,.isup inner bounds for constraints
% A             structure containing sparse coefficient matrix
%   .inf,.sup   bounds for coefficients
%   .nz         incidence matrix
% hess         sparsity pattern of Hessian of Lagrangian
% ier           0 correctly processed
%               1 empty bounds
%               2 zero row in constraint matrix
%
function [u,A,hess,ier]=glcoef(P)

ier=0;
% generate constraint bounds
u.inf=P.CC(P.ibounds(:,1)).inf;
u.iinf=P.CC(P.ibounds(:,1)).sup;
u.isup=P.CC(P.ibounds(:,2)).inf;
u.sup=P.CC(P.ibounds(:,2)).sup;
empty=find(u.sup<u.inf);
if ~isempty(empty),
  disp('empty bounds in general constraints');
  gldisp('empty',empty);
  A=[];hess=[];
  ier=1;
  return;
end;

% generate coefficient matrix
nc=size(P.ibounds,1);
nx=length(P.Q.int);
nA=size(P.coef,1);
ne=size(P.basis,1);
nF=nx*(nx+1);
null=sparse([],[],[],nc,nF,nA);
A.inf=null;
A.sup=null;
A.nz=null;
for l=1:nA,
  % loop needed since several constants may contribute to the same entry
  ind=P.coef(l,:);
  i=ind(1);j=ind(2);k=ind(3);
  A.inf(i,j)=A.inf(i,j)+P.CC(k).inf;
  A.sup(i,j)=A.sup(i,j)+P.CC(k).sup;
  A.nz(i,j)=1;
end;

% detect separable constraints
inds=nx+1:nx+1:nx*(nx+1);       % index list for squares
indq=nx+1:nx*(nx+1);
nzs=sum(A.nz(:,inds),2);        % number of square terms/constr
nzb=sum(A.nz(:,indq),2)-nzs;    % number of bilinear terms/constr
%isep=find(nzb==0)';            % list of separable constraints

% find sparsity pattern of Hessian of Lagrangian
% and detect univariate constraints
% quadratic part

% compute the number snz of Hessian entries containing variable k
%snz=sum(A.nz(isep(1:nx)),1)+sum(A.nz(isep,inds),1);
hess=(sum(A.nz(:,indq),1)>0);
hess=reshape(hess,nx,nx);
%% nonquadratic part
%for i=1:ne,
%  k=P.basis(i,2);
%  %snz(:,k)=snz(:,k)+1;
%  hess(k,k)=1;
%end;
hess=hess|hess';
if ~isempty(find(sum(A.nz,2)==0))
  disp('zero row in constraint matrix');
  ier=2;return;
end;
  
