%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glconst.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [C,CC,err]=glconst(const,errmsg);
% rigorous evaluation of symbolic constants
%
% the resulting intervals contain the true constants, 
% but may be wider due to rounding errors, 
% or if the same constant appears repeatedly.
%  
% const         nC x 1 cell array of symbolic constants 
% errmsg        error message to be displayed in case of errors
%               (default: none)
%
% Currently interpretable constant formats are: 
%    - standard real number notation, enclosed in square brackets.
%    - intervals in square brackets: 
%      [<lowerbound>,<upperbound>] or [<shortinterval>]
%      <lowerbound> and <upperbound> must be standard real numbers,
%      or constants C(<index>) already defined.
%      <shortinterval> has either the Intlab syntax 
%          <integer>.<digits>_<optional exponent part>
%      with an uncertainty of +-1 unit in the last displayed place,
%      or the colon syntax as on p.29 of my book
%          A. Neumaier,
%          Interval Methods for Systems of Equations,
%          Cambridge Univ. Press, Cambridge 1990.
%    - infix arithmetic expressions defined by a string 
%      composing the constants pi and C(<index>) if already defined, 
%      using the operators +-*/^, parentheses, and standard functions.
%      Infix constants must also be enclosed by square brackets,
%      and may not contain blanks or line feeds.
%      IMPORTANT: Using real numbers as part of an infix expression
%      will not be recognized but may lead to loss of control of 
%      rounding errors! Bad examples include [10*0.1] and [3/7].
% 
% CC(i)         interval enclosure of constant i 
%               unlike most bounds in Gloptlab, 
%               this is a typed interval as in Intlab, with bounds
%               [CC(i).inf,CC(i).sup]
% C(i)          absolutely smallest value in CC(i)
% err           [] if correctly processed,
%               vector of error flags otherwise
%                  0 expression correctly recognized
%                  1 square bracket(s) missing
%                  2 expression contains a blank
%                  3 expression contains a line feed 
%                  4 expression contains more than one comma
%                  5 malformed lower bound
%                  6 malformed upper bound
%                  7 empty interval
%                  8 expression contains more than one underscore
%                  9 malformed Intlab short interval
%                 10 expression contains more than one colon
%                 11 malformed colon short interval
%                 50 malformed infix expression
%                 99 short interval (not yet recognized)
%
function [C,CC,err]=glconst(const,errmsg)

nC=size(const,1);
Csup=inf*zeros(nC,1);
C=infsup(-Csup,Csup); % interval vector, will ultimately be CC
err=zeros(nC,1);
%pi=4*atan(intval(1.0));

for i=1:nC+1,

  if i>1, 
    err(i-1)=ier;
  end;
  if i>nC, break; end;

  % get i-th expression 
  expr=const{i}; 

  % check minimal syntax
  if (expr(1)~='[') || (expr(end)~=']'), 
    % square bracket(s) missing
    ier=1;continue;
  end;
  if ~isempty(findstr(expr,' ')),
    % expression contains a blank
    ier=2;continue;
  end;
  if ~isempty(findstr(expr,char(10))),
    % expression contains a line feed 
    ier=3;continue;
  end;

  % test for standard interval
  commas=findstr(expr,',');
  if length(commas)>1,
    % expression contains more than one comma
    ier=4;continue;
  end;
  if ~isempty(commas),
    % standard interval
    [xinf,ct,ers]=sscanf(expr(2:commas-1),'%e');
    if ~isempty(ers), 
      % malformed lower bound
      ier=5;continue;
    end;
    [xsup,ct,ers]=sscanf(expr(commas+1:end-1),'%e');
    if ~isempty(ers), 
      % malformed upper bound
      ier=6;continue;
    end;
    if xinf>xsup,
      % empty interval
      disp('emptiness test not yet rigorous');
      % since intval has no error recovery for this case
      ier=7;continue;
    end;
    % constant is a good interval
    [C(i),err]=str2intval(expr);
    if err>0,
      disp('modify program to recognize inf, NaN, d D') 
      disp(expr)
      error('admissible interval not recognized');
    end;
    ier=0;continue;
  end;

  % test for Intlab short interval
  undersc=findstr(expr,'_');
  if length(undersc)>1,
    % expression contains more than one underscore 
    ier=8;continue;
  end;
  if ~isempty(undersc),
    % expression must be an Intlab short interval
    [C(i),err]=str2intval(expr(2:end-1));
    if err>0, 
      % malformed Intlab short interval
      ier=9;continue;
    end;
    ier=0;continue;
  end;

  % test for colon short interval
  colons=findstr(expr,':');
  if length(colons)>1,
    % expression contains more than one colon
    ier=10;continue;
  end;
  if ~isempty(colons),
    % expression must be a colon short interval, 
    if 0, 
      % malformed Intlab short interval
      ier=11;continue;
    end;
    % colon short interval not yet recognized
    ier=99;continue;
  end;

  % test for real number 
  [x,ct,ers]=sscanf(expr(2:end-1),'%e');
  if isempty(ers), 
    % constant is a single real number
    realn=expr(2:end-1);
    err=0;
    % recognize +-inf and NaN
    if strcmp(realn,'-inf'), 
      C(i)=-inf;
    elseif strcmp(realn,'+inf'), 
      C(i)=+inf;
    elseif strcmp(realn,'inf'), 
      C(i)=inf;
    elseif strcmp(realn,'NaN'), 
      C(i)=NaN;
    else
      [C(i),err]=str2intval(realn);
    end;
    if err>0,
      disp('modify program to recognize d D') 
      disp(expr)
      error('admissible real number not recognized');
    end;
    ier=0;continue;
  end;

  % now expr must be an infix expression
  ier=0;
  C(i)=eval(expr(2:end-1),'ier=50');
  if ier==50,
    % malformed infix expression
  end;

end;

if sum(err)==0, 
  % constants correctly read
  err=[]; 
  CC=C; % now CC is the interval vector
  % create real approximation
  C=inf(CC);Csup=sup(CC);
  ind=(CC<=0);C(ind)=Csup(ind);
  ind=~(ind|CC>=0);C(ind)=0;
  return; 
end;
if nargin==1, return;end;



% print error messages
disp(' ')
disp(errmsg);
disp(' ')
for i=1:length(err),
  if err(i)>0, 
    disp(['error in constant ',num2str(i)',':']);
    disp(const{i})
  end;
  switch err(i)
    case   0
      % good case
    case   1 
      disp('square bracket(s) missing');
    case   2 
      disp('expression contains a blank');
    case   3 
      disp('expression contains a line feed');
    case   4 
      disp('expression contains more than one comma');
    case   5 
      disp('malformed lower bound');
    case   6 
      disp('malformed upper bound');
    case   7 
      disp('empty interval');
    case   8 
      disp('expression contains more than one underscore');
    case   9 
      disp('malformed Intlab short interval');
    case  10 
      disp('expression contains more than one colon');
    case  11 
      disp('malformed colon short interval');
    case  50 
      disp('malformed infix expression');
    case  99 
      disp('short interval (not yet recognized)');
    otherwise
      error('unrecognized case');
  end;
end;


