%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gldefparser.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [P, err] = gldefparser(fname);
% reads a global optimization problem
%    min  C_shift+-F_opt (but C_shift if opt=0)
%    s.t. Aq(x) in \F,
%         x_i = uni_j(x_k), (i,j,k) in basis
%         x_int integer,
%         x_{set_k} in N_k (k=1:ns),
% specified as described in glinput.def
% x is the vector of independent variables to be found
%
% fname        file name of the .def file, with full path,.
%
% P            structure containing the internal problem definition
%              C is in P.CC (approx. in P.C)
%              A is in P.Q.A, \F is in P.Q.F,
%              q(x) has the components
%                   q_k             =  x_k        (k=1:nx)
%                   q_(j+nk)        =  x_j x_k    (j,k=1:nx)
%              and additional definitions:
%                   basis           in P.basis(t,:)=[j,k,i] and P.ucode,
%                   int             in P.int,
%                   set_k and N_k   in P.set*
%   *P.name    problem name
%   *P.desc    informal problem description
%   *P.const   nC x 1 cell array of symbolic constants
%    P.C(i)    real approximation of constant i
%    P.CC(i)   interval enclosure of constant i  (Intlab compatible)
%   *P.coef    nA x 3 matrix of indices [i,j,k] defining A(i,j)=C(k)
%   *P.xref    nx x nref array of reference vectors
%   *P.ibounds nc x 2 array with indices for constraint bounds
%   *P.set     ns x 1 cell array defining set constraints
%              N_k = {C(i) || i in P.set{k}}
%   *P.setx(k) = set_k, index of variable with x_{set_k} in N_k
%   *P.basis   ne x 3 array defining nonquadratic basis functions
%   *P.uni     nu x 1 cell array defining univariate functions
%    P.ucode   nu x 1 cell array of univariate operation codes
%   *P.table   nt x 1 cell array of tables
%   *P.shift   constant index  in objective function C_shift+type*u_opt
%   *P.opt     variable index  in objective function
%   *P.type    factor (0,1,-1) in objective function
%   *P.Q       structure encoding the quadratic part
%      .desc      same as P.desc
%      .A         sparse matrix of constraint coefficients
%        .inf,.sup  bounds for coefficients
%        .nz        incidence matrix
%      .hess      sparsity pattern of Hessian of Lagrangian
%      *.int      characteristic vector of integer variables
%      *.nxo      number of the original variables before conversion
%                 from .dag to .def
%      .F         structure with bounds for general constraints
%        .inf,.sup    outer enclosure
%        .iinf,.isup  inner enclosure
%      .vmask     nx x nx lower triangular 0-1 mask matrix, of
%                 bilinear and quadratic variable adjacency
%      .vperm     the variable permutation used to get the mask
%      .gaps      ng x 3 array with rows [k a b] specifying finite gaps
%                    x_k notin ]a,b[
%      .Bu        ne x 5 array with entries Bu(i,:)=[k,a,b,c,d]
%                 defining quadratic enclosures
%                    B_{nx*(nx+1)+i}(x) in a x_k^2 + b x_k + [c,d]
%                 for nonquadratic basis functions
%  P.origvarnams no x 1 cell of originalvariable names or empty
%  P.xbox        bound constraint box (currently set to infinite)
%  err           error message, if empty all is ok.

function [P, err] = gldefparser(fname)

% argcheck
if nargin<1, error('gldefparser: Not enough arguments!'); end

% init
P=[]; err='';
[fpath, probname]=fileparts(fname);

% check
if exist(fname,'file')~=2,
    err=['Parsing error: file ' fname ' was not found!'];
    return;
end

% expected information in a given reading state
expectlist=[
    ' 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'
    ' 2 % gloptlab input file                           '
    ' 3 problem identifier                              '
    ' 4 % problem size                                  '
    ' 5 list of sizes                                   '
    ' 6 % description                                   '
    ' 7 informal description                            '
    ' 8 % constants and their positions                 '
    ' 9 % list defining a constant                      '
    '10 list of indices defining matrix entries         '
    '11 % objective                                     '
    '12 list defining objective                         '
    '13 % integer constraints                           '
    '14 list defining an integer variable               '
    '15 % general constraints                           '
    '16 list of indices defining constraint bounds      '
    '17 unused                                          '
    '18 unused                                          '
    '19 unused                                          '
    '20 % set constraints                               '
    '21 list with index defining discrete variables     '
    '22 % list defining set constraint                  '
    '23 list of indices defining set constraint         '
    '24 % nonquadratic basis functions                  '
    '25 list of indices for nonquadratic basis functions'
    '26 % univariate functions                          '
    '27 % list defining univariate function             '
    '28 string defining univariate function             '
    '29 % tables                                        '
    '30 % list defining table type                      '
    '31 list defining table entries                     '
    '32 % original variable names                       '
    '33 list of the original variable names             '
    '34 % end                                           '
    '35 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'
    ];
endstate=35;

fid=fopen(fname);
line=0;
state=1; % nothing read so far

% initializations
nd=0;nC=0;nA=0;ni=0;nc=0;ns=0;ne=0;nu=0;nt=0;vn=-1;
ndread=0;nCread=0;nAread=0;niread=0;
ncread=0;nsread=0;neread=0;nuread=0;ntread=0;
vnread=0;% original variable names read
nlread=0; % number of lines read from a sublist
nCl=0;

while 1
    string=fgetl(fid);   % reads next line without line feed
    line=line+1;
    if isempty(strtok(string)), continue; end;

    if state>endstate,
        if string(1)~=-1,
            err=['unexpected line ',num2str(line),' after % end'];
        end;
        % parsing successfully completed
        break;
    end;
    if state==17,
        % skip unused state numbers
        state=20;
    end;
    expect=expectlist(state,:);
    expected_state=sscanf(expect(1:2),'%i');
    if state~=expected_state,
        err=['undefined state at line ',num2str(line)];
        break;
    end;
    if string(1)<0,
        err=['unexpected end of file at line ',num2str(line)];
        break;
    end;

    if expect(4)=='%',
        if string(1)~='%',
            err=['unexpected entry at line ',num2str(line)];
            break;
        end;
        if expect(6)=='l'
            % list with leading % expected
            switch state
                case 9  % list defining a constant
                    nCread=nCread+1;
                    [num,string]=strtok(string(2:end));
                    if sscanf(num,'%i')~=nCread,
                        err=['mismatch in numbering of constants '...
                            num2str(sscanf(num,'%i')) '!=' num2str(nCread)];
                        break;
                    end;
                    [num,const]=strtok(string);
                    nl=sscanf(num,'%i');
                    nAread=nAread+nl;
                    const=strtok(const);
                    if (const(1)~='[') || (const(end)~=']'),
                        err=['constant ' const ' has no correct brackets'];
                        break;
                    end;
                    P.const{nCread}=const;
                    if nl>0,
                        state=state+1;
                        nlread=0;
                    elseif nCread==nC,
                        state=state+2;
                    end;
                    continue;
                case 18 % list defining min constraint
                    nvread=nvread+1;
                    [num,string]=strtok(string(2:end));
                    if sscanf(num,'%i')~=nvread,
                        err=['mismatch in numbering of min constraints '...
                            num2str(sscanf(num,'%i')) '!=' num2str(nvread)];
                        break;
                    end;
                    num=strtok(string);
                    nl=sscanf(num,'%i');
                    if nl>0
                        state=state+1;
                        nlread=0;
                    elseif nvread==nv,
                        state=state+2;
                    end;
                    continue;
                case 22 % list defining set constraint
                    nsread=nsread+1;
                    [num,string]=strtok(string(2:end));
                    if sscanf(num,'%i')~=nsread,
                        err=['mismatch in numbering of set constraints '...
                            num2str(sscanf(num,'%i')) '!=' num2str(nsread)];
                        break;
                    end;
                    num=strtok(string);
                    nl=sscanf(num,'%i');
                    if nl>0,
                        state=state+1;
                        nlread=0;
                    elseif nsread==ns,
                        state=state+2;
                    end;
                    continue;
                case 27 % list defining univariate function
                    nuread=nuread+1;
                    [num,string]=strtok(string(2:end));
                    if sscanf(num,'%i')~=nuread,
                        err=['mismatch in numbering of univariate func '...
                            num2str(sscanf(num,'%i')) '!=' num2str(nuread)];
                        break;
                    end;
                    num=strtok(string);
                    nl=sscanf(num,'%i');
                    if nl>0,
                        state=state+1;
                        nlread=0;
                    elseif nuread==nu,
                        state=state+2;
                    end;
                    continue;
                case 30 % list defining table type
                    ntread=ntread+1;
                    [num,string]=strtok(string(2:end));
                    if sscanf(num,'%i')~=ntread,
                        err=['mismatch in numbering of tables '...
                            num2str(sscanf(num,'%i')) '!=' num2str(ntread)];
                        break;
                    end;
                    [num,string]=strtok(string);
                    nl=sscanf(num,'%i');
                    if nl>0,
                        state=state+1;
                        nlread=0;
                    elseif ntread==nt,
                        state=state+2;
                    end;
                    [num,string]=strtok(string);
                    P.table{ntread}.label=sscanf(num,'%i');
                    typ=strtok(string);
                    P.table{ntread}.typ=typ;
                    if typ(1)=='h',
                        % Hermite spline needs derivatives
                        P.table{ntread}.data=zeros(nl,3);
                    else
                        P.table{ntread}.data=zeros(nl,2);
                    end;
                    continue;
                otherwise
                    err=['undefined state ',num2str(state),...
                        ' in list with leading %'];
                    break;
            end;
        else
            % contents descriptor expected
            if string(2:5)~=expect(5:8),
                err=['unexpected descriptor at line ',num2str(line)];
                break;
            end;
            % desired descriptor found
            % skip states if corresponding list is empty
            if     state== 6 && nd==0, state= 8;
            elseif state== 8 && nC==0, state=11;
            elseif state==13 && ni==0, state=15;
            elseif state==15 && nc==0, state=17;
            elseif state==17 && nv==0, state=20;
            elseif state==20 && ns==0, state=24;
            elseif state==24 && ne==0, state=26;
            elseif state==26 && nu==0, state=29;
            elseif state==29 && nt==0, state=32;
            elseif state==32 && vn==0, state=34;
            else                      state=state+1;
            end;
            % compatibility patch for missing original variable names
            if state==32 && vn==-1, state=34; end
            continue
        end;
    end;

    % now we have a line without leading % sign
    switch state
        case 3 % problem identifier
            P.name=strtok(string);
            if ~strcmp(probname,P.name),
                err=['mismatch of probname and problem identifier '...
                    probname '!=' P.name];
                break;
            end;
            state=state+1;
        case 5 % list of sizes
            sizes=sscanf(string,'%i');
            nd=sizes(1);P.desc=cell(nd,1);P.Q.desc=[];P.xref=[];
            nC=sizes(2);P.const=cell(nC,1);P.C=[];P.CC=[];
            nA=sizes(3);P.coef=zeros(nA,3);P.Q.A=[];P.Q.hess=[];
            nx=sizes(4);P.Q.int=zeros(nx,1);
            no=sizes(5);P.Q.nxo=no;
            reffile=[fpath, '/' probname,'.ref'];
            if exist(reffile,'file')==2,
                P.xref=load(reffile);
                P.xref(:,1)=[];
                if size(P.xref,1)~=nx,
                    gldisp('nx',nx,'dimref',size(P.xref,1));
                    err='reference vectors have wrong dimension';
                    break;                
                end;            
            else
                P.xref=zeros(nx,0);
            end;
            ni=sizes(6);
            nc=sizes(7);P.ibounds=zeros(nc,2);P.Q.F=[];
            ns=sizes(8);P.set=cell(ns,1);P.setx=zeros(ns,1);P.Q.gaps=[];
            nB=sizes(9);ne=nB-nx-nx^2;P.basis=zeros(ne,3);
            nu=sizes(10);P.uni=cell(nu,1);P.ucode=[];
            nt=sizes(11);P.table=cell(nt,1);
            if length(sizes)>11,
                vn=sizes(12);
                if vn~=0 && vn~=no,
                    gldisp('no',no,'vn',vn);
                    err='original variable name list has wrong dimension.';
                    break;
                end;
                P.origvarnams=cell(vn,1);
            else
                % compatibility patch for missing original var name number
                P.origvarnams=[];
            end
            state=state+1;
        case 7 % informal description
            ndread=ndread+1;
            P.desc{ndread}=string;
            if ndread==nd, state=state+1;end;
        case 10 % list of indices defining matrix entries
            nCl=nCl+1;
            nlread=nlread+1;
            P.coef(nCl,3)=nCread;
            P.coef(nCl,1:2)=sscanf(string,'%i')';
            if nlread==nl,
                if nCread<nC, state=state-1;
                else          state=state+1;
                    if nAread~=nA,
                        err=['wrong number of matrix entries ' ...
                            num2str(nA) '!=' num2str(nAread)];
                        break;
                    end;
                end;
            end;
        case 12 % list defining objective
            num=sscanf(string,'%i')';
            P.shift=num(1);P.opt=abs(num(2));P.type=sign(num(2));
            state=state+1;
        case 14 % list defining an integer variable
            niread=niread+1;
            int=sscanf(string,'%i')';
            P.Q.int(int)=1; % set entry in characteristic vector
            if niread==ni, state=state+1;end;
        case 16 % list of indices defining constraint bounds
            ncread=ncread+1;
            num=sscanf(string,'%i');
            if num(1)~=ncread,
                gldisp('num',num,'ncread',ncread);
                err=['mismatch in numbering of constraint bounds ' ...
                    num2str(num(1)) '!=' num2str(ncread)];
                break;
            end;
            P.ibounds(ncread,:)=num(2:3)';
            if ncread==nc, state=state+1;end;
        case 21 % list with index defining finite variables
            nsread=nsread+1;
            P.setx(nsread)=sscanf(string,'%i')';
            if nsread==ns, state=state+1;nsread=0;end;
        case 23 % list of indices defining set constraint
            P.set{nsread}=sscanf(string,'%i')';
            num=length(P.set{nsread});
            if num~=nl,
                err='mismatch in length of set constraint';
                break;
            end;
            if nsread<ns, state=state-1;
            else          state=state+1;
            end;
        case 25 % list of indices for nonquadratic basis functions
            neread=neread+1;
            num=sscanf(string,'%i');
            if num(1)~=neread,
                err='mismatch in numbering of basis functions';
                break;
            end;
            P.basis(neread,:)=num(1:3)';
            if neread==ne, state=state+1;end;
        case 28 % string defining univariate function
            nlread=nlread+1;
            P.uni{nuread}= [P.uni{nuread},' ',string];
            if nlread==nl,
                if nuread<nu, state=state-1;
                else          state=state+1;
                end;
            end;
        case 31 % list of table entries
            nlread=nlread+1;
            num=sscanf(string,'%f');
            P.table{ntread}.data(nlread,:)=num';
            if nlread==nl,
                if ntread<nt, state=state-1;
                else          state=state+1;
                end;
            end;
        case 33 % original variable names
            vnread=vnread+1;
            [t,nam]=strtok(string);
            num=str2num(t);
            if isempty(num) || num~=vnread,
                err='mismatch in numbering of the variable names';
                break;
            end;
            P.origvarnams{vnread}=nam(2:end);
            if vnread==no, state=state+1;end;
        otherwise
            err=['undefined state ',num2str(state),...
                ' in list without leading %'];
            break;
    end;
end;
if ~isempty(err),
    % an error has occured
    %     disp('nlread,ndread,nCread,nAread,niread')
    %     disp([nlread,ndread,nCread,nAread,niread])
    %     disp('ncread,nvread,nsread,neread,nuread')
    %     disp([ncread,nvread,nsread,neread,nuread])
    err=['Problem file ' fname ' parsing error:' err ' at line('...
        num2str(line) '): ' string ', state:' num2str(state)...
        ', expected contents: ' expect];
end

fclose(fid);