%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glrandomprob.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function P=glrandomprob(nx, rel, type, bound, center);
% Create nc random constraints; if supplied bound is a fixed boundary point,
% center is the center of the ellipse and rel is a relation (<=, =, >=)
%
% nc            number of constraints in the problem
% rel           ncx1 string of the constraint relations '<', '=' or '>',
%               random relation if empty entry in cell
% type          ncx1 string of the the constraint types: 
%               'l' - linear, 'e' - ellipsoid, 'c' - conic
% bound         boundary point, optional 
% center        center point of the ellipse, optional
%
% P            structure containing the internal problem definition
%              C is in P.CC (approx. in P.C)
%              A is in P.Q.A, \F is in P.Q.F, 
%              B(x) has the components
%                   B_k        = x_k        (k=1:nx) 
%                   B_(j+nk)   = x_j x_k    (j,k=1:nx)
%                   B_i        = uni_j(x_k) (i=nx^2+nx+1:nB)
%              defined by P.basis(i,:)=[j,k] and P.ucode
%              int is in P.int, set_k and N_k are in P.set*
%   P.name     problem name
%   P.desc     informal problem description
%   P.basis    ne x 2 array defining nonquadratic basis functions(empty)
%   P.uni      nu x 1 cell array defining univariate functions(empty)
%   P.shift    constant index  in objective function (0)
%   P.opt      variable index  in objective function (0)
%   P.Q        structure encoding the quadratic part
%      .A         sparse matrix of constraint coefficients
%        .inf,.sup  bounds for coefficients
%        .nz        incidence matrix
%      .hess      sparsity pattern of Hessian of Lagrangian
%      .int       characteristic vector of integer variables
%      .F         structure with bounds for general constraints
%      .gaps      ng x 3 array with rows [k a b] specifying finite gaps
%                    x_k notin ]a,b[ (empty)
%      .Bu        ne x 5 array with entries Bu(i,:)=[k,a,b,c,d]
%                 defining quadratic enclosures for nonquadratic
%                 basis functions (empty)

%        .inf,.sup    outer enclosure

function P=glrandomprob(nx, rel, type, bound, center)

global glconf;          %global configuration variable
RANDCOEFR=[-5 5];       %range of the random matrix coeffs
RANDBOUNDR=[-5 5];      %range of the random boundary point
RANDMIDR=[-5 5];        %range of the random midpoint

%argcheck
if nargin < 3
    error('glrandomprob: not enough arguments!');
end
if nargin < 4, bound=[];end
if nargin < 5, center=[];end
% argcheck

% set constants from config
if isfield(glconf,'randcoefr'),RANDCOEFR=glconf.randcoefr;end 
if isfield(glconf,'randboundr'),RANDBOUNDR=glconf.randboundr;end
if isfield(glconf,'randmidr'),RANDMIDR=glconf.randmidr;end
% create constraints matrix A and the box F
nc=length(rel);
A=zeros(nc,nx+nx^2);
F.inf=-inf+zeros(nc,1);F.sup=inf+zeros(nc,1);
% create A
% random boundary point
%left side ||R*(x-c)||^2-||Rc||^2=A
typestr=[];relstr=[];

for con=1:nc
    % (1) SET OR RANDOMIZE THE BOUNDARY AND CENTER POINTS AND THE 
    % CONSTRAINT TYPE
    if isempty(bound),
        % random boundary point
        b=round(RANDBOUNDR(1)+(RANDBOUNDR(2)-RANDBOUNDR(1))*random(nx,1));
    else
        % fixed boundary point
        b=bound;
    end
    if isempty(center),
        % random center point
        c=round(RANDMIDR(1)+(RANDMIDR(2)-RANDMIDR(1))*random(nx,1));
    else
        % fixed center point
        c=center;
    end
    if all(c==b),
        % center equals the boundary, correct it.
        c=b-ones(nx,1); 
    end
    if isempty(type(con)) || type(con)=='r'
        % random type
        r=rand;
        if r < 1/3, type(con)='l';
        elseif r < 2/3, type(con)='e';
        else  type(con)='c';
        end
    end
    % (2) CREATE THE LEFT HAND SIDE OF THE CONSTRAINT
    if type(con)=='l'
        % linear constraint, set random linear coefficients
        while 1,
            p=round(RANDCOEFR(1)+(RANDCOEFR(2)-RANDCOEFR(1))*random(1,nx));
            if any(p~=0),
                A(con,1:nx)=p;
                break;
            end
        end
    elseif type(con)=='e' 
        % ellipsoid constraint
        % solve ||R(x-c)||^2 rel ||R(b-c)||^2
        % where R is a random nx x nx matrix
       while 1,
            R=round(RANDCOEFR(1)+(RANDCOEFR(2)-RANDCOEFR(1))*random(nx,nx));
            for i=1:nx
                % diag in [1 2]
                R(i,i)=1+round(abs(random));
            end
            if det(R'*R)~=0, break;end
        end
        % generate ellipse
        E=zeros(nx+1,nx+1);
        for i=1:nx
            for j=1:nx
                r=0;
                for k=1:nx
                    r=r+R(k,i)*R(k,j);
                end
                %lin
                E(i+1,1)=E(i+1,1)-r*c(j);
                E(j+1,1)=E(j+1,1)-r*c(i);
                %quad
                E(i+1,j+1)=r;
            end
        end
        p=glqmat2qvec(E);
        A(con,:)=p(2:end)';
    elseif type(con)=='c'
        % conic constraint
        A(con,:)=round(RANDCOEFR(1)+...
            (RANDCOEFR(2)-RANDCOEFR(1))*random(1,nx+nx^2));
    else
        error(['glrandomprob: unknown type: ' type{i} ' found!']);
    end
    % (3) CREATE THE CONSTRAINT BOUNDS
    xmat=b*b';bb=[b ; xmat(:)];
    aa=A(con,:);FF=infsup(aa,aa)*bb;
    if isempty(rel(con)) || rel(con)=='*'
        % random relation
        r=rand;
        if r < 1/3, rel(con)='<';
        elseif r < 2/3, rel(con)='>';
        else  rel(con)='=';
        end
    end
    if rel(con)=='<'
        F.inf(con)=-inf;F.sup(con)=sup(FF);
    elseif rel(con)=='>'
        F.inf(con)=inf(FF);F.sup(con)=inf;
    elseif rel(con)=='='
        F.inf(con)=inf(FF);F.sup(con)=sup(FF);
    else
        error(['glrandomprob: unknown realtion: ' rel(con) ' found!']);
    end
end
% generate problem definition
P.name = ['random_prob_' num2str(round(1000*rand))];
for con=1:nc
    relstr=[relstr rel(con) ' '];
    typestr=[typestr type(con) ' '];
end
if isempty(bound),boundstr='random boundary';
else boundstr=['bound=(' num2str(bound') ')'];
end
if isempty(center),centerstr='random center';
else centerstr=['center=(' num2str(center') ')'];
end
P.desc = [{['Random problem generated by GloptLab at ' date]};...
          {['Input parameters: nx=' num2str(nx) ', types=(' typestr...
            '), rels=(' relstr '), ' boundstr ', ' centerstr '.']}]; 
P.basis = [];
P.uni = [];
P.shift = 0;
P.opt = 0;
P.Q.A.inf = A; P.Q.A.sup = A;
P.Q.int = zeros(nx,1);
P.Q.nxo = nx;
P.Q=glfindspars(P.Q);
P.Q.F = F;
P.Q.Bu = [];
P.Q.gaps = [];
P.xbox.inf=-inf+zeros(nx,1);
P.xbox.sup=inf+zeros(nx,1);
P.Q.vmask=[];
P.Q.vperm=[];
P.Q.xref=[];
P.Q.haseq=any(F.sup-F.inf==0);
P.origvarnams=[];
P.solstatus=[];
