%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glread.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [P, errmsg] = glread(fname, box);
% reads a global optimization problem 
%    min  C_shift+-F_opt (but C_shift if opt=0)
%    s.t. Aq(x) in \F,
%         x_i = uni_j(x_k), (i,j,k) in basis
%         x_int integer,
%         x_{set_k} in N_k (k=1:ns),
% specified as described in glinput.def
% x is the vector of independent variables to be found 
%
% fname        file name of the .def file, with full path.
% box          read the box from a .glb ? 0=no 1=yes
%
% P            structure containing the internal problem definition
%              C is in P.CC (approx. in P.C)
%              A is in P.Q.A, \F is in P.Q.F, 
%              q(x) has the components
%                   q_k             =  x_k        (k=1:nx) 
%                   q_(j+nk)        =  x_j x_k    (j,k=1:nx)
%              and additional definitions:
%                   basis           in P.basis(t,:)=[j,k,i] and P.ucode,
%                   int             in P.int, 
%                   set_k and N_k   in P.set*
%     .name    problem name
%     .desc    informal problem description
%     .const   nC x 1 cell array of symbolic constants
%     .coef    nA x 3 matrix of indices [i,j,k] defining A(i,j)=C(k)
%     .xref    nx x nref array of reference vectors
%     .ibounds nc x 2 array with indices for constraint bounds 
%     .set     ns x 1 cell array defining set constraints
%              N_k = {C(i) | i in P.set{k}}
%     .setx(k) = set_k, index of variable with x_{set_k} in N_k
%     .basis   ne x 3 array defining nonquadratic basis functions
%     .uni     nu x 1 cell array defining univariate functions
%     .table   nt x 1 cell array of tables
%     .shift   constant index  in objective function C_shift+type*u_opt
%     .opt     variable index  in objective function 
%     .type    factor (0,1,-1) in objective function 
%     .Q       structure encoding the quadratic part
%      *.int      characteristic vector of integer variables
%      *.nxo      number of the original variables before conversion
%                 from .dag to .def
%  errmsg        error message, if empty all is ok.

function [P, errmsg] = glread(fname, box)

% argcheck
if nargin<1, error('glread: Not enough arguments!'); end
if nargin<2, box=1; end

P=[];errmsg=[];

if exist(fname,'file')~=2,
    % file not found
    errmsg=['Problem file ' fname ' not found!'];
elseif ~isempty(findstr(fname, '.glb')),
    % .glb file
    load(fname, '-mat');
elseif ~isempty(findstr(fname, '.def')),
    % .def file
    [P, errmsg] = gldefparser(fname);
else
    % unsupported file
    errmsg=['Problem file ' fname ' cannot be read, '...
        'it has an unsupproted file type!'];
end

if isempty(errmsg),
    % problem file read
    if (~isfield(P,'C') || isempty(P.C)) && isfield(P, 'const')
        % translate constants
        errmsgc='error in constant definitions:';
        [P.C,P.CC,err]=glconst(P.const,errmsgc);
        if ~isempty(err),
            errmsg=['Problem file ' fname ' parsing error: '...
                errmsgc '!'];
            return;
        end
    end
    
    if ~isfield(P,'ucode') || isempty(P.ucode)
        % translate univariate functions
        errmsgu='error in definition of univariate functions:';
        [P.ucode,ieru]=glucode(P.uni,errmsgu);
        if ieru>0,
            errmsg=['Problem file ' fname ' parsing error: '...
                errmsgu '!'];
            return;
        end
    end
    
    if ~isfield(P.Q,'F') || isempty(P.Q.F)
        % create coefficient matrix
        [P.Q.F,P.Q.A,hess,ierc]=glcoef(P);
        if ierc>0,
            errmsg=['Problem file ' fname ' parsing error: '...
                'bad general constraints!'];
            return;
        end
        P.Q.hess=hess;
        P.Q.desc=P.desc;
    end
    
    if ~box || ~isfield(P, 'xbox') || isempty(P.xbox)
        % set xbox
        P.xbox.inf=-inf+zeros(length(P.Q.int),1);
        P.xbox.sup=inf+zeros(length(P.Q.int),1);
    end

    if (~isfield(P.Q,'gaps') || isempty(P.Q.gaps)) && isfield(P,'setx')
        % create bounds and gaps
        [P.xbox, P.Q.gaps]=glgaps(P, P.xbox);
    end

    if (~isfield(P.Q,'Bu') || isempty(P.Q.Bu))
        % rigorous quadratic enclosure for nonquadratic basis functions
        [P.Q.Bu, P.xbox]=gluniq(P, P.xbox, 0);
    end

    % check dimensions
    nx=length(P.xbox.inf);nc=length(P.Q.F.inf);nB=(nx+1)*nx;
    if any(size(P.Q.A.inf)~=[nc nB])
        errmsg=['Problem file ' fname ' corrupt, dimesnsion mismatch!'];
        return
    end
    
    % find equalities
    P.Q.hasequ=any(P.Q.F.sup-P.Q.F.inf==0);
    
    if ~isfield(P.Q,'vmask'),
        % set varaible mask and perm for the conic programms
        P.Q.vmask=[];
        P.Q.vperm=[];
    end
    
    % delete found boxes && stack
    if isfield(P, 'boxes');P=rmfield(P,'boxes');end
    if isfield(P, 'bstack');P=rmfield(P,'bstack');end
    
    if isfield(P,'xref'),
        % check if all the reference points are feasible or not.
        % in case of nonfeasibility gives a warning and relaxes the problem
        % in order to make the point feasible.
        P.Q.F=glrefcheck(P, 0);
    end
    
    if ~isfield(P,'origvarnams')
        P.origvarnams=[];
    end
    
    P.solstatus=[];
end