%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glreadinfo.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [P, errmsg] = glreadinfo(prob);
%
% get the problem parameters for the given problems or reads them from 
% the given .glb or .def file.
%
% prob      problem definition struct of the file name of the .glb or 
%           .def file, with full path.
%
% info      problem information structure
%  .name    problem name
%  .desc    problem description
%  .nx      number of variables
%  .no      number of original variables (before conversion)
%  .ni      number of integer variables
%  .nc      number of constraints
%  .nB      number of basis functions (now == nA)
%  .ne      number of constants
%  .nu      number of univariate basis functions
%  .nA      number of rows in the matrix A
%  errmsg        error message, if empty all is ok.

function [info, errmsg] = glreadinfo(prob)

% argcheck
if nargin<1, error('glreadinfo: Not enough arguments!'); end

info=[];errmsg=[];
if isstruct(prob)
    %existing problem
    info.name=prob.name;
    info.desc=prob.desc;
    info.nx=length(prob.Q.int);
    info.no=prob.Q.nxo;
    info.ni=nnz(prob.Q.int);
    info.nc=size(prob.Q.A.inf,1);
    info.nB=size(prob.Q.A.inf,2);
    info.ne=info.nB-info.nx-info.nx^2;
    info.nu=length(prob.uni);
    info.nA=nnz(prob.Q.A.inf | prob.Q.A.inf);
else
    ind=find(prob=='.');ind=ind(end);
    pref=prob(1:ind-1);ext=prob(ind+1:end);
    if exist(prob,'file')~=2,
        % file not found
        errmsg=['Problem file ' prob ' not found!'];
    elseif strcmp(ext,'glb')
        % .glb file
        inffile=[pref '.inf'];
        if exist(inffile,'file')==2
            %info file found
            info=glload([prob(1:find(prob=='.')-1) '.inf']);
        else
            %load(prob, '-mat');
            [P, errmsg] = glread(prob,0);
            info.name=P.name;
            info.desc=P.desc;
            info.nx=length(P.Q.int);
            info.no=P.Q.nxo;
            info.ni=nnz(P.Q.int);
            info.nc=size(P.Q.A.inf,1);
            info.nB=size(P.Q.A.inf,2);
            info.ne=info.nB-info.nx-info.nx^2;
            info.nu=length(P.uni);
            info.nA=nnz(P.Q.A.inf | P.Q.A.inf);
        end
    elseif strcmp(ext,'def')
        % .def file
        fid=fopen(prob);str='';
        % get the name
        while isempty(strfind(str,'gloptlab input file'))
            str=fgetl(fid);
        end
        info.name=fgetl(fid);
        % get the parameters
        while isempty(strfind(str,'problem size'))
            str=fgetl(fid);
        end
        str=str2mat(fgetl(fid));sizes=sscanf(str,'%i');
        info.nC=sizes(2);info.nA=sizes(3);info.nx=sizes(4);info.no=sizes(5);
        info.ni=sizes(6);info.nc=sizes(7);info.ns=sizes(8);info.nB=sizes(9);
        info.nu=sizes(10);info.nt=sizes(11);info.ne=info.nB-info.nx-info.nx^2;
        % get the description
        while isempty(strfind(str,'description'))
            str=fgetl(fid);
        end
        info.desc=cell(0);
        in=fgetl(fid);
        while isempty(strfind(in,'constants'))
            info.desc=[info.desc {in}];
            in=fgetl(fid);
        end
        fclose(fid);
    else
        % unsupported file
        errmsg=['Problem file ' prob ' cannot be read, '...
            'it has an unsupproted file type!'];
    end
end
