%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcertcheck.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [certs, out, timer]=glcertcheck(P, x, certs, minreduc, round, prec, 
%                                   maxerr, timer, prt, deb)
%
% Throw out certificates of the task which results in a too small gain,
% recompute the remaining certificates.
%
% P         original problem, if the certs begin with a readproblem it
%           can be empty
% x         starting xbox, if the certs begin with a readproblem it
%           can be empty
% certs     certificate cell array to be checked
%  {cert}   certificate structure
%   .name   name of the certificate
%   .tname  name of the task used to aquire the certificate
%   .x      the new box after finishing the task
%   .(vars) different additional variables
% minreduc  minimal relative reduction of the x, a certificate has to 
%           bring in order to keep it in the certs cell array.
% round     round the data in the certificates 0 = off 1 = on
% prec      precision, the number of digits the data of the certs will be
%           rounded.
% maxerr    the maximum relative error the rounding process can result.
%           In some cases this parameter allows a rounding with less
%           digits than given by the parameter prec.   
% timer     timer structure (see gltimer)
% prt       text output flag       0 = no output 1 = output
% deb       debug output flag      0 = no output 1 = output
%
% certs     new certificate cell array or [] if the check failed
% out       text output as cell
% timer     timer structure (see gltimer)

function [certs, out, timer]=glcertcheck(P, x, certs, minreduc, round, ...
                                    prec, maxerr, timer, prt, deb)

% arg check
if nargin<4,error('glcertcheck: Not enough arguments!');end
if nargin<5,round=0;end
if nargin<6,prec=5;end
if nargin<7,maxerr=1e-5;end
if nargin<8,timer=glcreatetimer(inf, inf, 0, 0, inf, '', '');end
if nargin<9,prt=0;end
if nargin<10,deb=0;end

% text output
if prt, out={'*** CHECKING CERTIFICATES ***'};
else out=[];
end

% init
throwout=1;  % array of the indices of the certificates which have been
             % thrown out in the last step
totalthrow=0;% total number of certifictes thrown out
readsave=[]; % saved read problem certificate (if any exists)   
cn=length(certs);

if cn==0 || (cn==1 && strcmp(certs{1}.tname,'Read'))
    if prt,out=[out ; {'No certificates found, checking skipped.'}];end
    if deb, disp('No certificates found, checking skipped.');end
end

% remove and save read problem certificate (if exists)
if strcmp(certs{1}.tname,'Read')
    % read problem certificate found, save it and set the original
    % P and x
    if deb, disp('Read Problem certificate removed.');end
    readsave=certs{1};       % save cert
    certs(1)=[];             % remove cert from the certs cell array
    orig.P=readsave.P;       % save original problem
    orig.x=readsave.x;       % save starting x
    cn=cn-1;
else
    orig.P=P;                % save original problem 
    orig.x=x;                % save starting x
end


% loop until all tasks corresponding to the certificates in the certificate
% array improve x at least by the supplied minimal reduction (minreduc)
while 1
    %(1) init
    P=orig.P;               % reset to original problem 
    x=orig.x;               % reset to starting x
    nx=length(x);           % number of vars
    throwout=[];            % clear the throw out array
    if deb,
        s='Certificates (taskname): ';
        for i=1:cn, s=[s certs{i}.tname '(' certs{i}.task.name '), '];end
        disp(s);
    end
    %(2) process all certificates, and check the minreduc, create the 
    % throwout indices cell
    for i=1:cn
        cert=certs{i};
        if deb, disp(['(' num2str(i) ') Processing ' cert.tname '...']); end
        if strcmp(cert.tname,'Simplify')
            % simplify does not have to bring a gain and is kept if
            % it removes constraints and/or variables
            if any(size(P.Q.A.inf) ~= size(cert.P.Q.A.inf))
                % simplify changed the number of vars or constraints
                P=cert.P;       % P set to the simplified problem 
                x=cert.x;       % x set to the x after the simplification
                nx=length(x);   % new number of wars
            else
                % simplify did not do anything, throw it out
                throwout=[throwout i];
            end
        elseif strcmp(cert.tname,'Split')
            % split,
            for j=1:length(cert.boxes)
                subbox=cert.boxes{j};
                if isempty(subbox.startbox)
                    gldisp('subbox',subbox);
                    error('glcertcheck: empty startbox found in Split!');
                end
                if deb,disp(['Processing subbox no.' num2str(j) ' ...']);end
                [cert.boxes{j}.certs,out2,timer]=glcertcheck(P, ...
                    subbox.startbox, subbox.certs, minreduc, round, ...
                    prec, maxerr, timer, prt, deb);
            end
        elseif strcmp(cert.tname,'Merge Boxes (cluster)')
        elseif strcmp(cert.tname,'Unrigorous')
            if prt,
                out=[out ;{['ERROR: Unrigorous method found, '...
                    'no proof can be generated!']};
                    {'*CHECKIN CERTIFICATES END*'}];
            end
            if deb,
                disp(['ERROR: Unrigorous method found, '...
                    'no proof can be generated!']);
            end
            certs=[];
            return
        else
            %if isempty(cert.x), name=cert.name; end
            %calcualte reduc
            if isempty(cert.x), reduc=ones(nx,1);
            else reduc=glgain(x, cert.x);
            end
            
            %check mingain
            if all(reduc<minreduc), throwout=[throwout i];end
            
            %debug
            if deb,disp([cert.tname ' reduc: ' num2str(reduc')]);end
            
            %save reduc in cert
            certs{i}.reduc=reduc;
            
            %set new x
            x=cert.x;
        end
    end

    %(3) throw out the certificates with to small reduc
    if deb, 
        disp(['Indices of the thrown out certificates: ['...
            mat2str(throwout) ']']);
    end
    if isempty(throwout), break; end
    certs(throwout)=[];
    cn=length(certs);
    totalthrow=totalthrow+length(throwout);
    
    %(4) make new task list
    tasklist=cell(cn,1);
    for i=1:cn
        if deb,disp(['Generated task for ' certs{i}.tname '...']);end
        tasklist{i}=certs{i}.task;
        certs{i}=rmfield(certs{i}, 'task');
        tasklist{i}.cert=certs{i};
    end

    %(5)process tasklist, starttask=1, nopause=1, prt=0, gra=0, pro=1,
    if ~isempty(tasklist)
        [P, x, status, taskpos, timer, certs]=glproctskl(orig.P, orig.x,...
        tasklist, 1, timer, 1, 0, 0, 1); 
    end
end
if prt,
    out=[out ;{['Certificates thrown out:' num2str(totalthrow)]};
        {['Certificates remain:' num2str(length(certs))]}];
end
if ~isempty(readsave)
    % set back read problem certificate
    certs=[{readsave};certs];
end
