%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glproof.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glproof(certs)
%
% make a proof from the supplied certificate array.
%
% certs         certificate cell array

function glproof(certs, proofdir)

%argcechk
if nargin<2, error('glproof: Not enough arguments!');end
if ~strcmp(certs{1}.tname,'Read')
    error('glproof: Corrupt certificate array found!');
end
indexfile=[proofdir 'index.html'];

%make a proof directory if none exists, and confirmed
if exist(proofdir,'dir') ~= 7,
    h = questdlg('Directory for the proofs does not exists, Create it?',...
        'Create Directory','Yes','No','Yes');
    if strcmp(h,'Yes')
        mkdir(proofdir);
        copyfile(which('newindex.html'),indexfile);
    else
        warndlg('Directory needed to store the proofs!','Warning');
        return;
    end
end

%make Proof Index File if none exists
if exist(indexfile,'file')~=2
    copyfile(which('newindex.html'),indexfile);
end

%get the original problem
P=certs{1}.P;
xbox=certs{1}.x;
probname=P.name;
nx=length(P.Q.int);

%get result
result=certs{end};

%open new problem file
fid = fopen([proofdir probname '.html'],'w');
%title
out=['<HTML>\n'...
    '<HEAD>\n'...
    '<TITLE>Automaic Proof for ' probname '</TITLE>\n'...
    '</HEAD>\n'...
    '<BODY bgcolor="#ffffff">\n'...
    '<H1><P ALIGN=CENTER>Proof for ' probname '</H1>\n'];
out=[out '<A HREF="index.html">back</A><P>\n'...
    '<HR>\n'];
%write the original problem
[probstr, equnums]=glprob2str(P, xbox, 1, [], 1);
lastnum=max(equnums)+1;
box=[];
if strcmp(result.tname,'Split') || strcmp(result.tname,'Merge Boxes (cluster)')
    boxnum=0;
    if length(result.boxes)>1
        [b, boxnum]=glboxes2html(result.boxes, boxnum);
        probstr=[probstr 'satisfies one of <BR>\n' b];
    else
        box=result.boxes{1};
        probstr=[probstr 'satisfies <BR>\n' glbox2html(box, 1) '.<BR>\n'];
        boxnum=1;
    end
elseif strcmp(result.tname,'Feas')
    probstr=[probstr 'has a feasible point in <BR>\n' ...
        glbox2html(result.x, 1) '.<BR>\n'];
    boxnum=1;
else
    box=result.x;
    if ~isempty(box),
        probstr=[probstr 'satisfies <BR>\n' glbox2html(box, 1) '.<BR>\n'];
        boxnum=1;
    else
        probstr=[probstr 'has no solution.<BR>\n'];
        boxnum=0;
    end
end

if boxnum~=0,
    probstr=['<B>Theorem:</B> Any x &isin; R <sup> ' num2str(nx)...
        '</sup> with <BR>\n' probstr];
else
    probstr=['<B>Theorem:</B> For all x &isin; R <sup> ' num2str(nx)...
        '</sup> the problem <BR>\n' probstr];
end
out=[out probstr];
%proofs
out=[out '<HR><B>Proof:</B>\n'];
out=[out '<!-- Proofs begin -->\n'];
out=[out '<!-- Proofs end -->\n'];
%end
out=[out '<A HREF="index.html">back</A><P>\n'];
out=[out '</BODY>\n</HTML>\n'];
%write file
fprintf(fid,out);
%close file
fclose(fid);

%set problem entry in index when its not already there
%read index file
fid = fopen(indexfile,'r');
out='';in=0;found=0;
txt=['<P> - <A HREF="' probname '.html">' probname ...
    '</A> - proof generated at ' datestr(now) '<P>\n'];
while 1
    string=fgetl(fid);   % reads next line without line feed 
    if ~ischar(string),
        % end of file reached
        break;
    elseif strcmp(string,'<!-- Problems begin -->')
        % entered problem listing
        in=1;
    elseif in==1 && ~isempty(findstr(string,['>' probname '</A> - ']))
        % index already exists, replace it with the new index
        out=[out txt];
        found=1;
        continue;
    elseif in==1 && ~found && strcmp(string,'<!-- Problems end -->')
        out=[out txt];
    end
    out=[out string '\n'];
end
fclose(fid);
% rewrite the index file
fid = fopen(indexfile,'w');
fprintf(fid,out);
fclose(fid);


%process all certificates
for i=2:length(certs)
    %(1)make a proof entry from a certificate
    cert=certs{i};
    entry=[];
    if strcmp(cert.tname,'Unrigorous')
        %not a valid entry, (unrigorous)
        entry=[];
        return
    end

    %init
    entry.name=cert.name;
    entry.fname=[num2str(i-1) '_' cert.tname];
    entry.outtext=[entry.name '(unfinished)'];
    entry.text=[entry.name '(unfinished)'];

    %result boxes
    if i==length(certs), boxnum=0; end
    
    %make the text of the proof entry
    switch cert.tname
        case 'Read'
            error('glproof: Corrupt certificate array found!');
        case 'Simplify'
            %get new xbox
            xbox=cert.x;
            %get new problem
            P=cert.P;
            %remove equation numbers
            equnums(:)=0;
            %generate entry
            [probstr,equnums]=glprob2str(P,xbox,1,equnums,1);
            lastnum=max(equnums)+1;
            entry.outtext=['We put the constraints into the form:'...
                probstr];
            entry.text=['<B>Proof entry:</B><BR>\n'...
              'Problem simplified to:<BR>\n' probstr...
              'by removing bound constraints and unused variables.<BR>\n'];
        case 'SepPropagate'
            %get new xbox
            xbox=cert.x;
            %set new F
            P.Q.F=cert.F;
            if ~isempty(xbox)
                %get sizes
                nx=length(xbox.inf);
                nc=length(P.Q.F.inf);
                entry.outtext=['<I>' cert.name '</I> '...
                    '(see Chapter 5, Pages 18-22) using '];
                %constraint listing
                for i=1:nc
                    equnum=equnums(i);
                    if equnum~=0
                        entry.outtext=[entry.outtext '(<small>'...
                            num2str(equnum) '</small>), '];
                    end
                end
                %box listing
                cert
                I=find(cert.reduc>0)';
                for i=I
                    equnum=equnums(nc+i);
                    if equnum ~=0
                        entry.outtext=[entry.outtext '(<small>'...
                            num2str(equnum) '</small>), '];
                    end
                end
                entry.outtext=entry.outtext(1:end-2);
                entry.outtext=[entry.outtext ' reduces the bound '...
                    'constraints to:<BR>\n'];
                [b,equnums,lastnum]=glbox2html(cert.x,[],I,1,equnums);
                entry.outtext=[entry.outtext b '.\n'];
            else
                entry.outtext=['<I>' cert.name '</I> '...
                    '(see Chapter 5, Pages 18-22) shows that the '...
                    'problem is infeasible.\n'];
            end
            entry.text=['<B>Proof entry:</B><BR>\n'...
                'Problem changed to :<BR>\n' glprob2str(P,xbox,1)...
                'by completing a step of constraint propagation.\n'];
            %disp(entry.text)
            %disp(glbox2str(cert.x, 1:length(cert.x.inf),1))
        case 'LinPropagate'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 4, Pages 18-19) reduces the bound '...
                'constraints to <BR>\n' b '.\n'];
        case 'Ehull'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 6, Pages 29-31) reduces the bound '...
                'constraints to <BR>\n' b '.\n'];
        case 'LinearBound'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 4, Pages 16-18) reduces the bound '...
                'constraints to <BR>\n' b '.\n'];
        case 'LinearContract'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 4, Pages 11-15) reduces the bound '...
                'constraints to <BR>\n' b '.\n'];
        case 'LinearSolve'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 4, Pages 19-20) reduces the bound '...
                'constraints to <BR>\n' b '.\n'];
        case 'Conic'
            %get sizes
            nx=length(xbox.inf);
            nc=length(P.Q.F.inf);
            %outtext
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 7, Page 33-53) produces the inequality<BR>\n'...
                glqvec2str([-cert.Q.F.sup(1) cert.Q.A.inf],-1, 1)...
                '.<BR>\n'...
                'Applying constraint propagation gives '...
                'the new bound constraints <BR>\n'];
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[],I,1,equnums);
            entry.outtext=[entry.outtext b '.\n'];
            %text
            if exist([proofdir 'Reduce_Form_1.jpg'],'file')~=2
                copyfile(which('Reduce_Form_1.jpg'),...
                    [proofdir 'Reduce_Form_1.jpg']);
            end
            entry.text=['<B>Proof entry:</B><BR>\nLet <BR>\n'...
                '<img align=middle src="Reduce_Form_1.jpg"'...
                'alt="Form 1"><BR>\n'...
                'with beta=1, where <BR>\n'...
                '(1) !**!E_inf!%%2 and !**!E_sup!%%2 are the bounds of '...
                'the cofficient interval-matrix of the constraints '...
                'which have been trasformed to equalties'];
            if length(cert.slacks)==0
                entry.text=[entry.text '.<BR>\n'];
            elseif length(cert.slacks)==1
                entry.text=[entry.text ' by adding a slack variable x<small>'...
                    num2str(cert.slacks(1)) '</small>.<BR>\n'];
            else
                entry.text=[entry.text ' by adding ' num2str(length(cert.slacks))...
                    ' slack variables '...
                    'x<small>' num2str(cert.slacks(1)) '</small> ... '...
                    'x<small>' num2str(cert.slacks(end)) '</small>.<BR>\n'];
            end
            entry.text=[entry.text '(2) The vector u together with '...
                '!**!zeta!%%0 represents the quadratic approximation polynomial.<BR>\n'...
                '(3) The vector !**!z!%%0 and the matrix !**!Z!%%0 which is '...
                'strictly positive represent the multipicators for the bound and '...
                'the general constraints.<BR>\n'...
                '(4)The matrix !**!G!%%0 is symmetric and positive '...
                'semidefinite.<BR>\n'...
                'Since for all x in xbox the term z^TEq(x) is zero, (1 x)G(y)(1 x)^T >=0 '...
                '(1 xu-x x-xl)Z(1 xu-x x-xl)^T <0 we gain <BR>\n'...
                'u^Tq(x)<zeta, an enclosure for the bound constraints.<BR>\n'...
                'Completing this results:<BR>\n'...
                ' * ' glqvec2str([-cert.Q.F.sup(1) cert.Q.A.inf],-1, 1) ' for !**!u!%%0.<BR>\n'...
                'Standard constrtaint propagation used on the '...
                'new constraint(s) reduced the the bound constraints from '...
                '!**!xb!%%0 to !**!xbnew!%%0.<BR>\n'];
            entry.E_inf=cert.E.inf;entry.E_sup=cert.E.sup;
            entry.z=cert.z;
            entry.Z=cert.Z;
            entry.zeta=cert.zeta;
            entry.G=cert.B'*cert.B;
            entry.u=cert.u';
            entry.xb=[xbox.inf xbox.sup];
            entry.xbnew=[cert.x.inf xbox.sup];
            %set new xbox
            xbox=cert.x;
        case 'Split'
            [b,boxnum]=glboxes2html(cert.boxes, boxnum);
            entry.outtext=['By <I>' cert.name '</I> '...
                '(see Chapter 8, Pages 55-57) we get the following '...
                'box(es) <BR>\n' b '\n'];
        case 'Merge Boxes (cluster)'
            [b,boxnum]=glboxes2html(cert.boxes, boxnum);
            entry.outtext=['By merging the above boxes to '...
                '<I>clusters</I> of boxes '...
                '(see Chapter 7, Pages 57-59) we find '...
                '<BR>\n' b '\n'];
        case 'Merge Boxes (hull)'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I, 1, equnums);
            entry.outtext=['By computing the  <I>interval hull</I> '...
                'of the above boxes '...
                '(see Chapter 7, Pages 57-59) we find '...
                '<BR>\n' b '.\n'];
        case 'Feas'
            I=find(cert.reduc>0)';
            [b,equnums,lastnum]=glbox2html(cert.x,[], I,1, equnums);
            entry.outtext=['The algorithm <I>' cert.name '</I> '...
                '(see Chapter 11, Pages 71-74) verifies that there is a '...
                'feasible point in <BR>\n' b '.\n'];
    end
    %result boxes
    if i==length(certs), entry.outtext=[ entry.outtext...
            '<BR>This proves the assertion.']; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % write the proof entry in the proof file %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %open the problem file
    fid = fopen([proofdir probname '.html'],'r');
    out='';
    while 1
        string=fgetl(fid);   % reads next line without line feed
        if ~ischar(string),
            %end of file reached
            break;
        elseif strcmp(string,'<!-- Proofs end -->')
            %end of proofs reached, new entry index and outer text
            out=[out glsetproofvars(probname, proofdir, entry.outtext,...
                entry) '<P>See detailed <A HREF="' probname '_'...
                entry.fname '.html"> proof entry </A>.<P>\n<HR>\n'];
        end
        out=[out string '\n'];
    end
    %rewrite the problem file
    fid = fopen([proofdir probname '.html'],'w');
    fprintf(fid,out);
    fclose(fid);

    %create the entry file
    fid = fopen([proofdir probname '_' entry.fname...
        '.html'],'w');
    %title
    out=['<HTML>\n'...
        '<HEAD>\n'...
        '<TITLE>Proof entry of ' entry.name ' for the Problem '...
        probname '</TITLE>\n'...
        '</HEAD>\n'...
        '<BODY bgcolor="#ffffff">\n'...
        '<H1><P ALIGN=CENTER>Proof entry of ' entry.name ...
        ' for the Problem ' probname '</H1>\n <HR>\n'];
    out=[out glsetproofvars([probname '_' entry.fname], proofdir, ...
        entry.text, entry)];
    %end
    out=[out '<HR>\n'];
    out=[out '<A HREF="' probname '.html">back</A><P>\n'];
    out=[out '</BODY>\n</HTML>\n'];
    %write file
    fprintf(fid,out);
    %close file
    fclose(fid);
end

%--------------------------------------------------------------------------
%Set entry Vars
%--------------------------------------------------------------------------
function txt=glsetproofvars(fname, proofdir, text, entry)
%create htmls for the entry varibales, and set reference int text
%scan entry for variables
inds=strfind(text,'!**!')-1;%variable starting string
inde=strfind(text,'!%%')+4;%variable ending string
%text(inde(i)-1) is the output flag:
%                               0: only matrix output
%                               1: convert equations with qvec2str
%                               2: 0&1
varn=length(inds);
%set variable references in text and create variable htmls
if varn>0
    txt=text(1:inds(1));
    inds=[inds length(text)];
    for i=1:varn,
        %set variable reference
        var=text(inds(i)+5:inde(i)-5);
        txt=[txt '<A HREF="' fname '_' var '.html">' var '</A>'];
        txt=[txt text(inde(i):inds(i+1))];
        %create variable html
        flag=str2num(text(inde(i)-1));
        out2=['<HTML>\n'...
                '<HEAD>\n'...
                '<TITLE>The variable ' var '</TITLE>\n'...
                '</HEAD>\n'...
                '<BODY bgcolor="#ffffff">\n'...
                '<H1><P ALIGN=CENTER>The '];
        siz=size(entry.(var));
        if flag ==1,
            out2=[out2 'equation'];
        elseif siz(1)==1 && siz(2)==1,
            out2=[out2 'constant'];
        elseif siz(1)>1 && siz(2)>1,
            out2=[out2 'matrix'];
        else
            out2=[out2 'vector'];
        end
        out2=[out2 ' ' var '</H1>\n'];
        %matrix representation
        if flag == 0 || flag==2,
            if siz(1)>siz(2),
                entry.(var)=entry.(var)';
                out2=[out2 '<HR>\n' var '^T=<BR>\n<PRE><I>'];
            else
                out2=[out2 '<HR>\n' var '=<BR>\n<PRE><I>'];
            end
            ce=glmat2scell(entry.(var));
            for i=1:length(ce),
                out2=[out2 ce{i} '<BR>\n'];
            end
            if siz(1)>siz(2),entry.(var)=entry.(var)';end;
            out2=[out2 '</I></PRE>\n<HR>\n'];
        end
        %equation representation
        if flag == 1 || flag==2,
            if flag==1
                out2=[out2 '<HR>\n'];
            else
                out2=[out2 'Which represents: <P>\n'];
            end
            for i=1:size(entry.(var),1),
                out2=[out2 glqvec2str(entry.(var)(i,:), 0, 1) '<BR>\n'];
            end
            out2=[out2 '<HR>\n'];
        end
        out2=[out2 '<A HREF="' fname '.html">back</A><P>\n</BODY>\n</HTML>\n'];
        %write variable html
        fid2 = fopen([proofdir fname '_' var '.html'],'w');
        fprintf(fid2, out2);
        fclose(fid2);
    end
else
    txt=text;
end

function [out, equnums, nextnum]=glbox2html(box, bnum, I, multiline, equnums)
    %argcheck
    if nargin<1, error('glbox2html: not enough arguments!');end
    if nargin<2, bnum=[];end
    if nargin<3, I=1:length(box.inf);end
    if nargin<4, multiline=0;end
    if nargin<5, equnums=[];nextnum=0;
    else nextnum=max(equnums)+1; end
    
    if ~isempty(box),
        %init
        nx=length(box.inf);
        nc=length(equnums)-nx; 
        %make box number
        if ~isempty(bnum), out=['<I>(<small>b' num2str(bnum) '</small>) '];
        else, out='<I>';
        end
        %get box html string cell
        if ~isempty(equnums),
            [b, equn]=glbox2str(box, I, 1, zeros(nx,1), nextnum);
            equnums(nc+I)=equn(I);
            nextnum=max(equnums)+1;
        else b=glbox2str(box, I, 1);
        end
        %output all selected box components
        for i=1:length(b)
            out=[out '<small>' b{i} '</small>'];
            if i<length(b),
                out=[out ', '];
                if multiline, out=[out '<BR>\n'];end
            end
        end
        %finish box
        out=[out '</I>'];
    else
        out=' an empty box';
    end
    
function [out,boxnum]=glboxes2html(boxes, boxnum)
    out=[];
    for i=1:length(boxes);
        if isfield(boxes{i}, 'inf')
            %boxes gained from cluster
            boxnum=boxnum+1;
            out=[out glbox2html(boxes{i}, boxnum) ' or <BR>\n'];
        elseif isfield(boxes{i}, 'newbox') && boxes{i}.finish
            %boxes gained through split
            boxnum=boxnum+1;
            out=[out glbox2html(boxes{i}.newbox, boxnum) ' or <BR>\n'];
        end
    end
    out=[out(1:end-10) '.<BR>\n'];
    
    