%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmakesol.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function sol=glmakesol(P,xbox,pfname,stratname,tim)
%
% Create a solution structure.
%
% P             problem definition
%  .solstatus   solution status, see glsolstatus
% xbox          bound constraints
%  .inf,.sup    outer enclosure
%  .best        most feasible point (can be missing)
%    .x         nx x 1 point
%    .fx        nc x 1 function value Aq(x)
%    .fd        feasibility distance of the found point
%    .status    status of the search: feasible, local minima, maxiter
%    .zb        a box containig a feasible point, or empty if the verifica
%               tion failed
% pfname        problem filename, empty if unknown
% strategy      used strategy name
% tim           solution time
% prt           text output flag       0 = no output 1 = output
%
% sol           solution structure
%  .probname    problem name
%  .pfname      problem filename
%  .stratname   strategy name
%  .sfname      strategy filename
%  .nxo         number of the original variables before conversion
%               from .dag to .def
%  .xold        starting bound constraints
%   .inf, .sup  outer enclosures
%  .xbox        found bound constraints (or a cell of boxes)
%   .inf, .sup  outer enclosures
%  .best        most feasible point
%   .x          nx x 1 point
%   .fx         nc x 1 function value Aq(x)
%   .fd         feasibility distance of the found point
%   .status     status of the search: feasible, local minima, maxiter
%   .zb         a box containig a feasible point, or empty if the verifica
%               tion failed
%   .fobj       the value of the objective function in the best point
%               (missing if there is no objective)
%  .splits      total number of splits done while executing the strategy.
%  .status      problem status 
%  .time        solution time
%  .date        date
%  .cpuinfo     cpu information string

function sol=glmakesol(P, xbox, pfname, stratname, tim)

global glconf;      % global configuration variable
FEAS=1e-6;          % feasibility tolerance

% argcheck
if nargin<5, error('glmakesol: Not enough arguments!'); end

% set configuration
if isfield(glconf,'feast'),FEAS=glconf.feast;end 

% make solution
sol.probname=P.name;
if isempty(pfname); sol.pfname='unknown location';
else sol.pfname=pfname;
end
sol.stratname=stratname;
sol.sfname=[glconf.stratdir stratname '.str'];
sol.status=P.solstatus;
sol.splits=P.splits;
sol.time=tim;
[err, cpuinfo]=glsystem('cpuinfo'); % set the cpu information
if ~err, sol.cpuinfo=cpuinfo;
else sol.cpuinfo='Unknown';
end
sol.date=now;
if isfield(P.Q,'A') && isfield(P,'startxbox')
    % unsimplify
    if ~isempty(xbox), [P, xbox]=glunsimplify(P, xbox, 0); end
    sol.xold=P.startxbox;sol.xbox=[];
    nx=length(P.Q.int);
    Am=(P.Q.A.inf+P.Q.A.sup)/2;

    % check boxes
    if (isfield(P,'boxes') && ~isempty(P.boxes)) || ...
            (isfield(P,'bstack') && ~isempty(P.bstack))
        % find the interval hull of the boxes, this also sets the best point
        % found in the boxes an the stack inside of xbox.
        ain.P=P;ain.x=xbox;ain.tol=[];ain.drop=0;
        aout=glmerge_inthull(ain);
        xbox=aout.x;
        P=aout.P;
    end

    % set the best point
    if ~isempty(xbox) && isfield(xbox,'best')
        % use the found best point
        sol.best=xbox.best;
    else
        % create a best point
        if ~isempty(xbox), sol.best.x=glmid(xbox);
        else sol.best.x=zeros(nx,1);
        end
        [sol.best.fx,sol.best.fd]=gleval(Am, sol.best.x, P.Q.F, xbox);
        if(sol.best.fd<FEAS), sol.best.status='f';
        else sol.best.status='m';
        end
    end

    % set resulting bound constraint box(es)
    if isfield(P,'boxes')  && ~isempty(P.boxes), sol.xbox=P.boxes;
    elseif ~isempty(xbox), sol.xbox={xbox};
    end

    % set objective value in the best point
    if P.opt~=0, sol.best.fobj=sol.best.fx(P.opt);end

    if isfield(P,'rem') && ~isempty(P.rem.I)
        % reset removed variables
        for i=1:length(sol.xbox)
            xx=sol.xbox{i};
            x.inf(P.J,1)=xx.inf;x.sup(P.J,1)=xx.sup;
            x.inf(P.rem.I,1)=P.rem.x.inf;x.sup(P.rem.I,1)=P.rem.x.sup;
            sol.xbox{i}=x;
        end
        min(P.J,1)=sol.best.x;
        min(P.rem.I,1)=P.rem.x.inf;
        sol.min=min;
    end
end
sol.nxo=P.Q.nxo;