%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsol2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function str=glsol2str(sol)
%
% converts a solution to a string cell
%
% sol           solution structure
%  .probname    problem name
%  .pfname      problem filename
%  .stratname   strategy name
%  .sfname      strategy filename
%  .nxo         number of the original variables before conversion
%               from .dag to .def
%  .xold        starting bound constraints
%   .inf, .sup  outer enclosures
%  .xbox        found bound constraints
%   .inf, .sup  outer enclosures
%  .best        most feasible point
%   .x          nx x 1 point
%   .fx         nc x 1 function value Aq(x)
%   .fd         feasibility distance of the found point
%   .status     status of the search: feasible, local minima, maxiter
%   .zb         a box containig a feasible point, or empty if the verifica
%               tion failed
%   .fobj       the value of the objective function in the best point
%               (missing if there is no objective)
%  .splits      total number of splits done while executing the strategy.
%  .status      solution status vector
%  .time        solution time
%  .date        date
%  .cpuinfo     cpu information string
%
% str           string cell array

function str=glsol2str(sol)
global glconf;          % global configuration variable

PREC=6;                 % number precision

%argcechk
if nargin <1, error('glsol2str: Not enough arguments!');end

%set constants from config
if isfield(glconf,'sdigits'),PREC=glconf.sdigits;end

%convert
str={['Problem "' sol.probname '" at '  sol.pfname]};
str=[str ; {['Strategy used: "' sol.stratname '" at '  sol.sfname]}];

if glsolstatus(sol.status,'NORMAL') || glsolstatus(sol.status,'INFEAS')...
        || glsolstatus(sol.status,'TIMEOUT')
    % bound constraints
    if ~isempty(sol.xbox) && ~glsolstatus(sol.status,'INFEAS')
        str=[str;{'The initial bound constraints:  '};...
            {[' ' mat2str([sol.xold.inf sol.xold.sup], PREC)]};...
            { ' reduced to: '}];
        if iscell(sol.xbox)
            for i=1:length(sol.xbox)
                str=[str;{[' ' mat2str([sol.xbox{i}.inf sol.xbox{i}.sup],...
                    PREC)]}];
            end
        else
            str=[str;{[' ' mat2str([sol.xbox.inf sol.xbox.sup],PREC)]}];
        end
    else
        str=[str ; {'The constraints has been found infeasible!'}];
    end
    % best point
    str=[str;{ 'The best point found is '};...
         {[' ' mat2str(sol.best.x, PREC)]};...
         {[' with a feasibility distance of ' num2str(sol.best.fd) '.']}];
    if isfield(sol.best,'fobj')
        str=[str ; {' and the objective value is at this point is'};...
             {[' ' mat2str(sol.best.fobj, PREC) '.']}];
    end
    % verified point
    if isfield(sol.best,'zb') && ~isempty(sol.best.zb)
        str=[str;{'A verified feasible point can be found in the box:'};...
            {[' ' mat2str([sol.best.zb.inf sol.best.zb.sup],PREC) '.']}];
    end
    str=[str ; {['Total Splits: ' mat2str(sol.splits)]}];
end
stat=glsolstatus(sol.status);s='';
for i=1:length(stat), s=[s stat{i} ', ']; end
str=[str ; {['Status: ' s(1:end-2)]}];
str=[str ; {['Time used: ' glshowtime(sol.time)]}];
str=[str ; {['Cpu information; ' sol.cpuinfo]}];
str=[str ; {['Date: ' datestr(sol.date)]}];

