%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% gltestenvsol.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function gltestenvsol(sol, fpath)
%
% converts a solution to the Testenvironment format and writes the
% .tab and .res file needed for solcheck, no additional analyze script
% is required inthe Testenvironment
%
% sol           solution structure
%  .probname    problem name
%  .pfname      problem filename
%  .stratname   strategy name
%  .sfname      strategy filename
%  .nxo         number of the original variables before conversion
%               from .dag to .def
%  .xold        starting bound constraints
%   .inf, .sup  outer enclosures
%  .xbox        found bound constraints
%   .inf, .sup  outer enclosures
%  .best        most feasible point
%   .x          nx x 1 point
%   .fx         nc x 1 function value Aq(x)
%   .fd         feasibility distance of the found point
%   .status     status of the search: feasible, local minima, maxiter
%   .zb         a box containig a feasible point, or empty if the verifica
%               tion failed
%   .fobj       the value of the objective function in the best point
%               (missing if there is no objective)
%  .splits      total number of splits done while executing the strategy.
%  .status      problem status, see glprobstatus
%  .time        solution time
%  .date        date
%  .cpuinfo     cpu information string
% fpath         output path where the <probname>.tab and <probname>.res 
%               will be stored
%
% Testenvironment Notes:
% The results of the different solvers are assumed to be given in the files
% <problem>.tab. Each such file must consist of one line, where the first
% entry is the problem name, the second entry is the CPU time needed by
% the solver, the third entry is the objective value obtained (NaN in the
% case of a failure),the fourth entry is our version of the solution status
% ,all other entries are solver-specific and irrelevant for this program.
% Solution statuses:
% 3 = not accepted
% 2 = unresolved
% 1 = local
% 0 = global
%-1 = local and timeout reached
%-2 = unresolved and timeout reached,
%-3 = infeasible

function gltestenvsol(sol, fpath)

global glconf;      % global configuration variable
PREC=10;            % precision digits in output text

%argcechk
if nargin<2, error('gltestenvsol: Not enough arguments!'); end

% set constants from config
if isfield(glconf,'sdigits'),PREC=glconf.sdigits;end

%make .tab file TAB STATUS:
%0 STATUS_GLOBAL;
%1 STATUS_LOCAL;
%2 STATUS_UNRESOLVED;
%3 STATUS_NOTACCEPTED;
%-1 STATUS_TIMEOUT_LOCAL;
%-2 STATUS_TIMEOUT_UNRESOLVED;
%-3 STATUS_INFEASIBLE;
if glsolstatus(sol.status,'NONQUAD') || glsolstatus(sol.status,'DIMHIGH')
    stat=3;sol.time=0;
elseif glsolstatus(sol.status,'FEAS') || glsolstatus(sol.status,'VFEAS')
    stat=0;
elseif glsolstatus(sol.status,'TIMEOUT')
    stat=-2;
elseif glsolstatus(sol.status,'INFEAS')
    stat=-3;
else
    stat=2;
end
if isfield(sol, 'best') && isfield(sol.best,'fobj') && ~isempty(sol.best.fobj),
    obj=sol.best.fobj;
else obj=0;
end

% write the .tab file
fid=fopen([fpath sol.probname '.tab'],'w');
if fid == -1, 
    error(['gltestenvsol: Wrong path:' fpath]);
end
fprintf(fid,'%s\t%d\t%s\t%d\t%d',sol.probname,sol.time,...
    num2str(obj,PREC),stat,sol.splits);
fprintf(fid,'\n');
fprintf(fid,'%s','% Testenvironment .tab file generated by Gloptlab %');
fprintf(fid,'\n');
fprintf(fid,'%s',['% at ' datestr(now)]);
fprintf(fid,'\n');
fclose(fid);   

%create the contents of the .res file
str={['modelstatus = ' num2str(stat)]};

for i=1:sol.nxo
    if isfield(sol, 'best')
        str=[str; {['x(' num2str(i) ') = ' num2str(sol.best.x(i),PREC)]}];
    else str=[str; {['x(' num2str(i) ') = 0']}];
    end
end
% objective
str=[str; {['obj = ' num2str(obj,PREC)]}];
% feasibility
if stat==-3, str=[str; {'infeas = 1'}];
else str=[str; {'infeas = 0'}];
end
% timeout -> nonopt
if stat==-2, str=[str; {'nonopt = 1'}];
else str=[str; {'nonopt = 0'}];
end
str=[str; {['time = ' num2str(sol.time,PREC)]}];
str=[str; {['splits = ' num2str(sol.splits)]}];

% write the .res file
fid=fopen([fpath sol.probname '.res'],'w');
for i=1:length(str)
    fprintf(fid,'%c',str{i});
    fprintf(fid,'\n');
end
fprintf(fid,'%c',['% Number of the original variables: ' num2str(sol.nxo)]);
fprintf(fid,'\n');
fprintf(fid,'%c','% Testenvironment .res file generated by Gloptlab %');
fprintf(fid,'\n');
fprintf(fid,'%c',['% at ' datestr(now)]);
fprintf(fid,'\n');
fclose(fid);