%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glchktskl.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [status, msg] = glchktskl(tasklist)
%
% check a tasklist
%
% tasklist      tasklist cell vector, elements:task with
%   task.name   name of the task
%   task.prt    text output by completing the task
%   task.deb    debug
%   task.(vars) various task variables (mingain, small etc)
%   task.text   text decription of the task (for the gui)
%
% status        =-1 tasklist wrong
%               = 0 tasklist ok
%               = 1 tasklist empty
% msg           error message

function [status, msg]=glchktskl(tasklist)

%nesting the same task is not allowed for theese multiline tasks
NONEST={'Split','Postprocess'}; 

% argcheck
if nargin<1, error('glchktskl: Not enough arguments!');end
tn=length(tasklist);
status=0;msg=[];needpostp=[];haspostp=0;
if tn == 0, 
    % read problem is missing
    msg={'Empty tasklist found!'};
    status=-1;
    return;
elseif ~strcmp(tasklist{1}.name,'Read Problem'),
    % read problem is missing
    msg={'Read Problem task is missing!'};
    status=-1;
    return;
end

mtasks=[]; % cell of the task name of the open multiline tasks
mtinds=[]; % cell of begin indices of the open multiline tasks

for i=1:tn
    % process tasks
    task=tasklist{i};
    tname=task.name;
    if task.comm==1 || strcmp(tname,'Pause') || strcmp(tname,'Finish')
        % task is commented out or other control task, do not check
        continue;
    elseif strcmp(tname,'End')
        % end found check if there is any multiline task open
        if isempty(mtasks)
            msg=[msg ; {['! Strategy wrong: End outside of a multiline '...
                'task found at entry ' num2str(i) '!']}];
            status=-1;
        end
        continue;
    elseif strcmp(tname,'Read Problem')
        tstr='readprob';
    elseif length(tname)>6 && strcmp(tname(1:6),'Begin ')
        % multiline task begin
        bstr=tname;             % begin string like 'Begin Split'
        tstr=bstr(7:end);       % type string like 'Split'
        % check multipile or invalid postprocess
        if strcmp(tstr, 'Postprocess')
            if haspostp,
                msg=[msg ; {['! Strategy wrong: a second Postprocess '...
                'found at entry ' num2str(i) ', a maximum of one is '...
                'allowed in a strategy!']}];
                status=-1;
            else
                haspostp=1;
                if ~isempty(mtasks),
                    msg=[msg ; {['! Strategy wrong: Postprocess '...
                    'is not allowed inside a multiline at entry ' ...
                    num2str(i) '!']}];
                    status=-1;
                end
            end
        end
        % check nesting
        nfound=0;
        for n=1:length(NONEST)
            nstr=NONEST{n};
            if strcmp(tstr, nstr)
                % nesting not allowed, check
                for j=1:length(mtasks)
                    if strcmp(mtasks{j}, nstr)
                        nfound=1;
                        break;
                    end
                end
            end
            if nfound, break; end
        end
        if nfound,
            msg=[msg ; {['! Strategy wrong: nested ' tstr ...
                ' found at entry ' num2str(i) '!']}];
            status=-1;
        end
        % add the task in the open multinie list
        mtasks=[mtasks {tstr}]; 
        mtinds=[mtinds i];
        tstr=lower(tstr);
    elseif length(tname)>4 && strcmp(tname(1:4),'End ')
        if isempty(mtasks)
            % multiline task end without begin found
            msg=[msg ; {['! Strategy wrong: ' tname ' without Begin '...
                tname(5:end) ' found at entry ' num2str(i) '!']}];
            status=-1;
        else
            tstr=mtasks{end};
            tend=['End ' tstr];
            ind = mtinds(end);
            if ~strcmp(tname,tend)
                msg=[msg ; {['! Strategy wrong: illegal ' tname ...
                    ' found at entry ' num2str(i) ', ' tend ...
                    ' was expected!']}];
                status=-1;
            else
                if ind+1==i
                    msg = [msg ; {['Strategy warning: Empty ' tstr ...
                        ' found at entry ' num2str(ind) '!']}];
                end
                mtasks(end)=[];
                mtinds(end)=[];
            end
        end
        continue;
    elseif ~isempty(mtasks) && strcmp(tname,'Simplify')
        %simplify is not allowed inside a multiline task
        msg=[msg ; {['! Strategy wrong: Simplify at entry ' num2str(i)... 
            ' is inside a multiline task, which is not allowed!']}];
        status=-1;
        continue;
    else
        % normal task
        tstr=lower(tname);
        % special task tests
        if strcmp(tname,'Merge')
            split=0;
            for k=1:i-1
                if strcmp(tasklist{k}.name,'Begin Split')
                    split=1;
                    break;
                end
            end
            if ~ split
                msg = [msg ; {['Strategy warning: Merge Boxes without'...
                    'previous split found at entry ' num2str(i) '!']}];
            end
        elseif strcmp(tname,'Break') && strcmp(task.argin.mode,'postprocess')
            % break task set the need of postprocessing if mode is postprocess
            needpostp=[needpostp i]; 
        end
    end
    

    if ~isfield(task,'argin')
        msg=[msg ; {['! Strategy wrong: Missing input arguments for ' ...
            'the task: "' tname ' at entry ' num2str(i) '!']}];
        status=-1;
        continue;
    end

    % general argin check
    args=fieldnames(task.argin);
    anum=length(args);
    % generate the name of the function curresponding to the task
    if isfield(task.argin, 'method')
        % the task has an additional method
        if ~isempty(task.argin.method)
            % the task has a method selected, the name of the function
            % is gl(taskname)_(method)
            taskfun=['gl' tstr '_' task.argin.method];
            tname=[tname ' (' task.argin.method ')'];
        else
            % the task should have a method but none selected, skip
            msg=[msg; {['! Strategy wrong: Missing critical '...
                'parameter "method" in ' tname ' at index '...
                num2str(i)]}];
            status=-1;
            continue;
        end
    else
        % the task has no additional method name=gl(taskname)
        taskfun=['gl' tstr];
    end

    % check if the generated function exits or not
    if exist(taskfun,'file')~=2
        msg=[msg ; {['! Strategy wrong: Unkown task: ' tname ' at index '...
            num2str(i)]}];
        status=-1;
        continue;
    end

    % get the needed and optional arguments together with the default
    % values of the optional input params (last one is not used)
    ar=feval(taskfun);
    nargs=ar{1};oargs=ar{2};

    argfound=zeros(1,anum);

    % ckeck if all needed argins are there or not, whereas P and x is
    % always supplied
    for j=1:length(nargs)
        narg=nargs{j};
        if strcmp(narg,'P') || strcmp(narg,'x') ||...
                strcmp(narg,'tasklist') || strcmp(narg,'starttask'),
            continue;
        end
        ind=find(~argfound);
        for k=ind
            if(strcmp(narg,args{k}))
                argfound(k)=1;
                break;
            elseif k==ind(end)
                msg=[msg; {['! Strategy wrong: Missing critical '...
                    'parameter "' narg '" in ' tname ' at index '...
                    num2str(i)]}];
                status=-1;
                break;
            end
        end
    end

    % ckeck if all optional argins are there or not, give warnings
    for j=1:length(oargs)
        oarg=oargs{j};
        if strcmp(oarg,'gra') || strcmp(oarg,'pro') ||...
                strcmp(oarg,'nopause') || strcmp(oarg,'cert'),
            continue;
        end
        ind=find(~argfound);
        for k=ind
            if(strcmp(oarg,args{k}))
                argfound(k)=1;
                break;
            elseif k==ind(end)
                msg=[msg ; {['Strategy warning: Missing optional '...
                    'parameter "' oarg '" in ' tname ' at index '...
                    num2str(i)]}];
                break;
            end
        end
    end

    % check extra input pararms
    ind=find(~argfound);
    for k=ind
        arg=args{k};
        if ~strcmp(arg,'method') && ~strcmp(arg,'lsolver'),
            msg=[msg ; {['Strategy warning: Extra parameter "' ...
                arg '" in ' tname ' at index ' num2str(i)]}];
        end
    end
end

if ~strcmp(tasklist{tn}.name,'Finish')
    % end is missing
    msg=[msg ; {['! Strategy wrong: Finish task is missing from the end '...
        'of the strategy!']}];
    status=-1;
end

% check unfinished multiline tasks
for i=1:length(mtasks)
    msg=[msg ; {['! Strategy wrong: unfinished ' mtasks{i} ...
        ' found at entry ' num2str(mtinds(i)) '!']}];
    status=-1;
end

% check missing postprocessing
if ~isempty(needpostp) && ~haspostp,
    pn=length(needpostp);
    if pn==1, s=['task at the entry ' num2str(needpostp) ', '];
    else
        s='tasks at the entries ';
        for i=1:pn, s=[s num2str(needpostp(i)) ', '];end
    end
    msg=[msg ; {['! Strategy wrong: Prostprocessing step is needed '...
        'by the ' s 'but the strategy contains none!']}];
    status=-1;
end

if status==0, msg=[msg ; {'Strategy test passed!'}];
else msg=[msg ; {'Strategy test failed!'}]; end
