%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glprocplist.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [sols, P, xbox, probpos, strategy, taskpos, status, timer, certs]=
%    glprocplist(problist, strategy, soldir, nopause, startprob,
%    starttask, P, xbox, timer, defaultstrat, prt, testenv, gra, ghand, gran
%    pro, proofdir, stat, statfile, certs, errhand)
%
% processes the strategy for all problems in the problist
% (1) when a break or a pause in the tasklist reached returns all relevant
%  infromations: P, xbox, probpos, lastprob, strategy, taskpos, status,
%  (equnums, lastnum only if pro) of the actual problem
% (2) for tasklist without pause the cell vector sols is relevant
% (3) the text output (log) can be found in gloutput.out (if prt=1)
%
% problist      cell of problem fnames with full path and/or with problem
%               definition structures P
% startegy      startegy structure
%   .name       name of the strategy
%   .tasks      startegy cell, with element fields
%       .name   name of the task
%       .prt    text output by completing the task
%       .deb    debug
%       .(vars) various task variables (mingain, small etc)
%       .text   text decription of the task (for the gui)
% soldir        directory to store the solutions
% nopause       ==0 use pauses in the tasklists
%               ==1 ignore the pauses in the tasklists
%               ==2 ignore the pauses in the tasklists, but not at the end
% startprob     index of the first problem to be solved
% starttask     starttask process tasklist at this index
% P             if starttask~=1 the actual problem definition
% xbox          if starttask~=1 the actual bound constraints
% timer         timer structure (see gltimer)
% defaultstrat  strategy=default strategy will be set each time if a
%               problem is loaded
% prt           text output flag       0 = output turned of for all probs
% testenv       save .tab solution for the Testenvironment flag 0 = no save
% gra           graphics output flag   0 = no output 1 = output
% ghand         the handle of the figure in which the plot axe(s) should
%               be placed. If empty the current figure is used.
% gran          1 x 4 position vector [x,y,xw,yw] of the plot space in the
%               figure. If empty the whole space in the figure is used.
% pro           proof output flag      0 = no output 1 = output
% proofdir      directory to store the proofs
% stat          statistics output flag 0 = no output 1 = output
% statfile      file to which the statistic will be appended
% certs         certificates cell array
% errhand       error handling flag   0 = off 1 = on
%
% sols          solution cell vector, for sol fields see glsol2str
% P             last problem definition
% xbox          last bound constraint
% probpos		last position in the problem list
% strategy      actual strategy, must not be the input strategy since it
%               can be another loaded one
% taskpos		last position in the startegy (can be a vector if subtasks)
% status        = 0 proceed with the next task
%               = 1 stop the strategy (pause or user break)
%               = 2 break this tasklist (glbreak->break tasklist, infeas)
%               = 3 break the multiline (glbreak->break multiline)
%               = 4 continue with postprocess (glbreak->postprocess,timeout)
%               = 5 finish the strategy (glbreak->finish strategy, problem
%                   not accepted: dim too high/nonquadratic found, after
%                   postprocess)
% certs         new certificates cell array

function [sols, P, xbox, probpos, strategy, taskpos, status, timer, certs]=...
    glprocplist(problist, strategy, soldir, nopause, startprob,...
    starttask, P, xbox, timer, defaultstrat, prt, testenv, gra, ghand, gran,...
    pro, proofdir, stat, statfile, certs, errhand)

% global variables
global glconf;      % global configuration variable
global gloutput;    % global output structure

% argcechk
if nargin<2,error('glprocplist: Not enough arguments!');end
if nargin<3,
    if isfield(glconf,'soldir'),soldir=glconf.soldir;
    else error('glprocplist: Not enough arguments!')
    end
end
if nargin<4,nopause=0;end
if nargin<5,startprob=1;end
if nargin<6,starttask=1;end
if nargin<7,P=[];end
if nargin<8,xbox=[];end
if nargin<9,timer=glcreatetimer(inf, 15, 1, 0, inf, '', '');end
if nargin<10,defaultstrat=strategy;end
if nargin<11,prt=0;end
if nargin<12,testenv=0;end
if nargin<13,gra=0;end
if nargin<14,ghand=[];end
if nargin<15,gran=[];end
if nargin<16,pro=0;end
if nargin<17,
    if pro,
        if isfield(glconf,'proofdir'),proofdir=glconf.proofdir;
        else error('glprocplist: Not enough arguments!');
        end
    else proofdir=[];
    end
end
if nargin<18,sta=0;end
if nargin<19,
    if sta,
        if isfield(glconf,'statfile'),statfile=glconf.proofdir;
        else error('glprocplist: Not enough arguments!');
        end
    else statfile=[];
    end
end
if nargin<20,certs=cell(0);end
if nargin<21,errhand=0;end

% init
sols=[];probpos=1;taskpos=1;status=0;
pn=length(problist);tn=length(strategy.tasks);
if gra
    % init the plot
    if ~isempty(ghand), gloutput.graphics.plotfig=ghand;end
    if ~isempty(gran), gloutput.graphics.plotspac=gran;end
end

% task && problemlist check
if pn==0
    % empty tasklist
    gloutput.out=[gloutput.out;{'ERROR: glprocplist: problem list empty!'}];
    return;
elseif starttask==tn,
    % last position in tasklist -> next problem first task
    starttask=1;startprob=startprob+1;
elseif starttask~=1 && (isempty(P) || isempty(xbox))
    % if not at the first task with no actual problem definition and a
    % valid box, begin at the first task
    starttask=1;
end

% pause at end flag: true if at the end of the strategy is an
% uncommented pause
pause_at_end = strcmp(strategy.tasks{tn-1}.name, 'Pause') && ...
    ~strategy.tasks{tn-1}.comm;

% process all problems from the index startprob
for probpos=startprob:pn
    if starttask==1,
        timer=gltimer(timer, 'r');
        P=problist{probpos};
        xbox=[];
    end
    if errhand,
        % with error handling
        try
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % PROCESSING TASKLIST
            % process all tasks for this problem
            [P, xbox, status, taskpos, timer, certs2]=glproctskl(P, xbox,...
                defaultstrat.tasks, starttask, timer, nopause, prt, gra, pro);
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % certificates
            if pro, certs=[certs; certs2]; end
        catch
            err=lasterror;
            disp(err.message);
            for i=1:length(err.stack)
                st=err.stack(i);
                disp([st.file ':' st.name ':' num2str(st.line)]);
            end
            msg=['*FATAL ERROR IN ' P.name ': '];
            d=err.stack;
            for i=1:length(dbstack)
                msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
            end
            msg=[msg ' -> ' err.message];
            glerrorlog(msg);
            P=[];
            gloutput.out=[gloutput.out ; {msg}];
            gloutput.brief=[gloutput.brief ; {msg}];
            % force all remaining outputs
            timer=gltimer(timer, 'f', P, []);
            % reset the timer
            timer=gltimer(timer, 'r');
            starttask=1;
            continue;
        end
    else
        % No error handling
        %TEST
        %F=[P.Q.F.inf,P.Q.F.sup]
        %save C:\Work\Matlab\Gloptlab\testP P;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % PROCESSING TASKLIST %
        % process all tasks for this problem
        [P, xbox, status, taskpos, timer, certs2]=glproctskl(P, xbox,...
            strategy.tasks, starttask, timer, nopause, prt, gra, pro);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % certificates
        if pro, certs=[certs; certs2]; end
    end

    % stop the timer, force outputs
    timer=gltimer(timer, 'f', P, xbox);
    timer=gltimer(timer, 'p');
    
    % the strategy has been paused; if at the end and the last task 
    % is a pause we continue like in the case normal termination then 
    % break at the end of the loop.
    if status==1 && (sum(taskpos)~=tn || ~pause_at_end), break; end
    
    % EXTRACTING PROBLEM
    %if isfield(P.Q,'A'),
    %    basedir=[fileparts(which('Gloptlab.m')) filesep];
    %    fname=[basedir '\SolverTests\Extracted\' P.name '_ex.mat'];
    %    glextracttest(fname);
    %end
    
    % strategy finished
    if prt
        gloutput.out=[gloutput.out ; {'Strategy finished.'}];
        gloutput.out=[gloutput.out ; {char(zeros(1,75)+45)}];
    end

    % CREATE SOLUTION, TRANSFORM BACK TO EQUS %
    if isstruct(problist{probpos}), pfname=[];
    else pfname=problist{probpos};
    end
    sol=glmakesol(P, xbox, pfname, strategy.name, timer.ellapsed);
    timer=gltimer(timer, 's');
    if prt,
        out=glsol2str(sol);
        gloutput.out=[gloutput.out; out; {char(zeros(1,75)+45)}];
        gloutput.brief=[gloutput.brief; {char(zeros(1,55)+45)}; out];
    end
    % store solution in solution vector
    sols=[sols {sol}];

    % save solution file
    sfname=[soldir P.name '.gls'];glsavesol(sol, sfname);
    if prt, gloutput.out=[gloutput.out;{['Solution saved in ' sfname]}];end
    if testenv,
        % also write .tab and .res file needed in the test environment
        gltestenvsol(sol, soldir);
        if prt,
            gloutput.out=[gloutput.out;
                {['Testenvironment ' soldir sol.probname '.tab saved.']}];
            gloutput.out=[gloutput.out;
                {['Testenvironment ' soldir sol.probname '.res saved.']}];
        end
    end

    if pro,
        % check certificates
        tim=glcreatetimer(inf, inf, 0, 0, inf, '', '');
        [ncerts,out2]=glcertcheck([],[],certs, glconf.proo.minreduc,...
            glconf.proo.round, glconf.proo.digits, glconf.proo.maxerr,... 
            tim, glconf.proo.prt, glconf.proo.deb);
        tim=gltimer(tim, 'c', []);
        if prt,
            gloutput.out=[gloutput.out;out2;{['Certificates checked in '... 
                num2str(tim.ellapsed) ' seconds.']}];
            
        end
        if ~isempty(ncerts)
            % save certificates
            save([soldir sol.probname '.cer'],'certs');
            if prt,
                gloutput.out=[gloutput.out; {['Certificate file ' ...
                    soldir sol.probname '.cer saved.']}];
            end
            % generate proof
            glproof(ncerts, proofdir);
            if prt,
                gloutput.out=[gloutput.out;...
                    {['Proof generated in the directory ' proofdir ]}];
            end
        end
    end

    if stat && ~glsolstatus(P.solstatus,'READERR'),
        % append problem statistic to the statistics file
        if exist(statfile,'file')==2, load(statfile,'-MAT');
        else d=cell(0);
        end
        % search if the problem already exist of not
        found=0;
        for i=1:length(d)
            if strcmp(d{i}.name, P.name), found=1;break;end
        end
        % create entry
        dat.name=P.name;
        if glsolstatus(sol.status,'NORMAL'), dat.acc=1;
        else dat.acc=0;
        end
        dat.nx=length(P.Q.int);dat.nxo=P.Q.nxo;dat.nonq=length(P.uni);
        % append or replace entry
        if ~found,
            d=[d;{dat}];
        else d{i}=dat;
        end
        % save statistic
        save (statfile,'d','-MAT');
        if prt,
            gloutput.out=[gloutput.out;...
                {['Statistic entry set in ' statfile '.']}];
        end
    end

    if probpos < pn, timer=gltimer(timer, 'f', P, xbox); end

    % -5=nx too high,-4=nonquad found,-1=paused,4=prob infeas,10=finish
    % tolast=[-5 -4 -1 0 4 10];
    if sum(taskpos)==tn && pause_at_end && nopause ~= 1
        % at the (end-1)th entry of the tasklist and this task is a pause
        % stop at the end of the problem
        break;
    end
    % next problem
    starttask=1;
end
% force all remaining outputs
timer=gltimer(timer, 'f', P, xbox);
