%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glproctskl.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [P, x, status, taskpos, timer, certs]=glproctskl(P, x, tasklist,
%    starttask, timer, nopause, prt, gra, pro)
%
% Process all tasks of the given tasklist for the problem P.
%
% P             problem definition
% P.solstatus   nst x 1 vector of solution status codes see glsoltatus
% x             bound constraints
% tasklist      tasklist cell vector, elements:task with
%   task.name   name of the task
%   task.prt    text output by completing the task
%   task.deb    debug
%   task.(vars) various task variables (mingain, small etc)
%   task.text   text decription of the task (for the gui)
%   task.cert   (optional) certificate of the previous run of this task
% starttask     starttask index, if a vector (2:end) will be passed to a
%               subtask (currently only to while)
% timer         timer structure (see gltimer)
% nopause       ==1 ignores the pauses in the tasklist
% prt           text output flag       0 = output turned off for all tasks
% gra           graphics output flag   0 = no output 1 = output
% pro           proof output flag      0 = no output 1 = output
% certs         certificates cell array
%
% P             new problem definition
% x             new bound constraints
% status        = 0 proceed with the next task
%               = 1 stop the strategy (pause or user break)
%               = 2 break this tasklist (glbreak->break tasklist, infeas)
%               = 3 break the multiline (glbreak->break multiline)
%               = 4 continue with postprocess (glbreak->postprocess,timeout)
%               = 5 finish the strategy (glbreak->finish strategy, problem
%                   not accepted: dim too high/nonquadratic found, read
%                   error, after postprocess)
%
% taskpos		position in the tasklist (when stopped by a pause)

function [P, x, status, taskpos, timer, certs]=glproctskl(P, x, tasklist,...
    starttask, timer, nopause, prt, gra, pro)

% global variables
global glconf;      % global configuration variable
global glbreak;     % gloptlab break flag
global gloutput;    % global output structure

% constants
FEAS=1e-6;          % feasibility tolerance

% argcechk
if nargin<3, error('glproctskl: Not enough arguments!');end
if nargin<4, starttask=1;end
if nargin<5, timer=[];end
if nargin<6, nopause=0;end
if nargin<7, prt=0;end
if nargin<8, gra=0;end
if nargin<9, pro=0;end

% set constants from config
if isfield(glconf,'feast'),FEAS=glconf.feast;end

% check wrong bounds
if ~isempty(x) && any(x.inf>x.sup)
    gldisp('x',[x.inf x.sup]);
    error ('glproctskl: wrong bound constraints!');
end

% init
if isempty(glbreak), glbreak=0;end
status=0;               % status indicator
taskpos=1;              % position where the tasklist execution ended
certs=cell(0);          % certificate cell array
tn=length(tasklist);    % number of the tasks in the tasklist

% check if the tasklist is empty or not
if tn == 0, return; end

% if not in a split but there are boxes on the stack, jump to the split
% in the tasklist or merge the boxes
% if isstruct(P) && isfield(P, 'bstack') && ~isempty(P.bstack)
%     % check if not already in a split
%     insp=0;
%     for i=1:starttask
%         if strcmp(tasklist{i}.name, 'Begin Split'), insp=1;
%         elseif strcmp(tasklist{i}.name, 'End Split'), insp=0;
%         end
%     end
%     if ~insp && ~strcmp(tasklist{starttask}.name, 'Merge Boxes')
%         % not in a split, search for a split in the strategy
%         % beginning from the starttask
%         for i=starttask:tn
%             if strcmp(tasklist{i}.name, 'Begin Split')
%                 starttask=i;break;
%             end
%         end
%         if i==tn
%             % merge boxes as hull do not
%             % drop the boxes and the stack
%             ain.P=P;ain.x=x;ain.tol=[];
%             ain.drop=0;ain.deb=0;ain.prt=1;
%             ain.gra=gra;ain.pro=pro;
%             aout=glmerge_inthull(ain);
%             P=aout.P;x=aout.x;
%             gloutput.out=[gloutput.out ;
%                 {'Hull of the found boxes and the stack created.'};
%                 {'The stack has been discarded:'}; aout.out];
%         elseif prt
%             gloutput.out=[gloutput.out ; {'Jumping to Split!'}];
%         end
%     end
% end

% prepare starttask
if length(starttask)==1 && starttask~=1
    % starttask is a single index and not at the beginning of the
    % tasklist, find possible subtasks (currently Split & While) and
    % if some found build a starttask vector
    ind=0;
    for j=1:starttask-1
        if ~isempty(strfind(tasklist{j}.name,'Begin ')), ind=[ind j];end
        if ~isempty(strfind(tasklist{j}.name,'End ')), ind=ind(1:end-1);end
    end
    ind=[ind starttask];starttask=[];
    for j=1:length(ind)-1, starttask=[starttask (ind(j+1)-ind(j))]; end
end
j=starttask(1);starttask=starttask(2:end);

% proces all task in the tasklist, beginning with the starttask
while 1
    if j>tn
        % all task processed, return with status=0
        status=0; break;
    end
    % redraw
    if ~nopause, drawnow; end
    % processed task
    task=tasklist{j};

    % pre execution checks
    if task.comm
        % task commented out, continue with next task
        j=j+1; continue;
    elseif glbreak,
        % user break
        if prt,
            gloutput.brief=[gloutput.brief ; {'User break!'}];
            gloutput.out=[gloutput.out ; {'User break!'}];
        end
        glbreak=0; taskpos=j; status=1; return;
    elseif strcmp(task.name,'Pause')
        % pause
        if nopause,
            % the nopause flag is set continue with the next task
            j=j+1; continue;
        else
            if prt,
                gloutput.brief=[gloutput.brief ; {'Strategy paused!'}];
                gloutput.out=[gloutput.out ; {'Strategy paused!'}];
            end
            taskpos=j+1;status=1;return;
        end
    elseif strcmp(task.name,'Finish')
        % finish task
        taskpos=j;status=5;return;
    end

    % get the certificate of the task if exitsts
    if isfield(task, 'cert')
        cert=task.cert;
        task=rmfield(task, 'cert');
    else cert=[];
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % process a the task
    argin=task.argin;
    argin.nopause=nopause;
    % brief output
    if prt,
        if isfield(argin, 'method')
            txt=[task.name '(' argin.method ')...started'];
        else txt=[task.name '...started'];
        end
        gloutput.brief=[gloutput.brief; {txt}];
    end
    % text output flag for this task
    tprt=(~isfield(task.argin,'prt') | task.argin.prt) & prt;
    % determine if the task is a begin-end task or not
    ctask=length(task.name)>6 && strcmp(task.name(1:6),'Begin ');

    if ctask && isempty(cert),
        % this task is a begin-end task, set taskname, sublist and
        % starttask for while and split
        bstr=task.name;         % begin string like 'Begin Split'
        tstr=bstr(7:end);       % type string like 'Split'
        estr=['End ' tstr];     % correspondig end str:'End Split'
        if strcmp(tstr, 'While'), nest=1; % nesting allowed (While)
        else nest=0;                      % nesting not allowed (Split)
        end
        % find the end of the subtasklist
        taskend=0;notclosed=0;
        for k=(j+1):tn
            if strcmp(tasklist{k}.name,bstr) && tasklist{k}.comm==0,
                if nest, notclosed=notclosed+1;
                else
                    error(['Strategy wrong: mutiple ' bstr ...
                        ' at entry ' num2str(k) '!']);
                end
            elseif strcmp(tasklist{k}.name,estr) && tasklist{k}.comm==0
                if nest,
                    if notclosed==0, taskend=k; break;
                    else notclosed=notclosed-1;
                    end
                else
                    taskend=k;
                    break;
                end
            end
        end
        if taskend==0,
            % the end of the subtasklist has not been found
            error(['Strategy wrong: ' estr ' not found!']);
        end
        % create the subtasklist and the starttask for the begin-end
        % task
        argin.tasklist=tasklist((j+1):(taskend-1));
        argin.starttask=starttask;
        starttask=1;           % clear the starttask
        tnam=lower(tstr);      % name of the task in lowercase letters
    elseif ctask,
        % this task is a begin-end task with a certificate
        taskend=j;
        argin.tasklist=cert.tasklist;
        starttask=1;           % clear the starttask
        argin.starttask=starttask;
        tnam=lower(task.name(7:end));% name of the task in lowercase letts
    elseif strcmp(task.name,'Read Problem')
        tnam='readprob';
    else
        % set task name for the normal tasks
        tnam=lower(task.name); % name of the task in lowercase letters
    end

    % generate the name of the function curresponding to the task
    if isfield(argin, 'method')
        % the task has an additional method
        if ~isempty(argin.method)
            % the task has a method selected, the name of the function
            % is gl(taskname)_(method)
            taskfun=['gl' tnam '_' argin.method];
        else
            % the task should have a method but none selected, skip
            gloutput.out=[gloutput.out ;
                {['No method selected, task ' task.name ' skipped']}];
            continue;
        end
    else
        % the task has no additional method name=gl(taskname)
        taskfun=['gl' tnam];
    end

    % check if the generated function exits or not
    if exist(taskfun,'file')~=2
        error (['Unkown Task: ' taskfun ' at index ' num2str(j)]);
    end
    
    % check the timer
    timer=gltimer(timer, 'c', P, x);
    
    % complete argin
    argin.P=P; argin.x=x;
    argin.prt=tprt; argin.gra=gra; argin.pro=pro;
    argin.cert=cert;
    argin.timer=timer;

    % save old x
    x_old=x;
    
    %****************************
    % call the task function
    argout=feval(taskfun, argin);
    %****************************
    
    %******************************
    % process the output arguments
    if isfield(argout,'P'), P=argout.P; end
    if isfield(argout,'x'),
        if isempty(argout.x), x.infeasible=true;
        else x=argout.x;
        end
    end
    if ~isempty(x) && ~isfield(x,'best') && isfield(x_old,'best')
        % best point must not be in x
        x.best=x_old.best;
    end
    if isfield(argout,'best') && isfield(x,'best') && argout.best.fd<x.best.fd
        x.best=argout.best;
    end
    if isfield(argout,'F')
        if isempty(argout.F), x.infeasible=true;
        else P.Q.F=argout.F;
        end
    end
    if isfield(argout,'E'),
        % linear relaxation of the problem
        P.L.E=argout.E;P.L.b=argout.b;P.L.cind=argout.cind;
    end
    if isfield(argout,'boxes') || isfield(argout,'bstack'),
        % boxes and the stack generated by the split
        P.boxes=argout.boxes;
        P.bstack=argout.bstack;
    end
    if isfield(argout,'splits'),P.splits=P.splits+argout.splits; end
    if status~=5
        % no finish reached, get the status
        if isfield(argout,'status'), status=argout.status;
        else status=0;
        end
    end
    if isfield(argout, 'out') && ~isempty(argout.out)
        % redirect output
        gloutput.out=[gloutput.out ; argout.out];
    end

    % check critical statuses, infeasibility and wrong bounds
    if glsolstatus(P.solstatus,'READERR') || glsolstatus(...
            P.solstatus,'NONQUAD') || glsolstatus(P.solstatus,'DIMHIGH')
        if tprt,
            sstr=glsolstatus(P);
            txt=['*** ' sstr{1} ', strategy terminated!'];
            gloutput.out=[gloutput.out ; {txt}];
            gloutput.brief=[gloutput.brief ; {txt}];
        end
        status=5;
    elseif isfield(x,'infeasible')
        % problem is infeasible, break the tasklist
        P=glsolstatus(P,'INFEAS',1);
        if tprt,
            txt=['*** Task (' num2str(j) '): ' task.name ...
                ' found that the problem is infeasible, strategy terminated!'];
            gloutput.out=[gloutput.out ; {txt}];
            gloutput.brief=[gloutput.brief ; {txt}];
        end
        status=2;
    elseif any(x.inf>x.sup) || any(isnan(x.inf)) ||...
            any(isnan(x.sup))
        % wrong x bounds found, error
        gldisp('x',[x.inf x.sup])
        error (['glproctskl: Task: ' task.name ' at index ' num2str(j) ...
            ' resulted wrong bound constraints!'])
    elseif any(P.Q.F.inf>P.Q.F.sup) || any(isnan(P.Q.F.inf)) ||...
            any(isnan(P.Q.F.sup))
        % wrong F bounds found, error
        gldisp('F',[P.Q.F.inf P.Q.F.sup])
        error (['glproctskl: Task: ' task.name ' at index ' num2str(j) ...
            ' resulted wrong constraint bounds!'])
    elseif tprt
        % correct bounds, problem feasible, set new bounds for the
        % substituted variables
        [P.Q.Bu, x, out2] = gluniq(P, x, tprt);
        if prt, gloutput.out=[gloutput.out ; out2]; end
        if ~strcmp(task.name,'Read Problem') && ~strcmp(task.name,'Simplify')...
                && ~strcmp(task.name,'Project')
            [xgain, Fgain, gout]=glgain(x_old, x, argin.P, P.Q.F);
            gloutput.out=[gloutput.out ; gout];
            gloutput.brief=[gloutput.brief;...
                {[task.name '...done, reduc: ' mat2str(xgain,10)]};...
                {['  Box:' glbox2str(x, 1:length(x.inf), 3)]}];
            if isfield(x,'best')
                gloutput.brief=[gloutput.brief;...
                    {['  Feas distance:' num2str(x.best.fd)]}];
            end
        else
            gloutput.brief=[gloutput.brief; {[task.name '...done.']}];
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%
    % certificates
    if pro && ~isempty(argout.cert)
        if ~iscell(argout.cert) && (~isfield(argout.cert,'tname') ||...
                ~isfield(argout.cert,'x'))
            % wrong certificate
            gldisp('cert',argout.cert)
            error(['glproctskl: wrong certificate produced for: '...
                task.name ' at entry ' num2str(j) '!']);
        else
            % save the certificate
            if iscell(argout.cert),
                % the certificate is a collection of more certificates
                for i=1:length(argout.cert),
                    if ~ctask,
                        % the task is not a begin end task, therefore set
                        % the task of each new certificate as the current
                        % task
                        argout.cert{i}.task=task;
                    elseif ~isfield(argout.cert{i},'task')
                        error(['glproctskl: tasklist is missing in the '...
                            'certificate: ' task.name ' at entry '...
                            num2str(j) '!']);
                    end
                end
                % add the new certificates to the certificate array
                certs=[certs; argout.cert];
            else
                % single certificate, set the task
                argout.cert.task=task;
                if ctask,
                    % save the subtasklist in the certificate
                    argout.cert.tasklist=argin.tasklist;
                end
                certs=[certs; {argout.cert}];
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%
    % check the timer
    [timer,tstatus]=gltimer(timer, 'c', P, x);
    if tstatus == -1,
        P=glsolstatus(P,'TIMEOUT',1);
        status=5;
    end;

    %%%%%%%%%%%%%%%%%%%%%%%%
    % status checks
    if status==0,
        % normal termination, proceed executing the tasklist
        if ctask,
            % begin-end task, jump to the task after the end of the task
            j=taskend+1;
        else
            % simple task
            j=j+1;
        end
    elseif status==1,
        % stop the strategy (pause or user break),
        if ctask,
            % begin-end task, add the position to taskpos and return
            taskpos=[j argout.taskpos];
        else taskpos=j;
        end
        break;
    elseif status==2
        % break this tasklist (glbreak->break tasklist, infeas),
        % absorb status
        status=0; taskpos=tn; break;
    elseif status==3,
        % break the multiline (glbreak->break multiline)
        taskpos=tn; break;
    elseif status==4,
        % continue with postprocess (glbreak->postprocess,timeout)
        % search for a postprocess in the remaining tasklist
        post=0;
        for k=tn:-1:j+1
            if strcmp(tasklist{k}.name,'Begin Postprocess')
                % postprocess found, execute it.
                j=k;post=1;break;
            end
        end
        if post, continue;
        else taskpos=tn; break;
        end
    elseif status==5,
        % finish reached
        if (isfield(P,'boxes') && ~isempty(P.boxes)) ||...
                (isfield(P, 'bstack') && ~isempty(P.bstack))
            % there are boxes a stack of boxes, checck if there is a merge
            % task in the remaining tasklist
            merge=0;
            for k=tn:-1:(j+1)
                if strcmp(tasklist{k}.name,'Merge')
                    % there is a merge task remaining, execute it.
                    j=k;merge=1;break;
                end
            end
            if merge, continue; end
        end
        % no boxes or no merge for the boxes found, return
        taskpos=tn;break;
    else
        error(['glproctskl: Unknown status code: ' num2str(status)...
            'found after executing the task: ' task.name ' at index '...
            num2str(j) '!']);
    end
end

