%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% glcondition_tiny_box.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glcondition_tiny_box(argin)
%
% Execute the sublist if the condition is met. The condition is met if
% x is tiny.
%
% * needed argin structure input fields *
% P         problem definition
% x         bound constraints
% tasklist  tasklist cell vector
% starttask starting task can be a vector if subtask are present
% not       if not 0 the condition is negated
%
% * optional argin structure input fields *
% nopause   ==1 ignores the pauses in the tasklist
% prt       text output flag       0 = no output 1 = output
% deb       debug output flag      0 = no output 1 = output
% gra       graphics output flag   0 = no output 1 = output
% pro       certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P         new problem definition
% x         new bound constraints
% status    = 0 proceed with the next task
%           = 1 stop the strategy (pause or user break)
%           = 2 break this tasklist (glbreak->break tasklist, infeas)
%           = 3 break the multiline (glbreak->break multiline)
%           = 4 continue with postprocess (glbreak->postprocess,timeout)
%           = 5 finish the strategy (glbreak->finish strategy, problem 
%               not accepted: dim too high/nonquadratic found, after 
%               postprocess)
% taskpos   curent task position, can be a vector if subtask are present
% cert      certificates cell array

function argout=glcondition_tiny_box(argin)
    
global gloutput;    % global output structure
global glconf;      % gloptlab configuration file

TINY=1e-6;          % tiny enties, used as maximal relative width
                    % to treat a box component always small
                    
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'tasklist'};{'starttask'};{'not'}]; 
thisfun=mfilename;

% optional input fields and their default values
oinp=[{'nopause'};{'prt'};{'deb'};{'gra'};{'pro'}];
dval=[{    0    };{  0  };{  0  };{  0  };{  0  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

% init argout
argout.P=argin.P;argout.x=argin.x;argout.status=0;
argout.timer=argin.timer;
if argin.pro, 
    %init certificates
    argout.cert=cell(0);
end

% init variables
starttask=argin.starttask;
if isempty(starttask), starttask=1; end

if argin.prt,
    if argin.nopause, out={'*** IF START (nopause) ***'};
    else out={'*** IF START ***'};
    end
    if argin.not, out=[out; {'Condition: if the box is not small'}];
    else out=[out; {'Condition: if the box is small'}];
    end
    gloutput.out=[gloutput.out ; out];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% evaluate condition
if isfield(argin.x,'status')
    condmet=(~argin.not & argin.x.status=='t') | ...
        (argin.not & argin.x.status~='t');
else
    width=argin.x.sup-argin.x.inf;
    condmet=(~argin.not & width<=TINY) | (argin.not & width>TINY);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if condmet,
    [argout.P, argout.x, argout.status, argout.taskpos, argout.timer, certs2]=...
        glproctskl(argout.P, argout.x, argin.tasklist, starttask, ...
        argout.timer, argin.nopause, argin.prt, argin.gra, argin.pro); 
else
    argout=[];
end
if argin.prt,
    if condmet,
        gloutput.out=[gloutput.out;{'*** IF END ***, Condition was met.'}];
    else
        gloutput.out=[gloutput.out;{'*** IF END ***, Condition was not met.'}];
    end
end