%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glpostprocess.m %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glpostprocess(argin)
%
% Execute the sublist for postprocessing the results.
%
% * needed argin structure input fields *
% P         problem definition
% x         bound constraints
% tasklist  tasklist cell vector
% starttask starting task can be a vector if subtask are present
%
% * optional argin structure input fields *
% nopause   ==1 ignores the pauses in the tasklist
% prt       text output flag       0 = no output 1 = output
% deb       debug output flag      0 = no output 1 = output
% gra       graphics output flag   0 = no output 1 = output
% pro       certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P         new problem definition
% x         new bound constraints
% status    =-3 timeout
%           =-2 user break
%           =-1 paused
%           = 0 normal termination
%           = 1 maxiter reached, 
%           = 2 small box,
%           = 3 mingain not reached
%           = 4 problem infeasible
%           = 5 feasible point found
%           = 6 verified feasible point found
%           = 7 allowed time exceeded
%           = 8 break reached
%           = 9 end reached
%           =10 finish reached
% taskpos   curent task position, can be a vector if subtask are present
% cert      certificates cell array

function argout=glpostprocess(argin)
    
global gloutput;    % global output structure
                    
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'tasklist'};{'starttask'}]; 
thisfun=mfilename;

% optional input fields and their default values
oinp=[{'nopause'};{'prt'};{'deb'};{'gra'};{'pro'}];
dval=[{    0    };{  0  };{  0  };{  0  };{  0  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.P=argin.P;argout.x=argin.x;argout.status=0;
argout.timer=argin.timer;
if argin.pro, 
    %init certificates
    argout.cert=cell(0);
end

% init variables
starttask=argin.starttask;
if isempty(starttask), starttask=1; end

if argin.prt,
    if argin.nopause, out={'*** POSTPROCESS START (nopause) ***'};
    else out={'*** POSTPROCESS START ***'};
    end
    gloutput.out=[gloutput.out ; out];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% process the subtasklist
[argout.P, argout.x, argout.status, argout.taskpos, argout.timer, certs2]=...
    glproctskl(argout.P, argout.x, argin.tasklist, starttask, ...
    argout.timer, argin.nopause, argin.prt, argin.gra, argin.pro);
if argin.prt, gloutput.out=[gloutput.out;{'*** POSTPROCESS END ***'}];end

