%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glconic_bound.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glconic_bound(argin)
%
% Try to find enclosing ellipsoid with midpoint in the origin, for CSP
% given in P by solving a conic program.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% reddir        string representing the direction(s) from which the conic
%               box reduction should be done (unused)
% csolver       name of the used solver
% Zset          type of setting the pattern of the matrix Z in the conic p.
% verify        verification on?
% weisl         weight slack variables?
% intround      user intround?
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%    .E         equality constraints
%     .inf,.sup outer enclosure
%    .slacks    slack variables
%    .B         nx+1 x nx+1 permuted lower triangular matrix with B'B=G
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
%    .u         quadratic enclosure coefficients
%    .zeta      zeta constant of the enclosure
%    .info      information structure
%       .pinf   primal infeasibility measure
%       .dinf   dual infeasibility measure
%       .numerr numretical errors
%       .rigor  indicates that this certificate is rigorous
%
% * argout structure fields *
% F             general constraint bounds
% x             new bound constraints
% out           text output
% cert          certificate cell

function argout=glconic_bound(argin)

global glbreak;   % gloptlab break flag              
global glconf;    % global configuration variable
TINY=1e-6;        % tiny enties
FEAS=1e-6;        % feasibility tolerance

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'reddir'};{'csolver'};{'Zset'};{'verify'};{'weisl'};];
thisfun=mfilename;

% optional input fields and their default values
oinp=[{'intround'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isempty(glbreak),            glbreak=0;end
if isfield(glconf,'tiny'),      TINY=glconf.tiny;end
if isfield(glconf,'feast'),     FEAS=glconf.feast;end 

% init argout
argout.x=argin.x;argout.F=argin.P.Q.F;
if argin.prt,
    argout.out=[{'*** CONIC BOUND REDUCTION ***'};...
        {['Solver: ' argin.csolver ]}];
end
if argin.pro,
    if isempty(argin.cert)
        argout.cert=cell(0);
    else argout.cert=argin.cert;
    end
end

% init variables
nx=length(argin.x.inf);               % number of vars
if isempty(argin.cert), hascert=0;
else hascert=1;
end

% get reduction direction inidieces
if hascert, vind=find(argin.cert.u==1);
elseif ~strcmp(argin.reddir,'auto')
    [ok, vind]=glparsevar(argin.reddir,-nx:nx,-nx:nx);
    if ~ok
        error('glconic_bound: splitvar string has wrong format!');
    elseif isempty(vind) && argin.prt,
        argout.out = [argout.out ; {['No valid reduction directions '...
            'found, conic bound skipped!']}];
        return;
    end
    if argin.prt,
        argout.out = [argout.out ; {['Reduction directions: '...
            num2str(vind)]}];
    end
end

%iterate through all selected directions
for i=vind
    % check user break
    if glbreak, break; end

    % create the vector u with uq(x) is describing the bounding direction
    u=zeros(1+nx+nx^2,1);
    if i>0,u(i+1)=1;
    else u(-i+1)=-1;
    end

    % text output for the reduction direction
    if argin.prt,
        if i>0
            argout.out=[argout.out;{['--> Reducing from the direction: '...
                'x_' num2str(i) ' <---']}];
        else
            argout.out=[argout.out;{['--> Reducing from the direction: '...
                '-x_' num2str(-i) ' <---']}];
        end
    end
        
    if isempty(argin.cert)
        % no certificate, use conic solver to get an approximative
        % solution, else the supplied certificate is used
        % initialize Z mask
        if strcmp(argin.Zset,'auto')
            if ~isempty(argin.P.Q.vmask),
                vmask=argin.P.Q.vmask;vperm=argin.P.Q.vperm;
            else argin.Zset='full';
            end
        end
        if strcmp(argin.Zset,'full'),
            vmask=sparse(tril(ones(nx,nx)));vperm=[];
        elseif strcmp(argin.Zset,'quad')
            vmask=sparse(1:nx,1:nx,ones(1,nx),nx,nx,nx);vperm=[];
        elseif ~strcmp(argin.Zset,'auto')
            error('glconic: Wrong Z masking mode found!');
        end
        % call glguesscut with the previously computed u and with beta=1
        [cert, cin]=glguesscut(argin.P, u, 1, argin.x, argin.csolver, ...
            argin.weisl, vmask, vperm, argin.deb);
        % output
        if argin.deb,
            argout.out=[argout.out; ...
                {[argin.csolver ' solves the conic program:']};...
                glcprogr2str(cin)];
        end
        % complete certificate
        if argin.pro,
            cert.name='Conic Bound';
            cert.tname='Conic';
            cert.F=argout.F;
            cert.x=argout.x;
        end
    else cert=argin.cert;
    end

    % check and output concic solver errors
    if cert.info.numerr==1
        if argin.prt,
            argout.out = [argout.out;{['Conic solver encountered numerical '...
                'problems, the results are inaccurate!']}];
        end
    elseif cert.info.numerr==2
        if argin.prt,
            argout.out = [argout.out;{['Conic solver encountered numerical '...
                'problems and completely failed!']}];
        end
        continue
    end
    
    %compute no and ns
    slacks=cert.slacks;ns=length(slacks);
    x=cert.xb;nx=length(x.inf);no=nx-ns;

    % interpret and output the results of the conic solver
    if cert.info.pinf<FEAS && cert.info.dinf<FEAS,
        if argin.prt,
            argout.out = [argout.out ; {['Enclosure found, zeta:'...
                num2str(cert.zeta)]}];
        end
    elseif cert.info.pinf>FEAS && cert.info.dinf<FEAS
        if argin.prt,
            argout.out = [argout.out ; {['Enclosure failed: Primal conic '...
                'program is infeasible, infeasiblity check needed!']}];
        end
        if ~argin.verify,
            argout.x=[];cert.x=[];
            if iscell(argout.cert), argout.cert=[argout.cert; {cert}];
            else argout.cert=cert;
            end
            return
        end;
    else
        if argin.prt,
            argout.out = [argout.out ; {['Enclosure failed: Dual conic '...
                'program is infeasible!']}];
        end
        continue
    end

    % verify results
    [ain.P.Q, cert, uu]=glverify(cert, x, argin.intround, ...
        argin.verify, argin.deb);
    ain.P.basis=[];ain.P.uni=[];ain.P.opt=0;

    % update the certificate
    if argin.pro, cert.Q=ain.P.Q; end

    % interpret and output the results of the verification
    if all(ain.P.Q.A.inf==0) && all(ain.P.Q.A.sup==0)
        if argin.prt,
            argout.out =[argout.out;{['The results of the conic solver are '...
                'unusable, the lhs of the enclosure is 0!']}];
        end
        continue;
    end

    % graphics output of the found cut
    if argin.gra && no==2,
        % remove slacks for plotting
        ind=1:no;ind2=ind;
        for j=1:no,ind2=[ind2 j*(nx)+ind];end
        % plot the cut
        glqvecplot([-ain.P.Q.F.sup(1) ain.P.Q.A.sup(ind2)],...
            'LineSpec', 'g--');
    end

    % debug for constraint propagation
    if argin.deb, disp(glqvec2str([-ain.P.Q.F.sup ain.P.Q.A.inf],-1)); end

    % use constraint propagation on the verified results to get new bounds
    % create problem definition for CP and use CP
    ain.x=x;ain.bilappr='a';ain.full=1;
    ain.prt=0;ain.deb=argin.deb;
    aout=glpropagate_separable(ain);
    x=aout.x;

    % interpret and output the results of the constraint propagation
    if isempty(x)
        % box is empty
        if argin.prt,
            argout.out=[argout.out ;{'Infeasibility verified!'}];
        end
        argout.x=[];cert.x=[];
        if argin.pro
            if iscell(argout.cert), argout.cert=[argout.cert; {cert}];
            else argout.cert=cert;
            end
        end
        return;
    end
    if any(x.inf>x.sup)
        % wrong x bounds
        gldisp('x',[x.inf x.sup]);
        error('glconic_bound: propagate resulted wrong bounds!');
    end

    % concatenate the new x with the old x
    for j=1:length(argout.x.inf)
        if argout.x.inf(j)<x.inf(j)
            argout.x.inf(j)=x.inf(j);
        end
        if argout.x.sup(j)>x.sup(j)
            argout.x.sup(j)=x.sup(j);
        end
    end

    % check the x after the concatenation
    if any(argout.x.inf>argout.x.sup)
        % box is empty
        if argin.prt,
            argout.out=[argout.out ;{'Infeasibility verified!'}];
        end
        argout.x=[];cert.x=[];
        if iscell(argout.cert), argout.cert=[argout.cert; {cert}];
        else argout.cert=cert;
        end
        return;
    end
    if argin.gra
        % graphics output: plot the new xbox
        glboxplot(argout.x);
    end
    
    % concatenate the slack entries of x with the old F gaining a new F
    for j=1:ns
        if argout.F.inf(slacks(j))<x.inf(no+j)
            argout.F.inf(slacks(j))=x.inf(no+j);
        end
        if argout.F.sup(slacks(j))>x.sup(no+j)
            argout.F.sup(slacks(j))=x.sup(no+j);
        end
    end
    
    % update the certificate
    if argin.pro,
        cert.x=argout.x;
        cert.F=argout.F;
        % add new certificate to the cert array
        if iscell(argout.cert), argout.cert=[argout.cert; {cert}];
        else argout.cert=cert;
        end
    end

    % complete text output
    if argin.prt,
        txt='Conic bound reduction ';
        if argin.verify,
            txt=[txt '(verified) '];
            if argin.intround,
                txt=[txt '(intround) '];
            else
                txt=[txt '(no intround) '];
            end
        else
            txt=[txt '(not verified) '];
        end
        if ~all(cert.u==0)
            txt=[txt 'with weights ' mat2str(cert.u',3)];
        end
        argout.out=[argout.out;{txt}];
        txt='';
        if ~isempty(slacks)
            txt=[txt '( '];
            for j=1:length(slacks)
                txt=[txt 'x_' num2str(no+j) '->eq_'...
                    mat2str(slacks(j),3) ', '];
            end
            txt=[txt(1:end-2) ' slack variables ) '];
        end
        if argin.weisl, txt=[txt '(weighted) '];end
        uu(abs(uu)<10*TINY)=0;
        argout.out = [argout.out ; {[txt 'results:']};...
            {'Unrounded sedumi output: '};{glqvec2str(uu,-1)};...
            {'Without small entries: '};{glqvec2str(uu,-1)}];
        if argin.verify,
            argout.out = [argout.out ; {'Verified sedumi output: '}];
        else
            argout.out = [argout.out ; {'Unverified sedumi output: '}];
        end
        uu2=[-ain.P.Q.F.sup ain.P.Q.A.inf];
        argout.out = [argout.out ; {glqvec2str(uu2,-1)}];
    end
end

if argin.gra
    % graphics output: plot the new xbox
    glboxplot(argout.x);
end