%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% glconic_feasibility.m %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glconic_feasibility(argin)
%
% Make an feasiblity test of the CSP given in P by solving a conic program.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% csolver       name of the used solver
% Zset          type of setting the pattern of the matrix Z in the conic p.
% verify        verification on?
% weisl         weight slack variables?
% intround      user intround?
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%    .E         equality constraints
%     .inf,.sup outer enclosure
%    .slacks    slack variables
%    .B         nx+1 x nx+1 permuted lower triangular matrix with B'B=G
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
%    .u         quadratic enclosure coefficients
%    .zeta      zeta constant of the enclosure
%    .info      information structure
%       .pinf   primal infeasibility measure
%       .dinf   dual infeasibility measure
%       .numerr numretical errors
%       .rigor  indicates that this certificate is rigorous
%
% * argout structure fields *
% F             general constraint bounds
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glconic_feasibility(argin)

global glbreak;   % gloptlab break flag              
global glconf;    % global configuration variable
FEAS=1e-6;        % feasibility tolerance

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'csolver'};{'Zset'};{'verify'};{'weisl'};];

% optional input fields and their default values
oinp=[{'intround'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isempty(glbreak),            glbreak=0;end
if isfield(glconf,'feast'),     FEAS=glconf.feast;end 

% init argout
argout.x=argin.x;argout.F=argin.P.Q.F;
if argin.prt,
    argout.out=[{'*** CONIC FEASIBILITY TEST ***'};...
        {['Solver: ' argin.csolver ]}];
end
if argin.pro && ~isempty(argin.cert), argout.cert=argin.cert; end

% init variables
nx=length(argin.x.inf);               % number of vars

if isempty(argin.cert)
    % no certificate, use conic solver to get an approximative
    % solution, else the supplied certificate is used
    % initialize Z mask
    if strcmp(argin.Zset,'auto')
        if ~isempty(argin.P.Q.vmask),
            vmask=argin.P.Q.vmask;vperm=argin.P.Q.vperm;
        else argin.Zset='full';
        end
    end
    if strcmp(argin.Zset,'full'),
        vmask=sparse(tril(ones(nx,nx)));vperm=[];
    elseif strcmp(argin.Zset,'quad')
        vmask=sparse(1:nx,1:nx,ones(1,nx),nx,nx,nx);vperm=[];
    elseif ~strcmp(argin.Zset,'auto')
        error('glconic: Wrong Z masking mode found!');
    end
    % call glguesscut with u =[] and with beta=0
    [argout.cert, cin]=glguesscut(argin.P, [], 0, argin.x, argin.csolver, ...
        argin.weisl, vmask, vperm, argin.deb);
    % output
    if argin.prt,
        argout.out=[argout.out; ...
            {[argin.csolver ' solves the conic program:']};...
            glcprogr2str(cin)];
    end
    % complete certificate
    if argin.pro,
        argout.cert.name='Conic Feasibility';
        argout.cert.tname='Conic';
        argout.cert.F=argin.P.Q.F;
        argout.cert.x=argin.x;
    end
end

% check and output conic solver errors
if argout.cert.info.numerr==1
    if argin.prt,
        argout.out = [argout.out;{['Conic solver encountered numerical '...
            'problems, the results are inaccurate!']}];
    end
elseif argout.cert.info.numerr==2
    if argin.prt,
        argout.out = [argout.out;{['Conic solver encountered numerical '...
            'problems and completely failed!']}];
    end
    return
end

% new nx, compute no and ns
x=argout.cert.xb;nx=length(x.inf);
slacks=argout.cert.slacks;ns=length(slacks);
no=nx-ns;

% interpret and output the results of the conic solver
if argout.cert.info.pinf<FEAS && argout.cert.info.dinf<FEAS,
   if argin.prt,
        argout.out = [argout.out ; {['Infeasibility check success, '...
            'rigorousity check is needed!']}];
    end
    if ~argin.verify,argout.x=[];argout.cert.x=[];return;end;
elseif argout.cert.info.pinf>FEAS && argout.cert.info.dinf<FEAS
    if argin.prt,
        argout.out =[argout.out; {['Infeasibility check failed: Primal '...
            'conic program is infeasible, infeasiblity could not be '...
            'verified!']}];
    end
    return
else
    if argin.prt,
        argout.out = [argout.out ; {['Infeasibility check failed: Dual '...
            'conic program is infeasible, infeasiblity could not be '...
            'verified!']}];
    end
    return
end

% verify results
[ain.P.Q, argout.cert, uu]=glverify(argout.cert, x, argin.intround, ...
    argin.verify, argin.deb);
ain.P.basis=[];ain.P.uni=[];ain.P.opt=0;

% update the certificate
if argin.pro, argout.cert.Q=ain.P.Q; end

% interpret and output the results of the verification
if all(ain.P.Q.A.inf==0) && all(ain.P.Q.A.sup==0)
    if argin.prt,
        argout.out =[argout.out;{['The results of the conic solver are '...
            'unusable, the lhs of the enclosure is 0!']}];
    end
    return;
end

% graphics output of the found cut
if argin.gra && no==2,
    % remove slacks for plotting
    ind=1:no;ind2=ind;
    for i=1:no,ind2=[ind2 i*(nx)+ind];end
    % plot cut
    glqvecplot([-ain.P.Q.F.sup(1) ain.P.Q.A.sup(ind2)],'LineSpec','g--');
end

% debug for constraint propagation
if argin.deb, disp(glqvec2str([-ain.P.Q.F.sup ain.P.Q.A.inf],-1)); end

% use constraint propagation on the verified results to get new bounds
% create problem definition for CP and use CP
ain.x=x;ain.bilappr='a';ain.full=1;
ain.prt=0;ain.deb=argin.deb;
aout=glpropagate_separable(ain);
x=aout.x;

% interpret and output the results of the constraint propagation
if isempty(x)
    % box is empty
    if argin.prt,argout.out=[argout.out ;{'Infeasibility verified!'}];end
    argout.x=[];
    argout.cert.x=[];
    return;
end
if any(x.inf>x.sup)
    % wrong x bounds
    gldisp('x',[x.inf x.sup]);
    error ('glconic_feasibility: propagate resulted wrong bounds!')
end

% concatenate the new x with the old x
for i=1:length(argout.x.inf)
    if argout.x.inf(i)<x.inf(i)
        argout.x.inf(i)=x.inf(i);
    end
    if argout.x.sup(i)>x.sup(i)
        argout.x.sup(i)=x.sup(i);
    end
end

% check the x after the concatenation
if any(argout.x.inf>argout.x.sup)
    % box is empty
    if argin.prt,argout.out=[argout.out ;{'Infeasibility verified!'}];end
    argout.cert.x=[];
    return;
end
if argin.gra
    % graphics output: plot the new xbox
    glboxplot(argout.x);
end


% concatenate the slack entries of x with the old F gaining a new F
for i=1:ns
    if argout.F.inf(slacks(i))<x.inf(no+i)
        argout.F.inf(slacks(i))=x.inf(no+i);
    end
    if argout.F.sup(slacks(i))>x.sup(no+i)
        argout.F.sup(slacks(i))=x.sup(no+i);
    end
end

% update the certificate
if argin.pro,
    argout.cert.x=argout.x;
    argout.cert.F=argout.F;
end

% complete text output
if argin.prt,
    txt='Conic feasibilty test ';
    if argin.verify,
        txt=[txt '(verified) '];
        if argin.intround,
            txt=[txt '(intround) '];
        else
            txt=[txt '(no intround) '];
        end
    else
        txt=[txt '(not verified) '];
    end
    argout.out = [argout.out ; {[txt 'results:']};...
        {'Unrounded sedumi output: '};{glqvec2str(uu,-1)};...
        {'Without small entries: '};{glqvec2str(uu,-1)}];
    if argin.verify,
        argout.out = [argout.out ; {'Verified sedumi output: '}];
    else
        argout.out = [argout.out ; {'Unverified sedumi output: '}];
    end
    uu2=[-ain.P.Q.F.sup ain.P.Q.A.inf];
    argout.out = [argout.out ; {glqvec2str(uu2,-1)}];
end