%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glguesscut.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [cert, cin]=glguesscut(P, u, beta, x, csolver, weisl, vmask, 
%                                 vperm, prt);
%
% If the problem is in the inequality form transform it to the eq from.
% Guess a cut (beta=1) or make an infeasiblity test (beta=0)
% of the CSP given in A. Return a not verified certificate cert.
% Used by the methods glconic_feasibility, glconic_bound and
% glconic_fixellipsoid
%
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% u             1+(nx+nx^2) vector representing the polynom used in the
%               enclosure
% beta          1 indicates enclosure, 0 infeasiblity guess
% x             nx dim structure with bounds on x (optional)
%   .inf,.sup   outer enclosure
% csolver       name of the used solver
% weisl         weight slack variables?
% vmask         variable mask found through glbilmask and stored in P.vmask
% vperm         variable permutation used to obtain the Zmask, P.vperm
% prt           output level 0=none 1=some 2=debug
%
% cert          certificate structure
%    .E         problem transformed to equalites Eq(x)=0
%    .slacks    1 x nx_new slack variable indexes: x_(nx+i) used for the
%               constraint slack(i) when two indexes for the same consr
%               happen means for the upper bound slack(i) lower slack(i+1)
%    .xb        nx_new dim structure with bounds on x
%    .B         nx+1 x nx+1 permuted lower triangular matrix with B'B=G
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
%    .u         quadratic enclosure coefficients
%    .beta      1 indicates enclosure, 0 infeasiblity guess
%    .zeta      zeta constant of the enclosure
%    .E         equality constraints
%     .inf,.sup outer enclosure
%    .info      information structure
%       .pinf   sedumi primal infeasibility
%       .dinf   sedumi dual infeasibility
%       .numerr sedumi error
%       .rigor  indicates that this certificate is rigorous
% cin           conic program definition (see gloptsolve)
function [cert, cin]=glguesscut(P, u, beta, x, csolver, weisl, vmask,...
                                vperm, prt)

global glconf;      % global configuration variable
INFTY=1e5;          % positive and negative infinity
TINY=1e-6;          % tiny enties

%argcheck
if nargin<3, error('glguesscut: Not enough arguments!'); end
if nargin<4, x=[]; end
if nargin<5, csolver='SeDuMi';end
if nargin<6, weisl=1; end
if nargin<7, vmask=[]; end
if nargin<8, vperm=[]; end
if nargin<9, prt=0; end
if prt==2, deb=1; else deb=0; end

%set constants from config
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

%init
if prt, disp('*glguesscut* START'), end
if isempty(x),
    x.sup=inf+zeros(length(P.Q.int),1);x.inf=-x.sup;
end

%--------------------------------------------------------------------------
% prepare
%--------------------------------------------------------------------------

% remove unbounded constraints (objective included)
bounded=find(P.Q.F.inf>-INFTY | P.Q.F.sup<INFTY);
P.Q.A.inf=P.Q.A.inf(bounded,:);P.Q.A.sup=P.Q.A.sup(bounded,:);
P.Q.F.inf=P.Q.F.inf(bounded,:);P.Q.F.sup=P.Q.F.sup(bounded,:);
% tranform constraints to equs
[x, slacks, E]=glineq2eq(x, P.Q.F, P.Q.A, deb);
% correct the slacks indices
slacks=bounded(slacks);
nx=length(x.inf);           % new total number of vars with all slacks
ns=length(slacks);          % new number of slacks vars
no=nx-ns;                   % number of original vars
if prt, disp([num2str(ns) ' slack variables added']);end

%prepare u
if beta==0
    %feasibility test
    u=zeros(nx+nx^2+1,1);
    u(1)=1;
else
    %enclosure
    unew=zeros(nx+nx^2+1,1);
    unew(1:no+1)=u(1:no+1);
    for i=1:no,
        unew(i*nx+2:i*nx+1+no)=u(i*no+2:(i+1)*no+1);
    end
    u=unew;
end

%weight slacks & vars
if weisl,
    ns=length(slacks);
    %weight quadratic terms of non slack vars
    %with infinite box
    for i=1:nx-ns
        %weight quadratic an linear terms of non slack
        %variables, with infinite box
        if x.inf(i)<-INFTY && x.sup(i)>INFTY
            qind=1+(i*(nx+1));
            if u(qind)==0,u(qind)=TINY^2;end
        elseif x.inf(i)<-INFTY && u(i+1)==0, u(i+1)=-TINY;
        elseif x.sup(i)>INFTY && u(i+1)==0, u(i+1)=TINY;
        end
    end
    %weight linear terms of slacks with infinite box
    for i=nx-ns+1:nx
        if x.inf(i)<-INFTY,u(i+1)=-TINY;
        elseif x.sup(i)>INFTY,u(i+1)=TINY;
        end
    end
end
G0=-glqvec2qmat(u);

%find index set of the non redundant constraints
A.inf=E.inf(:,2:end);A.sup=E.sup(:,2:end);
b.inf=E.inf(:,1);b.sup=E.sup(:,1);
xb.inf=[x.inf;zeros(nx^2,1)];xb.sup=[x.sup;zeros(nx^2,1)];
for i=1:nx
    for j=1:nx
        res=infsup(x.inf(i),x.sup(i))*infsup(x.inf(j),x.sup(j));
        xb.inf(i*nx+j)=res.inf;xb.sup(i*nx+j)=res.sup;
    end
end
if deb, gldisp('no',no,'nx',nx,'Ainf',A.inf,'x',[xb.inf xb.sup]); end;
[C, cperm, rind, exc]=glgaussjord(A,b,xb,deb);

%remove redundant constraints
E.inf=E.inf(rind,:);E.sup=E.sup(rind,:);
if prt && exc~=0, 
    disp([ 'Number of constraints reduced by ' num2str(exc)]),
end
if deb, 
    gldisp('C',C,'cperm',cperm,'rind',rind,'exc',exc,'E',full(E.inf)); 
end;

if prt, 
    gldisp('Einf',E.inf,'x',[x.inf';x.sup'],'slacks',slacks,...
        'u',glqvec2str(u),'beta',beta);
end

% make the mask matrix for Z
nze=2*nx+1;
%linear terms (orig +slacks)
Zmask=sparse(2:nze,ones(1,nze-1),ones(1,nze-1),nze,nze,2*nx^2+3*nx+1);
%bilinear terms of the orig vars
vmask=vmask(1:no,1:no);
Zmask(2:no+1,2:no+1)=vmask;
Zmask(nx+2:nx+no+1,2:no+1)=(vmask+vmask')~=0;
Zmask(nx+2:nx+no+1,nx+2:nx+no+1)=vmask;
%delete diagonal entries
for i=2:nze, Zmask(i,i)=0;end
if ~isempty(vperm), Zperm=[1 vperm+1 no+2:nx+1];
else Zperm=[];
end
 
%compute the G_i-s for Z and z
[G, R]=glmakeZz((E.inf+E.sup)/2,Zmask,x,prt);

% produce G{ng1+nc+1) if beta ~=0 from -beta*zeta
if beta~=0
    Gz=sparse([],[],[],nx+1,nx+1,1);
    Gz(1,1)=-beta;
    G=[G ;{Gz}];
    if prt, gldisp('Gz',Gz); end
end

% create conic problem
nZ=nnz(R);         %number of entries in Z
ny=length(G);      %number of y_i
if beta~=0
    %enclosure 
    cin.obj.b=sparse([],[],[],ny,1,1);
    cin.obj.b(ny,1)=beta;
else
    %feasibility
    cin.obj.b=sparse([],[],[],ny,1,nZ);
end
% non negativity constraints
cin.lin.A=[eye(nZ) zeros(nZ,ny-nZ)];
cin.lin.b(1:nZ,1)=-inf;cin.lin.b(1:nZ,2)=0;
% semidefinite constraints
cin.semi{1}.G0=G0;
cin.semi{1}.G=G;

%--------------------------------------------------------------------------
% solve
%--------------------------------------------------------------------------
[y, xb, cert.info]=gloptsolve(csolver, cin, prt);

if cert.info.numerr==2, return; end
%--------------------------------------------------------------------------
% reconstruct
%--------------------------------------------------------------------------
nR=size(R,1);           %size of R
nY=nnz(R);              %number of y-s in Z
%remove tiny entries in y 
y(abs(y)<TINY)=0;
%reconstruct Z
cert.Z=sparse([],[],[],nR,nR,nY);
for i=1:nR
    for j=1:nR
        if R(i,j) ~= 0
            cert.Z(i,j)=y(R(i,j));
        end
    end
end
%reconstruct z and zeta
if beta ~= 0, cert.z=y(nY+1:ny-1); cert.zeta=y(ny);
else cert.z=y(nY+1:ny);cert.zeta=[];
end
%reconstruct G
Gn=G0;
if prt, glqvec2str(glqmat2qvec(G0)),end;
for k=1:ny
    Gn=Gn-G{k}*y(k); %Z, E beta
    if prt,
        disp([ num2str(y(k)) '*('...
                glqvec2str(glqmat2qvec(-G{k})) ')']);
    end
end

%permute Gn
if prt, Gnorig=Gn; end
if ~isempty(Zperm), Gn=Gn(Zperm, Zperm);end

%remove tiny entries in Gn
diagGn=max(0,diag(Gn));
zerodiagentries=find(diagGn<=TINY*max(diagGn))';
for i=zerodiagentries,
    Gn(i,:)=0;Gn(:,i)=0;
end;
sc=sqrt(diagGn);
Gn(abs(Gn)<TINY*(sc*sc'))=0;

%construct B via cholesky factorization
fac=1e-10;
ptrace=1;
while ptrace>0
    Gn=Gn+diag(max(realmin,fac*diagGn));
    [cert.B,ptrace] = chol(Gn);
    fac=fac*10;
end
for i=zerodiagentries(:),
    cert.B(i,i)=0;
end;

%unpermute cholesky factor
if ~isempty(Zperm),cert.B(:,Zperm)=cert.B;end
if prt, gldisp('eigGn',eig(Gn),'B_err',norm(cert.B'*cert.B-Gnorig,1)); end
%--------------------------------------------------------------------------
% finish certificate
%--------------------------------------------------------------------------
%cert.B, cert.Z, cert.z, cert.zeta, cert.info already set.
cert.E=E;
cert.slacks=slacks;
cert.xb=x;
cert.u=u;
cert.beta=beta;
cert.info.rigor=0;

%--------------------------------------------------------------------------
% interpret results
%--------------------------------------------------------------------------
if cert.info.pinf==0 && cert.info.dinf==0,
    %feasiblity problem
    if beta==0,
        if prt,
            disp(['Infeasibility check sucess: Problem is infeasible, '...
                  'rigorousity check needed!']);
        end
    %enclosure
    else
         if prt,
            disp(['Enclosure found: Problem is feasible, '...
                  'zeta:' num2str(cert.zeta)]);
        end
    end
elseif cert.info.pinf==1 && cert.info.dinf==0
    %feasiblity problem
    if beta==0,
        if prt,
            disp(['Infeasibility check failed: Primal problem is infeasible, '...
                  ', infeasiblity could not be verified!']);
        end
    %enclosure
    else
         if prt,
            disp(['Enclosure failed: Primal problem is infeasible, '...
                  'infeasiblity check needed!']);
        end
    end
else
    if prt, disp('Dual Problem infesible!');end
end
if prt, disp('*glguesscut* END'), end
