%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glguessell.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [cert, cin]=glguessell(P, u, x, csolver, weisl, vmask, ...
%                                 vperm, prt);
%
% If the problem is in the inequality form transform it to the eq from.
% Find a bounding ellipsoid with radom midpoint, for CSP given in P
% by calling the function glguesscut and return the results in a not 
% verified certificate.
%
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% u             nx x 1 vector representing the weigths in the ellipsoid
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% csolver       name of the used solver
% weisl         weight slack variables?
% vmask         variable mask found through glbilmask and stored in P.vmask
% vperm         variable permutation used to obtain the Zmask, P.vperm
% prt           output level 0=none 1=some 2=debug
%
% cert          certificate structure
%    .E         problem transformed to equalites Eq(x)=0
%    .slacks    1 x nx_new slack variable indexes: x_(nx+i) used for the
%               constraint slack(i) when two indexes for the same consr
%               happen means for the upper bound slack(i) lower slack(i+1)
%    .xb         nx_new dim structure with bounds on x
%    .G         nx+1 x nx+1 matrix (psd? triang)
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
%    .zeta0     zeta0 constant
%    .zeta      nx x 1 vetor
%    .delta     delta constant
%    .u         nx x 1 vector representing the weigths in the enclosure
%    .E         equality constraints
%     .inf,.sup outer enclosure
%    .info      information structure
%       .pinf   primal infeasibility measure
%       .dinf   dual infeasibility measure
%       .numerr numretical errors
%       .rigor  indicates that this certificate is rigorous
% x             nx:new dim structure with bounds on x (optional)
% cin           conic program definition (see gloptsolve)

function [cert, cin]=glguessell(P, u, x, csolver, weisl, vmask,...
                                vperm, prt)

global glconf;      % global configuration variable
INFTY=1e5;          % positive and negative infinity
TINY=1e-6;          % tiny enties

%argcheck
if nargin<2, error('glguessell: Not enough arguments!'); end
if nargin<3, x=[]; end
if nargin<4, csolver='SeDuMi';end
if nargin<5, weisl=1; end
if nargin<6, vmask=[]; end
if nargin<7, vperm=[]; end
if nargin<8, prt=0; end
if prt==2,   deb=1; else deb=0; end

%set constants from config
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

%init
if prt, disp('*glguessell* START'), end
if isempty(x),
    x.sup=inf+zeros(length(P.Q.int),1);x.inf=-x.sup;
end

%--------------------------------------------------------------------------
% prepare
%--------------------------------------------------------------------------


%remove unbounded constraints (objective included)
bounded=find(P.Q.F.inf>-INFTY | P.Q.F.sup<INFTY);
P.Q.A.inf=P.Q.A.inf(bounded,:);P.Q.A.sup=P.Q.A.sup(bounded,:);
P.Q.F.inf=P.Q.F.inf(bounded,:);P.Q.F.sup=P.Q.F.sup(bounded,:);
%tranform constraints to equs
[x, slacks, E]=glineq2eq(x, P.Q.F, P.Q.A, deb);
% correct the slacks indices
slacks=bounded(slacks);
nx=length(x.inf);           %new total number of vars with all slacks
ns=length(slacks);          %new number of slacks vars
no=nx-ns;                   %number of original vars
if prt && ns, disp([num2str(ns) ' slack variables added']);end
u=[u ; zeros(ns,1)];        %resize u


%prepare u
if weisl,
    %weight slacks variables
    ns=length(slacks);
    %weight linear terms of slacks with infinite box
    for i=nx-ns+1:nx
        if x.inf(i)<-INFTY,u(i)=-TINY;
        elseif x.sup(i)>INFTY,u(i)=TINY;
        end
    end
end

if deb, 
    gldisp('Einf',E.inf,'x',[x.inf';x.sup'],'slacks',slacks,...
        'u',glqvec2str(u),'beta',beta);
end

%find index set of the non redundant constraints
P.Q.A.inf=E.inf(:,2:end);P.Q.A.sup=E.sup(:,2:end);
b.inf=E.inf(:,1);b.sup=E.sup(:,1);
xb.inf=[x.inf;zeros(nx^2,1)];xb.sup=[x.sup;zeros(nx^2,1)];
for i=1:nx
    for j=1:nx
        res=infsup(x.inf(i),x.sup(i))*infsup(x.inf(j),x.sup(j));
        xb.inf(i*nx+j)=res.inf;xb.sup(i*nx+j)=res.sup;
    end
end
if deb,
    gldisp('nn',nn,'nx',nx,'Ainf',A.inf,'x',[xb.inf xb.sup]);
end;
[C, cperm, rind, exc]=glgaussjord(P.Q.A,b,xb,deb);

%remove redundant constraints
E.inf=E.inf(rind,:);E.sup=E.sup(rind,:);
nc=size(E.inf,1);           %new number of constraints
if prt && exc~=0, 
    disp([ 'Number of constraints reduced by ' num2str(exc)]);
end
if deb, 
    gldisp('C',C,'cperm',cperm,'rind',rind,'exc',exc,'E',full(E.inf));
end

%create sedumi input
linv=(sum(P.Q.hess(1:no,1:no),1)==0);     %linear original vars
lind=find(linv==1);qind=find(linv==0);%index of linear & quadratic orig v
nl=length(lind);nq=length(qind);      %number of linear & quadratic orig v

%MAKE G ALL G_i must be symmetric and (nx+1)x(nx+1)
%make G0 from -||w_o x_o||^2+w_s'x_s -> will be used in the vector c
G0=sparse([],[],[],nx+1,nx+1,nx+ns);%constant terms matrix
G0(qind+1,qind+1)=-diag(u(qind).^2);%-||w_o x_o||^2 for quadratic terms
if ~isempty(lind)
    %-w_ox_o for linear original variables
    G0(lind+1,1)=-u(lind)'./2;
    G0(1,lind+1)=-u(lind)'./2;
end
%-w_ox_o for slack variables
G0(no+2:nx+1,1)=-u(no+1:nx)'./2;G0(1,no+2:nx+1)=-u(no+1:nx)'./2;

% make the mask matrix for Z
nze=2*nx+1;
%linear terms (orig +slacks)
Zmask=sparse(2:nze,ones(1,nze-1),ones(1,nze-1),nze,nze,2*nx^2+3*nx+1);
%bilinear terms of the orig vars
Zmask(2:no+1,2:no+1)=vmask;
Zmask(nx+2:nx+no+1,2:no+1)=(vmask+vmask')~=0;
Zmask(nx+2:nx+no+1,nx+2:nx+no+1)=vmask;
%delete diagonal entries
for i=2:nze, Zmask(i,i)=0;end
if ~isempty(vperm), Zperm=[1 vperm+1 no+2:nx+1];
else Zperm=[];
end

%make GZz-s from Z and z
[GZz, R]=glmakeZz((E.inf+E.sup)/2,Zmask,x,0);

%make Gzeta-s from 2zeta'(w_q*x_q)
Gzeta=cell(nq,1);
for i=1:nq,
    k=qind(i);
    Gzeta(i)={sparse([k+1,1],[1,k+1],[-u(k) -u(k)],nx+1,nx+1,2)};
end

%make Gdelta from delta
Gdelta=sparse(1,1,-1,nx+1,nx+1,1);

%make Gzeta0 from zeta0
Gzeta0=sparse([],[],[],nx+1,nx+1,1);

%make Gzeta1 from zeta1
Gzeta1=sparse([],[],[],nx+1,nx+1,1);

%make G : merge the different Gs (all but G0)
G=[GZz;{Gzeta0};{Gzeta1};Gzeta;{Gdelta}];
nZ=nnz(R);         %number of entries in Z
nZz=nZ+nc;         %number of entries in Z and z
ny=length(G);      %number of y_i
if prt,
    gldisp('nx',nx,'no',no,'ns',ns,'nl',nl,'nq',nq,'nc',nc,'nZ',nZ,...
        'nZn',nZz,'ny',ny,'G0',G0,'nGZz',size(GZz,1));
end
% PRODUCE A : ny x (nonneg:nZ+1)+(rconic:nq+2)+(semi:(nx+1)^2)%
%Arows=nZ+1+(nq+2)+(nx+1)^2;
%calculate the number of entries in A=
%nC=(nZ+1)+(nq+2);                   %(nonnegs)+(rconic)
%for i=1:ny, nC=nC+nnz(G{i}); end    %+(semi)
%A=sparse([],[],[],ny,Arows,nC);     %generate A

zeta0ind=nZz+1;
zeta1ind=nZz+2;
zetaind=nZz+3:nZz+nq+2;
deltaind=ny;

%% set non-negativity constraints for -Z>=0, zeta1<=1/2
% cin.lin.A=sparse([1:nZ+1],     [1:nZ,zeta1ind],     [ones(1,nZ+1)],...
%     nZ+1,ny,nZ+1);
% cin.lin.b=sparse([1:nZ+1,nZ+1],[ones(1,nZ+1),2],[zeros(1,nZ)-inf,-inf,1/2],...
%     nZ+1,2,nZ+2);

%% set non-negativity constraints for Z<=0, zeta1<=1/2, zeta0+delta>=0
cin.lin.A=sparse([1:nZ+2,nZ+2], [1:nZ,zeta1ind,zeta0ind,deltaind],...
    ones(1,nZ+3),nZ+2,ny,nZ+3);
cin.lin.b=sparse([1:nZ+1,nZ+1,nZ+2], [ones(1,nZ+1),2,2],...
    [zeros(1,nZ+1)-inf,1/2,inf], nZ+2,2,nZ+3);

%% set rconic constraints for 1/2*||zeta||^2<zeta0zeta1
cin.quadr{1}.A=sparse(1:nq,  zetaind, -ones(1,nq),nq, ny, nq);%zeta
cin.quadr{1}.b=sparse([],[],[],nq,1,0);
cin.quadr{1}.c=sparse(1, zeta0ind, -1, 1, ny, 1);        %zeta0
cin.quadr{1}.d=0;
cin.quadr{1}.e=sparse(1, zeta1ind, -1, 1, ny, 1);        %zeta1
cin.quadr{1}.f=0;

%% set semidef constraints from G
cin.semi{1}.G0=G0;
cin.semi{1}.G=G;
% set objective b = zeta0+delta
cin.obj.b=sparse([zeta0ind,deltaind],[1,1],[1,1],ny,1,2);

%--------------------------------------------------------------------------
% solve
%--------------------------------------------------------------------------
[y, xb, cert.info]=gloptsolve(csolver, cin, prt);

%--------------------------------------------------------------------------
% reconstruct
%--------------------------------------------------------------------------
if ~isempty(y)
    nR=size(R,1);      %size of R
    %remove tiny entries in y
    y(abs(y)<TINY)=0;
    %reconstruct Z
    cert.Z=sparse([],[],[],nR,nR,nZ);
    for i=1:nR
        for j=1:nR
            if R(i,j) ~= 0
                cert.Z(i,j)=y(R(i,j));
            end
        end
    end
    %reconstruct z
    cert.z=y(nZ+1:nZz);
    %reconstruct zeta0
    cert.zeta0=y(nZz+1);
    %reconstruct zeta1
    cert.zeta1=y(nZz+2);
    %reconstruct zeta
    cert.zeta=y(nZz+3:nZz+nq+2);
    %reconstruct delta
    cert.delta=y(nZz+nq+3);
    %reconstruct G
    Gn=G0;
    if prt, glqvec2str(glqmat2qvec(G0)),end;
    for k=1:ny
        Gn=Gn-G{k}*y(k); %Z, E beta
        if prt,
            disp([ num2str(y(k)) '*('...
                glqvec2str(glqmat2qvec(-G{k})) ')']);
        end
    end

    %permute Gn
    if prt, Gnorig=Gn; end
    if ~isempty(Zperm), Gn=Gn(Zperm, Zperm);end

    %remove tiny entries in Gn
    diagGn=max(0,diag(Gn));
    zerodiagentries=find(diagGn<=TINY*max(diagGn))';
    for i=zerodiagentries,
        Gn(i,:)=0;Gn(:,i)=0;
    end;
    sc=sqrt(diagGn);
    Gn(abs(Gn)<TINY*(sc*sc'))=0;

    %construct B via cholesky factorization
    fac=1e-10;
    ptrace=1;
    while ptrace>0
        Gn=Gn+diag(max(realmin,fac*diagGn));
        [cert.B,ptrace] = chol(Gn);
        fac=fac*10;
    end
    for i=zerodiagentries(:),
        cert.B(i,i)=0;
    end;

    %unpermute cholesky factor
    if ~isempty(Zperm),cert.B(:,Zperm)=cert.B;end
    if prt, 
        gldisp('eigGn',eig(Gn),'B_err',norm(cert.B'*cert.B-Gnorig,1));
    end
end

%--------------------------------------------------------------------------
% finish certificate
%--------------------------------------------------------------------------
%cert.B, cert.Z, cert.z, cert.zeta, cert.zeta0, cert.zeta1, cert.delta,
%cert.info already set.
cert.E=E;
cert.slacks=slacks;
cert.xb=x;
cert.u=u;
cert.info.rigor=0;

%--------------------------------------------------------------------------
% interpret results
%--------------------------------------------------------------------------
if prt,
    if cert.info.numerr==1,disp('Inaccurate Resuts');end
    if cert.info.numerr==2,
        disp('Serious numeriacal errors!');
    elseif cert.info.pinf<1 && cert.info.dinf<1,
        disp('Enclosure found: Problem is feasible.');
    elseif cert.info.pinf==1 && cert.info.dinf<1
        disp(['Enclosure failed: Primal problem is infeasible, '...
            'infeasiblity check needed!']);
    else
        disp('Dual Problem infesible!');
    end
    disp('*glguessell* END')
end