%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmakeZz.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [G, R]=glmakeZz(E, Zmask, xbox, prt);
%
% internal function for calcuating G_i, for making Z and z
%
% E             nc x (nxn^2+nxn) equality constraints with nxn=nx+nslack
% Zmask         nx^2+1 x nx^2+1 sparse mask matrix for Z, indicating
%               which entries could be set in Z
% xbox          nxn x 1 bound constraints
%   .inf,.sup   outer enclosures
% u             quadratic approximation vector of uq(x)
% beta          constant 0 for feasibilty test else 1
% prt           output level
%
% G             ny cell containing the (nxn+1) x (nxn+1) matrices G_i
%               with ny=nZ+nz(+1), (+1) appends for enclosures (beta~=0)
% R             (1+2nxn) x (1+2nxn) recostruction matrix with nnz(R)=nZ
%               where nZ is number of nonzeroes in the matrix Z

function [G, R]=glmakeZz(E, Zmask, xbox, prt)

global glconf;      %gloptlab configuration file
INF=1e5;            % positive and negative infinity used for determining
                    % which rows and cols of Z are left zero
% argcheck
if nargin<3, error('glmakeZz: Not enough arguments!');end
if nargin<4, prt=0;end

%set constants from config
if isfield(glconf,'infty'),INF=glconf.infty;end 

% init
nx=length(xbox.inf);        %number of vars
nc=size(E,1);               %number of constraints
ny=1;                       %conts the number of the y-s
nymax= 2*nx^2+3*nx+2+nc;    %maximum number of the y-s
if prt, gldisp('nx',nx,'nc',nc,'nymax',nymax); end
R=sparse([],[],[],2*nx+1,2*nx+1,2*nx^2+3*nx+1);
%R=zeros(2*nx+1,2*nx+1);
G=cell(nymax,1);

% PRODUCE G{k} %
% produce G{k} for k=1:(2*nx+1)*(nx+1) form
% [1;xbox.sup-x;x-xbox.inf]'Z[1;xbox.sup-x;x-xbox.inf]
% and ignore variables where G{k} contains infinity or NaN
ind=[nx+1;2*nx+1;(nx^2+5*nx+2)/2;(3*nx^2+5*nx+2)/2;2*nx^2+3*nx+1];
if prt, gldisp('ind',ind); end

for k=1:ind(5),
    if k==1 && Zmask(k,1),
        %constant term 
        if prt, disp(['PHASE 1, k=' num2str(k)]); end
        G{ny}=sparse([],[],[],nx+1,nx+1,1);
        G{ny}(1,1)=1;
        %traspose to force symetry and sign
        G{ny}=-(G{ny}+G{ny}')/2;
        %create reconstruction matrix
        R(k,1)=ny;
        if prt, gldisp('Gi',G{ny}); end
        ny=ny+1;
    elseif 1<k && k<=ind(1) && Zmask(k,1) && xbox.sup(k-1)<INF,
        %linear terms (1)
        if prt, disp(['PHASE 2, k=' num2str(k)]); end
        G{ny}=sparse([],[],[],nx+1,nx+1,3);
        G{ny}(1,1)=xbox.sup(k-1);
        G{ny}(k,1)=-1;
        %traspose to force symetry and sign
        G{ny}=-(G{ny}+G{ny}')/2;
        %create reconstruction matrix
        R(k,1)=ny;
        if prt, gldisp('Gi',G{ny}); end
        ny=ny+1;
    elseif ind(1)<k && k<=ind(2) && Zmask(k,1) && xbox.inf(k-nx-1)>-INF,
        %linear terms(2)
        if prt, disp(['PHASE 3, k=' num2str(k)]); end
        G{ny}=sparse([],[],[],nx+1,nx+1,3);
        G{ny}(1,1)=-xbox.inf(k-nx-1);
        G{ny}(k-nx,1)=1;
        %traspose to force symetry and sign
        G{ny}=-(G{ny}+G{ny}')/2;
        %create reconstruction matrix
        R(k,1)=ny;
        if prt, gldisp('Gi',G{ny}); end
        ny=ny+1;
    elseif ind(2)<k && k<=ind(3),
        %non-linear terms upper triangle
        %find index k->(i,j)
        i=k-ind(2);j=1;t=nx;
        while i>t,i=i-t;j=j+1;t=t-1;end
        i=nx-(t-i);
        %check infinity bounds
        if Zmask(1+i,1+j) && xbox.sup(i)<INF && xbox.sup(j)<INF
            if prt,
                disp(['PHASE 4, k=' num2str(k)]);
                disp(['conversion:' num2str(k) '->(' num2str(i) ',' num2str(j) ')']);
            end
            G{ny}=sparse([],[],[],nx+1,nx+1,7);
            G{ny}(1,1)=xbox.sup(i)*xbox.sup(j);
            G{ny}(1+i,1)=-xbox.sup(j);
            G{ny}(1+j,1)=G{ny}(1+j,1)-xbox.sup(i);
            G{ny}(1+i,1+j)=1;
            %traspose to force symetry and sign
            G{ny}=-(G{ny}+G{ny}')/2;
            %create reconstruction matrix
            R(1+i,1+j)=ny;
            if prt, gldisp('Gi',G{ny}); end
            ny=ny+1;
        elseif prt,
            disp('Infinite bound found -> variable ignored');
        end
    elseif ind(3)<k && k<=ind(4)
        %non-linear terms lower square
        %find index k->(i,j)
        i=k-ind(3);j=floor(i/nx)+1;i=mod(i,nx);
        if i==0,i=nx;j=j-1;end
        %check infinity bounds
        if Zmask(1+nx+i,1+j) && xbox.inf(i)>-INF && xbox.sup(j) <INF
            if prt,
                disp(['PHASE 5, k=' num2str(k)]);
                disp(['conversion:' num2str(k) '->(' num2str(i) ',' num2str(j) ')']);
            end
            G{ny}=sparse([],[],[],nx+1,nx+1,7);
            G{ny}(1,1)=-xbox.inf(i)*xbox.sup(j);
            G{ny}(1+i,1)=xbox.sup(j);
            G{ny}(1+j,1)=G{ny}(1+j,1)+xbox.inf(i);
            G{ny}(1+i,1+j)=-1;
            %traspose to force symetry and sign
            G{ny}=-(G{ny}+G{ny}')/2;
            %create reconstruction matrix
            R(1+nx+i,1+j)=ny;
            if prt, gldisp('Gi',G{ny}); end
            ny=ny+1;
        elseif prt,
            disp('Infinite bound found -> variable ignored');
        end
    elseif ind(4)<k && k<=ind(5),
        %non-linear terms lower triangle
        %find index k->(i,j)
        i=k-ind(4);j=1;t=nx;
        while i>t,i=i-t;j=j+1;t=t-1;end
        i=nx-(t-i);
        %check infinity bounds
        if Zmask(1+nx+i,1+nx+j) && xbox.inf(i)>-INF && xbox.inf(j) >-INF
            if prt,
                disp(['PHASE 6, k=' num2str(k)]);
                disp(['conversion:' num2str(k) '->(' num2str(i) ',' num2str(j) ')']);
            end
            G{ny}=sparse([],[],[],nx+1,nx+1,7);
            G{ny}(1,1)=xbox.inf(i)*xbox.inf(j);
            G{ny}(1+i,1)=-xbox.inf(j);
            G{ny}(1+j,1)=G{ny}(1+j,1)-xbox.inf(i);
            G{ny}(1+i,1+j)=1;
            %traspose to force symetry and sign
            G{ny}=-(G{ny}+G{ny}')/2;
            %create reconstruction matrix
            R(1+nx+i,1+nx+j)=ny;
            if prt, gldisp('Gi',G{ny}); end
            ny=ny+1;
        elseif prt,
            disp('Infinite bound found -> variable ignored');
        end
    elseif prt,
        disp('Infinite bound found -> variable ignored');
    end
end
% produce G{k} for k=ng1+1:ng1+nc form z'Eq(x)
for i=1:nc
    if prt, disp(['PHASE 7, k=' num2str(k)]); end
    G{ny}=-glqvec2qmat(E(i,:));
    if prt, gldisp('Ei',E(i,:),'Gi',G{ny}); end
    ny=ny+1;
end
G(ny:end)=[];
if prt, gldisp('ny',ny,'R',full(R)); end
