%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glverify.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Q,cert,uu]=glverify(cert,xbox,intround,verify,prt);
% verifies the results of glfind... and makes the results rigorous by
% using interval arithmetic. The interval matrix P represents a polynom
% p(x)= (1 x)'P(1 x) with interval coeffs.
%
% cert          certificate structure
%    .B         nx+1 x nx+1 permuted cholesky factor of G, G=B'B
%    .Z         2nx+1 x 2nx+1 triangular matrix
%    .z         nc dim vector
%    .zeta      zeta constant
%    .u         1 x (nx+1)nx+1 vector rep. the quadratic bounding
%    .E         simplified constraints
% xbox          nx x 1 bounds on x
%    .inf,.sup  interval bounds 
% intround      =1 indicates intround is used
% verify        =1 cut verification
% prt           output level 0=none 1=some 2=debug
% 
% Q                     structure encoding the cut
%      .A               1 x (nx+nx^2) sparse matrix of the cut coeffs
%        .inf,.sup      bounds for coefficients
%      .F               1 x 1 structure with bounds for general constrs
%        .inf,.sup      outer enclosure
%      .int             characteristic vector of integer variables
% cert
%     .info       information structure
%       .rigor    indicates that this certificate is rigorous
% uu              unrounded sedumi output

function [Q,cert,uu] = glverify(cert, xbox, intround, verify, prt)

global glconf;          % global configuration variable
TINY=1e-6;              % tiny enties
%intround costants
SIGMA=1e-6;
QMAX=10;
MAXITER=1000;
%nround constants
NMAX=50;


% ARGCHECK %
if nargin<2,error('glmakecut: Not enough arguments!');end
if nargin<3,intround=0;end
if nargin<4,verify=0;end
if nargin<5,prt=0;end

%set config
if isfield(glconf,'tiny'),TINY=glconf.tiny;end
if isfield(glconf,'roundsig'),SIGMA=glconf.roundsig;end 
if isfield(glconf,'roundmaxq'),QMAX=glconf.roundmaxq;end
if isfield(glconf,'roundmaxit'),MAXITER=glconf.roundmaxit;end 
if isfield(glconf,'roundmaxn'),NMAX=glconf.roundmaxn;end

if prt, disp('*qlmakecut* START'), end

%remove infinity bounds
xbox.inf(isinf(xbox.inf))=0;xbox.sup(isinf(xbox.sup))=0;
nx=size(cert.B,1)-1;
%unrounded sedumi output
uu=getu(cert.B,cert.Z,xbox,cert.z,cert.E.inf);
if prt, 
    gldisp('Z',cert.Z,'z',cert.z,'zeta',cert.zeta,'B',cert.B);
    disp('Certificate:');
    gldispcert(cert,xbox);
    disp('Unrounded sedumi output: ');
    disp(glqvec2str(uu,-1));
end

if intround,
    %scale y with intround
    y=[reshape(cert.Z,size(cert.Z,1)*size(cert.Z,2),1);cert.z]';
    [p,q,err] = glintround(y,SIGMA,inf,QMAX,MAXITER);
    if prt, 
        gldisp('y',y,'q',q,'p',p,'err',err,...
            'y_scale_err',max(abs(y-(1/q)*p)));
    end;
    B2=sqrt(q)*cert.B;
    %rescale B
    %find an 0<n<NMAX so that 
    %norm(round(n^2*R'*R)/(n^2)-q*cert.G, inf) minimal
    n=0;minnorm=inf;
    for i=1:NMAX
        B3=round(i*B2);
        no=norm((B3'*B3)/(i^2)-B2'*B2, inf);
        if no<minnorm, minnorm=no;n=i;B=B3;end
    end
    y2=round(n^2*p);
    if prt,
        gldisp('minnorm',minnorm,'n',n,'B',B,'B_total_err',...
            max(max(abs(B'*B/(n^2)-B2'*B2))),'y2',y2,'y_total_err',...
            max(abs(y-(1/(q*n^2))*y2)));
    end;
    cert.B=B;
    %rewrite Z and z
    zsiz=size(cert.Z);
    cert.Z=reshape(y2(1:zsiz(1)*zsiz(2)),zsiz(1),zsiz(2));
    cert.z=y2(zsiz(1)*zsiz(2)+1:end);
    if prt, gldisp('Z',cert.Z,'z',cert.z); end
end

if verify,
    %convert to intervals
    xbox.inf=infsup(xbox.inf-eps,xbox.inf+eps);
    xbox.sup=infsup(xbox.sup-eps,xbox.sup+eps);
    B=infsup(cert.B,cert.B);
    Z=infsup(cert.Z,cert.Z);
    z=infsup(cert.z,cert.z);
    E=infsup(cert.E.inf,cert.E.sup);
    %compute u
    u=getu(B,Z,xbox,z,E);
    if prt,
        disp('Corrected sedumi output: ');
        disp(glqvec2str(u.sup,-1));
    end
    %define Q structure
    Q.A.inf=u(2:end).inf';Q.A.sup=u(2:end).sup';    
    Q.F.inf=-inf;Q.F.sup=-(u(1).inf);
    cert.info.rigor=1;
else
    %compute u
    u=getu(cert.B,cert.Z,xbox,cert.z,cert.E.inf);
    %not verifyed, remove tiny entries
    u(abs(u)<100*TINY)=0;
    if prt,
        disp('Uncorrected sedumi output (tiny entries removed): ');
        disp(glqvec2str(u,-1));
    end
    %define Q structure
    Q.A.inf=u(2:end)';Q.A.sup=u(2:end)';
    Q.F.inf=-inf;Q.F.sup=-u(1);
    cert.info.rigor=0;
end
Q.int=zeros(nx,1);Q.Bu=[];
Q=glfindspars(Q);Q.gaps=[];

if prt, 
    %format short e
    %p1=[Q.F.sup Q.A.inf];
    %p1(abs(p1)<TINY)=0;
    %disp('Sedumi output (tiny entries removed): ');
    %disp(glqvec2str( p1, -1));
    %A_inf=Q.A.inf, A_sup=Q.A.sup, F=[Q.F.inf,Q.F.sup],
    %P_min=min(min(abs(Q.A.inf(Q.A.inf~=0)))),
    %P_max=max(max(abs(Q.A.inf))),
    %format short
    disp('*qlmakecut* END'),
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Get u %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function u=getu(B,Z,xbox,z,E)
nx=length(xbox.inf);
%part matrix Z
w1=Z(2:nx+1, 1);
w2=Z(nx+2:2*nx+1, 1);
Z11=Z(2:nx+1, 2:nx+1);
Z21=Z(nx+2:2*nx+1, 2:nx+1);
Z22=Z(nx+2:2*nx+1, nx+2:2*nx+1);
%compute the constant, linear and quadratic terms for Z
const=Z(1,1)+xbox.sup'*(w1+Z11*xbox.sup) ...
      -xbox.inf'*(w2+Z21*xbox.sup-Z22*xbox.inf);
lin=-w1+w2-(Z11+Z11')*xbox.sup+Z21*xbox.sup+Z21'*xbox.inf...
      -(Z22+Z22')*xbox.inf;
quad=Z11-Z21+Z22;
% compute z'Eq(x) NOT GOOD CREATE E.inf and E.sup from constr2eq,
% give sedumi (E.inf+E.sup)/2
if isintval(E)
    zE=infsup(zeros(nx+1,nx+1),zeros(nx+1,nx+1));
    for i=1:size(E,1)
        coni=glqvec2qmat(E.inf(i,:));
        cons=glqvec2qmat(E.sup(i,:));
        zE=zE-infsup(coni,cons)*z(i);
    end
else
    zE=zeros(nx+1,nx+1);
    for i=1:size(E,1)
        zE=zE-glqvec2qmat(E(i,:))*z(i);
    end
end
%compute P
P=-(B'*B-[[const;lin],[zeros(1,nx);quad]]+zE);
%transform P to vector u
u=glqmat2qvec(P);
