%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glehull.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glehull(argin)
%
% Compute the ellipse hull for each strict convex constraint and 
% concatenate the found hulls hopefully getting a tighter x.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
%
% * optional argin structure input fields *
% scal          scale the problem (=1) or not (=0)
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .... TODO
%
% * argout structure fields *
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glehull(argin)

global glconf;    % global configuration variable
INFTY=1e5;        % positive and negative infinity

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'}];

% optional input fields and their default values
oinp=[{'scal'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  1   };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'infty'),     INFTY=glconf.infty;end 

% init argout
argout.x=[];
if argin.prt, argout.out={'*** ELLIPSOID HULL ***'}; end
if argin.pro, 
    if isempty(argin.cert)
        argout.cert.name='Ellipsoid Hull';
        argout.cert.tname='Ehull';
        argout.cert.x=[];
    else argout.cert=argin.cert;
    end
end

% init variables
nc=size(argin.P.Q.A.inf,1);     % number of constraints
nx=length(argin.P.Q.int);       % number of all variables
x=argin.x; argout.x=argin.x;

if argin.scal && isfield(argin.P,'scal')
    format short g;
    if argin.deb, gldisp('Aun',full(argin.P.Q.A.sup)); end
    [argin.P.Q.A, argin.P.Q.F, x]=glscale(argin.P.Q.A, ...
        argin.P.Q.F, argin.P.scal.E, x, argin.P.scal.x,...
        ones(size(argin.P.scal.c)), 0);
    if argin.deb, gldisp('Asc',full(argin.P.Q.A.sup)); end
    if argin.prt, 
        argout.out=[argout.out ; {'Variable scaling applied!'}];
    end
    format short;
end

% find and bound variables which are only linear in a constraint
nlind=ones(nc,nx);  % index of vars which are not only linear
i2=1:nx;
for i=1:nc, 
    for j=1:nx,
        if all(argin.P.Q.A.sup(i,j*nx+1:(j+1)*nx)==0) &&...     %bil1_sup==0
                all(argin.P.Q.A.sup(i,i2*nx+j)==0) &&...        %bil2_sup==0
                all(argin.P.Q.A.inf(i,j*nx+1:(j+1)*nx)==0) &&...%bil1_inf==0
                all(argin.P.Q.A.inf(i,i2*nx+j)==0)              %bil2_inf==0  
            if (argin.P.Q.A.sup(i,j)~=0 || argin.P.Q.A.inf(i,j)~=0)
                % linear term is set, bound linear term and 
                % set it into the Fbox
                c=infsup(argin.P.Q.A.inf(i,j),argin.P.Q.A.sup(i,j))*...
                    infsup(x.inf(j),x.sup(j));
                argin.P.Q.F.inf(i)=argin.P.Q.F.inf(i)+(-c.sup);
                argin.P.Q.F.sup(i)=argin.P.Q.F.sup(i)+(-c.inf);
            end
            nlind(i,j)=0;  % remove variable index from the index matrix
        end
    end
end
% resolve two-sided constraints into inequalities
% first all lower bounds, then all upper bounds

% NEGATIVE INDEXES COMMENTED OUT, USABLE MAYBE LATER FOR EXCLUSION
il=(argin.P.Q.F.inf>-INFTY);
iu=(argin.P.Q.F.sup<INFTY);                % pos and neg indexes
iil=find(il);iiu=find(iu);
nl=nnz(il);nu=nnz(iu);                     % nl+nu max 2*nc (all two sided)
An=[-argin.P.Q.A.sup(il,:);-argin.P.Q.A.inf(iu,:)]; % lower part
Ap=[argin.P.Q.A.inf(il,:);argin.P.Q.A.sup(iu,:)];   % upper part
gamma=[-argin.P.Q.F.inf(il);-argin.P.Q.F.sup(iu)];  % left sided constant
% now the constraints are put in the form: [-An Ap]q(x)+gamma <=0
if argin.deb, 
    gldisp('An',full(An),'Ap',full(Ap),'gamma',gamma,'nl',nl,'nu',nu); 
end

% iterate through each constraint ( checking positive definitness still
% missing)
for i=1:nl+nu
    if argin.deb, 
        if i<=nl, dir=1;
        else dir=-1;
        end
        disp('Upper part:');
        disp(glqvec2str([gamma(i) Ap(i,:)],dir)),
        disp('Lower part:');
        disp(glqvec2str([gamma(i) -An(i,:)],dir)),
    end
    % get indices of nonlinear variables for this constraint
    if i<=nl, ind=find(nlind(iil(i),:));
    else ind=find(nlind(iiu(i-nl),:));
    end
    % remove linear or non used variables, and transform the constraint to
    % x^t[Gn Gp]x+[cn -cp]^tx+gamma<=0
    Pn=glqvec2qmat([-gamma(i) An(i,:)]);Pp=glqvec2qmat([gamma(i) Ap(i,:)]);
    Gn=Pn(ind+1,ind+1);Gp=Pp(ind+1,ind+1);
    if argin.deb,
        delim='\!&\!'; % '&' or '\!\!&\!\!';
        opt.delim=delim;
        opt.prec=3;
        opt.prt=2;
        glmat2tex(Gp, 'Gp', opt);
    end
    cn=Pn(ind+1,1)+Pn(1,ind+1)';cp=Pp(ind+1,1)+Pp(1,ind+1)';
    % crude check of positive definiteness
    if all(diag(Gp)<=0)
        % possibly negative definite,
        if i<=nl,
            if argin.deb, 
                disp('?negative definite?, direction ok (-sign) !');
            end
            %lower part (chg sign)
            Gp=-Gp;Gn=-Gn;cn=-cn;cp=-cp;gamma(i)=-gamma(i);
        else
            %upper part (dont enc)
            if argin.deb, disp('?negative definite?, direction wrong!');end
            continue;
        end
    elseif all(diag(Gp)>=0)
        % possibly positive definite,                
        if i<=nl,
            % lower part (dont enc)
            if argin.deb, disp('?positive definite?, direction wrong!');end
            continue;                              
        else
            % upper part (enc)
            if argin.deb, disp('?positive definite?, direction ok!');end
        end
    else
        % not positive or negative definite, do not enclose
        if argin.deb, disp('indefinite!');end
        continue;
    end
    if argin.deb, 
        gldisp('Gn',full(Gn),'Gp',full(Gp),'cn',cn,'cp',cp,'gamm',-gamma(i)); 
    end
    [x, delta, w, R, feas]=glellipse(Gn, Gp, cn, cp, -gamma(i), argin.deb);
    if feas==0
        % constraint is infeasible
        argout.x=[];
        break;
    elseif feas==-1
        %constraint is not strictly convex
        if argin.deb,
            argout.out=[argout.out ; ...
                {['Constraint ' num2str(i) ' is not provably ' ...
                'strictly convex:']}];
            gldisp('feas',feas,'eigG',eigs(Gp));
        end
        continue;
    end
    
    % new xbox
    dw=delta*w;
    xnew.inf=-inf+zeros(nx,1);xnew.sup=inf+zeros(nx,1);
    xnew.inf(ind)=-dw+x.inf;xnew.sup(ind)=dw+x.sup;
    if argin.deb, gldisp('dw',dw,'newx',[xnew.inf xnew.sup]);end
    % scale back the new xbox
    if argin.scal && isfield(argin.P,'scal')
        bscal=argin.P.scal.x;
        xnew.inf=-(-(xnew.inf).*bscal);xnew.sup=xnew.sup.*bscal;
        if argin.deb, gldisp('newx_backscaled',[xnew.inf xnew.sup]);end
    end
    
    if argin.gra && nx==2,
        % original ellipse
        Eorig=glqmat2qvec(Pp(1:nx+1,1:nx+1));
        % approximating ellipse
        Q=zeros(nx,nx);Q(ind,ind)=R'*R;
        xn=zeros(nx,1);xn(ind,1)=x.sup;
        l=-2*(xn'*Q)';c=xn'*Q*xn-delta^2;
        Eappr=glqmat2qvec([[c zeros(1,length(l))];[l Q]]);
        if argin.scal && isfield(argin.P,'scal')
            % Reverse the scaling
            qx=ones(7,1);
            for k=1:6,
                qx(k+1)=prod((1./argin.P.scal.x').^argin.P.scal.E(k,:));
            end
            Eorig=Eorig.*qx;
            Eappr=Eappr.*qx;
        end
        % plot the ellipsoids
        glqvecplot(Eorig, 'LineSpec', 'g--');
        glqvecplot(Eappr, 'LineSpec', 'r-.');
    end
    if argin.prt,
        argout.out=[argout.out ; {['Constraint ' num2str(i) ' bounded!']}];
    end
    if argin.gra && argin.prt,
        % plot xnew
        glboxplot(xnew, 'Color', 'g');
    end
    % concatenate the output x with xbnew for all used variables
    for j=1:length(ind)
        if argout.x.inf(ind(j))<xnew.inf(ind(j))
            argout.x.inf(ind(j))=xnew.inf(ind(j));
        end
        if argout.x.sup(ind(j))>xnew.sup(ind(j))
            argout.x.sup(ind(j))=xnew.sup(ind(j));
        end
    end
    if any(argout.x.inf>argout.x.sup)
        argout.x=[];
        break;
    end
end

% graphics output
if isempty(argout.x) 
    if argin.prt,
        argout.out=[argout.out ; {'The problem was found infeasible!'}];
    end
else
    if argin.gra, glboxplot(argout.x); end
    if argin.prt,
        owid=argin.x.sup-argin.x.inf;owid(isnan(owid))=inf;
        nwid=argout.x.sup-argout.x.inf;nwid(isnan(nwid))=inf;
        red=find(owid>nwid)';
        if ~isempty(red)
            argout.out = [argout.out ; {['Box components ' num2str(red) ...
                ' reduced, new box:']} ; glbox2str(argout.x)];
        else
            argout.out = [argout.out ; {['No better bounds gained, '...
                'the box is still:']} ; glbox2str(argout.x)];
        end
    end 
end

% certificate
if argin.pro, 
    argout.cert.x=argout.x;
end