%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glellipse.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,delta,w,R,feas]=glellipse(An, Ap, an, ap, alpha, prt);
%
% generates a rigourous upper bounds ||R(x-xp)||_2<=delta
% and |x-xp|<=delta*w on a strict convex equation 
% x'[-An Ap]x+[-an ap]'x<=alpha of nx variables
% 
% An,Ap     nx x nx sym matrices of the positive and negative quadratic co
% an,ap     nx x 1 vector of the linear coeffs
% alpha     constant therm
% prt       output flag
%
% x         nx x 1 interval vector
% delta     constant
% w         nx x 1 vector
% R         cholesky factor of [-An, Ap] with x'Ax >= ||Rx||^2
% feas      indicates if the constraint is 1:feasible 0:infeasible -1:not
%           strict convex

function [x,delta,w,R,feas]=glellipse(An, Ap, an, ap, alpha, prt)

% argcheck
if nargin <5, error('qlellipse: Not enough arguments!'); end
if nargin<6,prt=0;end

% init
x=[];delta=[];w=[];feas=-1;

% cholesky factor of [An Ap]
[perm,R,ier]=gldchol(An,Ap,2);
if ier, 
    %G not positive definite
    return;
end 
R(perm,perm)=R;
if prt, 
    res=Ap-R'*R;
    gldisp('perm',perm,'R',R,'res',res,'detres',det(res));
end
%find w
[w,beta]=gltinv(R);
if prt, gldisp('w',w, 'beta',beta); end
zs=R'\((ap-an)/4); % midpoint of [-an,ap]/2
xs=-R\zs;
if prt, gldisp('zs',zs,'xs',xs); end
%find delta
x=infsup(xs,xs);
z=infsup(zs,zs);
Rx=R*x;
a=infsup(-an,ap);
gamma=sup(norm(z+Rx,2)+w'*abs(a/2-R'*z));
g=infsup(gamma,gamma);
Delta=sup(g^2+alpha-a'*x-Rx'*Rx);
if prt, gldisp('gamma',gamma,'Delta',Delta); end
if Delta<0
    %constraint infeasible
    feas=0;R=[];
    return
end
feas=1;
delta=gamma+sqrt(Delta);
if prt, gldisp('delta',delta); end
