%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gltinv.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [u,beta,C]=gltinv(A, C);
% computes rigorous bound u>=|x| for ||Ax||_2<=1,
% where A is lower or upper triangular
%
% beta is close to 1 if the bound is accurate, 
%         0 if the bound is infinite (ill-conditioned or singular A)
% C is the preconditioner of A which can be given or computed.

function [u,beta,C]=gltinv(A,C)

%argcheck
if nargin<1,error('gltinv: Not enough arguments!');end
if nargin<2, C=[]; end
    
n=size(A,1);
%if min(abs(diag(A)))==0, 
%  disp('zero diagonal entry in A')
%  u=inf*ones(n,1);beta=0;
%  return;
%end;

if norm(A-eye(n),1)==0,
  % A=eye(n); |x_i|<=1
  u=ones(n,1);beta=1;
  return;
end;

if norm(A-diag(diag(A)),1)==0,
  % A is diagonal; |d_ix_i|<=1
  d=abs(diag(A));
  setround(1); % round up
  u=1./d;beta=1;
  return;
end;

%nl=norm(tril(A)-diag(diag(A)),1);
%nu=norm(triu(A)-diag(diag(A)),1);
%if nl>0 & nu>0,
%  lower_triangle_of_A=tril(A)-diag(diag(A))
%  upper_triangle_of_A=triu(A)-diag(diag(A))
%  error('A is not triangular')
%end;

% now A is nondiagonal triangular

if isempty(C)
    C=A\eye(n);  % inv(A) may give a nontriangular inverse
end
setround(1); % round up
H.inf=-(C*(-A));H.sup=C*A; 
% previous line could probably be speeded up a little 
% by using the triangular structure
dH=diag(H.inf);
if sum(dH<0)>0,
  gldisp('dA_dC_dH',[diag(A),diag(C),dH]);
  error('poor triangular inverse 1')
end;

% get negative comparison matrix
H1=max(abs(H.inf),abs(H.sup));
H1=H1-diag(diag(H1));H1=H1-diag(dH);

d=sqrt(diag(C*C'));
% previous line could probably be speeded up a little 
% by using the triangular structure
if max(d)==inf,
  % happened occasionally when n>500
  % disp('A severely ill-conditioned')
  u=inf*ones(n,1);beta=0;
  return;
end;
if sum(d<=0)>0,
  gldisp('dA_dC_dH',[diag(A),diag(C),dH]);
  error('poor triangular inverse 3')
end;
beta=-max((H1*d)./d);
% now <H>d >= beta d, with beta close to 1 
% in random tests we always had 1-beta<1e-6
if isnan(beta),
  gldisp('dA_dC_dH',[diag(A),diag(C),dH],'dd',[H1*d,d]);
  error('poor H-matrix certificate 1');
end;
if beta<=0,
  % disp('A severely ill-conditioned')
  u=inf*ones(n,1);beta=0;
  return;
end;

u=d./beta;
