%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfeasibility.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glfeasibility(argin)
%
% Uses a local solver to find an approximately feasible point of 
% the problem P. Verifies the point if needed. If sucessful a box \z
% surely containing a feasible point is returned. If no neary feasible
% point has been found an empty \z is returned.
% found and verified a feasible point (rigorous) - best.x is the found point,
%   zb surely contains a feasible point, xbox can be better than the old
% problem was found infeasible (rigorous) - best.x is the best found point,
%   zb and xbox is empty
% no point with feasibility distance < mindist found (rigorous) - best.x, zb 
%   is empty, xbox can be better than the old one
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
%   .xbox       original bound constraints after the simplification, 
%               needed for the verification
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% locsolver     name of the used local solver: 'BFGS' or 'grad proj'
% maxitm,maxitd maximum number of iterations allowed = maxitm*nx+maxitd
% verify        =1 if the verification is turned on
% multitime     this percentage of the time ellapsed is used for multistart
%               search
% delta         small verification constant < 0.5
% lsolver       linear solver name which is used for solving LPs, needed
%               only for the verification.
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .best       most feasible point found
%     .x        nx x 1 point
%     .fx       nc x 1 function value Aq(x)
%     .fd       feasibility distance of the found point
%     .status   status of the search: feasible, local minima, maxiter 
%   .C          inverse of B
%   .K          index set of the selected variables
%   .w          solution of the LP
%   .ver        fasible point verified=1 or 0 if not
%
% * argout structure fields *
% x             new bound constraints
%  .best        most feasible point found
%    .x         nx x 1 point
%    .fx        nc x 1 function value Aq(x)
%    .fd        feasibility distance of the found point
%    .status    status of the search: feasible, local minima, maxiter 
%    .zb        a box containig a feasible point, or empty if the verifica
%               tion failed
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glfeasibility(argin)

% global variables
global glconf;      % global configuration variable

% constants
FEAS=1e-6;          % feasibility tolerance
SCALING=1;          % TEMPORARY SCALING VECTOR

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'locsolver'};{'maxitm'};{'maxitd'};{'verify'};...
    {'delta'};{'lsolver'}];
% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set configuration
if isfield(glconf,'feast'),FEAS=glconf.feast;end

% init argout
if argin.prt,
    if argin.verify
        argout.out=[{'*** FIND AND VERIFY A FEASIBLE POINT ***'};...
            {['Local solver: ' argin.locsolver]};...
            {['Maximum iterations: ' num2str(argin.maxitm) '*nx+' ...
            num2str(argin.maxitd)]};...
            {['Minimum feasibility distance: ' num2str(FEAS)]};...
            {['Delta: ' num2str(argin.delta)]};...
            {['Linear solver: ' argin.lsolver]}];
    else
        argout.out=[{'*** FIND A FEASIBLE POINT ***'};...
            {['Local solver: ' argin.locsolver]};...
            {['Maximum iterations: ' num2str(argin.maxitm) '*nx+' ...
            num2str(argin.maxitd)]};...
            {['Minimum feasibility distance: ' num2str(FEAS)]}];
    end
end
if argin.pro
    if ~isfield(argin, 'cert') || isempty(argin.cert)
        argout.cert.name='Find a feasible point';
        argout.cert.tname='Feasibity';
    else argout.cert=argin.cert;
    end
else argout.cert=[];
end
% init variables
nx=length(argin.x.inf);               % number of vars
nc=size(argin.P.Q.A.inf,1);           % number of constraints
% try to find a feasible point
if ~argin.pro || ~isfield(argin, 'cert') || isempty(argin.cert)
    [best, argout.x]=glfindfeas(argin.P.Q.A, ...
            argin.P.Q.Fold, argin.x, argin.locsolver, argin.maxitm, ...
            argin.maxitd, argin.deb);
    if isempty(argout.x)  
        argout.x=argin.x;
        argout.x.infeasible=1;
    end
    argout.x.best=best;
    if argin.pro, argout.cert.x=argout.x; end
else argout.x=argin.cert.x;
end
% check the result
if isfield(argout.x,'infeasible')
    if argin.prt, argout.out=[argout.out ; {['Constraint propagation '...
            'found the problem infeasible!']}];end
elseif argout.x.best.fd>FEAS
    if argin.prt,
        argout.out=[argout.out;{'No approximative feasible point found!'};...
            {['Feasiblity distance of the found point = ' ...
            num2str(argout.x.best.fd)]}];
    end
    if argin.gra, 
        glboxplot(argout.x,'Color','r');
        zz.inf=argout.x.best.x;zz.sup=zz.inf;glboxplot(zz,'Color','k');
    end
else
    if argin.prt,
        argout.out=[argout.out ; {['Approximative feasible point: '...
            mat2str(argout.x.best.x,4) ' found!']};...
            {['Feasiblity distance = ' num2str(argout.x.best.fd)]}];
    end
    if argin.gra, 
        zz.inf=argout.x.best.x;zz.sup=argout.x.best.x;
        glboxplot(zz,'Color','g');
    end
    argout.P=glsolstatus(argin.P,'FEAS',1);
    if argin.verify
        % verify the found  approximative feasible point using
        % the original box P.startxbox
        [argout.x.best.zb, err, argout.cert]=glverifyfeas(...
            argin.P.Q.A, argin.P.Q.F,...
            argin.P.startxbox, argout.x.best.x, SCALING+zeros(nx,1), ...
            SCALING+zeros(nc,1), argin.delta, argin.lsolver, argin.deb,...
            argin.gra, argout.cert);
        if isempty(argout.x.best.zb)
            if argin.prt,
                argout.out=[argout.out ; {['Failed to verify the found '...
                    'feasible point: ' err '!']}];
            end
            return
        else
            if argin.prt, argout.out=[argout.out;{'The box:'};...
                    glbox2str(argout.x.best.zb) ; {'contains a feasible point'}];
            end
            argout.P=glsolstatus(argin.P,'FEAS',0);
            argout.P=glsolstatus(argin.P,'VFEAS',1);
            if argin.pro, argout.cert.x=argout.x; end
            % graphics
            if argin.gra, glboxplot(argout.x.best.zb,'Color','b'); end
        end
    end
end
