%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glfindfeas.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [best, x]=glfindfeas(A, F, x, locsolver, maxitm, maxitd, prt)
%
% Calls a local solver to find an approximately feasible point of
% the problem P.
%
% A             nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
% F             nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             bound constraints
%  .best        most feasible point found (or missing field if none found)
%    .x         nx x 1 point
%    .fx        nc x 1 function value Aq(x)
%    .fd        feasibility distance of the found point
%    .status    status of the search: (f)easible, (l)ocal minima,
%               (m)axiter, (e)rror, (p)ojected
% locsolver     name of the used local solver: 'BFGS' or 'grad proj'
% runc          number of runs=nx*runc, with random start point or
%               midpoint if runs=0;
% maxitm,maxitd maximum number of iterations allowed = maxitm*nx+maxitd
% prt           text output flag       0 = no output 1 = output
%
% best          most feasible point found
%  .x           nx x 1 point
%  .fx          nc x 1 function value Aq(x)
%  .fd          feasibility distance of the found point
%  .status      status of the found point: feasible, local minima, maxiter
% x             new rigorous bound constraints, empty if there is no
%               feasible point in the starting box


function [best, x]=glfindfeas(A, F, x, locsolver, maxitm, ...
    maxitd, prt)

global glconf;      % gloptlab configuration file
global gllocvars;   % global struct for passing P & x to glfeasdistfun
global glgradhelp;  % gradient helper (needed only for lbfgs-b)
global lbfgsiter;   % iteration counter (needed only for lbfgs-b)

MAXITM=5;           % maximum iterations mutiplier
MAXITD=20;          % maximum iterations shift
FEAS=1e-6;          % feasibility tolerance

hasfmincon=exist('fmincon','file')==2;

% argcheck
if nargin <3, error('glfindfeas: Not enough arguments!');end
if nargin <4,
    if hasfmincon,locsolver='fmincon'; 
    else locsolver='bfgs'; 
    end
end
if nargin <5, maxitm=MAXITM; end
if nargin <6, maxitd=MAXITD; end
if nargin <7, prt=0; end

% set configuration
if isfield(glconf,'feast'),FEAS=glconf.feast;end

% init
% the problem is in the inequ form: Aq(x) \in \F x \in \x
nx=length(x.inf); % number of vars of the ineq from
[nc, nA]=size(A.inf);       % number of constraints
% global information for the glfeasdistfun
gllocvars.A=(A.inf+A.sup)/2;
gllocvars.Al=gllocvars.A(:,1:nx);
gllocvars.Aq=cell(nc,1);indq=nx+1:nA;
for i=1:nc, gllocvars.Aq{i}=reshape(gllocvars.A(i,indq),nx,nx);end
gllocvars.F=F;

while 1,
    gllocvars.xb=x;
    iter=maxitd+nx*maxitm;
    solerr=0;
    % starting point
    if isfield(x,'best')
        % a feasible point has already been found, or the previously found
        % best point is contained in the current box
        if any(x.best.x < x.inf) || any(x.sup < x.best.x)
            % the found point is outside the given box, project and continue
            x.best.x=min(x.sup, max(x.inf,x.best.x));
            [x.best.fx, x.best.fd, x.best.fdv]=gleval(gllocvars.A, x.best.x, F, x);
            x.best.status='p';
            if prt, disp('Previous point projected!');end
        end
        if x.best.status(1)=='l' || x.best.status(1)=='e'
            % error or local minima was reached, use a random starting point.
            y=glrand(x);
            if prt, disp('Using a random starting point!');end
        else
            % maximum iteration reached, or fesible point found, use it as
            % the starting point.
            y=x.best.x;
            if prt, disp('Using previous point as the starting point!');end
        end
    else
        % no best found in the box, use the midpoint as the starting point.
        y=glmid(x);
        if prt, disp('Using the box midpoint as the starting point!');end
    end
    % feasible point search
    % (1) use the selected local solver, to find a feasible point
    if strcmp(locsolver, 'bfgs')
        [best.x, itc]=glprojbfgs(y,'glfeasdistfun',x.sup,x.inf,1e-10,iter);
        if itc<iter, best.status='l';
        else best.status='m';
        end
    elseif strcmp(locsolver, 'grad proj')
        [best.x, itc]=glgradproj(y,'glfeasdistfun',x.sup,x.inf,1e-10,iter);
        if itc<iter, best.status='l';
        else best.status='m';
        end
    elseif strcmp(locsolver, 'fmincon')
        if hasfmincon
            try
                options = optimset('GradObj','on','Display','off', ...
                    'MaxIter', iter, 'TolFun', 1e-10,'TolCon', 1e-10);
                [best.x, fx, exitflag] = fmincon(@glfeasdistfun,y,[],[],[],[],...
                    x.inf,x.sup,[],options);
                %exitflag
                % exitflag: Integer identifying the reason the algorithm
                % terminated.
                % 1: First-order optimality measure was less than options.TolFun,
                %    and maximum constraint violation was less than options.TolCon.
                % 2: Change in x was less than options.TolX.
                % 3: Change in the objective function value was less than
                %    options.TolFun.
                % 4: Magnitude of the search direction was less than 2*options.TolX
                %    and constraint violation was less than options.TolCon.
                % 5: Magnitude of directional derivative in search direction was
                %    less than 2*options.TolFun and maximum constraint violation
                %    was less than options.TolCon.
                % 0: Number of iterations exceeded options.MaxIter or number of
                %    functionevaluations exceeded options.FunEvals.
                % -1:The output function terminated the algorithm.
                % -2: No feasible point was found.
                if exitflag==0, best.status='m';
                else best.status='l';
                end
            catch
                err=lasterror;
                msg=['*FATAL ' locsolver ' ERROR: '];
                d=err.stack;
                for i=1:length(dbstack)
                    msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
                end
                msg=[msg ' -> ' err.message];
                glerrorlog(msg);
                solerr=1;
            end
        else
            error(['glfindfeas: fmincon selected a the local solver but '...
                'the Optimization Toolbox seems to be missing!']);
        end
    elseif strcmp(locsolver,'lbfgs-b')

        glgradhelp=1; %#ok<NASGU>
        try
            best.x = lbfgsb(y,x.inf,x.sup,'glfeasdistfun','glfeasdistgrad',...
                [],'lbfgsbitercount','maxiter',iter,'m',4,'factr',1e-12,...
                'pgtol',1e-5);
            if lbfgsiter<iter, best.status='l';
            else best.status='m';
            end
        catch
            err=lasterror;
            msg=['*FATAL ' locsolver ' ERROR: '];
            d=err.stack;
            for i=1:length(dbstack)
                msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
            end
            msg=[msg ' -> ' err.message];
            glerrorlog(msg);
            solerr=1;
        end
        glgradhelp=[];
    else error('glfindfeas: Unknown solver type!');
    end

    % (2) check the found point handle the solver errors or the nan in x
    if solerr || any(isnan(best.x))
        best.x=y;
        best.status='e';
        disp(['The solver ' locsolver ' produced a nan or a solver error!']);
    end
    if any(best.x < x.inf) || any(x.sup < best.x)
        % the found point is outside the given box
        xx=[x.inf x.sup],xf=best.x,
        error(['The solver ' locsolver ...
            ' found a solution outside the given bounds!']);
    end

    % (3) compute the function value and feasibility distance of the
    % found point
    [best.fx, best.fd, best.fdv]=gleval(gllocvars.A, best.x, F, x);

    % (4) check the feasibility distance
    if isfield(x,'best') && (best.fd-x.best.fd) > 0
        % the found point is not as good as the input, use the old one
        if prt, 
            disp(['Reveted to the old best point: fd-fd_old=' ...
                mat2str(best.fd-x.best.fd) '.']);
        end
        best=x.best;
    elseif best.fd<=FEAS,best.status='f';
    end
    if prt,
        gldisp('z',best.x,'fz',best.fx,'fd',best.fd,'status',best.status);
    end

    % (5) improve the xbox or verify the infeasibility by using CP on
    % fdv'*A \in fdv'*F
    if any(best.fdv~=0)
        a=best.fdv(1:nc)'*infsup(A.inf,A.sup);b=best.fdv(1:nc)'*infsup(F.inf,F.sup);
        ain.P.Q.A.inf=a.inf;ain.P.Q.A.sup=a.sup;
        ain.P.Q.F.inf=b.inf;ain.P.Q.F.sup=b.sup;
        ain.P.Q.int=zeros(nx,1);ain.P.Q.Bu=[];
        ain.P.Q=glfindspars(ain.P.Q);ain.P.Q.gaps=[];
        ain.P.basis=[];ain.P.uni=[];ain.P.opt=0;
        ain.x=x;ain.bilappr='a';ain.full=1;
        ain.prt=0;ain.deb=0;
        aout=glpropagate_separable(ain);
        x=aout.x;
        if prt,
            if ~isempty(x), gldisp('x',[x.inf, x.sup]);
            else disp('Empty box found');end
        end
    end
    if ~isempty(x) && (any(best.x < x.inf) || any(x.sup < best.x))
        % the found point is outside the given box, continue
        x.best=best;
        if prt, 
            disp(['Feasiblity improved the bounds, and the found point'...
                ' is outside the new box, repeating findfeas.']); 
        end
        continue;
    end
    break;
end