%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glverifyfeas.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [z, err, cert]=glverifyfeas(A, F, x, y, xscal, cscal, delta,
%                                      lsolver, prt, gra, cert)
%
% Find a box \z \in \x around the approximately feasible point y so that it
% surely contains a feasible point. If the returned /z is empty the
% feasibilty cannot be verified.
%
% A             nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
% F             nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 original bound constraint interval, after the 
%               simplification 
% y             an approximate feasible point
% xscal         nx x 1 variable scaling vactor
% cscal         nc x 1 constraint scaling vactor
% delta         mutiplyier determines the width of /z
% lsolver       linear solver name which is used for solving LPs
% prt           text output flag       0 = no output 1 = output
% gra           graphics output flag   0 = no output  1 = output
% cert          certificate structure
%   .C          inverse of B
%   .K          index set of the selected variables
%   .w          solution of the LP
%   .ver        fasible point verified=1 or 0 if not
%
% z             nx x 1 box containing a feasible point or empty
% err           error message or empty if everything went good

function [z, err, cert]=glverifyfeas(A, F, x, y, xscal, cscal, delta,...
                                     lsolver, prt, gra, cert)

global glconf;      % gloptlab configuration file

FEAS=1e-6;          % feasibility tolerance
DELTA=1e-4;         % delta constant
SCALING=1;          % TEMPORARY SCALING VECTOR

% argcheck
if nargin <4, error('glverifyfeas: Not enough arguments!');end
if nargin <5, xscal=SCALING+zeros(length(x.inf),1);end
if nargin <6, delta=DELTA; end
if nargin <7, lsolver='LpSolve';end
if nargin <8, prt=0; end
if nargin <9, gra=0; end
if nargin <10,cert=[]; end

% init
setround(1);
z=[];err=[];cert.ver=0;

% set constants from config
if isfield(glconf,'feast'), FEAS=glconf.feast;end 

% check
if delta>=0.5, error('glverifyfeas: delta>=0.5 !');end

% tranform constraints to equs
[x, slacks, E]=glineq2eq(x, F, A, 0);
nx=length(x.inf);ns=length(slacks);no=nx-ns;[nc, nE]=size(E.inf);
% also transfrom the approximate point
fy=gleval((A.sup+A.inf)/2, y, F);y(no+1:nx,1)=fy(slacks);
if any(y(no)<x.inf(no) | y(no)>x.sup(no)), 
    error('glverifyfeas: Incorrect y!');
end
if nc>nx, err='number of constraint > number of variables'; return ;end
    
% compute \z = \x conc [y-u, y+u], u=xscal*delta
u=[xscal ; cscal(slacks)]*delta;
%z.inf=-(u-y);z.sup=(y+u);
z.inf=max(x.inf,-(u-y));z.sup=min(x.sup,(y+u));
if prt, gldisp('z',[z.inf z.sup],'zwid',(z.sup-z.inf)'); end
if any(z.sup<z.inf)
    % inconsistent z, too many active constraints?
    err=['The box \z is inconsistent, z not in \x -> too many '...
        'constaints are active at the point z'];
    z=[];return;
end
if gra, 
    zo.inf=z.inf(1:no);zo.sup=z.sup(1:no);
    glboxplot(zo, 'Color', 'b');
end

% compute interval Jacobian
B=glderiv((E.inf(:,2:end)+E.sup(:,2:end))/2, infsup(z.inf,z.sup));
if prt, gldisp('B',full(B)); end

% find C and the index set rind of the selected variables using
% the Gauss-Jordan inversion 
if ~isfield(cert,'C')
    a.inf=zeros(nc,1);a.sup=a.inf;
    [C, K, rind, exc]=glgaussjord(B, a, x, 0);
    % K=sort(K);
    cert.C=C;cert.K=K;
else C=cert.C;K=cert.K;exc=0;
end
nK=length(K);
if prt, gldisp('C',C,'K',K,'rind',rind); end

% check the results of the inversion
if exc>0, err='Gauss-Jordan inversion exception';z=[];return;end
if nK~=nc, err='Gauss-Jordan inversion failed, |K|<nc';z=[];return;end
CB_K=C*B(:,K);CB_K_I=CB_K-eye(nc);
if prt, gldisp('CB_K',CB_K,'CB_K_I',CB_K_I); end
if norm(abs(CB_K_I),inf)>=1,err='|C*B(:,K)-I|_inf>=1';z=[];return;end

% compute J and split z into z_K and z_J
J=1:nx;J(K)=[];nJ=length(J);
zk.inf=z.inf(K);zk.sup=z.sup(K);
zj.inf=z.inf(J);zj.sup=z.sup(J);
if prt, gldisp('J',J,'zzk',[zk.inf zk.sup],'zzj',[zj.inf zj.sup]); end

% compute E2=D-CE with D and nc x nE matrix with D(K,K+1)=1
CE=C*infsup(E.inf, E.sup);
E2=sparse(1:nc,K+1,ones(1,nc),nc,nE,nc)-CE;
if prt,
    zb=infsup(z.inf,z.sup);
    gldisp('CEqx',CE*[1;zb;reshape(zb*zb',nx^2,1)])
end

% linearize E2q(x) in \z about y obtaining Ex+\a
[A, a]=gllinearize(E2(:,2:end), y, z, 0, 0, 0);
a.inf=-(-a.inf-E2.inf(:,1));a.sup=a.sup+E2.sup(:,1);
if prt, gldisp('A',A,'a',[a.inf a.sup],'awid',a.sup-a.inf); end

% elliminate x_K from Ex+\a obtaining Ax_J+\b
zb=infsup(z.inf, z.sup);
a2=infsup(a.inf,a.sup)+A(:,K)*zb(K); A=A(:,J);
b.inf=a2.inf;b.sup=a2.sup;
if prt, gldisp('A',A,'b',[b.inf b.sup]);end

% compute c with inward rounding, and tighten it by delta*100 percent
c.inf=zk.inf-b.inf;c.sup=-(b.sup-zk.sup);   % c:=zk (-) b rounded inward
if any(c.inf>c.sup), 
    err='Inward rounding of c resulted in a inconsistent box';
    z=[];return;
end

% check empty constraints
ind=find(sum(A~=0,2)==0)';
if any(c.inf(ind)>0) || any(c.sup(ind)<0)
    if prt, gldisp('ind',ind,'zero_in_c',c.inf(ind)<=0 & 0<=c.sup(ind));end
    % zero is not contained in the rhs -> problem is infeasible in z
    err='Problem was found infeasible in the box \z';z=[];return;
end
ind=find(sum(A~=0,2)~=0)';
A=A(ind,:);c.inf=c.inf(ind);c.sup=c.sup(ind);

if size(A,1)~=0
    % tighten c by delta*100 percent
    % c is guaranteed to contain 0
    d=c.sup-c.inf;
    dd=d*delta;
    c.inf=c.inf+dd;c.sup=-(dd-c.sup);
    
    if prt, gldisp('c',[c.inf c.sup],'c_width',c.sup-c.inf); end

    if ~isfield(cert,'w')
        % solve the linear program Aw \in c, w \in z_K
        cin.lin.A=[A;eye(nJ,nJ)];
        cin.lin.b=[[c.inf;zj.inf] [c.sup;zj.sup]];
        cin.obj.b=sparse([],[],[],nJ,1,1);
        %if prt, AA=cin.lin.A, bb=cin.lin.b, obj=cin.obj.b, end
        if prt, str=glcprogr2str(cin);for i=1:length(str);disp(str{i});end;end
        [w, xd, info]=gloptsolve(lsolver, cin);
        if prt, gldisp('w',w); end
        cert.w=w;
        cert.info=info;
    else w=cert.w;info=cert.info;
    end

    % interpret results
    if ~isempty(w) && (info.numerr==0 || info.numerr==1) &&...
            info.pinf<FEAS && info.dinf<FEAS
        Awb=A*w+infsup(b.inf,b.sup);
        % check if A*w+\b \in z_K still holds or not
        if any(zk.inf>Awb.inf & Awb.sup>zk.sup)
            err='A*w+\b \notin z_K';z=[];
            return;
        end
    else err='solving the LP failed';z=[];return;
    end
end

% remove slack entries from z
z.inf=z.inf(1:no);z.sup=z.sup(1:no);
            
% /z contains a feasible point
cert.ver=1;