%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinear_bound.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=gllinear_bound(argin)
%
% Solve a linear programm to reduce the infinite bound constraints.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% lsolver       linear solver name which is used for solving LP
% eqsolver      used equation solution method name:
%               available all .m files with prefix 'gllinprop_':
%               e.g. for using gllinprop_fast.m set eqsolver='fast'
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .cin        linear program solved to get the y
%   .y          solution of the linear program.
%
% * argout structure fields *
% F             general constraint bounds
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=gllinear_bound(argin)

global glconf;    % global configuration variable
INFTY=inf;        % infinity
FEAS=1e-6;        % feasibility tolerance
YCONST=1;         % set y > YCONST or y < -YCONST in the linear program

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'lsolver'};{'eqsolver'}];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

%argin.deb=1

% set constants from config
if isfield(glconf,'infty'), INFTY=glconf.infty;end
if isfield(glconf,'feast'), FEAS=glconf.feast;end

% init argout
argout.x=argin.x;argout.F=argin.P.Q.F;
if argin.prt,
    argout.out=[{'*** LINEAR BOUNDING ***'};{['Solver: ' argin.lsolver]};...
             {['Equation Solving: ' argin.eqsolver]}];
end
if argin.pro, 
    if isempty(argin.cert)
        argout.cert.name='Linear Bounding';
        argout.cert.tname='LinearBound';
        argout.cert.x=argin.x;
        argout.cert.F=argin.P.Q.F;
    else argout.cert=argin.cert;
    end
end

% init variables
nx=length(argin.x.inf);               % number of vars
if all(argin.x.inf(1:nx)>-INFTY) && all(argin.x.sup(1:nx)<INFTY)
    if argin.prt,
        argout.out=[argout.out ;{'All bounds are finite, task skipped!'}];
    end
    return 
end

% debug
if argin.deb,
    gldisp('Ainf',full(argin.P.Q.A.inf),'F',[argin.P.Q.F.inf argin.P.Q.F.sup],...
        'x',[argin.x.inf, argin.x.sup]);
end

% linearize the constraints
ain.P=argin.P;ain.x=argin.x;
ain.bilappr='a';ain.full=1;
ain.prt=argin.deb;ain.deb=argin.deb;
aout=glpropagate_linear(ain);
argout.x=aout.x;
if isempty(argout.x), 
    % linear cp found the problem infeasible
    return; 
end
argout.F=aout.F;
argout.E=aout.E;argout.b=aout.b;argout.cind=aout.cind;
if argin.deb, disp(aout.out); end

if all(argout.x.inf(1:nx)>-INFTY) && all(argout.x.sup(1:nx)<INFTY)
    if argin.prt,
        argout.out=[argout.out ;{['All bounds are finite, after the '...
            'linear constraint propagation, the task finished!']}];
    end
    if argin.gra, glboxplot(argout.x,'Color', 'r');end
    if argin.pro, argout.cert.x=argout.x; argout.cert.F=argout.F; end
    return 
end

% check the results of the linearization
if isempty(argout.x) 
    % constraints are infeasible
    if argin.pro, argout.cert.x=[]; end
    return;
else
    x.inf=argout.x.inf(1:nx);x.sup=argout.x.sup(1:nx);
    if argin.pro,argout.cert.x=argout.x; end
end

nx=length(x.inf);             % new number of vars
nc=size(argout.E,1);          % new number of constraints

% check if any constraints remain
if isempty(argout.E), 
    if argin.deb, gldisp('x',[x.inf, x.sup]); end
    if argin.prt,
        argout.out=[argout.out ;{['No valid linear constraints left '...
            'after the linear relaxation, bound constraints extracted, '...
            'the remaining part is skipped!']}];
    end
    %graphics
    if argin.gra, glboxplot(argout.x, 'Color', 'r');end
    if argin.pro, argout.cert.x=argout.x; argout.cert.F=argout.F; end
    return 
end
% check if any unbounded bounds
if all(x.inf>-INFTY) && all(x.sup<INFTY),
    if argin.prt,
        argout.out=[argout.out ; {['All bounds are finite after the '...
            'linearization, bound constraints extracted, the remaining '...
            'part is skipped!']}];
    end
    return 
end

if argin.deb, 
    gldisp('E',argout.E,'b',[argout.b.inf argout.b.sup],'x',[x.inf, x.sup]);
end

% INFO: the relaxed problem is now in the form Ex \in b , b is bounded at 
% least from one side
% create index sets
bli=argout.b.inf<=-INFTY;bui=argout.b.sup>=INFTY;
I_n=find(~bli &  bui)';                 % indices b.sup infinite -> y_i>0
I_p=find( bli & ~bui)';                 % indices b.inf infinite -> y_i<0
I  =[I_n I_p];   
nI=find(~bli & ~bui)';                  % indices b.inf, b.sup both finite
xli=x.inf<=-INFTY;
xui=x.sup>=INFTY;
N  =find( xli &  xui)';                 % \x_N unbounded->(E'y)_i=0
xsf=find( xli & ~xui)';                 % \x_xsf upwards bounded->(E'y)_i>0
xif=find(~xli &  xui)';                 % \x_xif downw bounded->(E'y)_i<0
nN=[xsf xif];                           % \x_nN bounded in at least one direction
xnf=find(xli | xui)';                   % \x_xnf has one infinite bound
xfin=find(~xli & ~xui)';                % \x_xfin is finite
if argin.deb,
    gldisp('I_n',I_n,'I_p',I_p,'nI',nI,'N',N,'nN',nN,'xsf',...
        xsf,'xif',xif,'xnf',xnf);
end

if ~isfield(argin.cert, 'y')
    % create bounds on y: Ry \in c:
    % set y_i>YCONST if b_i.inf>-inf b_i.sup=inf
    % set y_i<-YCONST if b_i.inf=-inf b_i.sup<inf
    nbif=length(I_n);nbsf=length(I_p);nBy=nbif+nbsf;
    R=sparse(1:nBy,[I_n I_p],ones(1,nBy),nBy,nc,nBy);
    c.inf=[YCONST+zeros(nbif,1);-inf+zeros(nbsf,1)];
    c.sup=[inf+zeros(nbif,1)   ;-YCONST+zeros(nbsf,1)];
    if argin.deb, gldisp('R',full(R),'c',[c.inf c.sup]); end
    % create bounds d for E'y \in d: 
    %   (E'y)_i=0 if \x_i = [-inf, inf]
    %   (E'y)_i>0 if x_i.inf=-inf && x_i.sup<inf
    %   (E'y)_i<0 if x_i.inf>-inf && x_I.sup=inf
    %   no restriction if x_i.inf=-inf && x_i.sup=inf
    d.inf=-inf+zeros(nx,1);d.sup=inf+zeros(nx,1);
    d.inf(N)=0;d.sup(N)=0;
    d.sup(xif)=-YCONST;d.inf(xsf)=YCONST;
    if argin.deb, gldisp('d',[d.inf d.sup],'E',full(argout.E(:,xnf)')); end
    % find zero cols in E(:,xnf)
    nxnf=xnf(~all(argout.E(:,xnf)==0,1)); % new index without zero cols
    if ~isempty(nxnf) || ~isempty(R)
        % make linear program [R; E(:,nxnf)'] y \in [c; d(nxnf)]
        cin.lin.A=[R;argout.E(:,nxnf)'];
        cin.lin.b=[[c.inf;d.inf(nxnf)] [c.sup;d.sup(nxnf)]];
        ny=size(cin.lin.A,2);
        cin.obj.b=sparse([],[],[],ny,1,0);
        cin.obj.b(I_n)=1;cin.obj.b(I_p)=-1;
        if argin.deb, disp(glcprogr2str(cin)), end
        % solve the LP
        [y, xd, info]=gloptsolve(argin.lsolver, cin);
        if argin.prt,
            argout.out=[argout.out ; ...
                {[argin.lsolver ' solved the linear program:']}; ...
                glcprogr2str(cin)];
        end
        if argin.pro, argout.cert.cin=cin; argout.cert.y=y; end
        % interpret results
        if ~isempty(y) && (info.numerr==0 || info.numerr==1) &&...
                info.pinf<FEAS && info.dinf<FEAS
            if argin.prt,
                argout.out=[argout.out;{['Solution for y found:'...
                    mat2str(full(y)')]}];
            end
            if argin.deb, gldisp('y',y); end;
        elseif argin.prt, argout.out=[argout.out; {'No y found!'}];
            return
        end
    elseif argin.prt,
        argout.out=[argout.out;{'Created an empty linear programm!'}];
        return
    end
        
else
    y=argin.cert.y;
end

% check if the restrictions y_i>0 if b_i.inf>-inf b_i.sup=inf and
% y_i<0 if b_i.inf=-inf b_i.sup<inf hold for the y found
if any(y(I_n)<=0) || any(y(I_p)>=0),
    if argin.prt, 
        argout.out=[argout.out; ...
            {'The solution found not useable, problem unbounded?'}];
    end
    return
elseif argin.prt,
    argout.out=[argout.out;{'The found solution is correct.'}];
end

% set A=E(:,xnf)), CA /approx I and compute y=y-C'*((C*A')\(A'*y)) 
% using interval arithmetics
C=pinv(argout.E(:,N));
if argin.deb, gldisp('C',C,'C*E',C*argout.E(:,N)); end
E=infsup(argout.E,argout.E);
%y=y-C'*((C*E(:,N))'\(E(:,N)'*y));
y=y-C'*(C*E(:,N))*(E(:,N)'*y);
if argin.deb, 
    ey=E(:,N)'*y;
    gldisp('y',[y.inf y.sup],'ydiam',diam(y),'E(:N)^T*y',[ey.inf ey.sup]); 
end

if ~isempty(nN)
    % solve y'Ax >= inf(y'*b) with CP to get bounds for the x_i bounded only
    % from one side
    Rs=infsup(zeros(1,nx),zeros(1,nx));Rs(1,xnf)=y'*E(:,xnf);
    if argin.deb, 
        gldisp('xnf', xnf, 'E(:,xnf)',E(:,xnf).inf,'y', y.inf,...
            'y^TE(:,xnf)',Rs.inf);
    end
    if any(Rs~=0)
        % there are some one sided x components
        % get the finite terms on the rhs
        xi=infsup(x.inf,x.sup);
        d=infsup(argout.b.inf, argout.b.sup)-E(:,xfin)*xi(xfin);
        nind=find(y.sup<0);pind=find(y.inf>0);
        bhat=zeros(nc,1);
        bhat(pind)=inf(d(pind));
        bhat(nind)=sup(d(nind));
%         if ~isempty(nind), F=F+y(nind)'*argout.b.sup(nind);end
%         if ~isempty(pind), F=F+y(pind)'*argout.b.inf(pind);end
        if argin.deb, 
            gldisp('b',[argout.b.inf argout.b.sup],'nind',...
                nind,'pind',pind,'b_hat',bhat);
        end
        ybhat=y'*bhat;
        % create problem definition for CP and use CP
        ain.P.Q.A.inf=[Rs.inf zeros(1,nx^2)];
        ain.P.Q.A.sup=[Rs.sup zeros(1,nx^2)];
        ain.P.Q.F.inf=ybhat.inf;ain.P.Q.F.sup=inf;
        ain.P.basis=[];ain.P.uni=[];ain.P.opt=0;
        ain.P.Q.int=zeros(nx,1);ain.P.Q.Bu=[];
        ain.P.Q=glfindspars(ain.P.Q);ain.P.Q.gaps=[];
        ain.x=x;ain.bilappr='a';ain.full=1;
        ain.prt=0;ain.deb=0;
        if argin.deb,
            gldisp('A',ain.P.Q.A.inf,'F',[ain.P.Q.F.inf ain.P.Q.F.sup],...
                'x',[x.inf x.sup]);
        end
        aout=glpropagate_separable(ain);
        xf=aout.x;Fnew=aout.F;
        if argin.deb,
            gldisp('Fnew',[Fnew.inf Fnew.sup],'xnew',[xf.inf xf.sup]);
        end
        if isempty(xf)
            % box is empty
            if argin.prt,
                argout.out = [argout.out ;{'Infeasibility verified!'}];
            end
            argout.x=[];
            if argin.pro,argout.cert.x=argout.x; end
            return
        end
        
        if argin.prt,
            argout.out=[argout.out;{'Constraint propagation for'};...
                glprob2str(ain.P);{'results'};glbox2str(xf)];
        end
        x=glconc(xf,x);
    elseif argin.deb,
        disp(['No one-sided bounded x components found, constraint '...
            'propagation skipped!']);
    end
end
if ~isempty(N)
    % find the un with un_i>=max((-C_ik\y_k).sup)) and
    % find the up with up_i<=min((-C_ik\y_k).inf))
    for i=1:size(C,1)
        %cc=-C(i,I)./y(I)';
        cc=-C(i,:)./y';
        un(i,1)=max(cc.sup);
        up(i,1)=min(cc.inf);
    end
    if argin.deb, 
        gldisp('up',up,'un',un,'b',[argout.b.inf argout.b.sup]);
    end
    
    % Cp=C+up*y', Cn=C+un*y'
    % CAx \in z
    % z=[inf(Cn*/argout.b-(Cn*E(:nN)*x(nN))),sup(Cp*/argout.b-(Cp*E(:nN)*x(nN))) ]:=z
    xi=infsup(x.inf, x.sup);
    bi=infsup(argout.b.inf,argout.b.sup);
    Cn=C+un*y';Cp=C+up*y';
    zl=glimult(Cn,bi)-glimult(glimult(Cn,argout.E(:,nN)),xi(nN));
    zu=glimult(Cp,bi)-glimult(glimult(Cp,argout.E(:,nN)),xi(nN));
    %zl=glimult(Cn,bi);zu=glimult(Cp,bi);
    z.inf=zl.inf;z.sup=zu.sup;
    % find x \in \xi with xi:=(CA)^-1*z
    %xi(N,:)=(C*argout.E(:,N))\infsup(z.inf,z.sup);
    xi(N,:)=pinv((C*argout.E(:,N)))*infsup(z.inf,z.sup);
    
    % CE_N=C*argout.E(:,N);B.inf=CE_N;B.sup=CE_N;
    % xn=feval('gllinprop_intlab', B, z, x);
    % xi(N,:)=infsup(xn.inf,xn.sup);
    
    % concatenate with the old x
    x=glconc(xi, x, 1:nx);
    if argin.deb, 
        gldisp('Cn_sup Cp_inf',[Cn.sup,Cp.inf],'z',[z.inf z.sup],'x',[x.inf x.sup]);
    end

    % old
    % zl=glimult(Cn,bi);zu=glimult(Cp,bi);
    % xi(N,:)=(C*argout.E(:,N))\infsup(z.inf,z.sup)-...
    % (C*argout.E(:,N))\((C*argout.E(:,nN))*xi(nN));
    
    if argin.prt,
            argout.out=[argout.out;...
                {'Caluculating two sided bounds from y result:'};...
                glbox2str(x)];
    end
end

argout.x.inf(1:nx)=x.inf;argout.x.sup(1:nx)=x.sup;

% plot x
if argin.gra, glboxplot(argout.x,'Color','r'); end

% update certificate
if argin.pro, argout.cert.x=argout.x; end
