%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinear_contract.m %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=gllinear_contract(argin)
%
% Linearize the constraint Aq(x) at z, and find new bounds on x.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% eqsolver      used equation solution method name:
%               available all .m files with prefix 'gllinprop_':
%               e.g. for using gllinprop_fast.m set eqsolver='fast'
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%
% * argout structure fields *
% F             general constraint bounds
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=gllinear_contract(argin)

thisfun=mfilename;
 
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'eqsolver'};];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.x=argin.x;argout.F=argin.P.Q.F;
if argin.prt,
    argout.out=[{'*** LINEAR CONTRACTION ***'};...
        {['Equation Solving: ' argin.eqsolver]}];
end
if argin.pro, 
    if isempty(argin.cert)
        argout.cert.name='Linear Contraction';
        argout.cert.tname='LinearContract';
        argout.cert.x=argin.x;
        argout.cert.F=argin.P.Q.F;
    else argout.cert=argin.cert;
    end
end

% init variables
nx=length(argin.x.inf);               % number of vars

% debug
if argin.deb, 
    gldisp('Ainf',full(argin.P.Q.A.inf),'F',...
        [argin.P.Q.F.inf argin.P.Q.F.sup],...
        'x',[argin.x.inf, argin.x.sup]);
end

% linearize the constraints
ain.P=argin.P;ain.x=argin.x;ain.bilappr='a';
ain.full=1;ain.prt=argin.deb;ain.deb=0;
aout=glpropagate_linear(ain);
argout.E=aout.E;argout.b=aout.b;argout.cind=aout.cind;
argout.x=aout.x;
if argin.deb, disp(aout.out); end

% check the results of the linearization
if isempty(argout.x) 
    % constraints are infeasible
    if argin.pro,argout.cert.x=[]; end
    return;
else
    argout.F=aout.F;
    x.inf=argout.x.inf(1:nx);x.sup=argout.x.sup(1:nx);
    if argin.pro,argout.cert.x=argout.x; end
end

nx=length(x.inf);             % new number of vars

% check if any constraints remain
if isempty(argout.E), 
    if argin.deb, gldisp('x',[argin.x.inf, argin.x.sup]); end
    if argin.prt,
    argout.out=[argout.out ;{['No valid linear constraints left '...
        'after the linear relaxation, bound constraints extracted, '...
        'the remaining part is skipped!']}];
    end
    %graphics
    if argin.gra, glboxplot(argout.x,'Color', 'r');end
    if argin.pro, argout.cert.x=argout.x; argout.cert.F=argout.F; end
    return 
end

if argin.deb, 
    gldisp('E',argout.E,'b',[argout.b.inf argout.b.sup],'x',[x.inf, x.sup])
end

% preconditioning and variable selection
if strcmp(argin.eqsolver,'intlab')
    % interval inversion needs no preconditioning or variable selection
    B.inf=argout.E;B.sup=argout.E;d=argout.b;cperm=[];
else
    % find a preconditioner and variable selection with gauss jordan 
    A2.inf=argout.E;A2.sup=argout.E;
    [C, cperm, rind, exc]=glgaussjord(A2, argout.b, x, 0);
    if isempty(cperm),
        if argin.prt,
            argout.out=[argout.out ;{'Gauss-Jordan inversion failed!'}];
        end
        % graphics
        if argin.gra, glboxplot(argout.x, 'Color', 'r');end
        if argin.pro, argout.cert.x=argout.x; argout.cert.F=argout.F; end
        return
    end
    nB=length(rind);
    if argin.deb, 
        gldisp('C',C,'cperm',cperm,'rind',rind,'exc',exc,'nB',nB)
    end;
    % compute \B:=C*\E(x) \in C*\b:=\d
    BB=C*infsup(argout.E,argout.E); dd=C*infsup(argout.b.inf,argout.b.sup);
    B.inf=BB.inf;B.sup=BB.sup;d.inf=dd.inf;d.sup=dd.sup;
    if argin.deb, gldisp('BB',BB,'dd',dd); end;
end

% solve \B \in \d with the selected equation solving method.
% if the fast method is used, the ith constraint will only solved for the
% variable x_cperm(i) all other terms are evaluated.
[x, F]=feval(['gllinsol_' argin.eqsolver], B, d, x, cperm, 0);
if isempty(x), argout.x=[];
else argout.x.inf(1:nx)=x.inf;argout.x.sup(1:nx)=x.sup;
end

% debug
if argin.deb, gldisp('x',[argin.x.inf, argin.x.sup]), end;

% graphics
if argin.gra, glboxplot(argout.x, 'Color', 'r');end

% certificate
if argin.pro, cert.x=argout.x; cert.F=argout.F; end
