%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinear_solve.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=gllinear_solve(argin)
%
% Solve linear programs to reduce the the bounding box, using the 3 most
% promising directions.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% lsolver       linear solver name which is used for solving LP
% eqsolver      used equation solution method name:
%               available all .m files with prefix 'gllinprop_':
%               e.g. for using gllinprop_fast.m set eqsolver='fast'
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .cins       linear program solved to get the y
%   .ys         solution of the linear programs.
%
% * argout structure fields *
% F             general constraint bounds
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=gllinear_solve(argin)

global glconf;    %global configuration variable
INFTY=inf;        %infinity
TINY=1e-6;        %tiny enties
FEAS=1e-6;        %feasibility tolerance

thisfun=mfilename;
 
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'lsolver'};{'eqsolver'}];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'infty'), INFTY=glconf.infty;end
if isfield(glconf,'tiny'),  TINY=glconf.tiny;end
if isfield(glconf,'feast'), FEAS=glconf.feast;end

% init argout
argout.x=argin.x;argout.F=argin.P.Q.F;
if argin.prt,
    argout.out=[{'*** LINEAR SOLVING ***'};{['Solver: ' argin.lsolver]};...
        {['Equation Solving: ' argin.eqsolver]}];
end
if argin.pro, 
    if isempty(argin.cert)
        argout.cert.name='Linear Solving';
        argout.cert.tname='LinearSolve';
        argout.cert.x=argin.x;
        argout.cert.F=argin.P.Q.F;
    else argout.cert=argin.cert;
    end
end

% init variables
nx=length(argin.x.inf);               % number of vars
% if any(argin.x.inf<=-INFTY) | any(argin.x.sup>=INFTY),
%     if argin.prt,
%         argout.out=[argout.out ; ...
%             {'Some bounds are infinite, LinSolve skipped!'}];
%     end
%     return 
% end

% debug
if argin.deb, 
    gldisp('Ainf',argin.P.Q.A.inf,'F',[argin.P.Q.F.inf argin.P.Q.F.sup],... 
    'x',[argin.x.inf, argin.x.sup]);
end

% linearize the constraints
ain.P=argin.P;ain.x=argin.x;ain.bilappr='a';
ain.full=1;ain.prt=argin.deb;ain.deb=argin.deb;
aout=glpropagate_linear(ain);
argout.E=aout.E;argout.b=aout.b;argout.cind=aout.cind;
argout.x=aout.x;argout.F=aout.F;
if argin.deb, disp(aout.out); end

% check the results of the linearization
if isempty(argout.x) 
    % constraints are infeasible
    if argin.pro,argout.cert.x=[]; end
    return;
else
    x.inf=argout.x.inf(1:nx);x.sup=argout.x.sup(1:nx);
    if argin.pro,argout.cert.x=argout.x; end
end

nx=length(x.inf);             % new number of vars
nc=size(argout.E,1);          % new number of constraints

% check if any constraints remain
if isempty(argout.E), 
    if argin.deb,gldisp('x',[x.inf, x.sup]); end
    if argin.prt,
        argout.out=[argout.out ;{['No valid linear constraints left '...
            'after the linear relaxation, bound constraints extracted, '...
            'the remaining part is skipped!']}];
    end
    %graphics
    if argin.gra, glboxplot(argout.x,'Color','r');end
    if argin.pro, argout.cert.x=argout.x; argout.cert.F=argout.F; end
    return 
end

if argin.deb
    gldisp('E',argout.E,'b',[argout.b.inf argout.b.sup],'x',[x.inf, x.sup]);
end

%get the variable indices sorted after their effects on the constraints
argin.P.Q.A.inf=[argout.E zeros(nc, nx^2)];
argin.P.Q.A.sup=[argout.E zeros(nc, nx^2)];
argin.P.Q.F.inf=argout.b.inf;argin.P.Q.F.sup=argout.b.sup;
ind=glvariation(argin.P, x, 'variation');
dirs=[ind(1:min(3,end))' -ind(1:min(3,end))'];  %search directions
nd=length(dirs);

% make linear programm without objective
% split to sided inequalities (since we also need the Lagrange multipliers)
% and add the bound constraints
bwid=argout.b.sup-argout.b.inf; xwid=x.sup-x.inf;
nind=find(argout.b.inf>-INFTY & argout.b.sup<INFTY & bwid>TINY);
nnb=length(nind);
xindlu=find(x.inf<=-INFTY & x.sup<INFTY);
xinduu=find(x.inf>-INFTY & x.sup>=INFTY);
xindeq=find(xwid<=TINY);
xindieq=find(x.inf>-INFTY & x.sup<INFTY & xwid>TINY);niq=length(xindieq);

nnx=length(xindlu)+length(xinduu)+2*length(xindieq)+length(xindeq);
nnc=nc+nnb+nnx;
cin.lin.A=[argout.E ; argout.E(nind,:); ...
    sparse(1:nnx,[xindlu' xinduu' xindeq' xindieq' xindieq'], ones(1,nnx), ...
    nnx,nx,nnx)];
b.inf=[argout.b.inf ; -inf+ones(nnb,1); x.inf(xindlu); x.inf(xinduu); ...
    x.inf(xindeq); x.inf(xindieq); -inf+ones(niq,1)]; 
b.sup=[argout.b.sup ; argout.b.sup(nind); x.sup(xindlu); x.sup(xinduu);...
    x.sup(xindeq); inf+ones(niq,1); x.sup(xindieq)]; 
b.sup(nind)=inf;
cin.lin.b=[b.inf b.sup];
cin.obj.b=sparse([],[],[],nx,1,0); % dummy objective
%if argin.deb, disp(glcprogr2str(cin)), end

Y=sparse([],[],[],nd,nnc,1);

% try to make all completely unbounded components bonded from one side
for i=1:length(dirs)
    vind=dirs(i);   % variable indices (signed)
    %find a bound for x_j from the direction dir(i)
    if argin.prt,
        str=['x_' num2str(abs(vind)) ' from the dir '...
            num2str(sign(vind)) ':'];
    end
    if vind > 0, cin.obj.b=sparse(vind,1,1,nx,1,1);
    else cin.obj.b=sparse(-vind,1,-1,nx,1,1);
    end
    [y, xd, info]=gloptsolve(argin.lsolver, cin);
    if argin.prt,
        argout.out=[argout.out ; {[argin.lsolver ...
            ' solved the linear program:']};glcprogr2str(cin)];
    end
    if argin.deb, disp(glcprogr2str(cin)),gldisp('y',y,'xd',xd); end
    %disp(glcprogr2str(cin))
    %y
    %xd
    %info
    %interpret results
    if ~isempty(y) && (info.numerr==0 || info.numerr==1) &&...
            info.pinf<FEAS && info.dinf<FEAS
        % bound found, save dual solution for verification
        % rounding factor: 108108000=2^5*3^3*5^3*7*11*13 approx 2^27
        nzind=xd~=0;rfact=108108000;%rfact=4*3*5*7*11;
        xd(nzind)=2^(-0)*round(xd(nzind)*(rfact/(realmin+norm(xd(nzind),inf))));
        Y(i,:)=xd';
        % set new bound
        % if vind>0, x.sup(vind)=-y(vind);
        % else x.inf(-vind)=-y(-vind);
        % end
        if argin.prt, 
            str=[str ' bound ' num2str(y(abs(vind))) ' was found!']; 
        end
    elseif argin.prt, str=[str ' no better bound found!'];
    end
    if argin.prt, argout.out=[argout.out ;{str}]; end
end

% verification
Ls=Y*infsup(cin.lin.A,cin.lin.A);
%Y
%Ls
ain.P.Q.A.inf=[Ls.inf zeros(nd,nx^2)];
ain.P.Q.A.sup=[Ls.sup zeros(nd,nx^2)];
F=glimult(Y,infsup(b.inf,b.sup));
% remove zero rows
zind=find(sum(abs(Ls),2)==0);
ain.P.Q.A.inf(zind,:)=[];ain.P.Q.A.sup(zind,:)=[];
if any(F.inf(zind)>0 | F.sup(zind)<0),
    error('gllinsolve: inconsistentcy detected!');
else F(zind)=[];
end

if any(size(F)==0),
    % no inprovements, return
    return;
end
    
% create problem definition for CP and use CP
ain.P.Q.F.inf=F.inf;ain.P.Q.F.sup=F.sup;
ain.P.basis=[];ain.P.uni=[];ain.P.opt=0;
ain.P.Q.int=zeros(nx,1);ain.P.Q.Bu=[];
ain.P.Q=glfindspars(ain.P.Q);ain.P.Q.gaps=[];
ain.x=x;ain.bilappr='a';ain.full=1;ain.prt=0;ain.deb=argin.deb;
if argin.deb, 
    disp('*Constraint generated:')
    ain.P.Q.A.inf
    ain.P.Q.F.inf
    s=glprob2str(ain.P);
    disp(s{1});
end
aout=glpropagate_separable(ain);
x=aout.x;Fnew=aout.F;

% interpret results
if isempty(x)
    % box is empty
    if argin.prt,
        argout.out = [argout.out ;{'Infeasibility verified!'}];
    end
    argout.x=[];
    if argin.pro, argout.cert.x=[]; end
    return
end
if argin.deb,
    gldisp('Ainf',ain.P.Q.A.inf,'Asup',ain.P.Q.A.sup,...
        'F',[ain.P.Q.F.inf ain.P.Q.F.sup],'xf',[x.inf x.sup]);
end
if argin.prt,
    argout.out=[argout.out;{'Constraint propagation for'};...
        glprob2str(ain.P);{'results'};glbox2str(x)];
end

%set resuts in argout.x
argout.x.inf(1:nx)=x.inf;argout.x.sup(1:nx)=x.sup;
P.Q.F=Fnew;

%plot x
if argin.gra, glboxplot(argout.x, 'Color', 'r'); end