%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinearize.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [E, c]=gllinearize(A, z, x, cen, int, prt);
%
% Linearizes the quadratic problem Aq(x) in \x around z by finding an
% interval (int=1) or scalar (int=0) matrix E and an interval vector c that
% (cen=0) A*q(x) \in Ex+c for all x \in \x      (normal form) or
% (cen=1) A*q(x) \in E(x-z)+c for all x \in \x  (centered form), holds.
%
% A             nc x nx^2+nx constraints without constant terms
%   .inf,.sup   outer enclosures
% z             nx x 1 vector to linearize the constraint at 
% x             bound constraints
%   .inf,.sup   outer enclosures
% cen           =0 indicates that the normal form  A*q(x) \in Ex+c and
%               =1 indicates that the centered form  A*q(x) \in E(x-z)+c is
%               computed
% int           =0 indicates that the linear interval coeffs are also
%               approximated
%               =1 leave the intervals in the linear coeffs
% prt           text output flag       0 = no output 1 = output
%
% E             nc x nx linearized (interval/scalat) constraint coeffs
% c             nc x 1 interval vector
%   .inf,.sup   outer enclosures

function [E, c]=gllinearize(A, z, x, cen, int, prt)

% argcheck
if nargin<5, error('gllinearize: Not enough arguments!');end
if nargin<6, prt=0;end

% init
nx=length(x.inf);            % number of vars
nc=size(A.inf,1);            % number of constraints
z=z(1:nx);
c.inf=zeros(nc,1);c.sup=zeros(nc,1);
if int, E.inf=zeros(nc,nx);E.sup=zeros(nc,nx);
else E=zeros(nc,nx);
end

% *linearize Aq(x) and obtain Ex \in \c*
for k=1:nc
    % linearize, obtain \E(k,:)*q(x) \in \con+\lin'x
    con=intval(0);                          % constant term
    lin=infsup(A.inf(k,1:nx),A.sup(k,1:nx));% linear terms
    xi=infsup(x.inf,x.sup);                 % bound constraints interval

    % linearize nonlinear terms
    for i=1:nx
        for j=1:nx
            ind=i*nx+j; % coeff index in c
            if A.inf(k,ind)~=0 && A.sup(k,ind)~=0
                % non zero coeff
                if i==j
                    % linearize quadratic terms
                    aa=(A.inf(k,ind)+A.sup(k,ind))/2;
                    u=2*aa*z(i);
                    % set linear term
                    lin(i)=lin(i)+u;
                    % forward propagation
                    % find upper & lower bounds q for aa z^2 + u z
                    xx.inf=x.inf(i);xx.sup=x.sup(i);
                    qu=glqforw(aa, -u, u, xx, 0);
                    ql=glqforw(-aa, u, -u, xx, 0);
                    % set constant term
                    con=con+infsup(-ql,qu);
                else
                    % linearize bilinear terms
                    bb=infsup(A.inf(k,ind),A.sup(k,ind));
                    u=bb*z(i);v=bb*z(j);
                    % set linear term
                    lin(i)=lin(i)+v;lin(j)=lin(j)+u;
                    % set constant term
                    w=(bb*xi(i).inf-u)*xi(j);
                    % workaround to fix intlab 0*[-inf inf]=NaN
                    if isnan(w), cl=-v*xi(i).inf;
                    else cl=w-v*xi(i).inf;
                    end
                    w=(bb*xi(i).sup-u)*xi(j);
                    if isnan(w), cu=-v*xi(i).sup;
                    else cu=w-v*xi(i).sup;
                    end
                    con=con+infsup(cl.inf,cu.sup);
                end
            end
        end
    end

    % process the found constant and linear part
    if int,
        % interval E required
        if cen,
            % centered form needed
            con=con+glimult(lin,z);
        end
        E.inf(k,:)=lin.inf;E.sup(k,:)=lin.sup;
    else
        % scalar E required
        b=lin.mid;res=lin-b;
        con=con+glimult(res,xi);
        if cen,
            % centered form needed
            con=con+glimult(b,z);
        end
        E(k,:)=b;
    end
   
    % set constant term
    c.inf(k)=con.inf;c.sup(k)=con.sup;
end
