%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinearize.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [E, c]=gllineariz2e(A, F, x, z, bilin, quad, lin, prt)
%
% Linearizes the quadratic problem Aq(x) \in F, x in \x around z by finding
% an interval (int=1) or scalar (int=0) matrix E and an interval vector c 
% that (cen=0) A*q(x) \in Ex+c for all x \in \x      (normal form) or
% (cen=1) A*q(x) \in E(x-z)+c for all x \in \x  (centered form), holds.
%
% A             nc x nx^2+nx constraints without constant terms
%   .inf,.sup   outer enclosures
% F            structure with bounds for general constraints
%   .inf,.sup   outer enclosure
% x             bound constraints
% z             nz x 1 cell of nx x 1 vectors to linearize the quadratic 
%               terms at this points.
%   .inf,.sup   outer enclosures
% bilin         approx binilear terms via intervals in the 'l'inear terms,
%               'q'uadratic terms, 'c'onstant or by adding an 'e'nvelope.
% quad          approx quadratic terms via intervals in the 'l'inear terms, 
%               in the 'c'onstant or by adding 'n'ew constraints.
% lin           approx linear terms via intervals in the 'c'onstant
%               or 'l'eave the intervals in the linear terms
% prt           text output flag       0 = no output 1 = output
%
% E             nc x nx linearized (interval/scalar) constraint coeffs
% c             nc x 1 interval vector
%   .inf,.sup   outer enclosures

function [E, c]=gllinearize2(A, F, x, z, bilin, quad, lin, prt)

% argcheck
if nargin<2, error('gllinearize: Not enough arguments!');end
if nargin<3 || isempty(z), z={glmid(x)};end
if nargin<4, bilin='l';end
if nargin<5, quad='l';end
if nargin<6, lin='c';end
if nargin<7, prt=0;end

% init
nx=length(x.inf);            % number of vars
nc=size(A.inf,1);            % number of constraints
nz=length(z);                % number of points for the tangents

% make separable structure
[sep, A, Bn, Bp, c]=glsepq(A, F, x, bilin, prt);
x=sep.x;

% now we have sparse separable interval constraints
%     A*x.^2 + [-Bn,Bp]*x  >= c
% sep          structure containing separable constraints 
%              Nq(x)>=c,  q_l(x) = a_l x_(k_l)^2 + b_l x_(k_l),  
%              b in [-bn,bp]  (the sign in bn saves time)
%   .il        finite lower bound indices of the original constraint
%   .iu        finite upper bound indices of the original constraint
%   .bil       constants calculated of bilinear terms which will be added 
%              to the right side constant of the split constraints
%   .N         (0,1) coefficient matrix 
%   .c         constant terms
%   .a         coefficients of the quadratic terms
%   .bn,.bp    coefficients of the linear terms
%   .k         index list defining the variable x_(k_l) in q_l(x)
%   .ns        number of new variables (~= 0 in case of envelope)
%   .nn        number of new constraints (~=0 in case of envelope)

%nn=sep.nn;ns=sep.ns;
%nc2=nc+nn;nx2=nx+ns;
%ni=length(sep.il);nu=length(sep.iu);
% A1=sparse([],[],[],nc2,nx2);
% A1(sep.il,:)=A(1:ni,:);
% A1(sep.iu,:)=A(ni+1:ni+nu,:);
% 
% Bn1=sparse([],[],[],nc2,nx2);
% Bn1(sep.il,:)=Bn(1:ni,:);
% Bn1(sep.iu,:)=Bn(ni+1:ni+nu,:);
% Bn1(nc+1:end,:)=-Bn(ni+nu+1:end,:);
% 
% Bp1=sparse([],[],[],nc2,nx2);
% Bp1(sep.il,:)=Bp(1:ni,:);
% Bp1(sep.iu,:)=Bp(ni+1:ni+nu,:);
% Bp1(nc+1:end,:)=-Bp(ni+nu+1:end,:);
% 
% L=infsup(Bn1,-Bp1);
% 
% A.inf=sparse([],[],[],nc2,nx2+nx2^2);
% A.sup=sparse([],[],[],nc2,nx2+nx2^2);
% A.inf(:,1:nx2)=Bn1;A.sup(:,1:nx2)=-Bp1;
% A.inf(:,nx2+1:nx2+1:end)=-A1;A.sup(:,nx2+1:nx2+1:end)=-A1;
% 
% F1.inf=-inf+zeros(nc2,1);F1.sup=inf+zeros(nc2,1);
% F1.inf(sep.il)=c(1:ni);F1.sup(sep.iu)=-c(ni+1:ni+nu);
% F1.inf(nc+1:end)=c(ni+nu+1:end);
% FF=[F1.inf F1.sup]
% F.inf=min(F1.inf,F1.sup);F.sup=max(F1.inf,F1.sup);
% 
% x=sep.x;
% 
% A1,Bn1,Bp1,AA=full(A.inf),FF=[F.inf F.sup]
% 
% if prt,
%     P.Q.A=A;
%     P.Q.F=F;
%     P.basis=[];P.uni=[];P.opt=0;
%     P.Q.int=zeros(nx2,1);P.Q.Bu=[];
%     P.Q=glfindspars(P.Q);P.Q.gaps=[];
%     P.uni=[];P.basis=[];P.shift=[];
%     str=glprob2str(P,x);
%     disp('Two sided constraints merged, new problem:');
%     for i=1:length(str), disp(str{i}); end
% end

[nc2,nx]=size(A);
% bound quadratic entries 
ns=0; % counter for the new variables
nn=0; % counter for the new inequs
nsubst=cell(0);% cell of (i, vnum, a) for variables substitution
nineq=cell(0);% cell of (k, mode) for the new inequs
xq=infsup(x.inf,x.sup).^2,
for i=1:nc2
    if prt, disp(['Inequality (' num2str(i) '):']); end
    % infinite bound constr box indexes
    %infind=find(x.sup>INF | x.inf<-INF); 
    for k=find(A(i,:)~=0)
        a=A(i,k);%bn=Bn(i,k);bp=Bp(i,k);
        % We have ax_k^2+bx_k >= c(i) with b \in [-bn, bp]
        switch quad(1),
            case 'n'
                % approximation mode 1=secant, 2=tangent, or 3=both
                if a>=0, mode=1;
                elseif a<0, mode=2;
                else mode=3;
                end
                % find out if the bilinear term x_k^2 is already marked
                % for approximation or not.
                vnum=0;
                for n=1:length(nineq)
                    nin=nineq{n};
                    if nin(1)==k,
                        % the quadratic term was already marked, change
                        % the mode if necessarry
                        omode=nin(2);
                        if omode~=3 && mode~=omode,
                            % change the mode to 'b'oth
                            nin(2)=3;nineq{n}=nin;
                            if omode==2,nn=nn+1;
                            else nn=nn+nz;
                            end
                        end
                        vnum=n;
                        break;
                    end
                end
                if vnum==0,
                    % introduce new variable
                    ns=ns+1;
                    vnum=ns;
                    nin=[k,mode];
                    nineq=[nineq; {nin}];
                    if mode==1, nn=nn+1;
                    elseif mode==2, nn=nn+nz;
                    else  nn=nn+1+nz;
                    end
                    if prt,
                    disp(['  (NEWC): Quadratic entry '...
                        'x_' num2str(k) '^2' ...
                        ' substituted by the new variable x_'...
                        num2str(nx+vnum) '!']);
                    end
                elseif prt,
                    disp(['  (NEWC): Quadratic entry '...
                        'x_' num2str(k) '^2' ...
                        ' substituted by the existing variable x_'...
                        num2str(nx+vnum) '!']);
                end
                nsubst=[nsubst; {[i,vnum,a]}];
            case 'c'
                % quadratic term is bounded by using interval mult with
                % directed rounding
                b=a*xq(k);b=b.sup;
                if prt,
                    disp(['  (APPR): Quadratic entry ' num2str(a) ...
                        'x_' num2str(k) '^2, for x_' num2str(k) ' in ['...
                        num2str(x.inf(k)) ',' num2str(x.sup(k))...
                        '] approximated: c=' num2str(c(i)) '+' ...
                        num2str(b) '=' num2str(c(i)+b) '!']);
                end
                c(i)=c(i)+b;
            case 'l'
                % linearize quadratic terms 
                sep.il
                sep.iu
                error('not implemented!');
            otherwise
                error(['glsepq: Unknown bilinear approximation mode "'...
                    quad '" found!']);
        end
    end
end

% add new inequs
if(ns>0)
    % substtute the new variables
    Bn2=sparse([],[],[],nc2,ns);
    Bp2=sparse([],[],[],nc2,ns);
    for n=1:length(nsubst)
        b=nsubst{n};
        i=b(1);vnum=b(2);
        Bn2(i,vnum)=-b(3); Bp2(i,vnum)=b(3);
    end
    % create new constraints
    Bp3=sparse([],[],[],nn,nx+ns);
    c2=zeros(ns*4,1);
    ec=1;
    for n=1:ns
        b=nineq{n};
        k=b(1);mode=b(2);
        if mode~=2
            % Add the secant:
            % y_n <= m*x_k+d ---> m*x_k-y_n >= -d
            %    m=x.sup(k)+x.inf(k), d=-x.sup(k)*x.inf(k)
            if ~isinf(x.sup(k)) && ~isinf(x.inf(k))
                Bp3(ec,nx+n)=-1;
                Bp3(ec,k)=x.sup(k)+x.inf(k);
                c2(ec)=x.sup(k)*x.inf(k);
                if prt,
                    disp(['Upper relaxing secant for x_' num2str(k)...
                        '^2 added.']);
                end
                ec=ec+1;
            end
        end
        if mode~=1
            % Add nz new tangents, for z{i}_k i=1...nz
            % y_n >= m*x_k+d ---> y_n-m*x_k >= d
            %    m=2*z{i}_k, d=-z{i}_k^2
            for i=1:nz
                zp=z{i};
                if ~isinf(zp)
                    Bp3(ec,nx+n)=1;
                    Bp3(ec,k)=-2*zp(k);
                    c2(ec)=-zp(k)^2;
                    if prt,
                        disp(['Lower relaxing tangent for x_' num2str(k)...
                            '^2 added for the point ' num2str(zp(k)) '.']);
                    end
                    ec=ec+1;
                end
            end
        end
    end
    nn=ec-1; % revised number of additional constraints 
    Bp3=Bp3(1:nn,:);
    c2=c2(1:nn);
    Bn3=-Bp3;
    % increase the number of variables by dimension by ns
    Bn=[Bn Bn2;Bn3];
    Bp=[Bp Bp2;Bp3];
    c=[c;c2];
    A=sparse([],[],[],nc2+nn,nx+ns,0);
    x.inf=[x.inf;-inf+zeros(ns,1)];
    x.sup=[x.sup;inf+zeros(ns,1)];
    %iu=[iu;ones(nn,1)];
    if prt,
        disp([num2str(ns) ' new variables and ' num2str(nn) ...
            ' new constraints added.']); 
    end
end

% process the linear part
[nc2,nx]=size(A);
xi=infsup(x.inf,x.sup);
for i=1:nc2
    li=infsup(-Bn(i,:),Bp(i,:))';
    if lin(1)=='c' && any(diam(li)>0)
        b=li.mid;Bn(i,:)=-b';Bp(i,:)=b';
        con=glimult((li-b)',xi);
        c(i)=c(i)+con.inf;
        if prt,
            disp(['The interval coefficients removed from constraint ' ...
                num2str(i) '.']); 
        end
    end
end

% now we have sparse separable interval constraints
%     A*x.^2 + [-Bn,Bp]*x  >= c
% translate into standard separable format
%     Nq(x)>=c,  q_l(x) = a_l x(k_l)^2 + b_l x(k_l), b in [-bn,bp]
[ii,kk]=find(Bn~=0|Bp~=0|A~=0);
nq=length(ii);
a=zeros(nq,1);bp=zeros(nq,1);bn=zeros(nq,1);sep.k=zeros(nq,1);
neq=size(c,1);N=sparse([],[],[],neq,nq,nq);
for l=1:nq,
  i=ii(l);k=kk(l);
  N(i,l)=1;
  a(l)=A(i,k);
  bn(l)=Bn(i,k);bp(l)=Bp(i,k);
  sep.k(l)=k;
end
% fill separable structure
sep.N=N;sep.c=c;sep.a=a;sep.bn=bn;sep.bp=bp;
sep.x=x;
sep.ns=ns;
sep.nn=nn;
% display polynomials
if prt,
    disp('Constraints:');
    gldispquad(a,infsup(-bn,bp),c,sep.k,N)
end


error('fdf');



% zp=z{1};zp=zp(1:nx);
% 
% 
% nx=nx2;
% nc=nc2;
% c.inf=zeros(nc,1);c.sup=zeros(nc,1);
% if lin=='l', E.inf=zeros(nc,nx);E.sup=zeros(nc,nx);
% else E=zeros(nc,nx);
% end
% 
% % *linearize Aq(x) and obtain Ex \in \c*
% for k=1:nc
%     % linearize, obtain \E(k,:)*q(x) \in \con+\lin'x
%     con=infsup(F.inf(k),F.sup(k));          % constant term
%     li=L(k,:);
%     xi=infsup(x.inf,x.sup);                 % bound constraints interval
% 
%     % linearize nonlinear terms
%     for i=1:nx
%         for j=1:nx
%             ind=i*nx+j; % coeff index in c
%             if A.inf(k,ind)~=0 && A.sup(k,ind)~=0
%                 % non zero coeff
%                 if i==j
%                     % linearize quadratic terms
%                     aa=(A.inf(k,ind)+A.sup(k,ind))/2;
%                     u=2*aa*zp(i);
%                     % set linear term
%                     li(i)=li(i)+u;
%                     % forward propagation
%                     % find upper & lower bounds q for aa z^2 + u z
%                     xx.inf=x.inf(i);xx.sup=x.sup(i);
%                     qu=glqforw(aa, -u, u, xx, 0);
%                     ql=glqforw(-aa, u, -u, xx, 0);
%                     % set constant term
%                     con=con+infsup(-ql,qu);
%                 else
%                     error('bilinear term found');
%                 end
%             end
%         end
%     end
%     % process the found constant and linear part
%     if lin=='l',
%         % interval E required
%         if cen,
%             % centered form needed
%             con=con+glimult(lin,z);
%         end
%         E.inf(k,:)=li.inf;E.sup(k,:)=li.sup;
%     else
%         % scalar E required
%         b=li.mid;res=li-b;
%         con=con+glimult(res,xi);
%         if cen,
%             % centered form needed
%             con=con+glimult(b,z);
%         end
%         E(k,:)=b;
%     end
%     c.inf(k)=con.inf;c.sup(k)=con.sup;
% end
% 
% if prt,
%     P.Q.A.inf=[E,zeros(nc,nx*nx)];P.Q.A.sup=P.Q.A.inf;
%     P.Q.F=c;
%     P.basis=[];P.uni=[];P.opt=0;
%     P.Q.int=zeros(nx2,1);P.Q.Bu=[];
%     P.Q=glfindspars(P.Q);P.Q.gaps=[];
%     P.uni=[];P.basis=[];P.shift=[];
%     str=glprob2str(P,x);
%     disp('Two sided constraints merged, new problem:');
%     for i=1:length(str), disp(str{i}); end
% end