%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmachino.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function x=glmachino(P, x, mingain, prt)
%
% Try to get a tight bounding box for each constraint separately by
% linearizing it and splitting the x in two subboxes. In the splitted
% boxes the constraint will be linearized and splitted again, until
% a minimum and a maximum of the constraint will be obtained.
%
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% mingain       minimum gain
%
% P             new problem definition
% x             new bound constraints
% out           text output
% cert          new certificate if none was supplied in input

function x=glmachino(P, x, mingain, prt)
                                              
% argcechk
if nargin <3, error('glmachino: Not enough arguments!');end
if nargin <4, prt=0; end

%init
setround(1);
nc=size(P.Q.A.inf,1);           % number of constraints        
cbox=x;

% process all constraints
for k=1:nc
    c.inf=P.Q.A.inf(k,:);c.sup=P.Q.A.sup(k,:);
    mode=0; % mode: 0 compute x1 and x2, save x2
            % mode: 1 use x1, compute always x1 until not enough gain
            % mode: 2 use x2, compute always x2 until not enough gain
    x1=[];x2=[];
    % begin splitting
    while 1,
        % (0) save the old cbox
        obox=cbox;
        % (1) linearize the problem in the current box about the midpoint
        % computing \cq(x) \in bx+\d (normal form and thin b)
        z=glmid(cbox);
        [b, d]=gllinearize(c, z, cbox, 0, 0, prt);
        % (1') transform: f(x) \in \d+b(x-z)= (\d-bz)+bx= \a+b(x-0)
        a.inf=-(-d.inf+b*z);a.sup=d.sup-b*z;
        %if prt, cc=c.inf, b, aa=[a.inf a.sup], end
        ab=abs(b);                      % abs(b)   row vector
        xmz=0.5*(cbox.sup-cbox.inf);   % |\x-z| z=mid(x)
        xw=cbox.sup-cbox.inf;           % wid(cbox)     col vector
        %(2) compute delta
        delta=(a.sup-a.inf)+b*xmz;
        if prt, gldisp('delta',delta); end
        %(3) find the x_i with abs(mid(b(i)))*(cbox.sup-cbox.inf)>delta 
        ind=ab.*xw'>delta;
        %if prt, ind, ii=find(ind), end
        %(4) check the index set
        if any(ind==1)
            %(5) the index set is not empty, split the cbox
            pind=b>0&ind; nind=b<0&ind;
            dp=delta./ab(pind)'; dn=delta./ab(nind)';
            %if prt, pind,nind, end
            if mode==0 || mode==1,
                x1=cbox;
                x1.sup(pind')=x1.inf(pind')+dp;
                x1.inf(nind')=-(-x1.sup(nind')+dn);
                if prt, gldisp('x1',[x1.inf x1.sup]); end
            end
            if mode==0 || mode==2,
                x2=cbox;
                x2.sup(nind')=x2.inf(nind')+dn;
                x2.inf(pind')=-(-x2.sup(pind')+dp);
                if prt, gldisp('x2',[x2.inf x2.sup]); end
            end
            % set new current box 
            if mode==2, cbox=x2;
            else cbox=x1;
            end
            % compute the gain
            if mode==0, 
                if prt, disp('Mode=0: finished, switching to mode=1 !');end
                mode=1;
            elseif all(glgain(obox, cbox))<mingain
                if mode==1,
                    if prt, 
                        disp('Mode=1: gain to small, switching to mode=2!');
                    end
                    mode=2;
                else
                    if prt, 
                        disp('Mode=2: gain to small, break, next constraint!');
                    end
                    break;
                end
            end
        elseif mode==1
            % the index set is empty, step to x2
            if prt, disp('Mode=1: empty index set, switching to mode=2 !');end
            mode=2;cbox=x2; continue;
        else
            % the index set is empty, mode==0 or mode==2, break
            if prt, 
                disp(['Mode=' num2str(mode) ': empty index set, break, '...
                    'next constraint!']);
            end
            break;
        end
    end
end
if prt, gldisp('x',[x.inf x.sup]); end;
