%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glprobing_tsplit.m %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glprobing_tsplit(argin)
%
% Tentatively splits the current box at the midpoint of a choosen direction.
% The substrategy is applied to the two boxes, then they are merged again. 
% The merging also stores gaps in the resulting box, but maximum one gap in
% each direction (the largest one) can be stored. 
% The directions where the reduction was significant (the minimum gain is 
% reached) are memorized and can be splitted again in the next round
% until there is no significant reduction, the maximum number of
% iteration is reached or all x components are small.
%
% * needed argin structure input fields *
% P             problem definition
% x             bound constraints
% tasklist      tasklist cell vector
% starttask     starting task can be a vector if subtask are present
% dirchooser    split direcction choosing method
% maxiter       maximum number of allowed iterations
% small         minimum realtive width of the remaining box: all(x)<small
%               finish while
% mingain       minimum gain in one ireation to make anoter round
%
% * optional argin structure input fields *
% nopause       ==1 ignores the pauses in the tasklist
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P             new problem definition
% x             new bound constraints
% status        = 0 proceed with the next task
%               = 1 stop the strategy (pause or user break)
%               = 4 continue with postprocess (glbreak->postprocess,timeout)
%               = 5 finish the strategy (glbreak->finish strategy, problem 
%                   not accepted: dim too high/nonquadratic found, after 
%                   postprocess)
% taskpos       curent task position, can be a vector if subtask are present
% cert          certificates cell array

function argout=glprobing_tsplit(argin)
    
global glbreak;     % gloptlab break flag
global gloutput;    % global output structure
global glconf;      % gloptlab configuration file
                    
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'tasklist'};{'starttask'};{'dirchooser'};{'maxiter'};...
    {'mingain'};{'small'}]; thisfun=mfilename;

% optional input fields and their default values
oinp=[{'nopause'};{'prt'};{'deb'};{'gra'};{'pro'}];
dval=[{    0    };{  0  };{  0  };{  0  };{  0  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.P=argin.P;argout.x=argin.x;argout.status=0;
argout.timer=argin.timer;
if argin.pro, 
    %init certificates
    argout.cert=cell(0);
end

% init variables
iter=0;nx=length(argout.x.inf);
gain=ones(1,2*nx);
gain(isinf(argout.x.inf'))=-1;gain(nx+find(isinf(argout.x.sup)))=-1;
starttask=argin.starttask;
if isempty(starttask), starttask=1; end

if argin.gra && nx>1,
    % initilaize the plot
    plot_init(argin.P,argin.x);
    plot_box('addfound', argin.x);
end

if argin.prt,
    if argin.nopause, out={'*** PROBING(tsplit) START (nopause) ***'};
    else out={'*** PROBING(tsplit) START ***'};
    end
    out=[out; {['Direction choosing method: ' num2str(argin.dirchooser)]};...
        {['Maxiter: ' num2str(argin.maxiter)]}; {['Mingain: ' ...
        num2str(argin.mingain)]};{['Small: ' num2str(argin.small)]}];
    gloutput.out=[gloutput.out ; out];
end
treason='Normal termination.';


% begin while
while 1
    for ind=find(gain>=argin.mingain)
        if ind<=nx, i=ind; %direction: lower
        else i=ind-nx;     %direction: upper
        end
        box=argout.x;  %slice
        box2=argout.x; %remainder
        wid=(box.sup(i)-box.inf(i))*argin.redfac; % width of the slice
        if ind<=nx, 
            % slice from the lower bound
            bound=box.inf(i)+wid;
            box.sup(i)=bound;box2.inf(i)=bound;
        else
            % slice from the upper bound
            bound=box.sup(i)-wid;
            box.inf(i)=bound;box2.sup(i)=bound;
        end
        
        if argin.gra && nx>1,
            plot_box('remfound', []);
            plot_box('highlight', box);
            plot_box('addfound', box2);
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % process tasklist
        [P2, box, argout.status, argout.taskpos, argout.timer, certs2]=...
            glproctskl(argout.P, box, argin.tasklist, starttask, ...
            argout.timer, argin.nopause, 0, 0, argin.pro);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % delete old starttask
        starttask=1;
        if argin.gra && nx>1,
            plot_box('remhighlight');% remove the highlight
            plot_box('remfound', []);
        end
        if isfield(box,'infeasible'),
            gain(ind)=1;argout.x=box2;
        else
            gain(ind)=1-(box.sup(i)-box.inf(i))/wid;
            if ind<=nx, argout.x.inf(i)=box.inf(i);
            else argout.x.sup(i)=box.sup(i);
            end
        end
        if argin.gra && nx>1,
            plot_box('addfound', argout.x);
        end
        % redraw
        if ~argin.nopause, drawnow; end
        
        %%%%%%%%%%%%%%%%%
        % STATUS CHECKS %
        %%%%%%%%%%%%%%%%%
        % check user break
        if glbreak, argout.status=1; end
        % end the while for: stop (1), break multiline(3), postprocess (4) or
        % finish(5)
        if argout.status>0,
            treason=glstatus2str(argout.status);
            break;
        end

        % certificates
        if argin.pro, argout.cert=[argout.cert; certs2]; end
    end
    %%%%%%%%%%%%%%%%%
    % check termination criteria
    if all(gain<argin.mingain)
        treason='Reduction factor was too low';
        argout.status=0; 
        break; 
    elseif iter==argin.maxiter
        treason='Maximum number of iteration exceeded'; 
        argout.status=0; 
        break; 
    end
    % next iteration
    iter=iter+1;
end
if argin.gra && nx>1,
    % finalize the plot
    plot_box('remfound', []);
    plot_finalize;
    glboxplot(argout.x);
end
if argin.prt,
    gloutput.out=[gloutput.out ; {['*** PROBING(tsplit) END *** Full steps '...
        'completed: ' num2str(iter) ', Termination reason: ' treason '.']}];
end
% absorb the break multiline status
if argout.status==3, argout.status=0;end

%%%%%%%%%%
%PLOTTING%
%%%%%%%%%%

% initialize the plot
function plot_init(P,refbox)
global glconf;      % gloptlab configuration file
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later

nx=length(gloutput.graphics.refbox.inf);
if length(refbox.inf)~=nx
    % input box and refbox dimension mismatch, the slack variables from
    % the input box must be removed!
    refbox.inf=refbox.inf(1:nx);refbox.sup=refbox.sup(1:nx);
end
% correct the reference box by cutting with the reference box
% from the config (if the intersection is not empty)
rbo=gloutput.graphics.refbox;
plot.refbox.inf=max(refbox.inf,rbo.inf);
plot.refbox.sup=min(refbox.sup,rbo.sup);
ind=find(plot.refbox.inf>plot.refbox.sup);
plot.refbox.inf(ind)=refbox.inf(ind);
plot.refbox.sup(ind)=refbox.sup(ind);
plot.is2d=nx==2;

if plot.is2d
    % 2D plot
    plot.SCOL=[0.8 1 1];     % color of the currently processed box (pause)
    plot.NCOL=[0.9 0.9 0.9]; % color of the not processed boxes (pause)
    plot.FCOL=[1 0.8 1];     % color of the found box
    plot.FFCOL=[1 0 0];      % color of the found full box
    plot.REFMODE=0;          % reference mode, used in plotting the boxes:
    % 0=reference box is the actual graphic view
    % 1=reference box is the current xbox
    if isfield(glconf,'sboxcol'),plot.SCOL=glconf.sboxcol;end
    if isfield(glconf,'nboxcol'),plot.NCOL=glconf.nboxcol;end
    if isfield(glconf,'fboxcol'),plot.FCOL=glconf.fboxcol;end
    if isfield(glconf,'ffboxcol'),plot.FFCOL=glconf.ffboxcol;end
    plot.hstack=[];
    plot.fhand=[];
    plot.high=[];
else
    % HD plot
    if isfield(glconf,'xind'),xind=glconf.xind;
    else xind=gloutput.graphics.xind;
    end
    glplot(P, [], 'Refbox', plot.refbox, 'IndexSet', xind);
    gloutput.graphics.fillbox=1;
    gloutput.graphics.plotdata=1;
    gloutput.graphics.plotc=1;
    plot.high=[];
    drawnow;
end

% add a box to the plot and store its handle into the store
function plot_box(command, box)
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if plot.is2d
    % 2D plot
    switch command
        case 'highlight'
            if plot.REFMODE, h=glboxplot(box, 'Color', SCOL,...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box, 'Color', plot.SCOL, 'Fill', inf);
            end
            plot.high=h;
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'remhighlight'
            if ~isempty(plot.high) && ishandle(plot.high)
                delete(plot.high);
            else return; % no highlited box
            end
        case 'addfound'
            %color=plot.FFCOL;
            color=plot.FCOL;
            if plot.REFMODE, h=glboxplot(box, 'Color', color, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box,'Color', color, 'Fill', inf);
            end
            plot.fhand=[plot.fhand h];
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'remfound'
            delete(plot.fhand(end));
            plot.fhand=plot.fhand(1:end-1);
        case 'addfeas'
            color='g';
            if plot.REFMODE, h=glboxplot(box, 'Color', color, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box, 'Color', color, 'Fill', inf);
            end
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        otherwise
            error(['glsplit_depthfirst:plot_box: Unknown '...
                'command found: ' command]);
    end
else
    % HD plot
    switch command
        case 'highlight'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=0;
            plot.high=box;
        case 'remhighlight'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=0;
            if ~isempty(plot.high) && ~isempty(plot.high)
                box=plot.high;
            else return; % no highlited box
            end
        case 'addfound'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=1;
        case 'remfound'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=-1;
        case 'addfeas'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=1;
        case 'addstack'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=1;
        case 'remstack'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=-1;
        otherwise
            error(['glsplit_depthfirst:plot_box: Unknown '...
                'command found: ' command]);
    end
    if gloutput.graphics.fillbox>0,plot.high=box;
    else plot.high=[];
    end
    glboxplot(box);
end


% remove a all boxes from the plot
function plot_clear(bstack, boxes)
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if plot.is2d
    % 2D plot
    for i=1:length(plot.hstack),
        delete(plot.hstack(i));
    end
    plot.hstack=[];
    % delete found box handles
    delete(plot.fhand);
elseif 0,
    % HD plot (turned off)
    gloutput.graphics.fillbox=0;
    gloutput.graphics.plotc=-1;
    for i=length(bstack):-1:1,
        glboxplot(bstack{i});
    end
    for i=1:length(boxes),
        glboxplot(boxes{i});
    end
end
plot_finalize;

% finalize the plot
function plot_finalize
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if isfield(plot, 'is2d')
    if ~plot.is2d
        % HD plot
        gloutput.graphics.fillbox=2;
        gloutput.graphics.plotdata=1;
        gloutput.graphics.plotc=0;
    end
    plot_box('remhighlight');
    clear global plot
    drawnow;
end

