%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% glproject_project.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glproject_project(argin)
%
% Transform the problem in its projective form if it is not already
% transformed.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% cond          condition when the projection should be done
%               'always'  or 'infinite' = transform only if there are
%               any infinite components x_j with j \in chkvars
%
% * optional argin structure input fields *
% chkvars       this argin is only needed if the cond=='infinite' and
%               gives the set of components of x which has to be checked. 
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .P          new problem definition
%
% * argout structure fields *
% x             new bound constraints
% P             new problem definition
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glproject_project(argin)

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'cond'}];

% optional input fields and their default values
oinp=[{'chkvars'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  'orig' };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
if argin.prt,
    argout.out=[{'*** PROJECTIVE TRANSFORMATION ***'};...
        {['Projection condition: ' argin.cond]}];
    if strcmp(argin.cond, 'infinite')
        argout.out=[argout.out; {['Check variables: ' argin.chkvars]}];
    end
end
if argin.pro,
    % make a normal certificate
    argout.cert.name='Projective Transformation';
    argout.cert.tname='Project';
    argout.cert.x=argout.x;
end

% init variables
nx=length(argin.x.inf);               % number of vars
nc=size(argin.P.Q.A.inf,1);           % number of constraints

% check if the projection is already done or not
if isfield(argin.P,'project')
    if argin.prt
        argout.out=[argout.out ; {['The problem '...
            'is already in the projective form!']}];
    end
    return
end

% check if the projection condition is met or not
if strcmp(argin.cond,'infinite')
    % init chkvars
    [ok, cind]=glparsevar(argin.chkvars, 1:argin.P.Q.nxo, 1:nx);
    if ~ok, error([thisfun ': chkvars string has wrong format!']); end
    if all(~isinf(argin.x.inf(cind))) && all(~isinf(argin.x.sup(cind)))
        if argin.prt
            argout.out=[argout.out ; {['Variables: x_{' num2str(cind) ...
                '} checked:']};{['The checked vars have all finite '...
                'bounds, the projective transformation is skipped!']}];
        end
        return
    end
end

% Transform the problem
A.inf=sparse([],[],[],nc+1,(nx+1)*(nx+2),nnz(argin.P.Q.A.inf)+nx);
A.sup=sparse([],[],[],nc+1,(nx+1)*(nx+2),nnz(argin.P.Q.A.sup)+nx);
ind=[];ind2=[];
for i=1:nx+1,
    ind=[ind (nx+1)*i+1+(1:nx)];
    ind2=[ind2 (nx+2)*i];
end
A.inf(1:nc,ind)=argin.P.Q.A.inf;A.inf(nc+1,ind2)=ones(1,nx+1);
A.sup(1:nc,ind)=argin.P.Q.A.sup;A.sup(nc+1,ind2)=ones(1,nx+1);
if argin.deb
    gldisp('nx',nx,'nc',nc,'Aold',full(argin.P.Q.A.inf),'Anew',full(A.inf));
end
%create new F
F.inf=[min(argin.P.Q.F.inf,0) ; 1];F.sup=[max(argin.P.Q.F.sup,0) ; 1];
if argin.deb
    gldisp('Fold',[argin.P.Q.F.inf argin.P.Q.F.sup],'F',[F.inf F.sup]);
end
%create new xbox
argout.x.inf=[0;-ones(nx,1)]; argout.x.sup=ones(nx+1,1);
if argin.deb,
    gldisp('xold',[argin.x.inf argin.x.sup],'x',[argout.x.inf argout.x.sup]);
end
%save old F and x and set new values for A, F and x
argout.P=argin.P;
argout.P.project.F=argin.P.Q.F; argout.P.project.x=argin.x;
argout.P.Q.A=A; argout.P.Q.F=F; argout.P.Q.int=[0;argin.P.Q.int];
%set new sparsity and non zero pattern
argout.P.Q=glfindspars(argout.P.Q);

% text output
if argin.prt,
    argout.out=[argout.out ; {'Transformed problem:'}; ...
        glprob2str(argout.P, argout.x)];
end

% certificate
if argin.pro,
    argin.cert.x=argout.x;
    argin.cert.P=argout.P;
end