%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% glproject_unproject.m %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glproject_unproject(argin)
%
% Reverts the projective transformation, extracting a new x and F.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .P          new problem definition
%
% * argout structure fields *
% x             new bound constraints
% P             new problem definition
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glproject_unproject(argin)

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
if argin.prt,
    argout.out={'*** REVERT PROJECTIVE TRANSFORMATION ***'};
end
if argin.pro,
    % make a normal certificate
    argout.cert.name='Revert Projective Transformation';
    argout.cert.tname='UnProject';
    argout.cert.x=argout.x;
end

% init variables
nx=length(argin.x.inf);               % number of vars
nc=size(argin.P.Q.A.inf,1);           % number of constraints

% check if the projection is already done or not
if ~isfield(argin.P,'project')
    if argin.prt
        argout.out=[argout.out ; {['The problem '...
            'is not in the projective form, task skipped!']}];
    end
    return
end

% Transform problem back
ind=[];
for i=1:nx, ind=[ind nx*i+1+(1:(nx-1))];end
A.inf=argin.P.Q.A.inf(1:(nc-1),ind);
A.sup=argin.P.Q.A.sup(1:(nc-1),ind);
if argin.deb, 
    gldisp('nx',nx,'nc',nc,'Aold',full(argin.P.Q.A.inf),'Anew',full(A.inf));
end
%create new F
lind=find(argin.P.Q.F.inf(1:end-1)<=0);
F.inf=argin.P.project.F.inf; F.inf(lind)=argin.P.Q.F.inf(lind);
uind=find(argin.P.Q.F.sup(1:end-1)>=0);
F.sup=argin.P.project.F.sup; F.sup(uind)=argin.P.Q.F.sup(uind);
if argin.deb,
    gldisp('Fold',[argin.P.Q.F.inf argin.P.Q.F.sup],'F',[F.inf F.sup]);
end
%create new xbox
if argin.x.inf(1)>0
    %\Z_0 does not contain the 0, all bounds are now finite
    zi=infsup(argin.x.inf,argin.x.sup);
    zi=zi(2:end)/zi(1);
    argout.x.inf=max(zi.inf, argin.P.project.x.inf);
    argout.x.sup=min(zi.sup, argin.P.project.x.sup);
else argout.x=argin.P.project.x;
end
if argin.deb,
    gldisp('xold',[argin.x.inf argin.x.sup],'x',[argout.x.inf argout.x.sup]);
end
%delete old xbox and F set new values for A, F and x
argout.P=rmfield(argin.P, 'project');
argout.P.Q.A=A;argout.P.Q.F=F;argout.P.Q.int=argout.P.Q.int(2:end);
%set new sparsity and non zero pattern
argout.P.Q=glfindspars(argout.P.Q);

%graphics output
if argin.gra,
    glboxplot(argout.x);
end

% text output
if argin.prt,
    argout.out=[argout.out ; {'Problem transformed back to:'}; ...
        glprob2str(argout.P, argout.x)];
end

% certificate
if argin.pro,
    argin.cert.x=argout.x;
    argin.cert.P=argout.P;
end