%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glgaps.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,gaps]=glgaps(P,x);
% finds (nargin=1) or improves (nargin=2) rigorous bounds and gaps 
% for variables from integer and set constraints in problem P.
%
% Integer constraints are only used to widen gaps, not to create them.
% (But integer gaps may be relevant in SDPs.) 
% 
% P              structure containing the internal problem definition
%   P.CC(i)      interval enclosure of constant i  (Intlab compatible)
%   P.Q.int      characteristic vector of integer variables
%   P.set        ns x 1 cell array defining set constraints
%                N_k = {C(i) | i in P.set{k}}
%   P.setx(k)    = set_k, index of variable with x_{set_k} in N_k
% x.inf,x.sup    nx x 1 vector of bounds on variables
%
% gaps           ng x 3 array with rows [k a b] specifying finite gaps
%                    x_k notin ]a,b[ 
%
function [x,gaps]=glgaps(P,x)

% initial box 
if nargin==1,
  % initially, no bound constraints are known
  nx=length(P.Q.int);
  x.sup=inf+zeros(nx,1);
  x.inf=-x.sup;
else
  % check integer bounds
  ind=find(P.Q.int==1);
  x.inf(ind)=ceil(x.inf(ind));
  x.sup(ind)=floor(x.sup(ind)); 
end;

ng=0;
ns=length(P.setx);
gaps=[];  %new
for k=1:ns,
  i=P.setx(k);
  values=P.CC(P.set{k}); % bounds for values of x_i
  nv=length(values);
  if nv==0,
    x=[];gaps=[]; 
    return; 
  elseif nv==1,
    x.inf(i)=values.inf;
    x.sup(i)=values.sup;
    continue;
  end;
  % now nv>1
  % find lower end of the first gap
  [vl,perm]=sort(values.inf);
  vu=values.sup(perm);
  if P.Q.int(i),
    % values must be integers 
    vl=ceil(vl);vu=floor(vu); 
  end;
  x.inf(i)=max(x.inf(i),vl(1));
  j=0;
  while 1,
    j=j+1;a=vu(j);
    while j<nv,
      b=vl(j+1);
      if b>a, break; end
      j=j+1;a=max(a,vu(j));
    end;
    if j==nv || b>=x.sup(i),
      x.sup(i)=min(x.sup(i),a);
      break;
    end;
    ng=ng+1;
    gaps(ng,:)=[i,a,b];
  end;
end;

