%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gllinsol_fast.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [x, F]=gllinsol_fast(A, F, x, vind, prt)
%
% Constraint propagation for a linear CSP,
% Ax \in \F, A \in \A, x \in \x
%
% A             nc x nx constraint coeff matrix
%     .inf,.sup outer enclosures
% F             nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%     .inf,.sup outer enclosures
% vind          1 x nc index set of the selected variables \in 1,...,nx,
%               select the variable x_vind(i) for the ith constraint
% prt           text output flag       0 = no output 1 = output
%
% x             nx x 1 new bound constraints
% F             nc x 1 new bounds of general constraints

function [x, F]=gllinsol_fast(A, F, x, vind, prt)

% argcechk
if nargin<4, error('gllinsol_fast: Not enough arguments!');end
if nargin<5, prt=0;end

% init
nx=length(x.inf);       % number of vars
nc=length(F.inf);       % number of constraints

% checks
if any(size(A.inf)~=[nc,nx]) || size(vind,2)~=nc,
    error('gllinsol_fast: Dimension mismatch!');
end

setround(1);            % roundup
xn=infsup(x.inf,x.sup); % interval verion of x
A=infsup(A.inf,A.sup);  % interval verion of A
Fn=glimult(A,xn);
Fn=infsup(max(Fn.inf,F.inf),min(Fn.sup,F.sup)); % new interval F

% constraint propagation for the ith constraint results for all
% x(j) with b(i,j)~=0:
% x.inf(j)=min( x(j).inf, (F.inf(i)-sum_(k~=j) (b(i,k)*x(k)).inf)
for i=1:nc
    c=-A(i,:).*xn';
    s1n=[0 cumsum(-c.inf)];s1p=[0 cumsum(c.sup)];
    s2n=[0 cumsum(-c.inf(end:-1:1))];s2p=[0 cumsum(c.sup(end:-1:1))];
    j=vind(i);
    if A.inf(i,j)>0 %should be 1 or 0 because of preconditioning
        %linear term set, bound linear term
        res=infsup(-(s1n(j)+s2n(end-j)-Fn(i).inf),...
            s1p(j)+s2p(end-j)+Fn(i).sup); %residual
        res=res/A(i,j);
        l=max(xn(j).inf, res.inf);
        u=min(xn(j).sup, res.sup);
        if u<l
            if prt, disp('Constraints are inconsistent!');end
            xbox=[];
            if pro,cert.xbox=xbox; end
            return
            %j, xn_err=[l u], tt=verifylss(A,Fn);
            %test=[max(xn(j).inf,tt(j).inf) min(xn(j).sup,tt(j).sup)]
            %error('gllcontract:constraints inconsistent')
        end
        xn(j)=infsup(l,u);
    end
end
% concatenate with the old x
x=glconc(x, xn);
Fn=glimult(A,infsup(x.inf, x.sup));
F.inf=Fn.inf;F.sup=F.sup;
