%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glpropagate_linear.m %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glpropagate_linear(argin);
%
% Compute the linear relaxation
%               Ex \in \b and x \in \x_new 
% of the quadratic constraint satisfaction problem
%               Aq(x) \in \F and x \in \x.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             bound constraints
%   .inf,.sup   outer enclosures
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure, if supplied, it will be checked
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .E          linearized constraint coeffs
%   .c          linearization interval constant
%
% * argout structure fields *
% x             new bound constraints, empty if the problem is infeasible
%   .inf,.sup   outer enclosures
% F             new bounds of the general constraints
%   .inf,.sup   outer enclosures
% E             nc_new x no linearized constraint coeffs
% b             nc_new x 1 constant interval vector
%   .inf,.sup   outer enclosures
% cind          indices of the remaining constraints with nc_new:=|cind|.
%               nc_new must not equal nc since after the linearization, 
%               the infinite, the empty and the bound constraints will be
%               removed. The removed bound constraints are stored in 
%               the returned x.
% out           text output string cell
% cert          new certificate, with the same fields as in the input

function argout=glpropagate_linear(argin)

% global variables and constants
global glconf;    % global configuration variable
INFTY=inf;        % infinity

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'}];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'infty'), INFTY=glconf.infty;end

% init argout
argout.x=[];argout.F=argin.P.Q.F;argout.E=[];argout.b=[];argout.cind=[];
if argin.prt,argout.out={'*** LINEAR CONSTRAINT PROPAGATION ***'};end
if argin.pro, 
    argout.cert.name='Linear Constraint Propagation';
    argout.cert.tname='LinPropagate';
    argout.cert.x=[];
    argout.cert.F=argin.x;
end

% debug
if argin.deb, 
    gldisp('A',argin.P.Q.A.inf,'x',[argin.x.inf argin.x.sup],... 
        'F',[argin.P.Q.F.inf argin.P.Q.F.sup]);
end

% init variables
x=argin.x;A=argin.P.Q.A;F=argin.P.Q.F;
setround(1);                    % round up
z=glmid(x);                     % midpoint of the linearization
nx=length(x.inf);               % number of vars
nc=size(A.inf,1);               % number of constraints

% linearize the left hand side in \x about z by computing Aq(x) \in Ex+\c
% in the normal form and thin E, Aq(x) \in Ex+c
if ~isfield(argin.cert, 'E')
    [E, c]=gllinearize(A, z, x, 0, 0, argin.deb);
    if argin.pro, argout.cert.E=E; argout.cert.c=c; end
else E=argin.cert.E;c=argin.cert.c;
end

% new contraints bounds F:=F_old \conc (E\x+c)
r=glimult(E,infsup(x.inf,x.sup));
F.inf=max(F.inf,-(-r.inf-c.inf));F.sup=min(F.sup,r.sup+c.sup);

% test feasibility
if any(F.inf>F.sup), 
    if argin.prt,
        argout.out=[argout.out ; {'Constraints are inconsistent!' }];
    end
    return; 
end

% use standard CP to improve the linearization Ex \in b:=F-c and
% to get better bound constraints
b.inf=-(-F.inf+c.sup);b.sup=F.sup-c.inf;
E2.inf=E;E2.sup=E;
[x, b]=gllinsol_standard(E2, b, x, [], argin.prt);
if isempty(x), return; end

% remove unbounded constraints
cind=1:nc;
ind=find((~isnan(b.inf) & ~isnan(b.sup)) & (b.inf>-INFTY | b.sup<INFTY))';
if argin.deb, gldisp('bounded_ind',ind); end
E=E(ind,:);b.inf=b.inf(ind);b.sup=b.sup(ind);cind=cind(ind);

% check empty constraints
ind=find(sum(E~=0,2)==0)';
if any(b.inf(ind)>0) || any(b.sup(ind)<0)
    % zero is not contained in the rhs -> problem is infeasible
    if prt,argout.out=[argout.out ; {'Constraints are inconsistent!' }];end
    return;
end

% remove empty constraints
ind=find(sum(E~=0,2)~=0)';
if argin.deb, gldisp('nonempty_ind',ind); end
E=E(ind,:);b.inf=b.inf(ind);b.sup=b.sup(ind);cind=cind(ind);

% extract bound constraints
ind=find(sum(E~=0,2)==1)';
for i=ind
    j=find(E(i,:)~=0);  % variable index
    % cut new variable bound with the previously found one
    setround(1);
    h=infsup(b.inf(i),b.sup(i))/E(i,j);
    x.inf(j)=max(x.inf(j),h.inf);x.sup(j)=min(x.sup(j),h.sup);
end

% check the extracted bound constraints
if any(x.inf>x.sup), 
    if prt,argout.out=[argout.out ; {'Constraints are inconsistent!' }];end
    return; 
end

% remove bound constraints
ind=find(sum(E~=0,2)~=1)';
if argin.deb, gldisp('notboundc_ind',ind); end
E=E(ind,:);b.inf=b.inf(ind);b.sup=b.sup(ind);cind=cind(ind);
nc=size(E,1);

% set argout
argout.x=x;argout.F=F; 
argout.E=E; argout.b=b; argout.cind=cind;

if argin.pro, 
    argout.cert.x=argout.x;
    argout.cert.F=argout.F;
end

% debug
if argin.deb, 
    gldisp('E',argout.E,'b',[argout.b.inf argout.b.sup],...
        'x',[argout.x.inf argout.x.sup]);
end;

% graphics
if argin.gra,
    glboxplot(argout.x);
    if nc>0 && nx==2,
        % plot the linear relaxations
        C.Q.A.inf=[argout.E zeros(nc,nx^2)];C.Q.A.sup=C.Q.A.inf;
        C.Q.F=argout.b;C.Q.int=zeros(1,nx);C.Q.nxo=2;
        glplot(C, [], 'ConColor','g');
        if argin.deb,
            % plot the remaining constraints
            C.Q.A.inf=A.inf(ind,:);C.Q.A.sup=A.sup(ind,:);
            C.Q.F.inf=F.inf(ind);C.Q.F.sup=F.sup(ind);
            C.Q.int=zeros(1,nx);C.Q.nxo=2;
            glplot(C, [], 'ConColor','r');
        end
    end
end