%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% glpropagate_separable.m %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glpropagate_separable(argin)
%
% Separates constraints and uses constraint propagation in order to obtain
% a tighter \x and \F.
% 
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
%   .int        nx x 1 vector, x_i integral if int(i)=1
%   .gaps       ng x 3 array with rows [k a b] specifying finite gaps
%               x_k notin ]a,b[
% x             bound constraints
%   .inf,.sup   outer enclosures
% bilappr       approximate the bilinear terms by 'c'onstant
%               'l'inear 'e'nvelope or 'q'uadratic terms or 'a'utomatically.
%
% * optional argin structure input fields *
% full          mode flag              0 = forward   1 = foward+backward
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure, if supplied, it will be checked
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%
% * argout structure fields *
% x             new bound constraints, empty if the problem is infeasible
%   .inf,.sup   outer enclosures
% F             new bounds of the general constraints
%   .inf,.sup   outer enclosures
% out           text output string cell
% cert          new certificate, with the same fields as in the input

function argout=glpropagate_separable(argin)

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'bilappr'}];

% optional input fields and their default values
oinp=[{'full'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  1   };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.x=[];argout.F=[];
if argin.prt,argout.out={'*** SEPARABLE CONSTRAINT PROPAGATION ***'};end
if argin.pro, 
    argout.cert.name='Separable Constraint Propagation';
    argout.cert.tname='SepPropagate';
    argout.cert.x=[];
    argout.cert.F=[];
end
step=0;
% rigorous quadratic enclosure for nonquadratic basis functions
argin.P.Q.Bu=gluniq(argin.P,argin.x);

% make separable structure
sep=glsepq(argin.P.Q.A, argin.P.Q.F, argin.x, argin.bilappr, argin.deb);
sep.int=argin.P.Q.int;sep.gaps=argin.P.Q.gaps;

while 1,
    % rigorous constraint propagation on separable quadratics
    [argout.x, argout.F, newgaps]=glqsred(sep, argin.P.Q.F, argin.full, argin.deb);

    % feasibility check
    if any(isempty(argout.x)),
        % problem infeasible
        if argin.prt,
            argout.out=[argout.out ; {'Constraints are inconsistent!' }];
        end
        return;
    elseif any(argout.x.inf>argout.x.sup)
        % wrong bounds
        gldisp('xol',[argin.x.inf argin.x.sup],'x',[argout.x.inf argout.x.sup]);
        error('glpropagate_separable: wrong bound constraints!');
    end;
    if sep.ns==0
        break;
    elseif step>0
        % remove slacks
        nx=length(argout.x.inf);
        nxo=nx-sep.ns;
        argout.x.inf=argout.x.inf(1:nxo);argout.x.sup=argout.x.sup(1:nxo);
        break;
    else
        sep.x=argout.x;
        argin.F=argout.F;
    end
    step=step+1;
end
% check whether trace points are in the box
%inbox=gltrace(argin.P.xref,x,'glqsred');

% graphics output
if argin.gra, glboxplot(argout.x); end
% set argout
if argin.pro, 
    argout.cert.x=argout.x;
    argout.cert.F=argout.F;
end

