%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqforw.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function q=glqforw(a, bp, bn x, prt, vind);
% find upper bounds u for q_i(z_i)=a_iz_i^2 + b_iz_i with z_i in x_i
%
% a,bp,bn          n x 1 vector of coefficients
% x.inf,.sup       nx dim box
% prt              output level 0=none 1=some 2=debug
% vind             n x 1 vector indicating variable dependecies 
%                  optional, used only for output
% 
% q                n x 1  upper bounds for q(x)
%
function q=glqforw(a, bp, bn, x, prt, vind)

%argcheck
if nargin <4, error('glqforw: Not enough arguments!');end
if nargin==4,prt=0;end
nx=size(x.inf,1);
if size(a,1)~=nx || size(bp,1)~=nx || size(bn,1)~=nx,
    gldisp('nx',nx,'a',a,'bn',bn,'bp',bp);
    error('glqforw: a, bp, bn, or x has wrong dimension!');
end
if nargin==5,vind=1:nx;end

%a, bp, bn, xb=[x.inf x.sup]
% rewrite as inequalities for nonnegative x
% by splitting after the xbox if necessary
indp=(x.sup>=0); %indexes of bound constraints having a positive part
indn=(x.inf<0); %indexes of bound constraints having a negative part
%rewrite inequ to have only positive xbox by splitting xbox
%and changing the sign of a,b, if neceseary
x.inf2=[max(x.inf(indp),0);max(-x.sup(indn),0)];
x.sup=[x.sup(indp);-x.inf(indn)];
x.inf=x.inf2;
a2=[a(indp);a(indn)];
b2=[bp(indp);bn(indn)];
np=sum(indp);nn=sum(indn);n=np+nn;
if prt,
    disp('Splitted variable bounds:');
    for i=1:length(x.inf),
        disp([num2str(x.inf(i)) ',' num2str(x.sup(i))]); 
    end
end

% indp, indn, n, a2, b2

% forward propagation
% find upper bounds u for az^2 + bz with z in x
setround(1); % round up;
u=max(x.inf.*(a2.*x.inf+b2),x.sup.*(a2.*x.sup+b2));
for k=1:n,
  if a2(k)~=0
    s=b2(k)/2;t=s/(-a2(k));
    if t>x.inf(k), 
      r=(-2*a2(k))*x.sup(k);
      if r>b2(k),
          u(k)=max(u(k),s*t);
      end;
    end;
  elseif b2(k)~=0
    u(k)=max(x.inf(k)*b2(k),x.sup(k)*b2(k));
  else
    u(k)=0;
  end
end;

% display results (upper bounds for a q(z))
if prt,
    disp('Upper bounds on the univariate expressions: (splitted)');
    gldispquad(a2,b2,u,[vind(indp);vind(indn)],[],1);
end

% find upper bounds q for q(z) by combining the split bounds
q=zeros(nx,1); % for zero z
q(indn)=-inf;q(indp)=u(1:np);
if np<length(u)
    q(indn)=max(q(indn),u(np+1:end));
end

% display results (combined upper bounds for a q(z))
if prt,
    disp('Combined upper bounds on the univariate expressions:');
    gldispquad(a,bn,q,vind,[],1);
end