%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glqsred.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,Fbox,reduc,newgaps,proof]=glqsred(sep,Fbox,mode,prt);
% rigorous constraint propagation on separable quadratics 
% (single sweep)
%
% reduces the box \x with a family of separable quadratic constraints 
% Nq(x)>=c, without losing any feasible point in the input box \x.
%
% sep              structure containing separable constraints 
%                  Nq(x)>=c,  q_l(x) = a_l x(k_l)^2 + b_l x(k_l),  
%                  b in [-bn,bp]  (the sign in bn saves time)
%   .il            lower bound not inf constraints indexes
%   .iu            upper bound not inf constraints indexes
%   .bil           constants calculated of bilinear terms which will be
%                  added to the right side constant of the split constraints
%   .N             (0,1) coefficient matrix 
%   .c             coefficient of constant term
%   .a             coefficient of quadratic terms
%   .bn,.bp        coefficients of the linear term (b in [-bn,bp])
%   .k             index list defining the variable in q_l(x)
%   .int           x_i integral if sep.int(i)=1
%   .gaps          ng x 3 array with rows [k a b] specifying finite gaps
%                  x_k notin ]a,b[
%   .x             bound constraints
%     .inf,.sup    
%   .ns            number of new variables (in case of envelope)
%   .nn            number of new constraints (in case of envelope)
% Fbox             bounds of general constraints
% mode             1 full(foward and backward) 0 only forward -1,only range
% proof            certificate of reduction (currently missing)
%
function [x,Fbox,newgaps,proof]=glqsred(sep,Fbox,mode,prt)

% args
if nargin<2
    error('glqsred: Not enough arguments!');
elseif nargin==2
    prt=0;
end;

proof=[];newgaps=[];
[nc,nq]=size(sep.N);
nco=nc-sep.nn;
x=sep.x;


% remove indirect addressing
% negative part is removed later
z.inf=x.inf(sep.k);
z.sup=x.sup(sep.k);
% forward propagation
% find upper bounds q for a z^2 + b z
q=glqforw(sep.a, sep.bp, sep.bn, z, prt, sep.k);
if any(isnan(q)), error('glqsred:Nan in forward step produced!');end
% check inconsistency
setround(1); % round up;
c=-inf+zeros(length(sep.k),1);
sumqu=zeros(nc,1);
% new summation technique is still missing
for i=1:nc,
    %%qu=sep.N(i,:)'.*q; % vector of the upper bounds for each term
    qind=find(sep.N(i,:)==1);qnz=length(qind);
    qu=sparse(qind,ones(1,qnz),q(qind),nq,1,qnz);
    if isempty(qu),
        % qu is empty check inconsistency
        if sep.c(i)>0 && mode ~=-1
            % constraint i is inconsistent
            x=[];newgaps=[];proof=[];return
        end
        continue; 
    end
    maxqu=max(qu);              % maximum of the upper bounds(can be inf)
    %minqu=min(qu);             % minimum of the upper bounds(>-inf)
    I=find(qu==maxqu);          % index set of maximas
    %J=find(qu==maxqu);         % index set of minimas
    nma=length(I);              % number of maximas
    %nmi=length(J);             % number of minimas
    K=ones(1,length(qu)); K(I)=0;% index set of all nonmax terms
    s=sum(qu(K==1));            % sum of the terms without max          
    sumqu(i)=s+maxqu*nma;       % sum(qu)
    r=sumqu(i)-sep.c(i); 
    if r<0 && mode ~=-1,        % inconsistency check if not range
        % constraint i is inconsistent
        %disp(['sum(q)-c<0 : ' num2str(s+maxqu) '-' num2str(sep.c(i)) '<0']);
        x=[];newgaps=[];proof=[];
        %disp(['glqsred: constraint ',num2str(i),' is inconsistent!']);
        return
    else
        % find rigorous lower bounds c on q
        % q_k(z_k) >= -(r_i-q_k) if N(i,k)=1, where r_i=N(i,:)*q-sep.c(i)
        if isinf(s)
            % sum is -inf-> -inf+inf would be undefined
            c(sep.N(i,:)~=0)=-inf;
        else
            for k=find(sep.N(i,:) ~= 0)
                if any(I==k), %index of maxqu reached
                    if nma ~=1,%more than one entries having value of maxqu
                        c(k)=-(s+maxqu*(nma-1)-sep.c(i));
                    else     %only one entry has the value of maxqu
                        c(k)=-(s-sep.c(i));
                    end
                else          %index of other than maxqu
                    c(k)=-(r-q(k));
                end
            end
        end
    end
end
% defined c=[-Q.F.inf(il);Q.F.sup(iu)] in sepq for il and iu
% indices, now get the new constraint bounds for the unsplitted
% constraints add the constants gained from the approximation of bilinear
% and univariate terms to sumq
sumqu=sumqu+sep.bil;
if prt, gldisp('Sum of the upper bounds for each constraint',sumqu); end
if any(isnan(sumqu)), error('glqsred:Nan in sumqu found!');end

fb=[Fbox.sup;-Fbox.inf];fb([sep.il;sep.iu])=sumqu(1:nco);
nc_old=length(Fbox.sup);
F.inf=-fb((nc_old+1):(2*nc_old),1);F.sup=fb(1:nc_old,1);
if any(F.sup<F.inf)
    gldisp('F',[F.inf F.sup],'Fwid',F.sup-F.inf);
    error('glqsred:The computed range has incorrect bounds!');
end

if mode==-1,
    % only the range is required, return
    Fbox=F;
    return;
end

% concatenate F with Fbox so get new constraint bounds
Fold=Fbox;
Fbox.inf=max(Fbox.inf,F.inf);Fbox.sup=min(Fbox.sup,F.sup);
if any(Fbox.sup<Fbox.inf)
    format short e
    gldisp('Fold',[Fold.inf Fold.sup],'Fran',[F.inf F.sup]',...
        'F',[Fbox.inf Fbox.sup],'Fdiff',Fbox.sup-Fbox.inf);
    error('F bounds are incorrect!');
end

if prt,
    disp('New bounds for the general constraints:'); 
    gldisp('FB',[Fbox.inf Fbox.sup]);
end;

if mode==0
    %only forward propagation required
    return;
end

% backward propagation
% display results (lower bounds on q)
if prt,
    disp('Backward propagate:');
    gldispquad(sep.a,sep.bp,c,sep.k);
end

% improve box bounds
% [z1,z2]=glqback(a,b,c);
% rigorous enclosure of all z>=0 with a z^2 + 2b z >=c (componentwise)
% into a union of two intervals \z1 and \z2 
% usually, \z2 is empty (=[inf,inf]) for both positive and negative parts
[zp1,zp2]=glqback(sep.a,sep.bp/2,c);
[zn1,zn2]=glqback(sep.a,sep.bn/2,c);

% merge positive and negative parts
z1.inf=[zp1.inf;zn1.inf];z1.sup=[zp1.sup;zn1.sup];
z2.inf=[zp2.inf;zn2.inf];z2.sup=[zp2.sup;zn2.sup];
kk=[sep.k;sep.k];

% display results
if prt,
    disp('Results of the backward propagation:');
    str=glbox2str(z1);for i=1:length(str), disp(str{i}); end
    if any(~isinf(z2.inf)) || any(~isinf(z2.sup))
        disp('    Second part:');
        zz2=z2;
        zz2.inf(isinf(zz2.inf))=-inf; % empty entries, set them to -inf inf
        str=glbox2str(zz2);for i=1:length(str), disp(str{i}); end
    else disp('   No second part found!');
    end
    gldisp('partind',kk');
end

% positive part of x, if not exists xp.sup(i)<0
xp.inf=max(x.inf,0);xp.sup=x.sup;
% negative part of x, if not exists xn.sup(i)<0
xn.inf=max(-x.sup,0);xn.sup=-x.inf;

% concatenate z with xp for the first (positive) part of z1 and z2
% and with xn for the second (negative) part of z1 and z2
% new gaps are stored in ngaps
ngaps=[];
for i=1:length(z1.inf)
    j=kk(i);
    if i<=length(kk)/2  
        % positive part
        if isinf(z2.inf(i))
            xp.inf(j)=max(xp.inf(j),z1.inf(i));
            xp.sup(j)=min(xp.sup(j),z1.sup(i));
        else
            ngaps=[ngaps; [kk(i) z1.sup(i) z2.inf(i)]];
        end
    else
        % negative part
        if isinf(z2.inf(i))
            xn.inf(j)=max(xn.inf(j),z1.inf(i));
            xn.sup(j)=min(xn.sup(j),z1.sup(i));
        else
            ngaps=[ngaps; [kk(i)  -z2.inf(i) -z1.sup(i)]];
        end
    end
end

% display results
if prt,
    disp('Negative part:');
    str=glbox2str(xn);for i=1:length(str), disp(str{i}); end
    disp('Positive part:');
    str=glbox2str(xp);for i=1:length(str), disp(str{i}); end
    gldisp('gaps',ngaps);
end

% build x from xp and xn
% new gaps are added to ngaps
out=[];
for i=1:length(x.inf)
    if xn.sup(i)>=0 && xp.sup(i)>=0 % both parts exists
        if xn.inf(i)>xn.sup(i) || isinf(xn.inf(i)), %inconsistent negative part
            out=[out ' incons_neg '];
            x.inf(i)=xp.inf(i);
            x.sup(i)=xp.sup(i);
        elseif xp.inf(i)>xp.sup(i) || isinf(xp.inf(i)), %inconsistent positive part
            out=[out ' incons_pos '];
            x.inf(i)=-xn.sup(i);
            x.sup(i)=-xn.inf(i);
        else % both parts consistent
            out=[out ' both_cons '];
            x.inf(i)=-xn.sup(i);
            x.sup(i)=xp.sup(i);
            if xn.inf(i) > 0 || xp.inf(i) > 0, %additional gap exists
                ngaps=[ngaps; [i  -xn.inf(i) xp.inf(i)]];
            end
        end
    elseif xn.sup(i)<0 %only positive part exits
        out=[out ' on_pos_part '];
        x.inf(i)=xp.inf(i);
        x.sup(i)=xp.sup(i);
    elseif xp.sup(i)<0 %only negative part exits
        out=[out ' on_neg_part '];
        x.inf(i)=-xn.sup(i);
        x.sup(i)=-xn.inf(i);
    end
end

% display results
if prt, disp(['Part mering:' out]); end

% check consistency
if any(x.inf>x.sup)
    % constraints are inconsistent
    x=[];newgaps=[];proof=[];
    return
end

%process gaps, if a gap contains one of the boundary of the
%corresponding /x_k, we reduce the box and discard the gap else
%it is stored in newgaps excluding the gaps totally outside of
%/x_k
for i=1:size(ngaps,1)
    k=ngaps(i,1);%variable index
    l=ngaps(i,2);%lower gap bound
    u=ngaps(i,3);%upper gap bound
    if x.inf(k)>l && x.inf(k)<=u %lower box boundary is contained in gap
        if x.sup(k)<u  %upper bound is also contained in gap
            if prt,
                disp(['Gap reduced bound constraint /x_' num2str(k) ...
                    ' to empty']);
                gldisp('x_k',[x.inf(k),x.sup(k)],'gaps_i',ngaps(i,:));
            end
            x=[];
            return
        end
        if prt,
            disp(['x_' num2str(k) ' lower: ' num2str(x.inf(k)) ' -> '...
                num2str(u)]);
        end
        x.inf(k)=u;
    elseif x.sup(k)<u && x.sup(k)>l %upper box boundary is contained in gap
        %no check of the lower needed, this was contained in the first case
        if prt,
            disp(['x_' num2str(k) ' upper: ' num2str(x.sup(k)) ' -> '...
                num2str(l)]);
        end
        x.sup(k)=l;
    elseif l>=x.inf(k) && u<=x.sup(k)  %gap is inside the box, add to gaps
        newgaps=[newgaps; ngaps(i,:)];
    end
end

% display results
if prt,
    disp('Final results:')
    str=glbox2str(x);for i=1:length(str), disp(str{i}); end
    gldisp('newgaps',newgaps,'F',[Fbox.inf Fbox.sup]);
end