%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glred.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [ran, F, x]=glred(A, x, F, prt)
% 
% FS some or all of the following
% (1) the range of Aq(x) where x \in \x, A \in \A
% (2) a new bound F' for Aq(x) \in F with x \in \x, A \in \A
% (3) new bound constrains x'
%
% A           nc x nx^2+nx constraint coeff matrix
%   .inf,.sup outer enclosures
% F           nc x 1 general constraint bounds
%   .inf,.sup outer enclosures
% x           nx x 1 bound constraints
%   .inf,.sup outer enclosures
% 
% ran         nc x 1 range of Aq(x)
%   .inf,.sup outer enclosures
% F           nc x 1 new general constraint bounds (or empty if infeasible)
%   .inf,.sup outer enclosures
% x           nx x 1 new bound constraints (or empty if infeasible)
%   .inf,.sup outer enclosures

function [ran, F, x]=glred(A, x, F, prt)

% argcheck
if nargin<2 || (nargin<3 && nargout>1)
    error('glred: Not enough arguments!');
end
if nargin<4, prt=0;end

setround(1)             % round up
[nc, nA]=size(A.inf);   % number of constraints
nx = length(x.inf);
% compute q(\x)
xi=infsup(x.inf,x.sup);
if prt, gldisp('x',[x.inf x.sup]); end
qi=xi*xi';qi=qi(:);qi(1:nx+1:end)=xi.^2;
qi=[xi;qi];
Qi=intval(zeros(nc,nA));
for i=1:nc,Qi(i,:)=qi'; end
if prt, gldisp('qx',[qi.inf qi.sup]); end

% compute \A(i,:).*q(\x), i=1,..,nc
S=A.inf~=0 | A.sup~=0;
Ai=infsup(A.inf,A.sup);Aqi=Ai;
Aqi(~S)=0;
Aqi(S)=Aqi(S).*Qi(S);
if prt, gldisp('Aqi_inf',Aqi.inf,'Aqi_sup',Aqi.sup); end

% compute the range of Aq(x) for x \in \x, A \in \A
ran.inf=-sum(-Aqi.inf,2);ran.sup=sum(Aqi.sup,2);
if prt, gldisp('range',[ran.inf ran.sup]); end

if nargout>1, 
    % compute new F
    F.inf=max(ran.inf,F.inf);F.sup=min(ran.sup,F.sup);
    if any(F.inf>F.sup)
        F=[];
        if prt, disp('The problem has been found infeasible!');end
        return;
    elseif prt, gldisp('F_new',[F.inf F.sup]); 
    end
end

if nargout>2
    % compute qsum with q_i(x_k) \in qsum_ik for i=1:nc, k=1:x
    for j=1:nA,
        ind=find(S(:,j));
        if ~isempty(ind)
            Aqiu=Aqi.sup;Aqiu(:,j)=0;Aqil=-Aqi.inf;Aqil(:,j)=0;
            b.sup=F.sup(ind)+sum(Aqil(ind,:),2);
            b.inf=-(-F.inf(ind)+sum(Aqiu(ind,:),2));
            bi=infsup(b.inf,b.sup);
            [i,k]=glmmod(j,nx);
            a=Ai(ind,j);
            if i==0,
                % linear term, x_k=intersect(b/A(:,j))
                bin=bi./a;
                xn.inf=max(bin.inf);xn.sup=min(bin.sup);
                if prt,
                    disp(['linear terms: ' mat2str(mid(a)) 'x_' ...
                        num2str(k) ' \in ' mat2str([bi.inf bi.sup])...
                        '->x_' num2str(k) ' \in '  mat2str([xn.inf xn.sup])]);
                end
                x.inf(k)=max(x.inf(k),xn.inf);
                x.sup(k)=min(x.sup(k),xn.sup);
            elseif i==k
                % quadratic term x_k=intersect(sqrt(b/A(:,j)))
                bia=bi./a;
                ind2=find(bia.sup>=0);
                if ~isempty(ind2)
                    sq=sqrt(bia.sup(ind2));
                    bin=infsup(-sq,sq);
                    xn.inf=max(bin.inf);xn.sup=min(bin.sup);
                    if prt,
                        s=['x_' num2str(k) ' \in ' ...
                            mat2str([xn.inf xn.sup])];
                    end
                elseif prt, s='no valid bound';
                end
                if prt,
                    disp(['quadratic terms: ' mat2str(mid(a)) 'x_' ...
                        num2str(k) '^2 \in ' mat2str([bi.inf bi.sup])...
                        '->' s]);
                end
                x.inf(i)=max(x.inf(i),xn.inf);
                x.sup(i)=min(x.sup(i),xn.sup);
            else
                % bilinear term x_k=intersect(b/(A(:,j)*\x_i))) and 
                % x_i=intersect(b/(A(:,j)*\x_k))) and 
                s='';
                axi=a*xi(i);ind2=find(axi.sup<0 | axi.inf>0);
                if ~isempty(ind2)
                    bik=bi(ind2)./axi(ind2);
                    xnk.inf=max(bik.inf);xnk.sup=min(bik.sup);
                    x.inf(k)=max(x.inf(k),xnk.inf);
                    x.sup(k)=min(x.sup(k),xnk.sup);
                    if prt,
                        s=['x_' num2str(k) ' \in ' ...
                            mat2str([xnk.inf xnk.sup])];
                    end
                end
                axk=a*xi(k);ind2=find(axk.sup<0 | axk.inf>0);
                if ~isempty(ind2)
                    bii=bi(ind2)./axk(ind2);
                    xni.inf=max(bii.inf);xni.sup=min(bii.sup);
                    x.inf(i)=max(x.inf(i),xni.inf);
                    x.sup(i)=min(x.sup(i),xni.sup);
                    if prt,
                        if ~isempty(s), s=[s ' and '];end
                        s=[s 'x_' num2str(i) ' \in ' ...
                            mat2str([xni.inf xni.sup])];
                    end
                end
                if prt,
                    if isempty(s), s='no vaild bounds';end
                    disp(['bilinear terms: ' mat2str(mid(a)) 'x_' ...
                        num2str(i) 'x_' num2str(k) ' \in ' ...
                        mat2str([bi.inf bi.sup]) '->' s]);
                end
            end
            if any(x.inf>x.sup)
                x=[];
                if prt, disp('The problem has been found infeasible!');end
                return;
            end
        end
    end
    if prt, gldisp('xnew',[x.inf x.sup]); end
end
