%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsepq.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [sep, A, Bn, Bp, c]=glsepq(A, F, x, bilappr, prt)
% creates a rigorous separable relaxation 
%    Bq(x)>=c,    q_l(x) = a_l x(k_l)^2 + b_l x(k_l)
% of a constraint satisfaction problem (coded in P)
%         AB(x) in \F, 
%         x_int integral,
%         x_{k_l} notin ]a_l,b_l[ (l=1:ng),
% restricted to x in \x (if nargin=2).
%
% Nononquadratic terms are treated only crudely, by simple forward interval
% evaluation of each term. 
% 
% A            sparse matrix of constraint coefficients
%    .inf,.sup bounds for coefficients
%    .nz       incidence matrix
% F            structure with bounds for general constraints
%       .inf,.sup   outer enclosure
% x.inf,x.sup  bound constraints for x in R^n (default: no bounds)
% bilappr      approximate the bilinear terms by 'c'onstant
%              'l'inear or 'q'uadratic terms, by adding the 'e'nvelope 
%              or 'a'utomatic.
% prt          output level
%
% sep          structure containing separable constraints 
%              Nq(x)>=c,  q_l(x) = a_l x_(k_l)^2 + b_l x_(k_l),  
%              b in [-bn,bp]  (the sign in bn saves time)
%   .il        finite lower bound indices of the original constraint
%   .iu        finite upper bound indices of the original constraint
%   .bil       constants calculated of bilinear terms which will be added 
%              to the right side constant of the split constraints
%   .N         (0,1) coefficient matrix 
%   .c         constant terms
%   .a         coefficients of the quadratic terms
%   .bn,.bp    coefficients of the linear terms
%   .k         index list defining the variable x_(k_l) in q_l(x)
%   .ns        number of new variables (~= 0 in case of envelope)
%   .nn        number of new constraints (~=0 in case of envelope)
% The sparse separable interval constraints
%     A*x.^2 + [-Bn,Bp]*x  >= c
% by fetching A, Bn, Bp, c.

function [sep, A, Bn, Bp, c]=glsepq(A, F, x, bilappr, prt)
global glconf;      %gloptlab configuration file
INF=1e5;            %positive and negative infinity

% argcheck
if nargin<2, error('glsepq: Not enough arguments!'); end
[nc,nB]=size(A.inf);nx=-0.5+sqrt(1+4*nB)/2;
if nargin<3, x.sup=inf+zeros(nx,1);x.inf=-x.sup; end
if nargin<4, bilappr='a'; end
if nargin<5, prt=0; end;

% set configuration
if isfield(glconf,'infty'),INF=glconf.infty;end 
% init
bilappr=bilappr(1);
if nx~=floor(nx), 
    error(['glsepq: A has invalid dimension :' num2str(nx) '?!']); 
end
if nc==0, error('glsepq: No constraints found!'); end

setround(1); % round up;

% resolve two-sided constraints into inequalities
% first all lower bounds, then all upper bounds
il=(~isinf(F.inf));iu=(~isinf(F.sup)); %pos and negative indexes
nl=nnz(il);nu=nnz(iu);              %nl+nu max 2*nc (all two sided)
nc2=nl+nu;                          %number of the new inequ constraints
An=[-A.inf(il,:);A.sup(iu,:)];  %lower part
Ap=[A.sup(il,:);-A.inf(iu,:)];  %upper part
c=[-F.inf(il);F.sup(iu)];       %left sided constant
% now [-An;Ap]q(x) - c >= 0, to be adapted at the end
if prt,
    P.Q.A.inf=-An;P.Q.A.sup=Ap;
    P.Q.F.inf=-c;P.Q.F.sup=inf+ones(nc2,1);
    P.basis=[];P.uni=[];P.opt=0;
    P.Q.int=zeros(nx,1);P.Q.Bu=[];
    P.Q=glfindspars(P.Q);P.Q.gaps=[];
    P.uni=[];P.basis=[];P.shift=[];
    str=glprob2str(P,x);
    disp('Two sided constraints resolved, new problem:');
    for i=1:length(str), disp(str{i}); end
end

% constants calculated of bilinear (bil) or quadratic (quad) terms which
% will be added to the right side
bil=zeros(nc2,1);
indb=1:(nB-nx);indb(1:nx+1:end)=[];indb=indb+nx;
% midpoint of x
z=glmid(x);
% bound bilinear entries 
ns=0; % counter for the new variables
nn=0; % counter for the new inequs
nsubst=cell(0);% cell of (i, vnum, bjkn, bjkp) for variables sustitution
nineq=cell(0);% cell of (j, k, mode) for the new inequs
for i=1:nc2
    if prt, disp(['Inequality (' num2str(i) '):']); end
    % infinite bound constr box indexes
    infind=find(x.sup>INF | x.inf<-INF); 
    for h=find(An(i,indb)~=0 | Ap(i,indb) ~=0)
        t=indb(h);
        [j,k]=glmmod(t,nx); % indices of the bilinear term x_jx_k
        bjkp=Ap(i,t);bjkn=An(i,t); % bjk \in [-bjkn,bjkp]
        % We have the inequality
        % Ap(i,j*(nx+1))x_k^2+bjk x_jx_k+Ap(i,k*(nx+1)) x_k^2 -c(i)>=0
        % with bik \in [-bjkn, bjkp]
        if bilappr=='a',
            % automatic method selection, find out the best method
            if (any(j==infind) || any(k==infind)), msel='q';
            else msel='c';
            end
        else
            msel=bilappr;
        end
        switch msel,
            case 'e'
                % approximation mode 1=upper,2=lower,3=both
                if bjkn<=0, mode=1;
                elseif bjkp<=0, mode=2;
                else mode=3;
                end
                % find out if the bilinear term x_jx_k is already marked
                % for approximation or not.
                vnum=0;
                for n=1:length(nineq)
                    nin=nineq{n};
                    if nin(1)==j && nin(2)==k,
                        % the bilinear term was already marked, change
                        % the mode if necessarry
                        omode=nin(3);
                        if omode~=3 && mode~=omode,
                            % change the mode to 'b'oth
                            nin(3)=3;nineq{n}=nin;
                            nn=nn+2;
                        end
                        vnum=n;
                        break;
                    end
                end
                if vnum==0,
                    % introduce new variable
                    ns=ns+1;
                    vnum=ns;
                    nin=[j,k,mode];
                    nineq=[nineq; {nin}];
                    if mode~=3, nn=nn+2;
                    else  nn=nn+4;
                    end
                    if prt,
                    disp(['  (ENV): Bilinear entry '...
                        'x_' num2str(j) 'x_' num2str(k) ...
                        ' substituted by the new variable x_'...
                        num2str(nx+vnum) '!']);
                    end
                elseif prt,
                    disp(['  (ENV): Bilinear entry '...
                        'x_' num2str(j) 'x_' num2str(k) ...
                        ' substituted by the existing variable x_'...
                        num2str(nx+vnum) '!']);
                end
                nsubst=[nsubst; {[i,vnum,bjkn,bjkp]}];
                
            case 'c'
                %bilinear term vars are bounded use interval mult with
                %directed rounding
                if bjkp>bjkn
                    b=bjkp*max([x.inf(j)*x.inf(k) ...
                        x.inf(j)*x.sup(k) x.sup(j)*x.inf(k)...
                        x.sup(j)*x.sup(k)]);
                else
                    b=bjkn*max([(-x.inf(j))*x.inf(k)...
                        (-x.inf(j))*x.sup(k) (-x.sup(j))*x.inf(k)...
                        (-x.sup(j))*x.sup(k)]);
                end
                if prt,
                    disp(['  (APPR): Bilinear entry '...
                        'b_' num2str(j) num2str(k) '=['...
                        num2str(-bjkn) ',' num2str(bjkp)...
                        '] approximated: c=' num2str(-c(i)) '-' ...
                        num2str(b) '=' num2str(-c(i)-b) '!']);
                end
                bil(i)=bil(i)+b;c(i)=c(i)+b;
            case 'l'
                % linearize bilinear terms
                bb=infsup(-bjkn,bjkp);
                u=bb*z(j);v=bb*z(k);
                % set linear term
                lj=infsup(-An(i,j),Ap(i,j))+v;
                Ap(i,j)=lj.sup;An(i,j)=-lj.inf;
                lk=infsup(-An(i,k),Ap(i,k))+u;
                Ap(i,k)=lk.sup;An(i,k)=-lk.inf;
                % set constant term
                u=zeros(1,4);
                u(1)=sup(((x.inf(j)-z(j))*x.inf(k)-z(k)*x.inf(j))*bb);
                u(2)=sup(((x.inf(j)-z(j))*x.sup(k)-z(k)*x.inf(j))*bb);
                u(3)=sup(((x.sup(j)-z(j))*x.inf(k)-z(k)*x.sup(j))*bb);
                u(4)=sup(((x.sup(j)-z(j))*x.sup(k)-z(k)*x.sup(j))*bb);
                u(isnan(u))=inf;
                b=max(u);
                bil(i)=bil(i)+b;c(i)=c(i)+b;
            case 'q'
                % some of the bound on x_j,x_k are infinite,
                % approximate the bilinear terms by quadratic ones
                aj=Ap(i,j*(nx+1));ak=Ap(i,k*(nx+1));
                % now we use ajx_k^2+bjkp x_jx_k+akx_k^2-c(i)>=0 (c1)
                if  (aj< 0 && ak<0)
                    % The quadratic coefficients are negative
                    % (This could be replaced by a better criteria)
                    % e.g. one sided and one of them less than zero etc.
                    % approximate by adding
                    % djk(x_j-betajk x_k)^2>0 with
                    % betajk=sign(bjkp)sqrt(aj/ak), djk=bjkp/(2betajk)
                    % to the left hand side of the inequality constraint
                    % (c1), obtaining aj=aj+djk, ak=ak+(bjkp*betajk)/2.
                    % ***************CHECK ROUNDING!*****************
                    if bjkp<=0, sig=-1;
                    elseif bjkn>=0, sig=1;
                    else sig=infsup(-1,1);
                    end
                    bi=infsup(-bjkn,bjkp);
                    betajk=sqrt(aj/ak);
                    djk=sig*bi/(2*betajk);

                    % new quadratic coeff for x_j
                    aj2=infsup(-An(i,j*(nx+1)),Ap(i,j*(nx+1)))+djk;
                    Ap(i,j*(nx+1))=aj2.sup;
                    An(i,j*(nx+1))=-aj2.inf;
                    bbeta=sig*(bi*betajk)/2;
                    % new quadratic coeff for x_k
                    ak2=infsup(-An(i,k*(nx+1)),Ap(i,k*(nx+1)))+bbeta;
                    Ap(i,k*(nx+1))=ak2.sup;
                    An(i,k*(nx+1))=-ak2.inf;
                    if prt,
                        disp(['  (ELL_1): Bilinear entry '...
                            'b_' num2str(j) num2str(k) '=['...
                            num2str(-bjkn) ',' num2str(bjkp) ']'...
                            ', elliminated. Quaratic coeffs changed: '...
                            'a_' num2str(j) '=[' ...
                            num2str(aj2.inf) ',' num2str(aj2.sup) '], '...
                            'a_' num2str(k) '=[' ...
                            num2str(ak2.inf) ',' num2str(ak2.sup) '].']);
                    end
                else
                    % One quadratic coefficient is non negative,
                    % use the old method: approximate by adding
                    % |bjkp/2|(x_j-sgn(bjkp)x_k)^2>0 to the
                    % left hand side of the inequality constraint (c1),
                    % obtaining aj=aj+|bjkp/2|, ak=ak+|bjkp/2|
                    m=abs(bjkp)/2;
                    Ap(i,j*(nx+1))=aj+m;
                    Ap(i,k*(nx+1))=ak+m;
                    if prt,
                        disp(['  (ELL_2): Bilinear entry '...
                            'b_' num2str(j) num2str(k) '=' num2str(bjkp)...
                            ', elliminated: a_' num2str(j) '=' ...
                            num2str(aj) '+' num2str(m) '=' ...
                            num2str(Ap(i,j*(nx+1))) ', '...
                            'a_' num2str(k) '=' num2str(ak) '+'...
                            num2str(m) '=' num2str(Ap(i,k*(nx+1))) '!']);
                    end
                end
            otherwise
                error(['glsepq: Unknown bilinear approximation mode "'...
                    msel '" found!']);
        end
    end
end

% split in quadratic (A) and linear part (Bn,Bp)
A=Ap(:,nx+1:nx+1:nB);
Bn=An(:,1:nx);
Bp=Ap(:,1:nx);
c=-c;                       % bring c to the left side

% add new inequs
if(ns>0)
    % substtute the new variables
    Bn2=sparse([],[],[],nc2,ns);
    Bp2=sparse([],[],[],nc2,ns);
    for n=1:length(nsubst)
        b=nsubst{n};
        i=b(1);vnum=b(2);
        Bn2(i,vnum)=b(3); Bp2(i,vnum)=b(4);
    end
    % create new constraints
    Bp3=sparse([],[],[],nn,nx+ns);
    c2=zeros(ns*4,1);
    ec=1;
    for n=1:ns
        b=nineq{n};
        j=b(1);k=b(2);mode=b(3);
        if mode~=1
            % lower bounding linear inequs:
            % y_n >= \ol x_jx_k +\ol x_jx_k -\ol x_j \ol x_k, trasformed to
            % --> y_n - \ol x_jx_k - \ol x_jx_k >= -\ol x_j \ol x_k.
            if ~isinf(x.inf(j)) && ~isinf(x.inf(k))
                Bp3(ec,nx+n)=1;
                Bp3(ec,k)=-x.inf(j);Bp3(ec,j)=-x.inf(k);
                c2(ec)=-x.inf(j)*x.inf(k);
                ec=ec+1;
            end
            % y_n >= \ul x_jx_k +\ul x_jx_k -\ul x_j \ul x_k, trasformed to
            % --> y_n - \ul x_jx_k - \ul x_jx_k >= -\ul x_j \ul x_k.
            if ~isinf(x.sup(j)) && ~isinf(x.sup(k))
                Bp3(ec,nx+n)=1;
                Bp3(ec,k)=-x.sup(j);Bp3(ec,j)=-x.sup(k);
                c2(ec)=-x.sup(j)*x.sup(k);
                ec=ec+1;
            end
        end
        if mode~=2
            % upper bounding linear inequs
            % y_n <= \ul x_jx_k +\ol x_jx_k -\ul x_j \ol x_k, trasformed to
            % --> -y_n + \ul x_jx_k + \ol x_jx_k >= \ul x_j \ol x_k.
            if ~isinf(x.inf(j)) && ~isinf(x.sup(k))
                Bp3(ec,nx+n)=-1;
                Bp3(ec,k)=x.inf(j);Bp3(ec,j)=x.sup(k);
                c2(ec)=x.inf(j)*x.sup(k);
                ec=ec+1;
            end
            % y_n <= \ol x_jx_k +\ul x_jx_k -\ol x_j \ul x_k, trasformed to
            % --> -y_n + \ol x_jx_k + \ul x_jx_k >= \ol x_j \ul x_k.
            if ~isinf(x.sup(j)) && ~isinf(x.inf(k))
                Bp3(ec,nx+n)=-1;
                Bp3(ec,k)=x.sup(j);Bp3(ec,j)=x.inf(k);
                c2(ec)=x.sup(j)*x.inf(k);
                ec=ec+1;
            end
        end
    end
    nn=ec-1; % revised number of additional constraints 
    Bp3=Bp3(1:nn,:);
    c2=c2(1:nn);
    Bn3=-Bp3;
    % increase the number of variables by dimension by ns
    Bn=[Bn Bn2;Bn3];
    Bp=[Bp Bp2;Bp3];
    c=[c;c2];
    A=[A sparse([],[],[],nc2,ns);sparse([],[],[],nn,nx+ns)];
    x.inf=[x.inf;-inf+zeros(ns,1)];
    x.sup=[x.sup;inf+zeros(ns,1)];
    bil=[bil;zeros(nn,1)];
    %iu=[iu;ones(nn,1)];
    if prt,
        disp([num2str(ns) ' new variables and ' num2str(nn) ...
            ' new constraints added.']); 
    end
end
% now we have sparse separable interval constraints
%     A*x.^2 + [-Bn,Bp]*x  >= c
% translate into standard separable format
%     Nq(x)>=c,  q_l(x) = a_l x(k_l)^2 + b_l x(k_l), b in [-bn,bp]
[ii,kk]=find(Bn~=0|Bp~=0|A~=0);
nq=length(ii);
a=zeros(nq,1);bp=zeros(nq,1);bn=zeros(nq,1);sep.k=zeros(nq,1);
neq=size(c,1);N=sparse([],[],[],neq,nq,nq);
for l=1:nq,
  i=ii(l);k=kk(l);
  N(i,l)=1;
  a(l)=A(i,k);
  bn(l)=Bn(i,k);bp(l)=Bp(i,k);
  sep.k(l)=k;
end
% fill separable structure
sep.il=il;sep.iu=iu;sep.bil=bil;
sep.N=N;sep.c=c;sep.a=a;sep.bn=bn;sep.bp=bp;
sep.x=x;
sep.ns=ns;
sep.nn=nn;
% display polynomials
if prt,
    disp('Constraints:');
    gldispquad(a,infsup(-bn,bp),c,sep.k,N)
end