%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glreadprob.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glreadprob(argin)
%
% Read the problem definition from the given file.
%
% * needed argin structure input fields *
% P             problem file name of the .def file, with full path
%               or a problem definition
%
% * optional argin structure input fields *
% box           read saved bound constraints box
% stra          read saved linked strategy
% cprt          clear diary before reading
% cdeb          clear debug window before reading
% accnq         accept problems with nonquadratic functions
% maxvars       accept problems with maximum so mutch vars
% inftya        values >infty (< -infty) are infinite (a*nx+b)
% inftyb        infty = a*nx+b (nx = number of vars)
% tiny          values in [-tiny tiny] are treated as zero
% feast         feasiblity tolerance, (locsol,split,conic)
% sprec         required precision of the external solvers
% minvar        variation constant var>minvar quadr else smear
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .P          new problem definition
%
% * argout structure fields *
% P             structure containing the internal problem definition,
%               (see Source/Help/Problem Definition.txt) or empty. 
% x             bound constraints
% status        =5 if any problems encountered while reading the problem
% out           text output

function argout=glreadprob(argin)

global gloutput;    % global output structure
global glconf;      % gloptlab configuration file

thisfun=mfilename;

% needed input fields & determine the function name
ninp={'P'};

% optional input fields and their default values
oinp=[{'box'};{'stra'};{'cprt'};{'cdeb'};{'accnq'};{'maxvars'};{'inftya'}];
dval=[{  1  };{  1   };{   1  };{   1  };{  0    };{    inf  };{   0    }];
oinp=[oinp;{'inftyb'};{'tiny'};{'feast'};{'sprec'};{'minvar'};{'prt'}];
dval=[dval;{   1e3  };{ 1e-6 };{  1e-7 };{ 1e-12 };{ 1e-3   };{ 0   }];
oinp=[oinp;{'deb'};{'gra'};{'cert'}];
dval=[dval;{ 0   };{  0  };{  []  }];

% argin check
if  nargin==0,
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1,
    error([thisfun ': Wrong number of input args!']);
end

% check the needed input arguments
for i=1:length(ninp)
    if ~isfield(argin,ninp{i})
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i})
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.x=[];argout.P=[];argout.status=0;
if argin.prt, argout.out={'*** READ PROBLEM ***'}; end

% certificate
if argin.pro,
    if isempty(argin.cert)
        argout.cert.name='Read Problem';
        argout.cert.tname='Read';
        argout.cert.x=[];
        argout.cert.P=[];
    else argout.cert=argin.cert;
    end
end

% get the problem info
[info, errmsg] = glreadinfo(argin.P);
if isempty(errmsg)
    nx=info.nx;nc=info.nc;no=info.no;nu=info.nu;
    if ~argin.accnq && nu>0
        % problem with nonquadratic should not be accepted
        argout.P.solstatus=[];argout.P=glsolstatus(argout.P,'NONQUAD',1);
        argout.P.name=info.name;argout.P.Q.nxo=no;argout.P.Q.int=zeros(1,nx);
        argout.P.uni=zeros(1,nu);
    elseif nx>argin.maxvars
        % problem dimension to high
        argout.P.solstatus=[];argout.P=glsolstatus(argout.P,'DIMHIGH',1);
        argout.P.name=info.name;argout.P.Q.nxo=no;argout.P.Q.int=zeros(1,nx);
        argout.P.uni=zeros(1,nu);
    elseif isstruct(argin.P)
        % problem definition already there
        argout.P=argin.P;
        if argin.prt
            argout.out=[argout.out ; {char(zeros(1,75)+45)}; ...
                {['Problem ' argin.P.name ' initialized!']}];
        end
    else
        % read the problem
        if argin.prt,
            argout.out=[argout.out ; {char(zeros(1,75)+45)}; ...
                {['Reading the problem ' argin.P '...']}];
        end
        % read problem
        [argout.P,errmsg]=glread(argin.P, argin.box);
        if isempty(errmsg)
            % text output
            if argin.prt
                % full output
                argout.out=[argout.out ; {['Problem file ' ...
                    argin.P ' read!']}];
            end
        end
    end
    % set the constants in the config
    glconf.infty=argin.inftya*nx+argin.inftyb;
    glconf.tiny =argin.tiny;
    glconf.feast=argin.feast;
    glconf.sprec=argin.sprec;
    % EXTRACTING PROBLEM
    % if isfield(argout.P.Q,'A'),
    %    glextracttest('clear');
    %    glextracttest(argout.P);
    % end
end
if ~isempty(errmsg),
    % error reading the info or the problem.
    msg=['*** ERROR: ' errmsg ];
    argout.out=[argout.out ; {msg}];
    argout.P.solstatus=[];[d,argout.P.name]=fileparts(argin.P);
    argout.P.Q.nxo=-1;
    argout.P=glsolstatus(argout.P,'READERR',1);
elseif ~glsolstatus(argout.P.solstatus,'NONQUAD') && ...
        ~glsolstatus(argout.P.solstatus,'DIMHIGH')
    argout.x=argout.P.xbox;argout.P.startxbox=argout.x;
    % text output
    if argin.prt
        argout.out=[argout.out ; ...
            {char(zeros(1,75)+45)};glprob2str(argout.P)];
        % brief output
        gloutput.brief=[gloutput.brief;...
            {char(zeros(1,length(argout.P.name)+7)+45)};...
            {['-->' argout.P.name '<--']};...
            {char(zeros(1,length(argout.P.name)+7)+45)};...
            {['nx=' num2str(nx) ', nxo=' num2str(argout.P.Q.nxo)...
            ', nc=' num2str(nc)]}];
    end
    % debug
    if argin.deb,
        gldisp('Ainf',argout.P.Q.A.inf,'Asup',argout.P.Q.A.sup,'F',...
            [argout.P.Q.F.inf argout.P.Q.F.sup],'x',...
            [argout.x.inf argout.x.sup]);
    end
    % graphics output
    if argin.gra,
        if isfield(gloutput, 'graphics') && ...
                isfield(gloutput.graphics,'ahand') && nx==2
            % clear the plot
            axes(gloutput.graphics.ahand(1));cla;
        end
        glplot(argout.P, argout.x);
        drawnow;
    end
    % load strategy defined for the loaded problem
    if argin.stra && isfield(argout.P,'stratname') && ~isempty(argout.P.stratname),
        sfile=[glconf.stratdir '/' argout.P.stratname '.str'];
        if exist(sfile,'file') == 2,
            load(sfile, '-mat');
            if argin.prt
                argout.out=[argout.out ; {['*Strategy file '...
                    argout.P.stratname '.str was read!*']}];
            end
        else
            argout.out=[argout.out ; {['*Warning strategy file '...
                argout.P.stratname '.str not found!*']}];
        end
    end
end
argout.P.splits=0;
% certificate
if argin.pro,
    argout.cert.x=argout.x;
    argout.cert.P=argout.P;
end