%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glselect.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [P,xbox]=glselect(P, I, J, xbox, prt)
% removes all bound constraints, removes all unused variables 
% from the subset Q.A(I) of the general constraints and returns the subset
% P            structure containing the internal problem definition
%   P.opt      new constraint index of the objective function
%   P.Q        structure encoding the quadratic part
%      .A         sparse matrix of constraint coefficients
%        .inf,.sup  bounds for coefficients
%      .int       characteristic vector of integer variables
%      .F         structure with bounds for general constraints
%        .inf,.sup    outer enclosure
%        .iinf,.isup  inner enclosure
%      .Bu              ne x 5 array with entries Bu(i,:)=[k,a,b,c,d]
%                       defining quadratic enclosures 
%                       B_{nx*(nx+1)+i}(x) in a x_k^2 + b x_k + [c,d]
%                       for nonquadratic basis functions
% I                     index set indiating the selected constraints
% J                     index set indiating the selected variables
% xbox                  nx dim structure with bounds on x (optional)
%   .inf,.sup           outer enclosure
% prt                   output level 0=none
%
% P                     structure containing the selected problem definition
%   P.opt               constraint index of the objective function
%   P.I                 index set indiating the remaining constraints
%   P.J                 index set indiating the remaining variables
%   P.rem.I             index set indiating the removed variables
%   P.rem.x             bounds of the removed variables (or empty)
%       .inf .sup
%   P.Q                 structure encoding the quadratic part of the subset
%      .A               nc_new x (nx_new+nx_new^2) sparse matrix of constraint coefficients
%        .inf,.sup      bounds for coefficients
%      .F               structure with bounds for general constraints
%        .inf,.sup      nc_new x 1 outer enclosure
% xbox                  nxnew dim structure with bounds on x 
%   .inf,.sup           outer enclosure

function [P,xbox]=glselect(P, I, J, xbox, prt)

%args
if nargin <3
    error('glselect: Not enough arguments!');
end
if nargin<4, xbox=[]; end
if nargin<5, prt=0; end

%init
nx=length(P.Q.int);    %number of initial variables
nu=size(P.Q.Bu,1);     %number of noquad-basis funs
if size(P.Q.A.inf,2) ~= nx+nx^2
    gldisp('nx',nx);
    error('glselect: P.Q.A.inf has wrong number of rows!');
end
if isempty(xbox)
    xbox.sup=inf+zeros(nx,1);
    xbox.inf=-xbox.sup;
end
if prt, gldisp('Ainf',P.Q.A.inf,'F',[P.Q.F.inf P.Q.F.sup],'I',I,'J',J); end

%--------------------------------------------------------------------------
%remove not desired(not given in I) , bound constraints
%and general constraints with infinite bounds (not objective)
ind=[]; %subset of the general (not bound) constraints of I
C=(P.Q.A.inf+P.Q.A.sup)/2; %mid of A
for i=I % for all constraints in the constraint index set
    if sum(C(i,1:nx)~=0)==1 && sum(C(i,nx+1:end)~=0)==0,
        %bound constraint found, concatenate with xbox and remove
        k=find(C(i,:)~=0);
        xbox.inf(k)=max(xbox.inf(k),P.Q.F.inf(i)/C(i,k));
        xbox.sup(k)=min(xbox.sup(k),P.Q.F.sup(i)/C(i,k));
        if i==P.opt
            %objectives should not be removed automatically but
            %set the bounds to infinity since it is in xbox
            P.Q.F.inf(i)=-inf;P.Q.F.sup(i)=inf;
            ind=[ind i];
        end
    elseif ~(isinf(P.Q.F.inf(i)) && isinf(P.Q.F.sup(i)) && i~=P.opt)
        %general constraint with infinite bounds or objective, dont remove
        ind=[ind i];
    end
end
P.Q.A.inf=P.Q.A.inf(ind,:);P.Q.A.sup=P.Q.A.sup(ind,:);
P.Q.F.inf=P.Q.F.inf(ind);P.Q.F.sup=P.Q.F.sup(ind);
nc=length(ind);  %new number of constraints
if prt && length(ind)~=length(I),
    disp([num2str(length(I)-length(ind)) ' bound constraint(s) removed!']);
    gldisp('nc_new',nc,'Ainf',P.Q.A.inf,'x',[xbox.inf';xbox.sup'],...
        'F',[P.Q.F.inf';P.Q.F.sup'],'int',P.Q.int);
end
I=ind; %store the new selection of the constraints

%--------------------------------------------------------------------------
%approximate and remove the non-selected variables
if nx ~= length(J)
    nxs=length(J);                      %number of selected variables
    nJ=1:nx;nJ(J)=[];                   %indices of the unselected vars
    E.inf=zeros(nc,nxs+nxs^2);
    E.sup=zeros(nc,nxs+nxs^2);          %new constraint matrix
    xb=infsup(xbox.inf,xbox.sup);       %interval xbox
    Fb=infsup(P.Q.F.inf,P.Q.F.sup);     %interval constraint bounds

    if prt, 
        gldisp('A',full(P.Q.A.inf),'F',[Fb.inf Fb.sup],'x',...
            [xb.inf xb.sup],'nx',nx,'nxs',nxs,'J',J,'nJ',nJ);
    end

    %(1) use constraint propagation on the unselected part to get the range
    %generate problem definition
    ind=nJ;ind2=ind;for i=ind,ind2=[ind2 i*nx+ind];end
    nP.Q.A.inf=P.Q.A.inf(:,ind2);nP.Q.A.sup=P.Q.A.sup(:,ind2);
    nxbox.inf=xbox.inf(ind);nxbox.sup=xbox.sup(ind);nnx=length(nxbox.inf);
    nP.basis = [];nP.uni = [];n.shift = 0;nP.opt = 0;
    nP.Q.int = zeros(nnx,1);nP.Q.nxo = nnx;nP.Q=glfindspars(nP.Q);
    nP.Q.F=P.Q.F;nP.Q.Bu = [];nP.Q.gaps = [];nP.Q.xref=[];
    if prt, 
        gldisp('ind2',ind2,'nnA',full(nP.Q.A.inf),...
            'nnxbox',[nxbox.inf nxbox.sup]); 
    end
    %compute range
    nF=glrange(nP, nxbox, 0);nF=infsup(nF.inf,nF.sup);
    if prt, gldisp('nnF',[nF.inf nF.sup]); end

    %(2) linearize bilinear terms
    for k=1:nc
        lin=infsup(P.Q.A.inf(k,J),P.Q.A.sup(k,J));   %linear terms
        for i=1:nx
            for j=1:nx
                ind=i*nx+j; %coeff index in A
                if P.Q.A.inf(k,ind) ~= 0 && P.Q.A.sup(k,ind) ~= 0
                    %non zero coeff
                    a=infsup(P.Q.A.inf(k,ind),P.Q.A.sup(k,ind));
                    if all(nJ~=i && nJ~=j)
                        %both vars selected, set quadratic/bilinear term
                        qind=find(J==i)*nxs+find(J==j);%index in E
                        E.inf(k,qind)=P.Q.A.inf(k,ind);
                        E.sup(k,qind)=P.Q.A.sup(k,ind);
                    elseif any(nJ==i)
                        %approximate the quadratic/bilinear part with
                        %an in xj linear term
                        lin(J==j)=lin(J==j)+a*xb(i);
                    elseif any(nJ==j)
                        %approximate the quadratic/bilinear part with
                        %an in xi linear term
                        lin(J==i)=lin(J==i)+a*xb(j);
                    end
                end
            end
        end
        %remove not tiny interval coeffs of the linear terms
        con=infsup(0,0);                             %constant part
        for i=1:nxs
            u=lin(i).mid;
            if (lin(i)-u)~=0, con=con+(lin(i)-u)*xb(i); end
            E.inf(k,i)=u;E.sup(k,i)=u;%set linear coeffs
        end
        %set new constraint bounds
        con=con+Fb(k)-nF(k);P.Q.F.inf(k)=con.inf;P.Q.F.sup(k)=con.sup;
    end
    P.Q.A=E;xbox.inf=xbox.inf(J);xbox.sup=xbox.sup(J);
    if prt, gldisp('Enew',full(E.inf)); end
    
    %remove the nonquadratics which are using unselected variables
    ind=[];                %indices of the selected nonquad funs
    for i=1:nu,
        if any(J==P.basis(i,2)),ind=[ind i];end
    end
    P.basis(ind,:)=[];     %remove unselected nonquads
    nx=length(xbox.inf);   %new number of variables
end

%--------------------------------------------------------------------------
% remove the unused variables
% (1)find indices of the used variables
ind=sum(abs(P.Q.A.inf)+abs(P.Q.A.sup),1);ind2=ind(1:nx);

% quadratic part
for i=1:nx,
    ind2=ind2+ind(i*nx+1:(i+1)*nx);
    ind2(i)=ind2(i)+sum(ind(nx*i+1:nx*(i+1)));
end

% % nonquadratic part
% for i=1:nu, ind2(P.basis(i,2))=1; end

ind=find(ind2~=0);  % used variable indices
uind=find(ind2==0); % unused variable indices

% (2) save the boxes for the removed variables
P.rem.I=uind;
P.rem.x.inf=xbox.inf(uind);P.rem.x.sup=xbox.sup(uind);

% (3) remove all col of the unused variables from P.Q.A and xbox
[P.Q.A, xbox]=glremovevars(P.Q.A, xbox, uind, 0);

% ind2=ind; %linear part
% %add used quadratic part
% for i=1:nx,
%     if any(ind==i), ind2=[ind2 ind+(i*nx)]; end
% end
% P.Q.A.inf=P.Q.A.inf(:,ind2);P.Q.A.sup=P.Q.A.sup(:,ind2);
% 
% %remove bound constraints of the not used variables
% xbox.inf=xbox.inf(ind);xbox.sup=xbox.sup(ind);

%(4) remove indices of the unused vars from J
uvar=J(uind);J(uind)=[];
if prt && length(ind)~=nx,
    disp(['Unused variable(s): x_{' num2str(uvar) '} removed!']);
    gldisp('nx_new',length(ind),'Ainf',P.Q.A.inf,'x',[xbox.inf xbox.sup],...
        'F',[P.Q.F.inf';P.Q.F.sup']);
end

%--------------------------------------------------------------------------
% set problem
P.I=I;P.J=J;            %constraint and variable index sets
P.Q.int=P.Q.int(J);     %shorten integer constraints indicator
P.opt=sum(I<=P.opt);    %new objective index
P.Q=glfindspars(P.Q);   %set new sparsity and non zero pattern


