%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsimplify.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glsimplify(argin)
%
% Simplify the constraints, remove the unused variables and the unbounded
% constraints. If needed the objective can be also removed, or bounded
% by using the reference points. The transformation in the equality form
% can be also requested.
%
% * needed argin structure input fields *
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
%   .xbox       original bound constraints after the simplification
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures
% scal          selected scaling method, none, Iter, LP, Hompack, Morgan
% lsolver       linear solver name which is used in the scaling with LP
% equ           equality transform fl  0 = no trans  1 = transform
% obj           'leave': leave objective function
%               'remove': remove objective function
%               'use refpoint': use reference(if exists) to set an upper 
%               bound of the objective function (and leave)
%
% * optional argin structure input fields *
% prt           text output flag       0 = no output 1 = output
% deb           debug output flag      0 = no output 1 = output
% gra           graphics output flag   0 = no output 1 = output
% pro           certificate output     0 = no cert   1 = make cert
% cert          certificate structure
%   .name       name of the certificate
%   .tname      name of the task used to aquire the certificate
%   .x          the new box after finishing the task
%   .P          new problem definition
%
% * argout structure fields *
% P             new problem definition
%   .I          index set indiating the remaining constraints
%   .J          index set indiating the remaining variables
%   ...
% x             new bound constraints
% out           text output
% cert          new certificate, with the same fields as in the input

function argout=glsimplify(argin)

global gloutput;        % global output structure

SBAS=10;                % scaling base

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'obj'}];

% optional input fields and their default values
oinp=[{'scal'};{'lsolver'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{'none'};{'LpSolve'};{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% init argout
argout.x=[];argout.P=[];
if argin.prt, argout.out={'*** PROBLEM SIMPLIFICATION ***'}; end
if argin.pro, 
    if isempty(argin.cert)
        argout.cert.name='Problem Simplification';
        argout.cert.tname='Simplify';
        argout.cert.x=[];
        argout.cert.P=[];
    else argout.cert=argin.cert;
    end
end

% init variables
[nc,nB]=size(argin.P.Q.A.inf);
nx=length(argin.P.Q.int);
if ~isfield(argin.P,'xref'),argin.P.xref=[];end
if argin.deb, 
    gldisp('Ainf',argin.P.Q.A.inf,'Asup',argin.P.Q.A.sup,...
        'F',[argin.P.Q.F.inf argin.P.Q.F.sup]);
end

% extract bound constraints remove constr with both infinite bounds
[argout.P, argout.x]=glselect(argin.P, 1:nc, 1:nx, argin.x, 0);
if length(argout.P.J)<nx,
    [nc,nB]=size(argout.P.Q.A.inf);
    nx=length(argout.P.Q.int);
end
obrem=0;obref=0;
if strcmp(argin.obj,'remove') && argout.P.opt~=0 && ...
        isinf(argout.P.Q.F.inf(argout.P.opt)) && ...
        isinf(argout.P.Q.F.sup(argout.P.opt));
    % remove the objective if desired, exists and has both infinite
    % bounds save ojective function
    argout.P.Q.obj.inf=argout.P.Q.A.inf(argout.P.opt,:);
    argout.P.Q.obj.sup=argout.P.Q.A.sup(argout.P.opt,:);
    argout.P.I=[1:argout.P.opt-1 argout.P.opt+1:nc];
    argout.P.Q.A.inf=argout.P.Q.A.inf(argout.P.I,:);
    argout.P.Q.A.sup=argout.P.Q.A.sup(argout.P.I,:);
    argout.P.Q.F.inf=argout.P.Q.F.inf(argout.P.I);
    argout.P.Q.F.sup=argout.P.Q.F.sup(argout.P.I);
    % delete objective index
    argout.P.opt=0;
    obrem=1;
elseif strcmp(argin.obj,'use refpoint') && argout.P.opt~=0 && ...
        size(argout.P.xref,2)>0
    % use reference vector to set the upper bound of the objective
    fx=inf;
    for i=1:size(argout.P.xref,2)
        f=gleval(infsup(argout.P.Q.A.inf(argout.P.opt,:),...
            (argout.P.Q.A.sup(argout.P.opt,:))), argout.P.xref(:,i));
        fx=min(f.sup,fx);
    end
    argout.P.Q.F.sup(argout.P.opt)=min(argout.P.Q.F.sup(argout.P.opt), fx);
    obref=1;
end

% remove unbounded constraints (objective excluded)
bounded=~(isinf(argout.P.Q.F.inf) & isinf(argout.P.Q.F.sup));
if argout.P.opt ~=0, bounded(argout.P.opt)=1;end
argout.P.Q.A.inf=argout.P.Q.A.inf(bounded,:);
argout.P.Q.A.sup=argout.P.Q.A.sup(bounded,:);
argout.P.Q.F.inf=argout.P.Q.F.inf(bounded,:);
argout.P.Q.F.sup=argout.P.Q.F.sup(bounded,:);

% save F in Fold
argout.P.Q.Fold=argout.P.Q.F;

% create variable mask with fillins in bilinear entries
[argout.P.Q.vmask, argout.P.Q.vperm]=glbilmask(argout.P.Q.A, 0);

% create scaling vectors
if ~strcmp(argin.scal,'none')
    A=argout.P.Q.A;x=argout.x;
    E=sparse(1:nx,1:nx,ones(nx,1),nB,nx,nB+nx); % n2 x nx monomial matrix
    ind=nx;
    for i=1:nx
        E(ind+1:ind+nx,:)=eye(nx);
        E(ind+1:ind+nx,i)=E(ind+1:ind+nx,i)+ones(nx,1);
        ind=ind+nx;
    end
    argout.P.scal.E=E;
    [argout.P.scal.x,argout.P.scal.c]=glscaling(A, E, x, argin.scal,...
        argin.lsolver, SBAS, 1, 0);
    As=glscale(A, argout.P.Q.F, E, x, argout.P.scal.x, ...
        argout.P.scal.c, 0);
    if argin.deb,
        gldisp('A',full(A.sup),'xs',full(argout.P.scal.x),...
            'cs',full(argout.P.scal.c),'Ass',full(As.sup));
    end
end

% set new sparsity and non zero pattern
argout.P.Q=glfindspars(argout.P.Q);

% set new bounds for the substituted variables
[argout.P.Q.Bu, argout.x] = gluniq(argout.P,argout.x,0);

% save the new x in P
argout.P.xbox=argout.x;
argout.P.startxbox=argout.x;

% text output
if argin.prt,
    txt=['Problem ' argout.P.name ' simplified: '];
    if ~isempty(argout.P.rem.I)
        txt=[txt 'variables removed: ' num2str(argout.P.rem.I) ', '];
    end
    if obrem,
        txt=[txt 'the objective function was removed '];
    elseif strcmp(argin.obj,'remove')
        txt=[txt 'bounded objective was not removed '];
    elseif obref,
        txt=[txt 'used references to set the objective upper bound '];
    elseif strcmp(argin.obj,'use refpoint')
        if argout.P.opt==0, txt=[txt 'no objective present '];
        else txt=[txt 'no references to set the objective '];
        end
    end
    if ~isempty(txt), argout.out=[argout.out ; {[txt(1:end-1) '!']}]; end;
    argout.out=[argout.out ; glprob2str(argout.P,argout.x)];
    if ~strcmp(argin.scal,'none')
        argout.out=[argout.out ; ...
         {['Scaling method used: ' argin.scal ', basis ' num2str(SBAS)]};...
         {['variables:' mat2str(log(argout.P.scal.x')/log(SBAS),3)]};...
         {['constraints:' mat2str(log(argout.P.scal.c')/log(SBAS),3)]}];
    end
end
if argin.deb, 
    gldisp('Ainf',argout.P.Q.A.inf,'Asup',argout.P.Q.A.sup,'F',...
        [argout.P.Q.F.inf argout.P.Q.F.sup],'x',[argout.x.inf argout.x.sup]);
end

% graphics output
if argin.gra, 
    nxp=min(length(argout.x.inf),argin.P.Q.nxo);
    if isfield(gloutput, 'graphics') && length(argin.x.inf)==2
        % clear the plot
        axes(gloutput.graphics.ahand(1));cla;
    end
    glplot(argout.P, argout.x, 'IndexSet', 1:nxp);
    drawnow; 
end

% certificate
if argin.pro,
    argout.cert.x=argout.x;
    argout.cert.P=argout.P;
end