%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmerge_cluster.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% argout=glmerge_cluster(argin)
%
% Find the clusters of the boxes given in P. The found cluster is returned
% together with its interval hull.
%
% * needed argin structure input fields *
% P
%   .boxes              1 x nb dimensional cell containig the input boxes
%                       of the same dimension
%   .stack              cell of unprocessed boxes, this field is
%                       optional, its contents are added to the boxes
%                       and the field is removed from argout.P!
% x                     nx x 1 bound constraints
%   .inf,.sup           outer enclosures
% tol                   maximum relative gap width between two boxes. 
%                       If two boxes have a gap smaller than this between 
%                       them they will be put in the same cluster.
%
% * optional argin structure input fields *
% prt                   text output flag       0 = no output 1 = output
% deb                   debug output flag      0 = no output 1 = output
% gra                   graphics output flag   0 = no output 1 = output
% pro                   certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P.boxes               1 x nb dimensional cell containig the found cluster
% x                     the found interval hull
%   .inf,.sup           outer enclosure
% out                   text output as cell
% cert                  produced certificate

function argout=glmerge_cluster(argin)

global glconf;    % global configuration variable
COL=[0 0 0];      % color of the plotted hull box

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'tol'}];

% optional input fields and their default values
oinp=[{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'clushullcol'), COL=glconf.clushullcol;end 

% init argout
argout.P=argin.P;argout.x=argin.x;
if argin.prt, argout.out={'*** FIND CLUSTERS ***'}; end

% check boxes
if isfield(argin.P,'boxes'), 
    boxes=argin.P.boxes;
    % drop boxes
    argout.P=rmfield(argout.P,'boxes');
else boxes=[];
end

% add the remaining stack to the boxes
if isfield(argin.P,'bstack')
    if ~isempty(argin.P.bstack)
        boxes=[boxes argin.P.bstack];
    end
    % drop the stack 
    argout.P=rmfield(argout.P,'bstack');
end

% find the cluster
nb=length(boxes);
if nb==0, 
    if argin.prt, argout.out=[argout.out; {'Problem has been found infeasible!'}]; end
    argout.x=[];
    argout.cert=[];
    return;
else
    nx=length(boxes{1}.inf);        %n umber of vars
    % cluster indicator: clu(i)=0 : undefined, clu(i)=nan : in currect (cn)
    % cluster but neighbours not determined, 0<clu(i)=n : box i in cluster n
    clu=sparse(1,1,nan,1,nb,nb);
    cn=1;                           % cluster counter
    ncn=nb;                         % number of not clustered boxes
    while ncn,
        % search for not processed boxes in current cluster
        ind=find(isnan(clu));
        if ~isempty(ind)
            % set cluster number fur current box in cluster
            k=ind(1);clu(k)=cn;ncn=ncn-1;
            % search for neighbours
            ind=find(clu==0);
            for i=ind
                neigh=1;% neighbourhood indicator
                for j=1:nx
                    tol=(boxes{i}.sup(j)-boxes{i}.inf(j)+...
                       boxes{k}.sup(j)-boxes{k}.inf(j))*argin.tol;
                    if boxes{i}.sup(j)+tol<boxes{k}.inf(j) || ...
                            boxes{i}.inf(j)>boxes{k}.sup(j)+tol
                        neigh=0;break;
                    end
                end
                if neigh, 
                    % neighbour found set to nan
                    clu(i)=nan;
                end
            end
        else
            % no other boxes in this cluster create new cluster
            ind=find(clu==0);k=ind(1);clu(k)=nan;cn=cn+1;
        end
    end
    ain.P.opt=argin.P.opt;ain.x=argin.x;ain.drop=0;ain.tol=[];
    for i=1:cn,
        % build cluster from the linear hull of each component
        ain.P.boxes=boxes(clu==i);
        aout=glmerge_inthull(ain);
        clus{i}=aout.x;
        clus{i}.status='a';
        clus{i}.depth=1;
    end
end

% text output
if argin.prt,
    if isempty(clus)
        argout.out=[argout.out;{'The cluster is empty!'}];
    else
        argout.out = {'The boxes merged to:'};
        for i=1:length(clus),
            argout.out=[argout.out; {['cluster (' num2str(i) '):']};...
                glbox2str(clus{i})];
        end
    end
end

% graphics output
if argin.gra && ~isempty(clus)
    for i=1:cn,glboxplot(clus{i}, 'Color', COL);end
end

% store the cluster
if cn==1,
    % only one box in the cluster, store in x 
    argout.x.inf=clus{1}.inf;argout.x.sup=clus{1}.sup;
else
    % more boxes in the cluster, store in boxes
    argout.P.boxes=clus;
    % compute new x by computing the interval hull of the cluster
    ain.P.opt=argin.P.opt;ain.P.boxes=clus;ain.x=argin.x;ain.tol=[];ain.drop=0;
    aout=glmerge_inthull(ain);
    argout.x=aout.x;
end

% certificate
if argin.pro, 
    argout.cert.name='Merge Boxes (cluster)';
    argout.cert.tname='Merge Boxes (cluster)';
    argout.cert.boxes=clus;
    argout.cert.x=argout.x;
end
