%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmerge_inthull.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% argout=glmerge_inthull(argin)
%
% Find the intervall hull of the boxes stored in P.boxes. 
%
% * needed argin structure input fields *
% P
%   .boxes              1 x nb dimensional cell containig the input boxes
%                       of the same dimension
%   .stack              a stack of unprocessed boxes, this field is
%                       optional, its contents are added to the boxes
%                       and the field is removed from argout.P!
% x                     nx x 1 bound constraints
%   .inf,.sup           outer enclosures
%
% * optional argin structure input fields *
% drop                  drop the boxes after creating the hull, the stack
%                       is always dropped (and added to the boxes)
% prt                   text output flag       0 = no output 1 = output
% deb                   debug output flag      0 = no output 1 = output
% gra                   graphics output flag   0 = no output 1 = output
% pro                   certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P                 new problem deffinition where the entries P.boxes and
%                   P.bstack could have been deleted if drop=1.
% x                 the found interval hull
%   .inf,.sup       outer enclosure
% out               text output as cell
% cert              produced certificate

function argout=glmerge_inthull(argin)

global glconf;    % global configuration variable
COL=[0 0 0];      % color of the plotted hull box

thisfun=mfilename;

% needed input fields & determine the function name 
ninp=[{'P'};{'x'}];

% optional input fields and their default values
oinp=[{'drop'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{  0   };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'boxhullcol'), COL=glconf.boxhullcol;end 

% init argout
argout.P=argin.P;

if argin.prt,
    if argin.drop, argout.out={'*** INTERVAL HULL *** (drop boxes)'};
    else argout.out={'*** INTERVAL HULL ***'};
    end
end

% check boxes
if isfield(argin.P,'boxes'), 
    boxes=argin.P.boxes;
    % drop boxes if needed
    if argin.drop
        if argin.prt,
            argout.out=[argout.out ; {'Original boxes discarded!'}];
        end
        argout.P=rmfield(argout.P,'boxes');
    end
else boxes=[];
end

% add the remaining stack to the boxes
if isfield(argin.P,'bstack')
    if ~isempty(argin.P.bstack)
        boxes=[boxes argin.P.bstack];
    end
    % drop the stack
    argout.P=rmfield(argout.P,'bstack');
    if ~argin.drop,
        % store the merged boxes and the stack in argout.P.boxes
        argout.P.boxes=boxes;
    end
end

nb=length(boxes);

if nb==0,
    argout.out=[argout.out; {'No boxes to merge!'}];
    argout.cert=[];
    return;
else 
    % find the interval hull, and find the best point (if any stored in the
    % boxes)
    nx=length(boxes{1}.inf);
    if ~isempty(argin.x) && isfield(argin.x,'best'), best=argin.x.best;
    else best=[];
    end
    % set the initial box to [inf -inf]
    argout.x.inf=inf+zeros(nx,1);argout.x.sup=-argout.x.inf;
    % find the interval hull of the boxes
    for i=1:nb
        box=boxes{i};
        li=argout.x.inf>box.inf;argout.x.inf(li)=box.inf(li);
        ui=argout.x.sup<box.sup;argout.x.sup(ui)=box.sup(ui);
        best=glbest(best, box, argin.P.opt);
    end
    if ~isempty(argin.x),
        % concatenate the found hull with the current box
        li=argout.x.inf<argin.x.inf;argout.x.inf(li)=argin.x.inf(li);
        ui=argout.x.sup>argin.x.sup;argout.x.sup(ui)=argin.x.sup(ui);
    end
        
end

% set the best point (if any)

if ~isempty(best), argout.x.best=best; end

% text output
if argin.prt,
    if isempty(argout.x), argout.out=[argout.out;{'The box is empty!'}];
    else
        argout.out=[argout.out; {'The boxes merged to the box:'} ; ...
            glbox2str(argout.x)];
    end
end

% graphics output
if argin.gra && ~isempty(argout.x)
    glboxplot(argout.x, 'Color', COL);
end

% certificate
if argin.pro,
    argout.cert.name='Merge Boxes (hull)';
    argout.cert.tname='Merge Boxes (hull)';
    argout.cert.x=argout.x;
end
