%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% glsplit_depthfirst.m %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glsplit_depthfirst(argin)
%
% Make bisections of /x and use the tasklist to reduce or eliminate the
% subboxes.
%
% * needed argin structure input fields *
% P          problem definition
% x          bound constraints
% tasklist   tasklist cell vector
% starttask  starting task can be a vector if subtask are present
% dirchooser the direction choosing method.
% maxdepth   maximum depth of the bisection
% splitvar   string inidicating the variables to be split e.g, 1,...,2,4
% asmall     minimum absolute width of the box parts,
%            min feasibilty distance
% smallm     margin between the absolute and relative small:
%            (1) if smallm>=0 for each interval a with mag(a)<=smallm the
%            absolute small and for mag(a)>=smallm the relative small is
%            calculated.
%            (2) if smallm<0 the automatical scaling factors times the
%            absolute value of the given margin is used as the margin between
%            the absolute and relative part.
% rsmall     minimum realtive width of the box parts
% smallcrit  the name of the criteria after which a box is found small or
%            not, currently 'width' and 'feasibility' can be chosen:
%            (1) width: d=wid(\x_c)./wid(\x) where \x is the the
%            box before the split) and \x_c is the the current box.
%            (2) feasibility: assume Aq(x) \in \F, x \in \x, and
%            \x_c \in \x is the current box. Using the range R=\Aq(\x_c)
%            of the current box we compute a feasibility distance as follows:
%            d_i=max(sup(R_i)-sup(F_i), inf(R_i)-inf(F_i)) for each i but
%            d_j=sup(R_j)-inf(R_i) for the objective.
%            (3) in both cases \x_c
%
% * optional argin structure input fields *
% mindist
% nopause    ==1 ignores the pauses in the tasklist
% prt        text output flag       0 = no output 1 = output
% deb        debug output flag      0 = no output 1 = output
% gra        graphics output flag   0 = no output 1 = output
% pro        certificate output     0 = no cert   1 = make cert
% cert       optional certificate structure !!!!!!!!!NOT USED!!!!!!!!
%   .name    name of the certificate
%   .tname   name of the task used to aquire the certificate
%   .boxes   the certificates of the split boxes
%
% * argout structure fields *
% splits    number of splits
% boxes     cell, found nonempty boxes
% boxes{i}  a found box
%  .inf,sup bounds
%  .status  status of the box: (i)nfinite, (f)ull, (m)ax depth reached, 
%           (t)iny, (s)mall, or (a)rbitrary.
%  .depth   split depth at which the box was found
%  .best    best point found inside the box (can be missing)
% bstack    cell, boxes remaining on the stack 
% bstack{i} a box which has not been processed by the split
%  .inf,sup bounds
%  .depth   split depth at which the box was found
%  .best    best point found inside the box (can be inherited by splitting 
%           the parent box)
% best          most feasible point found
%  .x           nx x 1 point
%  .fx          nc x 1 function value Aq(x)
%  .fd          feasibility distance of the found point
%  .status      status of the found point.
% status    = 0 proceed with the next task
%           = 1 stop the strategy (pause or user break)
%           = 4 continue with postprocess (glbreak->postprocess,timeout)
%           = 5 finish the strategy (glbreak->finish strategy, problem 
%               not accepted: dim too high/nonquadratic found, after 
%               postprocess)
% taskpos   curent task position, can be a vector if subtask are present
% cert      new certificate structute

function argout=glsplit_depthfirst(argin)

global glbreak;     % gloptlab break flag
global glconf;      % gloptlab configuration file
global gloutput;    % global output structure

INFTY=inf;          % infinity
TINY=1e-6;          % tiny enties, used as maximal relative width
% to treat a box component always small

% set constants from config
if isempty(glbreak), glbreak=0;end
if isfield(glconf,'infty'),INFTY=glconf.infty;end
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

thisfun=mfilename;

% needed input fields & determine the function name
ninp=[{'P'};{'x'};{'tasklist'};{'starttask'};{'dirchooser'};{'splitvar'};...
    {'maxdepth'};{'asmall'};{'smallm'};{'rsmall'};{'smallcrit'}];

% optional input fields and their default values
oinp=[{'nopause'};{'prt'};{'deb'};{'gra'};{'pro'};{'cert'}];
dval=[{   0     };{  0  };{  0  };{  0  };{  0  };{  []  }];

% argin check
if  nargin==0,
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1,
    error([thisfun ': Wrong number of input args!']);
end
% check the needed input arguments
for i=1:length(ninp)
    if ~isfield(argin,ninp{i})
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i})
        argin.(oinp{i})=dval{i};
    end
end

% init variables
P=argin.P;
nx=length(argin.x.inf);               % number of vars
nxo=P.Q.nxo;                    % number of original vars
tn=length(argin.tasklist);            % number of tasks
starttask=argin.starttask;
if isempty(starttask), starttask=1; end
splits=0;

% init split variable indices
[ok, vind]=glparsevar(argin.splitvar, 1:nxo, 1:nx);
if ~ok, error('glsplit_depthfirst: splitvar string has wrong format!');
elseif isempty(vind)
    gloutput.out = [gloutput.out ; {['No valid split components found,'...
        'check configuration for "begin split"!']}];
    return;
end
maxdepth=argin.maxdepth*length(vind); % init maxdepth
scount=zeros(nx,1);
% diameter reference for small
rl=argin.x.inf(vind);rl(rl<-INFTY)=-INFTY;
ru=argin.x.sup(vind);ru(ru>INFTY)=INFTY;
refdiam=ru-rl;

% init argout
argout.taskpos=tn;
argout.status=0;
argout.timer=argin.timer;
if argin.pro,
    % init certificates
    argout.cert.name='Branch and Bound';
    argout.cert.tname='Split';
    argout.cert.x=argin.x;
    argout.cert.boxes=[];
end

% best point found
if isfield(argin.x, 'best')
    argout.best=argin.x.best;
else
    argout.best.x=glmid(argin.x);
    [argout.best.fx, argout.best.fd, argout.best.fdv]=gleval(...
        (argin.P.Q.A.inf+argin.P.Q.A.sup)/2, argout.best.x, argin.P.Q.F, argin.x);
end

% output
if argin.prt,
    if argin.nopause, out={'*** SPLIT START (nopause) ***'};
    else out={'*** SPLIT START ***'};
    end
end

% output
if argin.prt,
    out=[out; ...
        {['Splitvar: ' argin.splitvar ', Maxdepth: ' num2str(maxdepth) ...
        ', Absolute small: ' mat2str((argin.asmall)',3) ...
        ', Relative small: ' mat2str((argin.rsmall)',3)]}];
    gloutput.out=[gloutput.out ; out];
end

% check tasklist
if tn==0
    % no tasks
    if argin.prt,
        gloutput.out=[gloutput.out ; {'Tasklist is empty, split skipped!'}];
    end
    return;
end

% init stack and boxes
if (~isfield(P,'bstack') || isempty(P.bstack)) && ...
        (~isfield(P,'boxes') || isempty(P.boxes))
    if argin.prt,
        gloutput.out=[gloutput.out ;{'Beginning split process...'}];
    end
    % no previous stack and boxes
    bstack=[];dep=0;boxes=[];box=argin.x;
else
    if argin.prt,
        gloutput.out=[gloutput.out ;{'Resuming split process...'}];
    end
    % check the stack
    if isfield(P,'bstack') && ~isempty(P.bstack)
        % restore the stack
        bstack=P.bstack;
        % pop last box, set it as the initial box
        [box, bstack]=pop(bstack);
        dep=box.depth;
    else
        bstack=[];dep=0;box=[];
    end
    % check the found boxes
    if isfield(P,'boxes')
        % push (m)ax depth reached, (s)mall, or (a)rbitrary. boxes to 
        % the stack, since theese parameters can have changed compairing to
        % a previous split.
        boxes=P.boxes;
        bind=[];
        for i=1:length(boxes)
            b=boxes{i};
            if b.status(1)=='s' || b.status(1)=='m' || b.status(1)=='a'
                % box is arbitrary, small or maxdepth reaxhed, push it on 
                % the stack and remember its index.
                bind=[bind i];
                bstack=push(bstack, b);
                dep=b.depth;
            end
        end
        % delete the boxes which have been pushed in the stack from the
        % boxes cell
        boxes(bind)=[];
    else boxes=[];
    end
    % check the starting box
    if isempty(box)
        if ~isempty(bstack)
            % pop last box, set it as the initial box
            [box, bstack]=pop(bstack);
            dep=box.depth;
        else
            % no boxes on the stack, terminate the split
            box=[];
            taskpos=tn;
            status=0;
        end
    end
end

if argin.gra && nx>1,
    % initilaize the plot, and reset old split picture using the bstack and
    % the boxes
    plot_init(P,argin.x,bstack,boxes);
end
treason='Normal termination.';
% split
while 1,
    if isempty(box),break;end

    if argin.gra && nx>1,
        plot_box('highlight', box); % highlight the current box
    end
    if ~argin.nopause,drawnow;end
    obox=box;
    % save split info in P
    P.boxes=boxes;
    P.bstack=bstack;
    P.splits=splits;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % process tasklist, no text output, no grpahic output
    [C, xf, status, taskpos, argout.timer, certs]=glproctskl(P,...
        box, argin.tasklist, starttask, argout.timer, argin.nopause, 0,...
        0, argin.pro);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isempty(xf),
        % xf cannot be empty, even if the problem is infeasible
        % the last box is returned but the box.infeas == true
        error('glsplit_depthfirst: empty box found!');
    end
    
    % set the best point
    if isfield(xf,'best') && argout.best.fd>xf.best.fd
       % new the best point
       argout.best=xf.best;
    end

    if argin.gra && nx>1,
        plot_box('remhighlight');% remove the highlight
    end
    %%%%%%%%%%%%%%%%%%%%%%%
    % CHECK THE FOUND BOX %
    %%%%%%%%%%%%%%%%%%%%%%%
    bstatus=boxanalyzer(xf, INFTY, vind, refdiam, argin.asmall,...
        argin.smallm, argin.rsmall, TINY, P.Q.F, C.Q.F, dep, maxdepth);
    % delete old starttask 
    starttask=1; xf.status=bstatus(1); xf.depth=dep;
    
    % EXTRACTING PROBLEM
    %glextracttest(xf);
    
    if bstatus(1)=='n'
        %(n)ot feasible: clear the box
        box=[];
    elseif bstatus(1)=='a'
        %(a)rbitrary, work on found box
        box=xf;
    else
        %(i)nfinite,(f)ull,(m)ax depth reached,(t)iny,(s)mall
        % save the found box and clear the current box
        boxes=[boxes {xf}];box=[];
    end

    % save certificate
    if argin.pro,
        c.name='Subbox';
        c.tname='Subbox';
        c.startbox=obox;
        c.newbox=xf;
        c.finish=bstatus(1) ~= 'a';
        c.certs=certs;
        argout.cert.boxes=[argout.cert.boxes; {c}];
    end
    
    % graphics output
    if argin.gra && bstatus(1)~='a' && bstatus(1)~='n' && nx>1,
        % add the found box to the plot
        plot_box('addfound', xf);
    end

    % text output
    if argin.prt
        if argin.deb,
            gloutput.out = [gloutput.out;{['* (' num2str(splits) ...
                '): ' glbox2str(obox, 1:nx, 3)]}];
        end
        if bstatus(1)~='n',
            % feasible box
            if argin.deb,
                gloutput.out = [gloutput.out;{['  reduced: ' ...
                    glbox2str(xf, 1:nx, 3)]};{['  gain: ' ...
                    mat2str(glgain(obox, xf),4)]}];
            end
            if bstatus(1)~='a'
                reason=[upper(bstatus(1)) bstatus(2:end)];
                wid=xf.sup(vind)-xf.inf(vind);
                if argin.deb
                    % relative diameter of each component
                    rd=zeros(size(refdiam));
                    rd(ind)=wid(ind)./refdiam(ind);
                    str = ['  ' upper(reason) '! wid=' mat2str(wid',4) ...
                        ', relwid=' mat2str(rd',4)];
                    if isfield(xf,'best')
                        str = [str ', x=' mat2str(xf.best.x',4) ...
                            ', fd=' mat2str(xf.best.fd,4)];
                    end
                else
                    str = ['(' num2str(splits) ') ' reason ': ' ...
                        glbox2str(xf, 1:nx, 3)];
                end
                gloutput.out = [gloutput.out ;{str}];
                bstr=['(' num2str(splits) '):' reason ': maxwid=' ...
                    num2str(max(wid))];
                if isfield(xf,'best')
                    bstr = [bstr ', fdbest=' mat2str(argout.best.fd,4)];
                end
                gloutput.brief = [gloutput.brief ;{bstr}];
            end
        else
            if argin.deb, gloutput.out = [gloutput.out;{'  EMPTY! '}];
            else
                gloutput.out = [gloutput.out;{['(' num2str(splits) ...
                    ') Empty box: ' glbox2str(obox, 1:nx, 3)]}];
            end
        end
    end
    %%%%%%%%%%%%%%%%%
    % STATUS CHECKS %
    %%%%%%%%%%%%%%%%%
    % check user break
    if glbreak, status=1; end
    % end the split if user break, break multiline, postprocess or finish.
    if status>0
        treason=glstatus2str(status);
        if ~isempty(box)
            % push the current box to the stack
            bstack=push(bstack, box);
        end
        % if the box is currently empty it will not be saved on the stack
        if argin.gra && nx>1,
            % delete the boxes on the stack and the found boxes from the
            % plot
            plot_clear(bstack, boxes);
        end
        if glbreak && argin.prt,
            gloutput.brief=[gloutput.brief;{'User break!'}];
        end
        break;
    end
    
    %%%%%%%%%%%%%%%%
    % POP OR SPLIT %
    %%%%%%%%%%%%%%%%
    if isempty(box)
        % the box is empty get one from stack
        if isempty(bstack)
            % box empty and no box on stack, split finished
            status=0;
            break;
        else
            % box empty pop another
            [box, bstack]=pop(bstack);
            if argin.gra && nx>1,
                % remove the popped box from the plot
                plot_box('remstack', box);
            end
            % set depth counter as the last depth entry of the depth vec
            dep=box.depth;
        end
    else
        % when maxdepth not reached and box not small, split box
        % into box and box2
        obox=box;
        % sort the components according to the given direction selection
        % method
        [ind, type]=glvariation(P, box, argin.dirchooser, scount);
        % leave only the components of the selected variables in the list
        ind2=[];
        for i=1:length(ind),
            if any(ind(i)==vind), ind2=[ind2 ind(i)];end
        end
        if ~isempty(ind2),
            % select the first of the remaining components
            ind=ind2(1);
            % increase the occurance of the selected, and reset the others
            sn=scount(ind)+1;scount=zeros(nx,1);scount(ind)=sn;
        else error('glsplit_depthfirst: direction choosing failed!');
        end
        % split the box in the selected component
        bxs=glboxpart(box, [ind;2]);
        if length(bxs)==2, box=bxs{1};box2=bxs{2};
        else box=bxs{1}; box2=[];
        end
        if ~isempty(box2) && argin.prt && argin.deb,
            gloutput.out = [gloutput.out ; ...
                {['  split (' type ') \x_' num2str(ind) '=[' num2str(obox.inf(ind))...
                ',' num2str(obox.sup(ind)) '] into ['...
                num2str(box.inf(ind)) ',' num2str(box.sup(ind)) '] and [' ...
                num2str(box2.inf(ind)) ',' num2str(box2.sup(ind)) ']']}];
        end
        splits=splits+1;
        % check boxes
        if ~isempty(box2)
            if any(box.inf>box.sup) || any(box2.inf>box2.sup)
                % wrong bounds
                gldisp('b1',[box.inf box.sup],'b2',[box2.inf box2.sup]);
                error ('glsplit_depthfirst: wrong splited boxes!')
            end
            % increase depth counter and push the second box
            dep=dep+1;box2.depth=dep;
            bstack=push(bstack, box2);
            if argin.gra && nx>1,
                % add the pushed box to the plot
                plot_box('addstack', box2);
            end
        else
            % INFINITE BOX splitted should not happen!
            gldisp('obox',[obox.inf obox.sup],'box',[box.inf box.sup]);
            gldisp('ind',ind,'INFTY',INFTY);
            argout.timer=gltimer(argout.timer, 'f', P, box);
            error('glsplit_depthfirst: infinite box splitted');
        end
    end
end

% output
argout.boxes=boxes;
argout.bstack=bstack;
argout.splits=splits;
argout.taskpos=taskpos;
argout.status=status;
if argin.prt,
    gloutput.out=[gloutput.out ; {['*** SPLIT END *** Total splits: ' ...
        num2str(splits) ', Termination reason: ' treason '.']}];
end

% absorb the end multine status
if argout.status==3, argout.status=0;end

if argin.gra && nx>1,
    % finalize the plot
    plot_finalize;
end

% Get the status of a box. 
% input: the found box, the infinity margin, the selected variable indices, 
% the reference box diameter, the absolute small, the small margin, the 
% relative small, the tiny margin, the old F, the new F, the current depth 
% and the maximal depth. 
% possible statuses: (n)ot feasible, (i)nfinite, (f)ull, 
% (m)ax depth reached, (t)iny, (s)mall, or (a)rbitrary.
function bstatus=boxanalyzer(box, infty, vind, refdiam, asmall, ...
    smallm, rsmall, tiny, Fold, Fnew, dep, maxdepth)
% argchk
if nargin<11, error('boxanalyzer: not enough arguments'); end
% find the box status
if isfield(box,'infeasible'), bstatus='not feasible';
elseif any(box.sup(vind)<=-infty) || any(box.inf(vind)>=infty)
    bstatus='infinite';
elseif all(Fold.inf<Fnew.inf) && all(Fnew.sup<Fold.sup)
    bstatus='full';
elseif dep==maxdepth
    bstatus='max depth reached';
else
    wid=box.sup(vind)-box.inf(vind);
    if max(wid)<tiny, bstatus='tiny';
    else
        ind=refdiam~=0;
        % relative diameter of each component
        rd=zeros(size(refdiam));
        rd(ind)=wid(ind)./refdiam(ind);
        if max(wid)<=smallm && max(wid)<=asmall
            bstatus='small(absolute)';
        elseif max(rd)<=rsmall
            bstatus='small(relative)';
        else
            bstatus='arbitrary';
        end
    end
end
%%%%%%%%%%%%%%%%%%
%STACK OPERATIONS%
%%%%%%%%%%%%%%%%%%

% push data to stack
function stack=push(stack, dat)
stack=[stack {dat}];

% pop last daat from stack
function [dat, stack]=pop(stack)
if isempty(stack)
    %no data on stack
    dat=[];
else
    dat=stack{end};
    stack(end)=[];
end

%%%%%%%%%%
%PLOTTING%
%%%%%%%%%%

% initialize the plot
function plot_init(P,refbox,bstack,boxes)
global glconf;      % gloptlab configuration file
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later

nx=length(gloutput.graphics.refbox.inf);
if length(refbox.inf)~=nx
    % input box and refbox dimension mismatch, the slack variables from
    % the input box must be removed!
    refbox.inf=refbox.inf(1:nx);refbox.sup=refbox.sup(1:nx);
end
% correct the reference box by cutting with the reference box
% from the config (if the intersection is not empty)
rbo=gloutput.graphics.refbox;
plot.refbox.inf=max(refbox.inf,rbo.inf);
plot.refbox.sup=min(refbox.sup,rbo.sup);
ind=find(plot.refbox.inf>plot.refbox.sup);
plot.refbox.inf(ind)=refbox.inf(ind);
plot.refbox.sup(ind)=refbox.sup(ind);
plot.is2d=nx==2;

if plot.is2d
    % 2D plot
    plot.SCOL=[0.8 1 1];     % color of the currently processed box (pause)
    plot.NCOL=[0.9 0.9 0.9]; % color of the not processed boxes (pause)
    plot.FCOL=[1 0.8 1];     % color of the found box
    plot.FFCOL=[1 0 0];      % color of the found full box
    plot.REFMODE=0;          % reference mode, used in plotting the boxes:
    % 0=reference box is the actual graphic view
    % 1=reference box is the current xbox
    if isfield(glconf,'sboxcol'),plot.SCOL=glconf.sboxcol;end
    if isfield(glconf,'nboxcol'),plot.NCOL=glconf.nboxcol;end
    if isfield(glconf,'fboxcol'),plot.FCOL=glconf.fboxcol;end
    if isfield(glconf,'ffboxcol'),plot.FFCOL=glconf.ffboxcol;end
    plot.hstack=[];
    plot.fhand=[];
    plot.high=[];
    if ~isempty(bstack) || ~isempty(boxes)
        % restore the plot of a previous splitting by drawing the
        % stack and the found boxes
        plot.hstack=[];
        for i=1:length(bstack)
            %plot stack
            if plot.REFMODE, h=glboxplot(bstack{i}, 'Color', plot.NCOL, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(bstack{i}, 'Color', plot.NCOL, 'Fill', inf);
            end
            plot.hstack=push(plot.hstack, h);
        end
        for i=1:length(boxes)
            % plot found box
            if boxes{i}.status=='f', col=plot.FFCOL;
            else col=plot.FCOL;
            end
            if plot.REFMODE, h=glboxplot(boxes{i}, 'Color', col, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(boxes{i}, 'Color', col, 'Fill', inf);
            end
            plot.fhand=[plot.fhand h];
        end
        hc=get(gca,'Children');
        nhn=length(boxes)+length(bstack);
        set(gca,'Children',hc([nhn+1:end,1:nhn]))
    end
else
    % HD plot
    if isfield(glconf,'xind'),xind=glconf.xind;
    else xind=gloutput.graphics.xind;
    end
    glplot(P, [], 'Refbox', plot.refbox, 'IndexSet', xind);
    gloutput.graphics.fillbox=1;
    gloutput.graphics.plotdata=1;
    gloutput.graphics.plotc=1;
    plot.high=[];
    if ~isempty(bstack) || ~isempty(boxes)
        % restore the plot of a previous splitting by drawing the
        % stack and the found boxes
        gloutput.graphics.fillbox=0;
        gloutput.graphics.plotc=1;
        for i=length(bstack):-1:1,
            glboxplot(bstack{i});
        end
        for i=1:length(boxes),
            glboxplot(boxes{i});
        end
    end
    drawnow;
end

% add a box to the plot and store its handle into the store
function plot_box(command, box)
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if isfield(plot,'is2d') && plot.is2d
    % 2D plot
    switch command
        case 'highlight'
            if plot.REFMODE, h=glboxplot(box, 'Color', SCOL,...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box, 'Color', plot.SCOL, 'Fill', inf);
            end
            plot.high=h;
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'remhighlight'
            if ~isempty(plot.high) && ishandle(plot.high)
                delete(plot.high);
            else return; % no highlited box
            end
        case 'addfound'
            if box.status(1)=='f', color=plot.FFCOL;
            else color=plot.FCOL;
            end
            if plot.REFMODE, h=glboxplot(box, 'Color', color, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box,'Color', color, 'Fill', inf);
            end
            plot.fhand=[plot.fhand h];
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'remfound'
            delete(plot.fhand(end));
            plot.fhand=plot.fhand(1:end-1);
        case 'addfeas'
            color='g';
            if plot.REFMODE, h=glboxplot(box, 'Color', color, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box, 'Color', color, 'Fill', inf);
            end
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'addstack'
            color=plot.NCOL;
            if plot.REFMODE, h=glboxplot(box, 'Color', color, ...
                    'Fill', inf, 'Refbox', plot.refbox);
            else h=glboxplot(box, 'Color', color, 'Fill', inf);
            end
            plot.hstack=push(plot.hstack, h);
            hc=get(gca,'Children');
            set(gca,'Children',hc([length(h)+1:end,1:length(h)]));
        case 'remstack'
            [h, plot.hstack]=pop(plot.hstack);
            delete(h)
        otherwise
            error(['glsplit_depthfirst:plot_box: Unknown '...
                'command found: ' command]);
    end
else
    % HD plot
    switch command
        case 'highlight'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=0;
            plot.high=box;
        case 'remhighlight'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=0;
            if ~isempty(plot.high) && ~isempty(plot.high)
                box=plot.high;
            else return; % no highlited box
            end
        case 'addfound'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=1;
        case 'remfound'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=-1;
        case 'addfeas'
            gloutput.graphics.fillbox=2;
            gloutput.graphics.plotc=1;
        case 'addstack'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=1;
        case 'remstack'
            gloutput.graphics.fillbox=0;
            gloutput.graphics.plotc=-1;
        otherwise
            error(['glsplit_depthfirst:plot_box: Unknown '...
                'command found: ' command]);
    end
    if gloutput.graphics.fillbox>0,plot.high=box;
    else plot.high=[];
    end
    glboxplot(box);
end


% remove a all boxes from the plot
function plot_clear(bstack, boxes)
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if plot.is2d
    % 2D plot
    for i=1:length(plot.hstack),
        [h, plot.hstack]=pop(plot.hstack);
        delete(h);
    end
    % delete found box handles
    delete(plot.fhand);
elseif 0,
    % HD plot (turned off)
    gloutput.graphics.fillbox=0;
    gloutput.graphics.plotc=-1;
    for i=length(bstack):-1:1,
        glboxplot(bstack{i});
    end
    for i=1:length(boxes),
        glboxplot(boxes{i});
    end
end
plot_finalize;

% finalize the plot
function plot_finalize
global gloutput;    % global output structure
global plot;        % global plot structure, will be removed later
if isfield(plot, 'is2d')
    if ~plot.is2d
        % HD plot
        gloutput.graphics.fillbox=2;
        gloutput.graphics.plotdata=1;
        gloutput.graphics.plotc=0;
    end
    plot_box('remhighlight');
    clear global plot
    drawnow;
end
