%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glb2q.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [r,c,d]=glb2q(x,P);
% creates, by sampling, rough approximate quadratic enclosures
%    F_(nx^2+nx+i)(x) = uni_j(x_k) in \r(i) + c(i)*x(k) + d(i)*x(k)^2
%                       for x in \x, [j,k]=P.basis(i,:)
% for nonquadratic basis functions in the global optimization problem
% described by P
%
% IMPORTANT: depends on P.xref, hence don't use it outside testing
% but only to create quadratic test problems
%
% x.inf,x.sup  bounds on x
% P              structure containing the internal problem definition
%   P.C(i)       real approximation of constant i
%   P.xref       nx x nref array of reference vectors
%   P.basis      ne x 2 array defining nonquadratic basis functions
%   P.ucode{j}   operation code for j-th univariate function, j>0
%      .o        row vector with operation code
%      .i        row vector with index list
%      .st       maximal stack size
%      .domain   [.inf,.sup] encloses relevant domain of argument
%      .range    [.inf,.sup] encloses known constraint on the range
%      .table    Hermite spline enclosure in relevant domain
%      supported univariate functions for j<0 are
%            uni_j(z) = sqrt(z) if j=-103,
%            uni_j(z) = exp(z)  if j=-104,
%            uni_j(z) = log(z)  if j=-105,
%            uni_j(z) = xlog(z) if j=-106, % xlog(x)=x*log(x), xlog(0)=0
%            uni_j(z) = sin(z)  if j=-111,
%            uni_j(z) = cos(z)  if j=-112,
%            uni_j(z) = atan(z) if j=-113,
%            uni_j(z) = abs(z)  if j=-151,
%            uni_j(z) = sign(z) if j=-152, % sign(0)=0
%            uni_j(z) = pos(z)  if j=-153, % pos(x)=max(x,0)
%            uni_j(z) = neg(z)  if j=-154, % neg(x)=max(-x,0)
%
% r.inf,r.sup  bounds on residual r
% c            coefficient vector of linear term
% d            coefficient vector of quadratic term
%
function [r,c,d]=glb2q(x,P)

fac=2^12;             % maximal size of a reasonable number
nref=size(P.xref,2);  % number of reference points

ne=size(P.basis,1);
r.inf=zeros(ne,1);
r.sup=zeros(ne,1);
c=zeros(ne,1);
d=zeros(ne,1);

for i=1:ne,
    j=P.basis(i,1);
    k=P.basis(i,2);
    low=x.inf(k);upp=x.sup(k);
    % restrict to a reasonable interval
    low=max(low,-fac);
    upp=min(upp,fac);
    if nref>0,
        % ensure that reference points are included
        low=min(low,min(P.xref(k,:)));
        upp=max(upp,max(P.xref(k,:)));
    end;

    % create sample argument list
    if low>=upp, % thin domain
        z=(low+upp)/2;
    elseif low>max(fac*upp,0)||low <min(fac*upp,0)
        z=low+((upp-low)/16)*0:16; % narrow domain
    elseif low>0,
        z=upp*2^-12:0;
    elseif low==0,
        z=[0,upp*2^-12:0];
    elseif upp<0,
        z=low*2^-12:0;
    elseif upp==0,
        z=[low*2^-12:0,0];
    else % interior zero
        z=[low*2^-12:0,0,upp*2^-12:0];
    end;
    z=sort(z);
    % add huge boundaries if present
    % but then the linear interpolant will be poor, hence don't!
    % if x.inf(k)<z(1),   z=[x.inf(k),z]; end;
    % if x.sup(k)>z(end), z=[z,x.sup(k)]; end;

    % evaluate and remove undefined points
    switch j
        case -103, fz=sqrt(z);
        case -104, fz=exp(z);
        case -105, fz=log(z);
        case -106, % xlog(x)=x*log(x), xlog(0)=0
            fz=zeros(size(z));ind=(z>0);
            fz(ind)=z(ind).*log(z(ind));
        case -111, fz=sin(z);
        case -112, fz=cos(z);
        case -113, fz=atan(z);
        case -151, fz=abs(z);
        case -152, fz=sign(z);
        case -153, fz=max(z,0);
        case -154, fz=max(-z,0);
        otherwise,  fz=glueval(P.ucode{j},z,P.C);
    end;
    ind=~(isnan(fz)|isinf(fz));
    z=z(ind);fz=fz(ind);nz=size(z,2);
    if nz==0,
        % function undefined, taken as zero
        continue;
    end;
    if nz==1,
        % function taken as constant
        r.inf(i)=fz;r.sup(i)=fz;
        continue;
    end;

    % form linear interpolant f0+slop*z at end points
    zlow=z(1);zupp=z(end);
    slop=(fz(end)-fz(1))/(zupp-zlow);
    f0=fz(1)-slop*zlow;
    if nz==2,
        % function taken as linear
        r.inf(i)=f0;r.sup(i)=f0;
        c(i)=slop;d(i)=0;
        continue;
    end;

    ind=ind(2:end-1);
    qlist=(fz(ind)-f0-slop*z(ind))./((z(ind)-zlow).*(z(ind)-zupp));
    d(i)=(min(qlist)+max(qlist)/2);
    c(i)=slop-d(i)*(zlow+zupp);
    if nref>0,
        % to get a tight test examples, use only reference points
        switch j
            case -103, fz=sqrt(z);
            case -104, fz=exp(z);
            case -105, fz=log(z);
            case -106, % xlog(x)=x*log(x), xlog(0)=0
                fz=zeros(size(z));ind=(z>0);
                fz(ind)=z(ind).*log(z(ind));
            case -111, fz=sin(z);
            case -112, fz=cos(z);
            case -113, fz=atan(z);
            case -151, fz=abs(z);
            case -152, fz=sign(z);
            case -153, fz=max(z,0);
            case -154, fz=max(-z,0);
            otherwise,  fz=glueval(P.ucode{j},z,P.C);
        end;
    end;
    rlist=fz-(c(i)+d(i)*z).*z;
    r.inf(i)=min(rlist);r.sup(i)=max(rlist);
end;
