%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glucode.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [ucode,ier]=glucode(uni,errmsg);
% reads the cell array uni defining univariate functions
% in reverse polish notation, specified as described in glinput.def.
% Translates it in to an operation code, 
% and corresponding index list and entry point table 
% for evaluating these functions using glueval.m 
%
% uni       nf x 1 cell array defining univariate functions
% errmsg    error message to be displayed in case of errors
%           (default: none)
%
% Currently interpretable tokens and their operation code are:
%           <index>   0     % index for constant
%           z         1     % independent variable
%           minus   101     % unary minus
%           sqr     102     % square
%           sqrt    103     % square root
%           exp     104     % exponential function
%           log     105     % natural logarithm
%           xlog    106     % xlog(x)=x*log(x); xlog(0)=0
%           sin     111     % sine
%           cos     112     % cosine
%           atan    113     % unary arc tangent
%           abs     151     % absolute value
%           sign    152     % sign; sign(0)=0
%           pos     153     % positive part; pos(x)=max(x,0) 
%           neg     154     % negative part; neg(x)=max(-x,0)
%           +       211     % binary addition
%           -       212     % binary subtraction
%           *       213     % multiplication
%           /       214     % division
%           ^       215     % power
%           min     216     % binary minimum       
%           max     217     % binary maximum
% In extended versions, the operation code oc must satisfy
%                oc<100     for fetch operations,
%           100<=oc<200     for unary operations,
%           200<=oc<300     for binary operations.
% 
% ucode{j}  structure with code for univariate function j
%   .o         row vector with operation code
%   .i         row vector with index list
%   .st        maximal stack size
%   .domain    [.inf,.sup] encloses relevant domain of argument
%   .range     [.inf,.sup] encloses known constraint on the range
%   .table     Hermite spline enclosure in relevant domain
%              (to be created in glu2tab.m)
% ier       0 correctly processed
%           1 unrecognizable token found
%           2 index <=0 found
%           3 attempt to read from empty stack
%           4 final stack has more than one entry
%
function [ucode,ier]=glucode(uni,errmsg)

nf=length(uni);
ucode=cell(nf,1);
ier=0;

for j=1:nf,
    string=uni{j};
    n=length(string);
    uo=zeros(1,n);no=0;
    ui=zeros(1,n);ni=0;
    err1=[];
    while 1,
        [op,string]=strtok(string);
        if isempty(op),
            ucode{j}.o=uo(1:no);
            ucode{j}.i=ui(1:ni);
            break;
        end;
        switch op
            case 'z',    oc=1;

            case 'minus',oc=101;
            case 'sqr',  oc=102;
            case 'sqrt', oc=103;
            case 'exp',  oc=104;
            case 'log',  oc=105;
            case 'sin',  oc=111;
            case 'cos',  oc=112;
            case 'atan', oc=113;
            case 'abs',  oc=151;
            case 'sign', oc=152;
            case 'pos',  oc=153;
            case 'neg',  oc=154;

            case '+',    oc=211;
            case '-',    oc=212;
            case '*',    oc=213;
            case '/',    oc=214;
            case '^',    oc=215;
            case 'min',  oc=216;
            case 'max',  oc=217;

            otherwise,    oc=0;  % <index>
                ni=ni+1;
                [ind,ct,err1]=sscanf(op,'%i');
                if isempty(err1),
                    ui(ni)=ind;
                else
                    % unrecognized token
                    err1=[err1,' ',op];
                    ui(ni)=NaN;
                end;
        end;
        no=no+1;
        uo(no)=oc;
    end;
    if ~isempty(err1),
        if ier==0, disp(errmsg); end;
        ier=1;
        disp('unrecognized tokens');disp(err1)
        disp(['in univariate function ',num2str(j)]);
        disp(uni{j});
    end;
end;
if ier==1, return; end;


% find stack sizes and check consistency
err2=[];
err3=[];
err4=[];
stacksize=0;
for j=1:nf
    if min(ucode{j}.i)<0, err2=[err2,j];end;
    i=0; % counter for top of stack
    uo=ucode{j}.o;
    for k=1:length(uo),
        cas=uo(k);
        if cas<100,
            i=i+1; % fetch operation
            stacksize=max(stacksize,i);
        elseif cas<200,
            % i unchanged, unary operation
        elseif cas<300,
            i=i-1; % binary operation
            if i<1, err3=[err3,j];break;end;
        else
            error('n-ary operations not implemented')
        end;
    end;
    if i>1, err4=[err4,j];end;
    ucode{j}.st=stacksize;
    ucode{j}.domain.inf=-inf;
    ucode{j}.domain.sup=inf;
    ucode{j}.range.inf=-inf;
    ucode{j}.range.sup=inf;
    ucode{j}.table=[]; % Hermite spline enclosure in relevant domain
    % (to be created in glu2tab.m with better domain)
end;


if ~isempty(err2)||~isempty(err3)||~isempty(err4),
    disp(errmsg);
end;
if ~isempty(err2),
    disp('attempt to read from empty stack in univariate functions')
    disp(err2);
    ier=2;
end;
if ~isempty(err3),
    disp('attempt to read from empty stack in univariate functions')
    disp(err3);
    if ier==0,ier=3;end;
end;
if ~isempty(err4),
    disp('final stack has more than one entry in univariate functions')
    disp(err4);
    if ier==0,ier=4;end;
end;
