%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glueval.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function fz=glueval(ucode,z,C);
% evaluate univariate function defined by the structure ucode,
% (typically a cell ucode{j} of the structure generated by glucode.m)
%
% ucode     structure with code for univariate function j>0
%   .o         row vector with operation code
%   .i         row vector with index list
%   .st        maximal stack size 
% z         argument; may be any typed structure for which the
%           operations in ucode are defined
%           if z is a row vector, all operations are done componentwise
% C         array of constants, must be real for floating point 
%           operations, interval for verified computations
%
% fz        function value, evaluated at z, of same type as z
%
function fz=glueval(ucode,z,C)

if size(z,1)>1, error('argument must be a row vector'); end;

on=ones(ucode.st,1);
s=z(on,:);     % stack (arbitrary initialization)
i=0;           % counter for top of stack
j=0;           % counter for index list

for k=1:length(ucode.o),
    switch ucode.o(k)
        case   0, i=i+1;j=j+1;s(i,:)=C(ucode.i(j)); % fetch constant
        case   1, i=i+1;      s(i,:)=z;             % fetch variable
        case 101, s(i,:)=-s(i,:);
        case 102, s(i,:)=s(i,:).^2;
        case 103, s(i,:)=sqrt(s(i,:));
        case 104, s(i,:)= exp(s(i,:));
        case 105, s(i,:)= log(s(i,:));
        case 106, % xlog
            ind=(s>0);s(i,ind)= s(i,ind).*log(s(i,ind));
            s(i,~ind)= 0;
        case 111, s(i,:)= sin(s(i,:));
        case 112, s(i,:)= cos(s(i,:));
        case 113, s(i,:)=atan(s(i,:));
        case 151, s(i,:)= abs(s(i,:));
        case 152, s(i,:)=sign(s(i,:));
        case 153, s(i,:)= max(s(i,:),0);   % pos(x)=max(x,0)
        case 154, s(i,:)= max(-s(i,:),0);  % neg(x)=max(-x,0)
        case 211, i=i-1;s(i,:)=s(i,:)+s(i+1,:);
        case 212, i=i-1;s(i,:)=s(i,:)-s(i+1,:);
        case 213, i=i-1;s(i,:)=s(i,:).*s(i+1,:);
        case 214, i=i-1;s(i,:)=s(i,:)./s(i+1,:);
        case 215, i=i-1;s(i,:)=s(i,:).^s(i+1,:);
        case 216, i=i-1;s(i,:)=min(s(i,:),s(i+1,:));
        case 217, i=i-1;s(i,:)=max(s(i,:),s(i+1,:));
        otherwise
            error('bad operation code');
    end;
end;
if i~=1,error('bad stack count');end;
fz=s(1,:);

