%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gluniq.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Bu,x,out]=gluniq(P,x,prt);
% creates rigorous quadratic enclosure for nonquadratic basis functions
% of a constraint satisfaction problem restricted to x in \x
%
% currently only by simple forward interval evaluation
% (i.e., zero linear and quadratic part)
%
% P            structure containing the internal problem definition
%              B(x) has the components
%                   B_k        = x_k        (k=1:nx)
%                   B_(j+nk)   = x_j x_k    (j,k=1:nx)
%                   B_i        = uni_j(x_k) (i=nx^2+nx+1:nB)
%              defined by P.basis(i,:)=[j,k] and P.ucode
%   P.basis    ne x 2 array defining nonquadratic basis functions
%   P.ucode    nu x 1 cell array of univariate operation codes
% x.inf,x.sup  bound constraints for x in R^n (default: no bounds)
%              on output improved box
% Bu           ne x 5 array with entries Bu(i,:)=[k,a,b,c,d]
%              defining quadratic enclosures
%                 B_{nx*(nx+1)+i}(x) in a x_k^2 + b x_k + [c,d]
%              for nonquadratic basis functions
% prt          1=text output, 0= no output
%
% out          output text cell
function [Bu,x,out]=gluniq(P,x,prt)

global xold; %global variable saving the previously evaluated box

%argcheck
if nargin<2, error('gluniq: Not enough arguments!');
elseif nargin==2,prt=0;
end
Bu=[];out=[];
if size(P.basis,1)==0, return;   %no nonquadratics exists
elseif isempty(xold) || length(xold.inf) ~= length(x.inf),
    %reset xold, not set or boxsize changed
    xold.sup=inf+zeros(size(x.sup));xold.inf=-xold.sup;
end
ne=size(P.basis,1);
Bu=zeros(ne,5);
for i=1:ne,
    if x.inf(i)<=xold.inf(i) && x.sup(i)>=xold.sup(i), out=[]; continue;end
    j=P.basis(i,1);
    k=P.basis(i,2);
    zz=infsup(x.inf(k),x.sup(k));
    switch j
        case -103, fzz=sqrt(max(zz,0));
        case -104, fzz=exp(zz);
        case -105, fzz=log(zz,0);
        case -106, % xlog(x)=x*log(x), xlog(0)=0
            disp('crude interval evaluation of xlog')
            zz=max(zz,0);
            if zz.inf>0,
                fzz=zz*log(zz);
            elseif zz.sup>0,
                zz=max(zz,realmin);
                fzz=zz*log(zz);fzz=infsup(fzz.inf,max(fzz.sup,0));
            else
                fzz=infsup(0,0);
            end
        case -111, fzz=sin(zz);
        case -112, fzz=cos(zz);
        case -113, fzz=atan(zz);
        case -151, fzz=abs(zz);
        case -152, fzz=sign(zz);
        case -153, fzz=max(zz,0);
        case -154, fzz=max(-zz,0);
        otherwise,  fzz=glueval(P.ucode{j},zz,P.C);
    end;
    Bu(i,1)=k;
    Bu(i,4)=fzz.inf;
    Bu(i,5)=fzz.sup;
    if Bu(i,2)==0 && Bu(i,3)==0
        ind=P.basis(i,3);
        x.inf(ind)=max(x.inf(ind),Bu(i,4));
        x.sup(ind)=min(x.sup(ind),Bu(i,5));
    else
        error('x dependent terms not yet implemented!');
    end
end;
if prt,
    out=[{'Setting in the definitions of nonquadratics results:'};...
        glbox2str(x)];
end
