%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glwhile.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function argout=glwhile(argin)
%
% executes tasklist on the problem P until: 
% (1) the maximum number of iterations has not been reached
% (2) all x components are small
% (3) the minimum gain in the last step has not been reached
% writes the output to glgloutput.out, if prt=1
%
% * needed argin structure input fields *
% P         problem definition
% x         bound constraints
% tasklist  tasklist cell vector
% starttask starting task can be a vector if subtask are present
% maxiter   maximum number of allowed iterations
% time      this percentage of the time ellapsed is used for the task
% small     minimum realtive width of the remaining box: all(x)<small
%           finish while
% mingain   minimum gain in one ireation to make anoter interation in (0,1)
%
% * optional argin structure input fields *
% nopause   ==1 ignores the pauses in the tasklist
% prt       text output flag       0 = no output 1 = output
% deb       debug output flag      0 = no output 1 = output
% gra       graphics output flag   0 = no output 1 = output
% pro       certificate output     0 = no cert   1 = make cert
%
% * argout structure fields *
% P         new problem definition
% x         new bound constraints
% status    = 0 proceed with the next task
%           = 1 stop the strategy (pause or user break)
%           = 4 continue with postprocess (glbreak->postprocess,timeout)
%           = 5 finish the strategy (glbreak->finish strategy, problem 
%               not accepted: dim too high/nonquadratic found, after 
%               postprocess)
% taskpos   curent task position, can be a vector if subtask are present
% cert      certificates cell array

function argout=glwhile(argin)
    
global glbreak;     % gloptlab break flag
global gloutput;    % global output structure
global glconf;      % gloptlab configuration file

TINY=1e-6;          % tiny enties, used as maximal relative width
                    % to treat a box component always small
                    
% needed input fields & determine the function name 
ninp=[{'P'};{'x'};{'tasklist'};{'starttask'};{'maxiter'};{'time'};...
    {'mingain'};{'small'}]; thisfun=mfilename;

% optional input fields and their default values
oinp=[{'nopause'};{'prt'};{'deb'};{'gra'};{'pro'}];
dval=[{    0    };{  0  };{  0  };{  0  };{  0  }];

% argin check
if  nargin==0, 
    argout=[{ninp};{oinp};{dval}];return
elseif nargin>1, 
    error([thisfun ': Wrong number of input args!']); 
end

% check the needed input arguments 
for i=1:length(ninp)
    if ~isfield(argin,ninp{i}) 
        error([thisfun ': Field ' ninp{i} ' is missing in argin!']);
    end
end

% if not found in argin, set the optional inputs at their default value
for i=1:length(oinp)
    if ~isfield(argin,oinp{i}) 
        argin.(oinp{i})=dval{i};
    end
end

% set constants from config
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

% init argout
argout.P=argin.P;argout.x=argin.x;argout.status=0;
argout.timer=argin.timer;
if argin.pro, 
    %init certificates
    argout.cert=cell(0);
end

% init variables
iter=1;
starttask=argin.starttask;
if isempty(starttask), starttask=1; end
width=argin.x.sup-argin.x.inf;
refdiam=argin.x.sup-argin.x.inf;      % diameter reference for small
% time
if argin.time>0,
    timer=gltimer(argin.timer, 'p');
    allowedtime=timer.ellapsed*argin.time;
    stime=toc;
end

if argin.prt,
    if argin.nopause, out={'*** WHILE START (nopause) ***'};
    else out={'*** WHILE START ***'};
    end
    out=[out; {['Maxiter: ' num2str(argin.maxiter)]}; {['Mingain: ' ...
        num2str(argin.mingain)]};{['Small: ' num2str(argin.small)]}];
    if argin.time>0,
        out=[out; {['Allowed time: ' num2str(allowedtime)]}];
    end
    gloutput.out=[gloutput.out ; out];
end
treason='Normal termination.';
% begin while
while 1,
    wo=width;               % save old width for reduc
    xold=argout.x;          % save old x for reduc 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % process tasklist
    [argout.P, argout.x, argout.status, argout.taskpos, argout.timer, certs2]=...
        glproctskl(argout.P, argout.x, argin.tasklist, starttask, ...
        argout.timer, argin.nopause, argin.prt, argin.gra, argin.pro); 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % delete old starttask
    starttask=1;
    
    %redraw
    if ~argin.nopause, drawnow; end
    % certificates
    if argin.pro, argout.cert=[argout.cert; certs2]; end
    
    %%%%%%%%%%%%%%%%%
    % STATUS CHECKS %
    %%%%%%%%%%%%%%%%%
    % check user break
    if glbreak, argout.status=1; end
    % end the while for: stop (1), break multiline(3), postprocess (4) or 
    % finish(5)
    if argout.status>0, 
        treason=glstatus2str(argout.status);
        break;
    end
    
    % compute width
    if isfield(argout.P,'boxes') && ~isempty(argout.P.boxes)
        % more boxes after split, compute total width
        width=zeros(length(argout.x.inf),1);
        for i=1:length(argout.P.boxes)
            width=width+argout.P.boxes{i}.sup-argout.P.boxes{i}.inf;
        end
        width=min(width, argout.x.sup-argout.x.inf);
    else
        %only x
        width=argout.x.sup-argout.x.inf;
    end
    %recalculate the diameter reference for small boxes
    infref=isinf(refdiam)&~isinf(width);
    refdiam(infref)=width(infref);
    nzref=refdiam~=0;
    warning off MATLAB:divideByZero
    reduc=width./wo;
    reduc(isnan(reduc))=1;  % infinity bounds without improvement
    reduc(isinf(xold.sup) & ~isinf(argout.x.sup))=0; % infinity bounds improved
    reduc(isinf(xold.inf) & ~isinf(argout.x.inf))=0; % infinity bounds improved
    reduc=1-reduc;
    warning on MATLAB:divideByZero
    
    %%%%%%%%%%%%%%%%%
    % check termination criteria
    if isempty(nzref) || all(max(width(nzref)./refdiam(nzref))<argin.small) || max(width)<TINY
        treason='Bounds are small';
        argout.P=glsolstatus(argout.P,'SMALLB',1);
        argout.status=0; 
        break; 
    elseif all(reduc<argin.mingain)
        treason='Reduction factor was too low';
        argout.status=0; 
        break; 
    elseif iter==argin.maxiter
        treason='Maximum number of iteration exceeded'; 
        argout.status=0; 
        break; 
    elseif argin.time>0 && toc-stime > allowedtime
        treason='Allowed time was exceeded'; 
        argout.status=0; 
        break; 
    end
    % next iteration
    iter=iter+1;
end
if argin.prt,
    gloutput.out=[gloutput.out ; {['*** WHILE END *** Full steps '...
        'completed: ' num2str(iter) ', Termination reason: ' treason '.']}];
end
% absorb the break multiline status
if argout.status==3, argout.status=0;end
