%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcprogr2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [string, equnums]=glcprogr2str(cin,html,prec,equnums,nextnum);
%
% converts the linear or conic program input cin to a string or a
% html representation
%
% FOR LINEAR AND CONIC PROGRAMS:
%   objective function min b*y
%       obj.b       1 x ny sparse vector
%   nl linear A*y \in \b
%       lin.A       nl x ny sparse matix
%       lin.b       nl x 2 matrix b(i)=[lineq.b(i,1) lineq.b(i,2)]
% ONLY FOR CONIC PROGRAMS:
%   nql quadratics ||b_i-A_i*y||<=d_i-c_i*y, i=1:nql
%       quadl       nql cell with quadl{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%   nqr quadratics ||b_i-A_i*y||^2<=u*v, with  u:=(d_i-c_i*y)>=0,
%                   v:=(f_i-e_i*y)>=0 for i=1:nqr
%       quadr       nqr cell with quadb{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%          .e       1 x ny sparse vector
%          .f       constant
%   ns semidefinite G0_i-sum(G_(i,j) y_j) psd, j=1:ny, i=1:ns
%       semi        ns cell with semi{i} having the fields
%          .G0      m x m sparse matrix
%          .G       ny cell with m x m sparse matrix entries
%  html         format as a html text (default=0)
%  equnums      nc+nx x 1 vector representing the current number of the
%               numerated constraints. first the general than the bound.
%  nextnum      next equation number
%
%  string       string cell representation of P or simple string if html==1
%  equnums      nc+nx x 1 vector representing the current number of equs

function [string, equnums]=glcprogr2str(cin, html, prec, equnums, nextnum)

global glconf;      %gloptlab configuration variable

%HTMLCOLOR='#008F8F' ;
INFTY=inf;          % infinity

% argcheck
if nargin<1,error('glcprogr2str: Not enough arguments!');end
if nargin<2,html=0;end
if nargin<3,prec=4;end
if nargin<4,equnums=[];end
if nargin<5,nextnum=max(equnums)+1;end

%set configuration
if isfield(glconf,'infty'),INFTY=glconf.infty;end 

% init
nx=length(cin.obj.b);
nl=0;nql=0;nqr=0;ns=0;
if (isfield(cin, 'lin') && ~isempty(cin.lin))
    nl=size(cin.lin.b,1);               %number of linear constraints
    %wid=cin.lin.b(:,2)-cin.lin.b(:,1);  %width of 
    if any(cin.lin.b(:,2)-cin.lin.b(:,1)<0),
        error('glcprogr2str: linear constr bounds wrong!');
    end
    if any(cin.lin.b(:,2)>=INFTY & cin.lin.b(:,1)<=-INFTY),
        error('glcprogr2str: unbounded linear constraints found!');
    end
end
if (isfield(cin, 'quadl') && ~isempty(cin.quadl)), nql=length(cin.quadl);end
if (isfield(cin, 'quadr') && ~isempty(cin.quadr)), nqr=length(cin.quadr);end
if (isfield(cin, 'semi') && ~isempty(cin.semi)), ns=length(cin.semi);end
equnums=zeros(nl,1);

%(1) create string representation of the basis funcs
unistr=cell((nx+1)*nx,1);
% linear terms
for i=1:nx
    if html,
        unistr{i}=['x<sub><small>' num2str(i) '</small></sub>'];
    else
        unistr{i}=['x_' num2str(i)];
    end
end
% quadratic terms
for i=1:nx
    for j=1:nx
        if i~=j
            if html,
                unistr{i*nx+j}=['x<sub><small>' num2str(i) ...
                    '</small></sub>x<sub><small>' ...
                    num2str(j) '</small></sub>'];
            else
                unistr{i*nx+j}=['x_' num2str(i) 'x_' num2str(j)];
            end
        else 
            if html,
                unistr{i*nx+j}=['x<sub><small>' num2str(i) ...
                    '</small></sub><sup>2</sup>'];
            else
                unistr{i*nx+j}=['x_' num2str(i) '^2'];
            end
        end
    end
end

%(2) create string representation of the constraints
% objective and linear terms
linstr=cell(nl+1,1);qlstr=cell(nql,1);qrstr=cell(nqr,1);sestr=cell(ns,1);
A=[cin.obj.b' ; cin.lin.A];
for i=1:nl+1
    out=Avec2str(A(i,:), A(i,:), unistr, prec);
    % normal constraint
    if nextnum,
        equnums(i,1)=nextnum;nextnum=nextnum+1;
        if html,out=['(<small>' num2str(equnums(i)) '</small>) ' out];
        else out=['(' num2str(equnums(i)) ') ' out];end
    end
    % rhs
    if i==1 
        if isempty(out), out='1';end
        out=[        'minimize ' out];
    elseif i==2, out=['s.t.     ' out];
    else out=[      '         ' out];
    end
    if i>1, 
        out=[out bound2str(cin.lin.b(i-1,1), cin.lin.b(i-1,2), html,prec)];
    end
    linstr{i}=out;
end
if nql>0
    %nql quadratics ||b_i-A_i*y||<=d_i-c_i*y, i=1:nql
    for i=1:nql
        in=cin.quadl{i};lstr='';
        %left side
        for j=1:size(in.A,1)
            a=[in.b(j) -in.A(j,:)];
            str=Avec2str(a, a, [{''};unistr], prec);
            if sum(a~=0)>1, str=['(' str ')'];end
            if html, str=[str '<sup>2</sup>'];
            else str=[str '^2'];
            end
            if j==1, lstr=str; 
            else lstr=[lstr ' + ' str];
            end
        end
        lstr=['sqrt(' lstr ')'];
        %right side
        u=[in.d -in.c];
        rstr=Avec2str(u, u, [{''};unistr], prec);
        %equation
        if html, out=['         ' lstr '<B> &le; </B>' rstr];
        else out=['         ' lstr ' <= ' rstr];
        end
        qrstr{i}=out;
    end
end
if nqr>0
    %nqr quadratics ||b_i-A_i*y||^2<=u*v, with  u:=(d_i-c_i*y)>=0,
    %und v:=(f_i-e_i*y)>=0 for i=1:nqr
    for i=1:nqr
        in=cin.quadr{i};lstr='';
        %left side
        for j=1:size(in.A,1)
            a=[in.b(j) -in.A(j,:)];
            str=Avec2str(a, a, [{''};unistr], prec);
            if sum(a~=0)>1, str=['(' str ')'];end
            if html, str=[str '<sup>2</sup>'];
            else str=[str '^2'];
            end
            if j==1, lstr=str; 
            else lstr=[lstr ' + ' str];
            end
        end
        %right side
        u=[in.d -in.c];
        rstr1=Avec2str(u, u, [{''};unistr], prec);
        if sum(u~=0)>1, rstr1=['(' rstr1 ')'];end
        v=[in.f -in.e];
        rstr2=Avec2str(v, v, [{''};unistr], prec);
        if sum(v~=0)>1, rstr2=['(' rstr2 ')'];end
        if sum(u~=0)>0 && sum(v~=0)>0, rstr=[rstr1 rstr2]; 
        else rstr='0';
        end
        %equation
        if html, out=['         ' lstr '<B> &le; </B>' rstr];
        else out=['         ' lstr ' <= ' rstr];
        end
        qrstr{i}=out;
    end
end
if ns>0
    %ns semidefinite G0_i-sum(G_(i,j) y_j) psd, j=1:ny, i=1:ns
    nns=0;
    for i=1:ns
        in=cin.semi{i};
        G0=glmat2scell(in.G0, prec);
        m=size(in.G0,1);n=round(m/2);
        nG=length(in.G);G=cell(nG,1);
        for j=1:nG, G{j}=glmat2scell(in.G{j}, prec); end
        for k=1:m
            if any(any(in.G0 ~= 0)), out=['         ' G0{k}];nz=1;
            else out='         ';nz=0;
            end
            for j=1:size(in.G)
                if any(any(in.G{j}~=0)),
                    val=mat2str(G{j}{k});
                    if k==n, 
                        if nz~=0, out=[out '+'];
                        else nz=0;
                        end
                        out=[out val unistr{j}];
                    else
                        spac=char(32+zeros(size(unistr{j})));
                        out=[out ' ' val spac];
                    end   
                end
            end
            if k==n, out=[out ' is psd.'];end
            nns=nns+1;
            sestr{nns,1}=out;
        end
    end
end
string=[linstr; qlstr; qrstr; sestr];
%(3) create string from cell for html documents,
if html,
    out='<PRE><I>';
    for i=1:length(string)
        out=[out string{i} '\n'];
    end
    out=[out '</I></PRE>\n'];
    string=out;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Avec2str %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% str=Avec2str(ainf, asup, basis, prec)
%
% make a string representation of a vector [a.inf a.sup]*basis
%
% [ainf,asup]   1xnB vectors
% basis         1xnB string cell representing the basis variables
% prec          precision of the numeric constants
%
% str         	string representation of [a.inf a.sup]*basis

function str=Avec2str(ainf, asup, basis, prec)

global glconf;      %gloptlab configuration variable

TINY=1e-6;          % tiny enties

% argcheck
if nargin<3,error('Avec2str: Not enough arguments!');end
if nargin<4,prec=4;end

%set configuration
if isfield(glconf,'tiny'),TINY=glconf.infty;end 

nz=0;str='';
if(asup-ainf<TINY)
    for j=1:length(asup)
        c=(asup(j)+ainf(j))/2;
        if c~=0,
            if nz~=0 && c>=0, str=[str '+'];
            else nz=1;
            end
            if c==-1, str=[str '-'];
            elseif c~=1, str=[str num2str(c,prec)];
            end
            str=[str basis{j}];
        end
    end
else
    for j=1:length(asup)
        if ainf(j)~=0 || asup(j)~=0,
            if nz~=0, str=[str '+'];
            else nz=1;
            end
            str=[str '[' ainf(j) ',' asup(j) ']' basis{j}];
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% bound2str %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% str=bound2str(xinf, xsup, html, prec)
%
% make a string of an (in)equation: ... \in [xinf, xsup]
%
% [xinf,xsup]   box in which the constraint or the variable is contained
% isvar         instr represents a single variable 0:no 1:yes
% html          format as a html text (default=0)
% prec          precision of the numeric constants
%
% str         	string representation of instr \in box

function str=bound2str(xinf, xsup, html, prec)

global glconf;      %gloptlab configuration variable

INFTY=inf;          % infinity
TINY=1e-6;          % tiny enties

% argcheck
if nargin<2,error('bound2str: Not enough arguments!');end
if nargin<3,html=0;end
if nargin<4,prec=4;end

%set configuration
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

str=[];
if xsup-xinf<=TINY
    if html, str=[str '<B> = </B>'];
    else str=[str ' = '];
    end
    str=[str num2str((xinf+xsup)/2,prec)];
elseif xsup<INFTY && xinf<=-INFTY
    if html, str=[str '<B> &le; </B>'];
    else str=[str ' <= '];
    end
    str=[str num2str(xsup,prec)];
elseif xsup>=INFTY && xinf>-INFTY
    if html, str=[str '<B> &ge; </B>'];
    else str=[str ' >= '];
    end
    str=[str num2str(xinf,prec)];
elseif xsup<INFTY && xinf>-INFTY
    if html, str=[str '<B> &isin; </B>'];
    else str=[str ' in '];
    end
    str=[str '[' num2str(xinf,prec) ',' num2str(xsup,prec) ']' ];
end