%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glineq2eq.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [x, slacks, E]=glineq2eq(x, F, A, prt)
%
% trasforms the the general constraints in equality constraints by adding
% slack variables. Finally the constarints have the form
% E(i,:)[1;x(1);...;x(nx);x(1)x(1);...;x(1)x(nx);...;x(nx)x(1);...;
%       x(nx)x(nx)]'=0
%
% x              no dim structure with bounds on x 
%   .inf,.sup    outer enclosure
% A              nc x (no+no^2) sparse matrix of constraint coeffs
%   .inf,.sup    bounds for coefficients
% F              structure with bounds for general constraints
%   .inf,.sup    outer enclosure
% prt            output level 0=none 1=output
%
% x              no+ns dim structure with bounds on x
%   .inf,.sup    outer enclosure
% slacks         1 x ns slack variable indexes: x_(nx+i) used for the
%                non-equality constraint slack(i)
% E              nc x (no+ns)+(no+ns)^2+1 matrix of equality constraints
%   .inf,.sup    outer enclosure

function [x, slacks, E]=glineq2eq(x, F, A, prt)

global glconf;      % global configuration variable
INF=1e5;            % positive and negative infinity
TINY=1e-6;          % tiny enties, used as maximal relative width
                    % to treat a constraint as equality constraint

% argcheck
if nargin<3, error('glineq2eq: Not enough arguments!'); end
if nargin<4, prt=1; end

% set constants from config
if isfield(glconf,'infty'),INF=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

% init
nx=length(x.inf); % number of variables
no=nx;            % number of original variables

% checks
if size(A.inf,2)~=nx*(nx+1),
    gldisp('nA',size(A.inf,2),'nA2',nx*(nx+1));
    error('glineq2eq: dimension mismatch!');
end

% convert to equality constraints
Fw=F.sup-F.inf;                                 % width of the box F
if any(Fw<0), error('glineq2eq: constraint bounds are inconsistent!');end
nc=size(A.inf,1);                               % number of constraints
con.inf=zeros(nc,1);                            % constant part of E
con.sup=zeros(nc,1);                            % constant part of E
lini=A.inf(:,1:nx); lins=A.sup(:,1:nx);         % linear part of E
quadi=A.inf(:,nx+1:nx+nx^2);quads=A.inf(:,nx+1:nx+nx^2);% quad part of E
slacks=[];                                      % slack variable indexes
for i=1:nc
    if max(Fw)<INF, small=TINY*max(Fw);         % one width is finite
    else small=TINY;                            % all widths are infinite         
    end
    if Fw(i)<=small
        if prt,disp(['Constraint ' num2str(i)...
                ' is already an equality constraint']);end
        con.inf(i)=-F.sup(i);con.sup(i)=-F.inf(i);
    else
        if prt,disp(['Constraint ' num2str(i)...
                ' is not an equality, adding slack']);end
        % con(i)=0 is already set
        lini=[lini zeros(nc,1)];lins=[lins zeros(nc,1)];
        nx=nx+1;lini(i,nx)=-1;lins(i,nx)=-1;
        slacks=[slacks i];
        x.inf=[x.inf; F.inf(i)];
        x.sup=[x.sup; F.sup(i)];
    end
end

if prt && nx~=no,
    disp(['A total of ' num2str(nx-no) ...
        ' new variables (slaks) added!']);
end

% build E
E.inf=[con.inf lini];E.sup=[con.sup lins];   % constant, linear terms
nxn=nx-no;
for i=1:no,                               % quadratic terms
    E.inf=[E.inf [quadi(:, (i-1)*no+1:i*no) zeros(nc,nxn)]];
    E.sup=[E.sup [quads(:, (i-1)*no+1:i*no) zeros(nc,nxn)]];
end
% add extra empty quadratic entries for slacks
E.inf=[E.inf zeros(nc,nx*(nxn))];E.sup=[E.sup zeros(nc,nx*(nxn))];
