%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glmat2tex.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function ce=glmat2tex(A, name, opt)
%
% converts matrix to a tex string representation
%
% A         a matrix
% name      name of the matrix can be empty
% opt       options structure with possible fileds
%   .delim  delimeter between the tex entries default is '&', use '\!&\!'
%           to save space
%   .rep    nrx2 cell, indicating that the numeric
%           rep{i}{1} will be replaced by the string rep{i}{2}
%   .head   1 x m string cell of col headers
%   .prec   precision of the numbers in the output (optional)
%   .cut    cut the values to this number chars (-1 no cut) 
%   .prt    write the tex matrix directly to the console
% 
% ce    text string cell array created from the matrix

function ce=glmat2tex(A, name, opt)

% argcheck
if nargin<1, error('glmat2tex: Not enough arguments!');end
if nargin<2, name=[];end
if nargin<3, opt=[];end

% set default options
if ~isfield(opt,'delim'),    opt.delim='&'; end
if ~isfield(opt,'rep'),      opt.rep=[]; end
if ~isfield(opt,'head'),     opt.head=[]; end
if ~isfield(opt,'prec'),     opt.prec=3; end
if ~isfield(opt,'cut'),      opt.cut=-1; end
if ~isfield(opt,'prt'),      opt.prt=0; end

if isfield(A,'inf')
    % interval matrix
    isint=1;
    [n,m]=size(A.inf);
else
    % normal matrix
    isint=0;
    [n,m]=size(A);
end
rn=length(opt.rep);
hn=~isempty(opt.head);
n2=n+hn;
ce=cell(n2+2,1);
onerow=(n==1 && hn==0);
if onerow==1,
    % only one row, set the delim to ' '
    opt.delim='~~ ';
end

for j=1:m
    ml=1;
    val=cell(n2,1);
    if hn==1
        val{1}=opt.head{j};
        ml=max(ml,length(val{1}));
    end
    for i=1:n
        if isint
            l=A.inf(i,j);u=A.sup(i,j);v=[];w=[];
            for k=1:rn
                if l==opt.rep{k,1} || (isnan(l) && isnan(opt.rep{k,1})),
                    v=opt.rep{k,2};
                    break;
                end
            end
            for k=1:rn
                if u==opt.rep{k,1} || (isnan(u) && isnan(opt.rep{k,1})),
                    w=opt.rep{k,2};
                    break;
                end
            end
            val{i+hn}='{[} ';
            if isempty(v),
                val{i+hn}=['{[} ' sprintf(['%0.' num2str(opt.prec) 'g'],l) ', '];
            else val{i+hn}=['{[} ' v ', '];
            end
            if isempty(w),
                val{i+hn}=[val{i+hn} sprintf(['%0.' num2str(opt.prec) 'g'],u) ' {]}'];
            else val{i+hn}=[val{i+hn} w ' {]}'];
            end
        else
            a=A(i,j);v=[];%ex=opt.prec*10;ex2=(opt.prec-1)*10;
            for k=1:rn
                if a==opt.rep{k,1} || (isnan(a) && isnan(opt.rep{k,1})),
                    v=opt.rep{k,2};
                    break;
                end
            end
            if isempty(v), val{i+hn}=sprintf(['%0.' num2str(opt.prec) 'g'],a);
            else val{i+hn}=v;
            end
            % additional cut
            if opt.cut>0
                if a>=0,
                    val{i+hn}=val{i+hn}(1:min([length(val{i+hn}) opt.cut]));
                else
                    val{i+hn}=val{i+hn}(1:min([length(val{i+hn}) opt.cut+1]));
                end
            end
        end
        ml=max(ml,length(val{i+hn}));
    end
    ml=ml+2;

    for i=1:n2
        ce{i+1}=[ce{i+1} val{i} char(32+zeros(1,ml-length(val{i})))...
            opt.delim ' '];
    end
end

if ~isempty(name), ce{1}=[name ':= '];
else ce{1}=[];
end
if onerow,
    % only one row, set the delim to ' '
    if m>1
        ce={[ce{1} '(' ce{2}(1:(end-length(opt.delim)-1)) ')']};
    else ce={[ce{1} ce{2}(1:(end-length(opt.delim)-1))]};
    end
        
else
    ce{1}=[ce{1} '\left(\bary{' char(99+zeros(1,m)) '}'];
    for i=2:n2+1, ce{i}=[ce{i}(1:(end-length(opt.delim)-1)) '\\'];end
    ce{n2+2}='\eary\right)';
end

if opt.prt, for i=1:length(ce), disp(ce{i});end;end
