%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% glprob2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [string, equnums]=glprob2str(P, xbox, mode, equnums, nextnum);
%
% converts the problem P to a string or html representation
%
% P            structure containing the internal problem definition
%   P.C(i)     real approximation of constant i
%   P.basis    ne x 2 array defining nonquadratic basis functions
%   P.uni      nu x 1 cell array defining univariate functions
%   P.shift    constant index  in objective function C_shift+type*u_opt
%   P.opt      variable index  in objective function  
%   P.Q        structure encoding the quadratic part
%      .A         sparse matrix of constraint coefficients
%        .inf,.sup  bounds for coefficients
%      .int       characteristic vector of integer variables
%      .F         structure with bounds for general constraints
%        .inf,.sup    outer enclosure
%        .iinf,.isup  inner enclosure
%  xbox         bound constraints for x (optional)
%        .inf,.sup    outer enclosure
%  mode         text output mode: 0=normal text, 1=html, 2=ampl format
%  equnums      nc+nx x 1 vector representing the current number of the
%               numerated constraints. first the general than the bound.
%  nextnum      next equation number
%
%  string       string cell representation of P or simple string if mode==1
%  equnums      nc+nx x 1 vector representing the current number of equs

function [string, equnums]=glprob2str(P, xbox, mode, equnums, nextnum)

global glconf;          % global configuration variable
HTMLCOLOR='#008F8F' ;   % html color
VSUBST=0;               % variable backsubstitution for univariate funcs
CLABELS=1;              % constraint labeling for ampl output
DIGITS=6;               % number of digits shown in each numeric value

% argcheck
if nargin<1,error('glprob2str: Not enough arguments!');end
if nargin<2,xbox.inf=[];xbox.sup=[];end
if nargin<3,mode=0;end
if nargin<4,equnums=[];end
if nargin<5,nextnum=max(equnums)+1;end

% init
nx=length(P.Q.int);
nu=length(P.uni);
[nc, nB]=size(P.Q.A.inf);
equnums=zeros(nc+nx,1);

% set constants from config
if isfield(glconf,'htmlcolor1'),HTMLCOLOR=glconf.htmlcolor1;end
if isfield(glconf,'vsubst'),VSUBST=glconf.vsubst;end
if isfield(glconf,'clabels'),CLABELS=glconf.clabels;end
if isfield(glconf,'digits'),DIGITS=glconf.digits;end

% create string representation of the basis functions
bas=cell(nB,1);                     % string representation of the basis.
[uni,J]=gluni2str(P,mode,DIGITS);   % univariate terms
if mode==2 
    % variable definitions for the ampl output
    if ~VSUBST, vars=cell(nx,1);
    else vars=cell(nx-nu,1);
    end
    vc=1;   % varable counter
end

if isfield(P,'origvarnams'), orig=P.origvarnams;
else orig=[];
end
        
% create string of the basis, and the variable definition for the ampl outp
for i=1:nx
    % linear terms
    ind1=find(J==i);
    if ~isempty(ind1) && VSUBST,
        % univariate func substituted as the basis
        bas{i}=uni{ind1};
    else
        bas{i}=glvar2str(i, mode, orig);
        if mode==2,
            % create variable definition string
            vars{vc}=['var ' setbounds(bas{i},xbox.inf(i),xbox.sup(i),...
                2,1,DIGITS) ';'];
            vc=vc+1;
        end
    end
    b1=bas{i};
    for j=1:nx
        % quadratic terms
        if i~=j
            ind2=find(J==j);
            if ~isempty(ind2) && VSUBST,
                bas{i*nx+j}=uni{ind2};
            elseif mode==2,
                bas{i*nx+j}=[b1 '*' glvar2str(j, mode, orig)];
            else
                bas{i*nx+j}=[b1 glvar2str(j, mode, orig)];
            end
        else
            if mode==1,    bas{i*nx+j}=[b1 '<sup>2</sup>'];
            else           bas{i*nx+j}=[b1 '^2'];
            end
        end
    end
end

% create string representation of the general constraints
A=(P.Q.A.inf+P.Q.A.sup)/2;
ncb=0;
if ~isempty(xbox) && size(xbox.inf,1)>0, ncb=nx+1; end
str=cell(nc+1+nu+1+ncb,1);
for i=1:nc
    % create the lhs of the constraint
    nz=0;out='';
    for j=1:nB
        c=A(i,j);
        if c~=0,
            if nz~=0 && c>=0, out=[out '+'];
            else nz=1;
            end
            if c==-1, out=[out '-'];
            elseif c~=1, out=[out num2str(c,DIGITS)];
            end
            if mode==2 && c~=1 && c~=-1, out=[out '*' bas{j}];
            else out=[out bas{j}];
            end
        end
    end
    if i==P.opt
        % objective function
        c=P.C(P.shift);
        if c~=0,
            if c>=0,out=[out '+'];end
            out=[out num2str(c,DIGITS)];
        end
        % equation number only if a non inf box is present
        if ~(isinf(P.Q.F.inf(i)) && isinf(P.Q.F.sup(i))) && ...
                ~isempty(nextnum),
            equnums(i,1)=nextnum;nextnum=nextnum+1; 
            if mode==1,
                out=['(<small>' num2str(equnums(i)) '</small>) ' out];
            else out=['(' num2str(equnums(i)) ') ' out];
            end
        end
        if mode==0,
           out=['minimize ' out];
        elseif mode==1,
           out=['<B><FONT COLOR="' HTMLCOLOR '">minimize </FONT></B>' out];
        elseif mode==2,
           if CLABELS, out=['minimize obj: ' out];
           else out=['minimize ' out];
           end
        end
        if (~isinf(P.Q.F.inf(i)) || ~isinf(P.Q.F.sup(i)))
            str{i+1}=setbounds(out, P.Q.F.inf(i), P.Q.F.sup(i),...
                mode, 0, DIGITS);
        else
            str{i+1}=out;
        end
    else
        % normal constraint
        if mode ~= 2 && ~isempty(nextnum)
            % equation numbering on
            equnums(i,1)=nextnum;nextnum=nextnum+1; 
            if mode==1,
                out=['(<small>' num2str(equnums(i)) '</small>) ' out];
            else 
                out=['(' num2str(equnums(i)) ') ' out];
            end
        end
        if mode==2, 
            % ampl constraint formatting
            out=setbounds(out, P.Q.F.inf(i), P.Q.F.sup(i), mode, 0,DIGITS);
            if CLABELS, str{i+1}=['s.t. c' num2str(i) ': ' out];
            else str{i+1}=['s.t. ' out];
            end
        else
            % non-ampl constraint formatting
            out=['         ' out];
            % print Fbox
            str{i+1}=setbounds(out,P.Q.F.inf(i),P.Q.F.sup(i),mode,0,DIGITS);
        end
    end
    if mode==2, str{i+1}=[str{i+1} ';']; end
end

% relocation of the objective
if P.opt >0 && P.opt <=nc
    % move objective to the front
    str{1}=str{P.opt+1};
    str{P.opt+1}=str{2};
    if mode==0,
        str{2}='    s.t.';
    elseif mode==1,
        str{2}=['    <B><FONT COLOR="' HTMLCOLOR '">s.t.</B></FONT>'];
    elseif mode==2,
        str{2}='';
    end
else str(1)=[];
end

% additional definitions or constraints of the univariate functions
if nu~=0 && ~VSUBST
    if mode==0,
        str{nc+2}='  define';
    elseif mode==1,
        str{nc+2}=[' <B><FONT COLOR="' HTMLCOLOR '">define</B></FONT>'];
    else
        str{nc+2}='';
    end
end

for i=1:nu
    ind=P.basis(i,3);b=uni{find(J==ind)};
    if ~VSUBST,
        % no univariate func substituted, add an additional definition
        vstr=glvar2str(ind, mode, orig);
        if mode==0 || mode==1,
            str{nc+2+i}=['         ' vstr ' := ' b];
        elseif mode==2,
            str{nc+2+i}=['s.t. c' num2str(nc+i) ': ' vstr '=' b  ';'];
        end
    elseif ~isempty(xbox) && size(xbox.inf,1) >0 && ...
            (~isinf(xbox.inf(ind)) || ~isinf(xbox.sup(ind)))
        % univariate func substituted but the bounds for the variable
        % are not infinite, add an additional constraint
        str{nc+2+i}=setbounds(b,xbox.inf(ind),xbox.sup(ind),mode,0, DIGITS);
        if mode==0,     str{nc+2+i}=['         ' str{nc+2+i}];
        elseif mode==1, str{nc+2+i}=['         ' str{nc+2+i}];
        elseif mode==2, str{nc+2+i}=['s.t. ' str{nc+2+i} ';'];
        end
    end
end


% bound constraints for the non-ampl output
if ~isempty(xbox) && size(xbox.inf,1) >0 && ~(all(isinf(xbox.inf)) && ...
        all(isinf(xbox.sup))) && mode~=2,
    if mode==0,     str{nc+nu+3}='    with';
    elseif mode==1, str{nc+nu+3}='    <B>with</B>';
    end
    if nextnum,
        [bstr, equn]=glbox2str(xbox,1:nx,mode,equnums(nc+1:end,1),nextnum);
        str(nc+3+nu+(1:length(bstr)))=bstr;
        equnums=[equnums(1:nc,1);equn];
    else
        bstr=glbox2str(xbox,1:nx,mode);
        str(nc+3+nu+(1:length(bstr)))=bstr;
    end
end

% add variable definitions for the ampl output
if mode==2, str=[vars;str];end

% remove empty entries from cell
ind=[];
for i=1:length(str)
    if ~isempty(str{i}),ind=[ind i];end
end
j=1;string=cell(length(ind),1);
for i=ind
    string{j}=str{i};
    j=j+1;
end

% create string from cell for html documents,
if mode==1,
    out='<PRE><I>';
    for i=1:length(string)
        out=[out string{i} '\n'];
    end
    out=[out '</I></PRE>\n'];
    string=out;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% setbounds %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% str=setbounds(instr, xinf, xsup, mode, isvar)
%
% make an ampl string of an (in)equation: instr \in [xinf, xsup]
%
% instr         string representing a constraint or a variable
% [xinf,xsup]   box in which the constraint or the variable is contained
% mode          text output mode: 0=normal text, 1=html, 2=ampl format
% isvar         instr represents a single variable 0:no 1:yes (ampl)
% digits        number of diitsg
%
% str         	string representation of instr \in box

function str=setbounds(instr, xinf, xsup, mode, isvar, digits)
% glratstr removed!!!
if ~isinf(xinf) && ~isinf(xsup)
    if xinf==xsup, 
        % equality
        if mode==1, str=[instr '<B> = </B>' num2str(xinf,digits)];
        else str=[instr ' = ' num2str(xinf,digits)];
        end
    else
        % in a box
        bs=['[' num2str(xinf,digits) ',' num2str(xsup,digits) ']'];
        if mode==0, str=[instr ' in ' bs];
        elseif mode==1, str=[instr '<B> &isin;; </B>' bs];
        elseif isvar,
            str=[instr ' >= ' num2str(xinf,digits) ', <= ' num2str(xsup,digits)];
        else str=[num2str(xinf,digits) ' <= ' instr ' <= ' num2str(xsup,digits)];
        end
    end
elseif ~isinf(xinf), 
    if mode==1, str=[instr '<B> &ge; </B>' num2str(xinf,digits)];
    else str=[instr ' >= ' num2str(xinf,digits)];
    end
elseif ~isinf(xsup), 
    if mode==1, str=[instr '<B> &le; </B>' num2str(xsup,digits)];
    else str=[instr ' <= ' num2str(xsup,digits)];
    end
else str=instr;
end