%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gluni2str.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [unistr, J]=gluni2str(P, mode, digits)
%
% Get the string representation for all the univariate function in P.
%
% P            structure containing the internal problem definition
%   P.basis    ne x 2 array defining nonquadratic basis functions
%   P.uni      nu x 1 cell array defining univariate functions
% mode         text output mode: 0=normal text, 1=html, 2=ampl format
% digits       number of digits show in numerical values
%
% unistr       string cell representation of the univariate function(s)
% J            index set of the substituted variables: J(2)=4 means
%              x_4:=unistr{2}

function [unistr, J]=gluni2str(P, mode, digits)

% argcheck
if nargin <1, error('gluni2str: Not enough arguments!');end
if nargin <2, mode=0;end

% init output and stack
unistr=cell(0);J=[];
stack=[];stack=clearstack(stack);   % string stack
brack=[];brack=clearstack(brack);   % bracketing indicator stack
                                    % 0=no brackests, 
                                    % 1=braketed,
                                    % 2=variable or positive constant
for i=1:length(P.Q.int)
    %  iterate through all variables
    if ~isempty(P.basis) && any(P.basis(:,3)==i)
        % substituted variable found
        ind=find(P.basis(:,3)==i); % index of basis function
        var=glvar2str(P.basis(ind,2), mode, P.origvarnams);
        bas=P.uni{ind};
        while 1,
            [op,bas]=strtok(bas);
            if isempty(op); break; end;
            switch op
                % variable
                case 'z',
                    stack=push(stack, var); brack=push(brack, 2);
                % unarian operators
                case 'minus',
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'sqr', 
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'sqrt',
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'exp', 
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'log',
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'sin', 
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'cos', 
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'atan',
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'abs',  
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'sign', 
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'pos',  
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                case 'neg',  
                    [dat, stack]=pop(stack);[br, brack]=pop(brack);
                    if br~=1, dat = ['(' dat ')']; end
                    stack=push(stack, [op dat]); brack=push(brack, 0);
                
                % binary operators
                case '+',    
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    stack=push(stack, ['(' dat2 op dat1 ')']);
                    brack=push(brack, 1);
                case '-',    
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    stack=push(stack, ['(' dat2 op dat1 ')']);
                    brack=push(brack, 1);
                case '*',    
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    if mode ==1 
                        stack=push(stack, [dat2  dat1]);
                    else
                        stack=push(stack, [dat2 op dat1]);
                    end
                    brack=push(brack, 0);           
                case '/',     
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    stack=push(stack, [dat2 op dat1]);
                    brack=push(brack, 0);
                case '^', 
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    if br1==0, dat1 = ['(' dat1 ')']; end
                    if br2==0, dat1 = ['(' dat1 ')']; end
                    if mode ==1 
                        stack=push(stack, [dat2 '<sup>' dat1 '</sup>']);
                    else
                        stack=push(stack, [dat2 op dat1]);
                    end
                    brack=push(brack, 0);
                case 'min',
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    stack=push(stack, [op '(' dat2 ',' dat1 ')']);
                    brack=push(brack, 0);
                case 'max',              
                    [dat1, stack]=pop(stack);[br1, brack]=pop(brack);
                    [dat2, stack]=pop(stack);[br2, brack]=pop(brack);
                    stack=push(stack, [op '(' dat2 ',' dat1 ')']);
                    brack=push(brack, 0);
                otherwise
                    % index of a constant
                    [ind,ct,err1]=sscanf(op,'%i');
                    if isempty(err1),
                        const=P.C(ind);
                        stack=push(stack,num2str(const,digits));
                        if const>=0, brack=push(brack, 2);
                        else brack=push(brack, 0);
                        end
                    else
                        % unrecognized token
                        error(['gluni2str: unrecognized token: ' op]);
                    end;
            end;
        end
        [txt, stack]=pop(stack);[br, brack]=pop(brack);
        if stacksize(stack) ~= 0 || isempty(txt), 
            error(['gluni2str: parsing error: ' bas]);
        end
        if br~=0, txt=txt(2:end-1); end
        unistr=[unistr;{txt}];J=[J i];
    end
end

%%%%%%%%%%%%%%%%%%
%STACK OPERATIONS%
%%%%%%%%%%%%%%%%%%

% push data to stack
function stack=push(stack, dat)
    stack.size=stack.size+1;
    stack.data{stack.size}=dat;

% pop last daat from stack
function [dat, stack]=pop(stack)
    if isempty(stack.data)
        %no data on stack
        dat=[];
    else
        dat=stack.data{stack.size};
        stack.data(stack.size)=[];
        stack.size=stack.size-1;
    end
    
% depth of the stack
function siz=stacksize(stack)
    siz=stack.size;
    
% clear the stack
function stack=clearstack(stack)
    stack.data=[];
    stack.size=0;