%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% glcin2sdpt3.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [blk,At,C,b,conv] = glcin2sdpt3(cin)
%
% conversion of the internal conic input format to the sdpt3 input format.
% Global constants:
% INFTY = positive and negative infinity
% TINY  = intervals of smaller width as TINY, are treated as points
%
% cin: *CONIC PROBLEMS INPUT FORMAT* 
%   ny dimension of y
%
%   objective function min b*y
%       obj.b       ny x 1 sparse vector
%
%   nl linear A*y \in \b
%       lin.A       nl x ny sparse matix
%       lin.b       nl x 2 matrix b(i)=[lin.b(i,1) lin.b(i,2)]
%
%   nql quadratics ||b_i-A_i*y||<=d_i-c_i*y, i=1:nql
%       quadl       nql cell with quadl{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%
%   nqb quadratics 1/2*||b_i-A_i*y||^2<=u*v, with  u:=(d_i-c_i*y)>=0,
%                   v:=(f_i-e_i*y)>=0 for i=1:nql
%       quadb       nqb cell with quadb{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%          .e       1 x ny sparse vector
%          .f       constant
%
%   ns semidefinite G0_i-sum(G_(i,j) y_j) psd, j=1:ny, i=1:ns
%       semi        ns cell with sem{i] having the fields
%          .G0      m x m sparse matrix
%          .G       ny cell with m x m sparse matrix entries
%
%   EXAMPLE
% 
%   min y1+y2        => (1 1)(y1 y2)'
%
%   y2>=0.2          => (0 1)(y1 y2)'\in [0.2 inf]
%   y1-y2=2          => (1 -1)(y1 y2)'\in [2 2]
%   y1^2+y2^2 <= 4   => ||0 - (1 1)(y1 y2)'|| <= 4 + (0 0)(y1 y2)'
%   psd              => (1 0; 0 1)+(1 2; 3 4)y1+(4 5; 5 6)y2 psd 
%
%   obj.b=sparse([1 1]);
%   lin.A=sparse([0 1;1 -1]);lin.b=[0.2 inf;2 2]
%   quadl{1}.A=sparse([1 1]);quadl{1}.b=[0];
%           quadl{1}.c=sparse([0 0]);quadl{1}.d=4;
%   quadb=[];
%   semi{1}.G0=[1 0; 0 1];semi{1}.G{1}=[1 2; 3 4];semi{1}.G{2}=[4 5; 5 6];
%   
%   SDPT3 DUAL INPUT FORMAT
%
%   max b^Ty
%   s.t.  cl-Al*y in Kl={y \in R^nl}                           linear terms
%         cq_i-Aq_i*y in Kq={y \in R^nq : y(1) >= ||y(2:ny)||} quadratics
%         cs_j-As_j*y in Ks={y \in R^(m^2): mat(y) is psd}     psd
%
% Al        is an ny x nl matrix,
% Aq_i      is an ny x nq_i matrix with 1<=i<=nnq,
% As_j      is ns_j x ns_j matrix with 1<=j<=nns,
% m         maximum of 1, nnq and nns
% k         the number of the different types used: unbound ('u'),
%           linear ('l'), quadratic('q') and semidefinite ('s'),
%           (max 4 min 1)
%
% blk       k x 2 cell with blk{i,1}='u','l','q' or 's', and blk{i,1} is
%           the dimension of the corresponding type.
% At        k x m cell of various matrices Al,Aq_i and As_j
% C         k cell of constants terms (matrices or vectors)
% b         ny objective vector
% conv      conversion info structure with entries
%   .lin    nl x 2 constraint transformation vector, lin(4)=[3 1] means 
%           that the 3rd linear constraint in the orginal representation is
%           coded in the 4th entry in the new repr with an operator >.
%           The operator types are given as -1 : <, 0 : =, 1 : >

function [blk,At,C,b,conv] = glcin2sdpt3(cin)

global glconf;      %gloptlab configuration file

INFTY=inf;          % infinity
TINY=1e-6;          % tiny enties

% argcheck
if nargin <1, error('glcin2sdpt3: Not enough arguments!'); end

%set configuration
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

%convert
conv.nl=0;l=0;ql=0;qr=0;s=0;
if (isfield(cin, 'lin') && ~isempty(cin.lin))
    wid=cin.lin.b(:,2)-cin.lin.b(:,1);
    if any(wid<0),error('cin2sedumi: linear constr bounds wrong!');end
    u=any(wid<=TINY);  %equalities present
    l=any(wid> TINY);  %inequs present
    conv.nl=size(cin.lin.b(:,1));
end
if (isfield(cin, 'quadl') && ~isempty(cin.quadl)), ql=length(cin.quadl);end
if (isfield(cin, 'quadr') && ~isempty(cin.quadr)), qr=length(cin.quadr);end
if (isfield(cin, 'semi') && ~isempty(cin.semi)), s=length(cin.semi);end
k=l+ql+qr+s;
m=max([1, ql, qr, s]);
conv.lin=[];

blk=cell(k,2);
At=cell(k,m);
C=cell(k,1);
b=-cin.obj.b; %since the input is in the dual format, max(-bx)
if size(b,1)<size(b,2),b=b';end

t=1;
if (isfield(cin, 'lin') && ~isempty(cin.lin))
    % (I) Convert linear terms
    if u, % (1) Convert equality constraints
        eind=find(wid<=TINY);
        At{t,1}=cin.lin.A(eind,:);
        C{t}=cin.lin.b(eind,1)+wid(eind)/2;
        blk{t,1}='u'; blk{t,2}=nnz(eind);
        conv.lin=[conv.lin; [eind zeros(length(eind),1)]];
        t=t+1;
    end
    if l, % (2) Convert inequality constraint
        iind=find(wid>TINY);
        % constraints with finite lower bounds
        infind=iind;supind=iind; % finite lower and upper bound indicators
        infind(cin.lin.b(iind,1)<-INFTY)=[];
        supind(cin.lin.b(iind,2)> INFTY)=[];
        At{t,1}=[-cin.lin.A(infind,:);cin.lin.A(supind,:)];
        C{t}=[-cin.lin.b(infind,1);cin.lin.b(supind,2)];
        blk{t,1}='l'; blk{t,2}=nnz(infind)+nnz(supind);
        conv.lin=[conv.lin; [infind ones(length(infind),1)]; ...
            [supind -ones(length(supind),1)]];
        t=t+1;
    end
end
if ql,
    % (II) Convert lquadratic terms
    for i=1:length(cin.quadl)
        At{t,1}=[cin.quadl{i}.c;cin.quadl{i}.A];
        C{t}=[cin.quadl{i}.d;cin.quadl{i}.b];
        blk{t,1}='q'; blk{t,2}=length(C{t});
        t=t+1;
    end
end
if qr,
    % (II) Convert rquadratic terms
    % since for u':=u+v and v':=u-v, u'^2-v'^2 = 2*u*v holds,
    % ||z||^2<=u*v=u'^2-v'^2 can be written as ||[z;v']||<=u'.
    % u'=(d+f)-(c+e)*y and v'=(d-f)-(c-e)*y
    for i=1:length(cin.quadr)
        cpe=cin.quadr{i}.c+cin.quadr{i}.e;
        dpf=cin.quadr{i}.d+cin.quadr{i}.f;
        At{t,1}=[cpe;                           %u' linear term (rhs)
                 cin.quadr{i}.c-cin.quadr{i}.e; %v' linear term
                 cin.quadr{i}.A];               %z  linear terms
        C{t}=[dpf;                              %u' constant term (rhs)
              cin.quadr{i}.d-cin.quadr{i}.f;    %v' constant term
              cin.quadr{i}.b];                  %z  constant terms
        blk{t,1}='q'; blk{t,2}=length(C{t});
        t=t+1;
        
        %add u>=0 v>=0
        %At{t,1}=[cin.quadr{i}.c;cin.quadr{i}.e];
        %C{t}=[cin.quadr{i}.d;cin.quadr{i}.f];
        %blk{t,1}='l'; blk{t,2}=2;
        %t=t+1;
        
        % add u+v>=0
        %At{t,1}=cpe;C{t}=dpf;blk{t,1}='l'; blk{t,2}=1;
        %t=t+1;
    end
end
if s,
    % (IV) Convert semidefinite terms
    for i=1:length(cin.semi)
        nx=size(cin.semi{i}.G0,1);
        C{t}=cin.semi{i}.G0;
        At{t,1}=sparse([],[],[],nx*(nx+1)/2,nx);
        bl{1,1}='s'; bl{1,2}=size(C{t},1);
        for j=1:length(cin.semi{i}.G)
            %%svec replaced by glsshape, to use old svec
            %%of SDPT3 svec.m, msxvec.c, msxvec.dll and msxvec.mexglx
            %%must be reachable
            %%At{t,1}(:,j)=svec(bl,cin.semi{i}.G{j})
            At{t,1}(:,j)=glsshape(cin.semi{i}.G{j});
        end
        blk{t,1}='s'; blk{t,2}=size(C{t},1);
        t=t+1;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsshape %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% return the svec of a symmetric matrix in a nx*(nx+1)/2 vector with 
% svec(A):={A11,sqrt(2)A21,A22,sqrt(2)A13,..,A33,sqrt(2)An1 ... Ann}
function vec=glsshape(A)
if (A+A')/2 ~= A, error('glsshape: matrices must be symmetic!'); end
nx=size(A,1);
vec=sparse([],[],[],nx*(nx+1)/2,1); k=1; sq=sqrt(2);
for i=1:nx
    vec(k:k+i-1)=A(i,1:i);
    vec(k:k+i-2)=vec(k:k+i-2)*sq;
    k=k+i;
end
    