%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% glcin2sedumi.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [At,b,c,K,conv] = glcin2sedumi(cin)
%
% conversion of the internal conic input format to the sedumi input format.
%
% Global constants:
% INFTY = positive and negative infinity
% TINY  = intervals of smaller width as TINY, are treated as points
%
% cin: *CONIC PROBLEMS INPUT FORMAT* 
%   ny dimension of y
%
%   objective function min b*y
%       obj.b       1 x ny sparse vector
%
%   nl linear A*y \in \b
%       lin.A       nl x ny sparse matix
%       lin.b       nl x 2 matrix b(i)=[lin.b(i,1) lin.b(i,2)]
%
%   nql quadratics ||b_i-A_i*y||<=d_i-c_i*y, i=1:nql
%       quadl       nql cell with quadl{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%
%   nqb quadratics ||b_i-A_i*y||^2<=u*v, with  u:=(d_i-c_i*y)>=0,
%                   v:=(f_i-e_i*y)>=0 for i=1:nql
%       quadb       nqb cell with quadb{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%          .e       1 x ny sparse vector
%          .f       constant
%
%   ns semidefinite G0_i-sum(G_(i,j) y_j) psd, j=1:ny, i=1:ns
%       semi        ns cell with sem{i] having the fields
%          .G0      m x m sparse matrix
%          .G       ny cell with m x m sparse matrix entries
%
%   EXAMPLE
% 
%   min y1+y2        => (1 1)(y1 y2)'
%
%   y2>=0.2          => (0 1)(y1 y2)'\in [0.2 inf]
%   y1-y2=2          => (1 -1)(y1 y2)'\in [2 2]
%   y1^2+y2^2 <= 4   => ||0 - (1 1)(y1 y2)'|| <= 4 + (0 0)(y1 y2)'
%   psd              => (1 0; 0 1)+(1 2; 3 4)y1+(4 5; 5 6)y2 psd 
%
%   obj.b=sparse([1 1]);
%   lin.A=sparse([0 1;1 -1]);lin.b=[0.2 inf;2 2]
%   quadl{1}.A=sparse([1 1]);quadl{1}.b=[0];
%           quadl{1}.c=sparse([0 0]);quadl{1}.d=4;
%   quadb=[];
%   semi{1}.G0=[1 0; 0 1];semi{1}.G{1}=[1 2; 3 4];semi{1}.G{2}=[4 5; 5 6];
%
% A, b, c, K: *Sedumi DUAL INPUT FORMAT*
%
%   max b^Ty
%   s.t.  c-A'*y in K
%
%   calling: [x, y, info] = sedumi(A, b, c, K, pars)   
%
%   Where c \in R^k x \in R^ny A is an ny x k matrix and K is a cone with
%       K=R^K.f x R+^K.l x (Qcone_1 x ... x Qcone_K.q) x (Rcone_1 x ... 
%               x Rcone_K.r) x (Scone_1 x ... x Scone_K.s)
%       with
%       Qcone_i={ (y1,y2) \in R x C^K.q[i]-1 : y1 >= ||y2|| }
%       Rcone_j={ (y1,y2,y3) \in R x R x C^K.r[j]-2 : y1y2 >= 1/2*||y3||^2
%                 , y1+y2>=0 }
%       Scone_k={ x \in C^(K.s[k]^2) : mat(x) is positive semidefinite }
%                   ( + Hermitian if in the primal form )
%       and k=K.f + K.l + sum(K.q) + sum(K.r) + sum(K.s^2) with
%
%   K.f           The number of free primal variables, number of equality
%                 constraints in the dual form
%   K.l           The number of the nonegativity constraints
%   K.q           A list of dimensions of quadratic cone constraints
%   K.xcomplex    A list of primal variables in the f,l,q,r blocks that are
%                 allowed to have nonzero imaginary part.
%   K.s           A list of orders of positive semidefiniteness constraints
%   K.scomplex    A list of matrix variables that are restriced to be
%                 Hermitian positive semidefinite
% conv      conversion info structure with entries
%   .lin    nl x 2 constraint transformation vector, lin(4)=[3 1] means 
%           that the 3rd linear constraint in the orginal representation is
%           coded in the 4th entry in the new repr with an operator >.
%           The operator types are given as -1 : <, 0 : =, 1 : >
%
%   CONVERSION :
%
%   K.f=nnz(lin.b(:,1)-lin.b(:,2)<=TINY)
%   K.l=nnz(lin.b(:,1)>-INFTY) + nnz(lin.b(:,2)<INFTY)-2*K.f
%   K.q(i)=length(quadl{i}.b)+1 , i=1:length(quadl)
%   K.r(i)=length(quadr{i}.b)+2 , i=1:length(quadr)
%   K.s(i)=size(semi.G0,1), i=1:length(semi)
%   k= K.f + K.l + sum(K.q) + sum(K.r) + sum(K.s^2)
%   A' is k x ny, c is k dim, b is ny dim
%
%   EXAMPLE CONVERTED:
%   K =  { l:3        q: 2      s: 2 }
%   At = [0 -1;-1 1;  0 0;1 1;  1 4; 3 5; 2 5; 4 6]
%   c =  [-0.2 -2 2    4 0       1 0 0 1]'
%   b =  [1 1]'
function [At,b,c,K,conv] = glcin2sedumi(cin)

global glconf;      %gloptlab configuration file

INFTY=inf;          % infinity
TINY=1e-6;          % tiny enties

% argcheck
if nargin<1, error('glcin2sedumi: Not enough arguments!'); end
deb=0;

%set configuration
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end
%convert
ny=length(cin.obj.b);
k=0;
conv.nl=0;
if (isfield(cin, 'lin') && ~isempty(cin.lin))
    wid=cin.lin.b(:,2)-cin.lin.b(:,1);
    if any(wid<0),error('glcin2sedumi: linear constr bounds wrong!');end
    f=nnz(wid<=TINY);
    l=nnz(cin.lin.b(:,1)>-INFTY) + nnz(cin.lin.b(:,2)<INFTY)-2*f;
    if f > 0, K.f=f; end; if l > 0, K.l=l; end; k=k+f+l;
    conv.nl=l+f;
end
if (isfield(cin, 'quadl') && ~isempty(cin.quadl))
  ind=1:length(cin.quadl);K.q(ind)=length(cin.quadl{ind}.b)+1;k=k+sum(K.q);
end
if (isfield(cin, 'quadr') && ~isempty(cin.quadr))
  ind=1:length(cin.quadr);K.r(ind)=length(cin.quadr{ind}.b)+2;k=k+sum(K.r);
end
if (isfield(cin, 'semi') && ~isempty(cin.semi))
  ind=1:length(cin.semi);K.s(ind)=size(cin.semi{ind}.G0,1);k=k+sum(K.s^2);
end

if deb, gldisp('K',K,'k',k); end

At=sparse([],[],[],k,ny,100);
c=sparse([],[],[],k,1);
b=-cin.obj.b; %since the input is in the dual format, max(-bx)
row=1;
conv.lin=[];


if (isfield(cin, 'lin') && ~isempty(cin.lin))
    % (I) Convert linear terms
    if isfield(K,'f')
        % (1) Convert equality constraints
        eind=find(wid<=TINY);
        At(row:row+K.f-1,:)=cin.lin.A(eind,:);
        c(row:row+K.f-1)=cin.lin.b(eind,1)+wid(eind)/2;
        conv.lin=[conv.lin; [eind zeros(length(eind),1)]];
        row=row+K.f;
    end
    if isfield(K,'l')
        % (2) Convert inequality constraint
        % constraints with finite lower bounds
        iind=find(wid>TINY);
        infind=iind;infind(cin.lin.b(iind,1)<-INFTY)=[];ni=length(infind);
        At(row:row+ni-1,:)=-cin.lin.A(infind,:);
        c(row:row+ni-1)=-cin.lin.b(infind,1);
        row=row+ni;
        % constraints with finite upper bounds
        supind=iind;supind(cin.lin.b(iind,2)> INFTY)=[];ns=length(supind);
        At(row:row+ns-1,:)=cin.lin.A(supind,:);
        c(row:row+ns-1)=cin.lin.b(supind,2);
        conv.lin=[conv.lin; [infind ones(length(infind),1)]; ...
            [supind -ones(length(supind),1)]];
        row=row+ns;
    end
end
if (isfield(cin, 'quadl') && ~isempty(cin.quadl))
    % (II) Convert lquadratic terms
    for i=1:length(cin.quadl)
        At(row,:)=cin.quadl{i}.c; c(row)=cin.quadl{i}.d; row=row+1;
        for j=1:size(cin.quadl{i}.A,1)
          At(row,:)=cin.quadl{i}.A(j,:);c(row)=cin.quadl{i}.b(j);row=row+1;
        end
    end
end
if (isfield(cin, 'quadr') && ~isempty(cin.quadr))
    % (III) Convert rquadratic terms
    for i=1:length(cin.quadr)
        At(row,:)=cin.quadr{i}.c; c(row)=cin.quadr{i}.d; row=row+1;
        At(row,:)=cin.quadr{i}.e; c(row)=cin.quadr{i}.f; row=row+1;
        for j=1:size(cin.quadr{i}.A,1)
          At(row,:)=cin.quadr{i}.A(j,:);c(row)=cin.quadr{i}.b(j);row=row+1;
        end
    end
end
if (isfield(cin, 'semi') && ~isempty(cin.semi))
    % (IV) Convert semidefinite terms
    for i=1:length(cin.semi)
        m=size(cin.semi{i}.G0,1)^2-1;
        c(row:(row+m))=reshape(cin.semi{i}.G0,m+1,1);
        for j=1:length(cin.semi{i}.G)
          At(row:row+m,j)=reshape(cin.semi{i}.G{j},m+1,1);
        end
        row=row+m;
    end
end
if deb, gldisp('A',full(At),'b',full(b),'c',full(c)); end
