%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% gllin2lpsolve.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [c,A,b,type,lb,ub,conv] = gllin2lpsolve(pin)
%
% conversion of the internal linear input format to the lpsolve input 
% format.
%
% Global constants:
% INFTY = positive and negative infinity
% TINY  = intervals of smaller width as TINY, are treated as points
%
% pin: *LINEAR PROBLEMS INPUT FORMAT* 
%   ny dimension of y
%
%   objective function min b*y
%       obj.b       1 x ny sparse vector
%
%   nl linear A*y \in \b
%       pin.A       nl x ny sparse matix
%       pin.b       nl x 2 matrix b(i)=[pin.b(i,1) pin.b(i,2)]
%
%   EXAMPLE
% 
%   min x1+x2        => (1 1)(x1 x2)'
%
%   x2>=0.2          => (0 1)(x1 x2)'\in [0.2 inf]
%   x1-x2=2          => (1 -1)(x1 x2)'\in [2 2]
%   x1+2x2 \in [1 4] => (1 2)(x1 x2)'\in [1 4]
%
%   resulting
%   obj.b=sparse([1 1]);
%   lin.A=sparse([0 1;1 -1;1 2]);lin.b=[0.2 inf;2 2;1 4]
%
% c,A,b,type,lb,ub: *LpSolve INPUT FORMAT*
%
% min  c^T x 
% s.t. Ax <> b (*)
%      <> is determined linewise by type: 
%         type = -1 : <
%         type =  0 : =
%         type =  1 : >
%     lb <= x <= ub
% conv      conversion info structure with entries
%   .nl     number of linear constaints in the original problem
%   .lin    na x 2 constraint transformation vector, lin(4)=[3 1] means 
%           that the 3rd linear constraint in the orginal representation is
%           coded in the 4th entry in the new repr with an operator >.
%           The operator types are given as -1 : <, 0 : =, 1 : >
%   .bound  nb x 1 bound constraint transformation cell, bound{1}=[2 4]
%           means that the first bound constraint entry was produced
%           by removing the constraint number 2: cx_1 \in \b_2 and 4:
%           cx_1 \in b_4 from the input form.
%   
%
%   CONVERSION :
%   (1)Objective
%   c=obj.b
%
%   (2)Bound constraints
%   nx=length(obj.b)                %number of the variables
%   ind(i)=(nnz(lin.A(:,i))==1);    %indices of the bound constarints
%   lb=-inf+zeros(0,nx);ub=lb=inf+zeros(0,nx)
%   lb(ind)=lin.b(ind,1);ub(ind)=lin.b(ind,2); %bound constraints set
%  
%   (3)Remove all bound constraints from A and b
%   lin.A(ind,:)=[];lin.b(ind)=[];
%
%   (4)Determine (in)equality type
%   eq=find(lin.b(2)-lin.b(1)<=TINY)%equalities
%   ll=find(lin.b(1)<-INFTY)        %lhs of (*) less than the rhs of (*)
%   lg=find(lin.b(2)>INFTY)         %lhs of (*) greater than the rhs of (*)
%   bo=find(lin.b(1)>-INFTY && lin.b(2)<INFTY) && bo=~eq %double inequs
%   extra=length(bo);%extra number of constraints because of the two sided
%   type=ones(1,size(lin.A,1)+extra);
%   type(eq)=0;type(ll)=-1;type(lg)=1;type(bo)=-1;
%
%   (5)Create the matrix A
%   A=[lin.A; lin.A(bo,:)]
%
%   (5)Create the vector b
%   b=zeros(1,length(type));
%   b(eq)=lin.b(eq,1);b(ll)=lin.b(eq,2);b(lg)=lin.b(eq,1);
%   b(bo)=lin.b(eq,2);b(end-extra:end)=lin.b(eq,1);
%
%   EXAMPLE CONVERTED:
%
%   c=[1 1]
%   lb=[-inf;0.2],ub=[inf;inf] -> bound constraints removed: A=[1 -1;1 2]
%   type=[0;-1;1] -> last entry becasue of the two sided ineq
%   A=[1 -1;1 2;1 2] -> new entry becasue of the two sided ineq
%   b=[2;4;1];
%
%   and its representing
%   min x1+x2        => (1 1)(x1 x2)'
%
%   x \in [-inf inf;0.2 inf]
%
%   x1-x2=2         
%   x1+2x2 <= 4
%   x1+2x2 >= 1

function [c,A,b,type,lb,ub,conv] = gllin2lpsolve(pin)

global glconf;      %gloptlab configuration file

INFTY=inf;          % infinity
TINY=1e-6;          % tiny enties

% argcheck
if nargin <1, error('gllin2lpsolve: Not enough arguments!'); end
deb=0;

%set configuration
if isfield(glconf,'infty'),INFTY=glconf.infty;end 
if isfield(glconf,'tiny'),TINY=glconf.tiny;end
%convert

% (1)Objective
c=pin.obj.b;
conv.lin=[];conv.bound=[];
if deb, gldisp('c',full(c)); end
if (isfield(pin, 'lin') && ~isempty(pin.lin))
    [nc,nx]=size(pin.lin.A);
    conv.nl=nc;
    conv.bound=cell(nx,1);
    if deb, 
        gldisp('nc',nc,'nx',nx,'A',full(pin.lin.A),'b',full(pin.lin.b));
    end
    if any((pin.lin.b(:,2)-pin.lin.b(:,1))<0),
        error('gllin2lpsolve: linear constr bounds wrong!');
    end
    %(2)Set the bound constraints
    %get the indices of the bound constarints
    lb=-inf+zeros(nx,1);ub=inf+zeros(nx,1);
    ind=[];%indices of the bound constrains
    for i=1:nc,
        j=find(pin.lin.A(i,:)~=0);   %non zero entries in this row
        if(length(j)==1)
            %only one non-zero enty -> bound constraint for x_j 
            %get the value of the zero entry in this row
            a=pin.lin.A(i,j);
            %compute bounds on x_j
            if a>=0, xjl=pin.lin.b(i,1)/a;xju=pin.lin.b(i,2)/a;
            else xjl=pin.lin.b(i,2)/a;xju=pin.lin.b(i,1)/a;
            end
            %set new lower and upper bound on x_j
            lb(j)=max(lb(j),xjl);ub(j)=min(ub(j),xju);
            conv.bound{j}=[conv.bound{j} i];
            ind=[ind i];
        end
    end
    if deb, gldisp('ind',ind,'x',[lb ub]); end
    
    %(3)Remove all bound constraints from A and b
    oldind=(1:nc)'; %old constraint numbering
    pin.lin.A(ind,:)=[];pin.lin.b(ind,:)=[];oldind(ind)=[];
    nc=size(pin.lin.A,1);
    if deb
        gldisp('nc',nc,'Anew',full(pin.lin.A),'bnew',full(pin.lin.b));
    end

    %(4)Determine (in)equality type
    wid=pin.lin.b(:,2)-pin.lin.b(:,1);
    %equalities
    eq=find(wid<=TINY);   
    %lhs of (*) less than the rhs of (*)
    ll=find(pin.lin.b(:,1)<=-INFTY);   
    %lhs of (*) greater than the rhs of (*)
    lg=find(pin.lin.b(:,2)>=INFTY);
    %two sided inequs without equalities
    bo=find(pin.lin.b(:,1)>-INFTY & pin.lin.b(:,2)<INFTY & wid>TINY);   
    %extra number of constraints because of the two sided
    ne=length(bo);
    %set types
    type=ones(size(pin.lin.A,1)+ne,1);
    type(eq,1)=0;     %equalities
    type(ll,1)=-1;    %(Ax)_i<=b_i
    type(lg,1)=1;     %(Ax)_i>=b_i
    type(bo,1)=-1;    %(Ax)_i<=b_i for the two sided
    %(Ax)_i>=b_i in the extra entries by default
    if deb, gldisp('eq',eq,'ll',ll,'lg',lg,'bo',bo,'type',type); end

    %(5)Create the matrix A
    A=[pin.lin.A; pin.lin.A(bo,:)];
    if deb, gldisp('A',full(A)); end
    
    %(6)conversion info stucture
    conv.lin=[[oldind; oldind(bo,1)] abs(type)];

    %(7)Create the vector b
    b=zeros(length(type),1);
    %equs, one sided inequs
    b(eq,1)=pin.lin.b(eq,1);
    b(ll,1)=pin.lin.b(ll,2);
    b(lg,1)=pin.lin.b(lg,1);
    %two sided inequs
    b(bo,1)=pin.lin.b(bo,2);
    b((end-ne+1):end,1)=pin.lin.b(bo,1);
    if deb, gldisp('b',full(b)); end
else
    error('gllin2lpsolve: linear program without constraints found!');
end

if ((isfield(pin, 'quadl') && ~isempty(pin.quadl)) ||...
   (isfield(pin, 'quadr') && ~isempty(pin.quadr)) ||...
   (isfield(pin, 'semi') && ~isempty(pin.semi)))
   error('gllin2lpsolve: linear program with non-linear constraints found!');
end
