%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gloptsolve.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x, y, info]=gloptsolve(solvername, cin);
%
% calls an external global solver named 'solvername' in oder to solve the 
% problem supplied in the structure cin
%
% solvername        name of the used solver: LpSolve, SeDuMi, or SDPT3
% cin               conic problems input structure, ny is the dim of y
% FOR ALL SOLVERS:
%   objective function min b*y
%     obj.b         1 x ny sparse vector
%   nl linear A*y \in \b
%       lin.A       nl x ny sparse matix
%       lin.b       nl x 2 matrix b(i)=[lineq.b(i,1) lineq.b(i,2)]
% ONLY FOR CONIC SOLVERS:
%   nql quadratics ||b_i-A_i*y||<=d_i-c_i*y, i=1:nql
%       quadl       nql cell with quadl{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%   nqb quadratics ||b_i-A_i*y||^2<=u*v, with  u:=(d_i-c_i*y)>=0,
%                   v:=(f_i-e_i*y)>=0 for i=1:nql
%       quadb       nqb cell with quadb{i} having the fields
%          .A       nq_i x ny sparse matrix
%          .b       nq_i x 1 sparse vector
%          .c       1 x ny sparse vector
%          .d       constant
%          .e       1 x ny sparse vector
%          .f       constant
%   ns semidefinite G0_i-sum(G_(i,j) y_j) psd, j=1:ny, i=1:ns
%       semi        ns cell with sem{i] having the fields
%          .G0      m x m sparse matrix
%          .G       ny cell with m x m sparse matrix entries
%
% x                 primal problem solution
% y                 dual problem solution (which form is always used)
% info              info structure of fields
%   .pinf           primal problem infeasible?
%   .dinf           dual problem infeasible?
%   .numerr         error flag: 0=ok, 1=inaccurate, 2=failed

function [x, y, info]=gloptsolve(solvername, cin, prt)

global glconf;      %global configuration variable
SPREC=1e-10;        %solver precision

% argcechk
if nargin <2, error('gloptsolve: Not enough arguments!'); end
if nargin <3, prt=0; end

% set constants from config
if isfield(glconf,'sprec'),SPREC=glconf.sprec;end

% output
if prt, disp(glcprogr2str(cin)); end

if strcmp(solvername,'LpSolve')
    %init
    if isfield(glconf,'lpsolvepath') && exist(glconf.lpsolvepath,'dir')==7
        if isempty(strfind(path,glconf.lpsolvepath))
            %init LpSolve
            gltoolinit('LpSolve', glconf.lpsolvepath);
        end
    else
        error('gloptsolve: wrong or non existent LpSolve path!');
    end
    % convert to LpSolve input form
    [c,A,b,type,lb,ub,conv] = gllin2lpsolve(cin);
    info.pinf=0;info.dinf=0;info.numerr=0;
    x=[];y=[];
    try
        lp = lp_maker (c,A,b,type,lb,ub,'',0,1);
        statuslpsolve=mxlpsolve('solve',lp);
        %get solution and status
        switch statuslpsolve
            case -2 % Out of memory
                % first unload problem from memory!
                mxlpsolve('delete_lp', lp);
                % now throw error
                error ('lp_solve has run out of memory during lp-relaxation');
            case 5 % bad accurarcy
                info.numerr=1;
                x = mxlpsolve('get_variables',lp);
                d = mxlpsolve('get_dual_solution',lp);
                y=glbacktrans(d, conv);
            case 0 % lp_solve has found an optimal solution
                %objective = mxlpsolve('get_objective',lp);
                x = mxlpsolve('get_variables',lp);
                d = mxlpsolve('get_dual_solution',lp);
                %basis = mxlpsolve('get_basis',lp,0);
                y=glbacktrans(d, conv);
            case 2 % infeasible
                info.pinf=1;
            case 3 % unbounded
                info.dinf=1;
            otherwise
                % first unload problem from memory!
                mxlpsolve('delete_lp', lp);
                % now throw error
                error (['An unknown status has occured: lpsolve status =',...
                    num2str(statuslpsolve)]);
        end;
        % delete lp from memory for all cases with well defined status messages;
        mxlpsolve('delete_lp', lp);
    catch
        err = lasterror;
        % Solver error
        info.pinf=0;info.dinf=0;info.numerr=2;
        x=[];y=[];
        msg='*FATAL LPSOLVE ERROR:';
        d=err.stack;
        for i=1:length(dbstack)
            msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
        end
        msg=[msg ' -> ' err.message];disp(msg);
        glerrorlog([{msg} ; {'Programm:'}; glcprogr2str(cin)]);
    end
elseif strcmp(solvername,'SeDuMi')
    %init
    if isfield(glconf,'sedumipath') && exist(glconf.sedumipath,'dir')==7
        if ~isfield(glconf,'actcsolver') || ...
                ~strcmp(glconf.actcsolver,solvername)
            %clean all conic solvers path
            glcleanpath;
            %set sedumi path
            gltoolinit('SeDuMi', glconf.sedumipath);
            %set actual solvername
            glconf.actcsolver=solvername;
        end
    else
        error('gloptsolve: wrong or non existent SeDuMi path!');
    end
    % convert to sedumi dual input form
    
    [A,b,c,K,conv] = glcin2sedumi(cin);
    % set output and precision
    if ~prt, pars.fid=0; end
    pars.eps=SPREC;
    if length(c)<=length(b)
        disp('Should have length(c) > length(b) in any sensible model');
        x=[];y=[];info=[]; return;
    end
    try
        % call sedumi
        [d,x,in]=sedumi(A',b,c,K,pars);

        %trasform langrangian muliplicators to the original format
        y=glbacktrans(d, conv);

        % output + show optimality condition
        if prt,
            res=c-A*x;
            gldisp('A',full(A'),'b',full(b),'c',full(c),'K',K,'res',...
                res,'opt',res'*x);
            if ~isfield(K,'f'), gldisp('eigRes',eigK(res,K)'); end
        end
        % set info
        info.pinf=in.pinf;
        info.dinf=in.dinf;
        info.numerr=in.numerr;
    catch
        err = lasterror;
        % Solver error
        info.pinf=0;info.dinf=0;info.numerr=2;
        x=[];y=[];
        msg='*FATAL SEDUMI ERROR:';
        d=err.stack;
        for i=1:length(dbstack)
            msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
        end
        msg=[msg ' -> ' err.message];disp(msg);
        glerrorlog([{msg} ; {'Programm:'}; glcprogr2str(cin)]);
    end
elseif strcmp(solvername,'SDPT3')
    %init
    if isfield(glconf,'sdpt3path') && exist(glconf.sdpt3path,'dir')==7
        if ~isfield(glconf,'actcsolver') || ...
                ~strcmp(glconf.actcsolver,solvername)
            %clean all conic solvers path
            glcleanpath;
            %set sdpt3 path
            gltoolinit('SDPT3', glconf.sdpt3path);
            %set actual solvername
            glconf.actcsolver=solvername;
        end
    else
        error('gloptsolve: wrong or non existent SDPT3 path!');
    end
   
    % convert todeb sdpt3 dual input format
    [blk,At,C,b,conv] = glcin2sdpt3(cin);
    
    %output
    if prt,
        gldisp('b',full(b));
        for i=1:length(C)
            disp([blk{i,1} ':' num2str(blk{i,2})])
            gldisp('Ci',full(C{i}));
            for j=1:size(At,2)
                gldisp(['A' num2str(j)],full(At{i,j}));
            end
        end
    end
    %conv=conv.lin,
    % set parameters
    OPTIONS.vers           = 1;
    OPTIONS.gam            = 0;
    OPTIONS.predcorr       = 1;
    OPTIONS.expon          = [1 1];
    OPTIONS.gaptol         = SPREC;%1e-8;
    OPTIONS.inftol         = 1e-8;
    OPTIONS.steptol        = 1e-6;
    OPTIONS.maxit          = 50;
    OPTIONS.printyes       = prt;%1;
    OPTIONS.scale_data     = 0;
    OPTIONS.randnstate     = 0;
    OPTIONS.spdensity      = 0.5;
    OPTIONS.rmdepconstr    = 1;
    OPTIONS.cachesize      = 256;
    OPTIONS.smallblkdim    = 15;
    
    try
        % call sdpt3
        [obj,dc,x,z,in] = sqlp(blk,At,C,b,OPTIONS);
        %info = [termination-code  iters  gap  pinfeas  dinfeas total-time]

        %transform the langrangian muliplicators to the original format
        d=[];
        for i=1:length(dc),
            d=[d; reshape(dc{i},size(dc{i},1)*size(dc{i},2),1)];
        end
        y=glbacktrans(d, conv);

        % set info
        stat=in(1);
        switch stat
            case -7, msg='Primal infeasibility has deteriorated too much.';
            case -6, msg='Maximum number of iterations reached.';
            case -5, msg='Progress is too slow.';
            case -4, msg='Schur complement matrix not positive definite.';
            case -3, msg='X, Z are not both positive definite.';
            case -2, msg='Steps in predictor too short.';
            case -1, msg='Lack of progress in predictor.';
            case  0, msg='Relative gap < infeasibility.';
            case  1, msg='Primal problem is suspected of being infeasible';
            case  2, msg='Dual problem is suspected of being infeasible';
            otherwise, msg=['Unknown SPDT status code: ' num2str(stat)];
        end
        if prt, disp(msg),gldisp('x',x,'y',y,'z',z); end
        if stat~=1,info.pinf=in(4);
        else info.pinf=1-in(4);
        end
        if stat~=2,info.dinf=in(5);
        else info.dinf=1-in(5);
        end
        info.numerr=0;
    catch
        err = lasterror;
        % Solver error
        info.pinf=0;info.dinf=0;info.numerr=2;
        x=[];y=[];
        msg='*FATAL SDPT3 ERROR:';
        d=err.stack;
        for i=1:length(dbstack)
            msg=[msg ' at ' d(i).file ' line ' num2str(d(i).line) ';'];
        end
        msg=[msg ' -> ' err.message];disp(msg);
        glerrorlog([{msg} ; {'Programm:'}; glcprogr2str(cin)]);
     end
else
    error(['Unkown solver : ' solvername '!']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcleanpath.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% glpath=glcleanpath;
%
% cleans the path of all conic solvers
%
% glpath    new path string

function glcleanpath
glpath=path;
%remove sedumi and sdpt3 path if set
if ~isempty([strfind(glpath,'SDPT3') strfind(glpath,'SeDuMi')])
    %remove old path (if any)
    ind=[0 find(glpath==':') length(glpath)+1];
    %cut path component conatinig the word 'SDPT3' or 'SeDuMi'
    for i=1:length(ind)-1
        if ~isempty([strfind(glpath(ind(i)+1:ind(i+1)-1),'SDPT3') ...
                strfind(glpath(ind(i)+1:ind(i+1)-1),'SeDuMi')]),
            glpath(ind(i)+1:min(ind(i+1),length(glpath)))='*';
        end
    end
    glpath(glpath=='*')=[];
    path(glpath);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glbacktrans.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% y=glbacktrans(y, conv)
%
% trasform langrangian muliplicators to the original format.
% non linear case not implemented!
%
% d         dual solution of the transformed problem
% conv      conversion structure with
%   .ln     number of linear constraints in the original formulation
%   .lin    na x 2 constraint transformation vector, lin(4)=[3 1] means 
%           that the 3rd linear constraint in the orginal representation is
%           coded in the 4th entry in the new repr with an operator >.
%           The operator types are given as -1 : <, 0 : =, 1 : >
%   .bound  nb x 1 bound constraint transformation cell, bound{1}=[2 4]
%           means that the first bound constraint entry was produced
%           by removing the constraint number 2: x_1 \in \b_2 and 3:
%           x_1 \in b_3 from the input form. (not used)
% y         dual solution of the original problem

function y=glbacktrans(d, conv)
%d
%conv.lin
%for i=1:length(conv.bound)
%    disp ([num2str(i) ':' mat2str(conv.bound{i})]);
%end
%error('ee');
y=zeros(conv.nl,1);
if ~isempty(conv.lin)
    ind1=find(conv.lin(:,2)>=0); %all ax>=b or ax=b
    ind2=find(conv.lin(:,2)<0);  %all ax<=b
    if ~isempty(ind1),y(conv.lin(ind1,1))=d(ind1);end
    if ~isempty(ind2),
        y(conv.lin(ind2,1))=y(conv.lin(ind2,1))-d(ind2);
    end
end
%no multiplikators for the bound constraints

