%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glbest.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function best=glbest(best, x, opt)
% 
% compare the given best point and the best point stored in the box x.
% The one with the lower feasibilty distance or better objective is
% returned.
%
% best          found point (can be empty)
%   .x          nx x 1 point
%   .fx         nc x 1 function value Aq(x)
%   .fd         feasibility distance of the found point
%   .status     status of the search: feasible, local minima, maxiter
%   .zb         a box containig a feasible point, or empty if the verifica
%               tion failed
% x             boxes of the same dimension (can be empty)
%   .inf        nx x 1 lower bounds
%   .sup        nx x 1 upper bounds
%   .best       most feasible point (can be missing), same fields as best
% opt           index of the objective function 1<=opt<=nc, when this arg
%               is given and both best points are feasible, the point
%               with the lower objective value is selected.
%
% best          the best point (can be empty)

function best=glbest(best, x, opt)

global glconf;      % global configuration variable
FEAS=1e-6;          % feasibility tolerance

% argcheck
if nargin<2, error('glbest: Not enough arguments!')
elseif isintval(x), error('glbest: Cannot be appied to intervals!');
end

% set configuration
if isfield(glconf,'feast'),FEAS=glconf.feast;end

if ~isempty(x) && isfield(x,'best')
    % x contains a best point
    if isempty(best), best=x.best; % best is empty use the best from x
    elseif length(best.x)~=length(x.best.x),
        error('glbest: Dimension mismatch!');
    elseif (nargin>2 && opt ~= 0) && best.fd<FEAS && x.best.fd<FEAS
        % objective index supplied and both solution are feasible
        if best.fx(opt) > x.best.fx(opt), best=x.best;end
    elseif best.fd > x.best.fd, best=x.best; % the best of x is better
    end
end
