%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glbilmask.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [M, perm]=glbilmask(Q, prt)
% 
% finds the mask of variable dependecies, trying to find an optimal
% permutation in order to have a minimal fillin of the bilinear terms
%
% A               1 x (nx+nx^2) sparse matrix 
%   .inf,.sup     optional bounds for coefficients
% prt             text output flag       0 = no output 1 = output
%
% M               nx x nx lower triangular 0-1 mask matrix, of
%                 bilinear and quadratic variable adjacency
% perm            the variable permutation used to get the mask
% Ad              diffrence between the original quadratic adjacency matrix
%                 and the found mask M

function [M, perm, Ad]=glbilmask(A, prt)

% argcheck
if nargin<1, error('glbilmask: Not enough arguments!'); end
if nargin<2, prt=0; end

%init
if isstruct(A), nx=((1+4*size(A.inf,2))^0.5-1)/2;
else nx=((1+4*size(A,2))^0.5-1)/2;
end
if floor(nx)~=nx, 
    gldisp('nx',nx);
    error('glbilmask: A has wrong number of rows!');
end

%nonzero pattern
if isstruct(A), nz=(A.inf~=0 | A.sup~=0);
else nz=A~=0;
end

%make adjacency matrix
Ad=(sum(nz(:,nx+1:nx*(nx+1)),1)>0);
Ad=sparse(reshape(Ad,nx,nx));
Ad=(Ad+Ad')~=0;
if prt, gldisp('Ad',full(Ad));end

%symmetric approximate minimum degree permutation.
Pamd = symamd(Ad);
[count,h,parent,post,Ramd] = symbfact(Ad(Pamd,Pamd));

%symmetric reverse Cuthill-McKee permutation.
Prcm = symrcm(Ad);
[count,h,parent,post,Rrcm] = symbfact(Ad(Prcm,Prcm));

if prt, 
    gldisp('Pamd',Pamd,'Ramd',full(Ramd),'nnzRamd',nnz(Ramd),'Prcm',Prcm,...
    'Rrcm',full(Rrcm),'nnzRrcm',nnz(Rrcm));
end

%choose best permutation
if nnz(Ramd)<nnz(Rrcm), perm=Pamd;M=Ramd';
else perm=Prcm;M=Rrcm';
end

%delete all 1 in the diag for only linear variabels
for i=1:nx, if sum(Ad(i,2:end))==0, M(i,i)=0; end; end

%compute diffrence between the original adjacency matrix and M
if nargout==3, Ad=tril(M-Ad(perm,perm)); end

%test
%nc=10,nx=5,A.inf=sparse(rand(nc,(nx+1)*nx)>0.9);
%A.sup=A.inf;full(A.inf);[M, perm, Ad]=glbilmask(A,1);perm,full(M),full(Ad)
