%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glboxpart.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [boxes,part]==glboxpart(x, part)
% 
% Partition a box into subboxes. Fully infinite components or points (where 
% x(i).sup=<-INF or x(i).inf>INF orx(i).inf=x(i).sup) are not splitted.
% Infinite components (where x(i).inf=<-INF or x(i).sup>INF) are splitted 
% in one (or two) fully infinite part(s), and n-1 (n-2) finite ones. 
% If the best point stored in the input box, then it is set into the 
% correct subbox.
%
% x         box
%   .inf    nx x 1 lower bounds
%   .sup    nx x 1 upper bounds
% part      2 x np partitioning matrix: the first row contains the 
%           directions in which the box has to be splitted, the second in 
%           how many parts each direction is partitioned. 
%           e.g. when part=[2 1;3 4], and the intervals x(1) and x(2) are
%           bounded: x(2) is splitted into 3 parts while x(1) into 4 parts,
%           resulting in a total of 12 subboxes.
%
% boxes     nn x 1 cell of subboxes, (nn=max(prod(part(2,:)))
% part      updated partitioning matrix, where for the part number for the
%           fully infinite components is set to 1.

function [boxes,part]=glboxpart(x, part)

global glconf;      % gloptlab configuration file
INFTY=1e4;          % infinity

% set constants from the config
if isfield(glconf,'infty'),INFTY=glconf.infty;end

% argcheck
if nargin <2, error('glboxpart: Not enough arguments!');end
nx=length(x.inf);
if size(part,1)~=2 || size(part,2)>nx,
    error('glboxpart: Wrong partitioning matrix!');
end

% init
infind=find(x.inf'>=INFTY | x.sup'<=-INFTY);
linfind=find(x.inf<-INFTY);uinfind=find(x.sup>INFTY);
eqind=find(x.inf'==x.sup');
% initialize the boxes cell
if isfield(x,'best')
    % best point found in the input box
    best=x.best;
    if  any(best.x < x.inf) || any(x.sup < best.x)
        % the found point is outside the given box, project
        best.x=min(x.sup, max(x.inf,best.x));
        best.status(1)='m';
    end
    % remove and store the best point
    box.inf=x.inf;box.sup=x.sup;
    boxes={box};
else boxes={x};best=[];
end



for i=1:size(part,2)
    ind=part(1,i);
    pn=part(2,i);
    if pn==1 || any(infind==ind) || any(eqind==ind)
        % partitioning into a single part, fully infinite component or 
        % component has zero width, do nothing
        part(2,i)=1;
        continue;
    elseif ind<=0 || ind>nx, 
        error('glboxpart: Illegal variable index found!');
    end
    boxes2=cell(0);
    linf=any(linfind==ind);uinf=any(uinfind==ind);midsp=linf&uinf&pn==2;
    for j=1:length(boxes)
        box=boxes{j};
        if midsp
            % [...<-INFTY INFTY<...] => [...<-INFTY 0] & [0 ...<INFTY]
            box2=box;box.sup(ind)=0;box2.inf(ind)=0;
            boxes2=[boxes2;box;box2];
            continue;
        end
        bparts=cell(pn,1);
        if linf
            % [...<-INFTY ...] => [...<-INFTY -INFTY] & ...
            box2=box;box2.sup(ind)=-INFTY;bparts{pn}=box2;pn=pn-1;
            box.inf(ind)=-INFTY;
        end
        if uinf
            % [... INFTY<...] => [INFTY INFTY<...] & ...
            box2=box;box2.inf(ind)=INFTY;bparts{pn}=box2;pn=pn-1;
            box.sup(ind)=INFTY;
        end

        % partition the box into pn subboxes
        wid=(box.sup(ind)-box.inf(ind))/pn;
        lb=box.inf(ind);
        ub=box.inf(ind)+wid;
        for k=1:pn-1
            box2=box; box2.inf(ind)=lb; box2.sup(ind)=ub;
            bparts{pn}=box2;pn=pn-1;
            lb=ub;ub=ub+wid;
        end
        box2=box; box2.inf(ind)=lb;bparts{pn}=box2;
        if abs(box.sup(ind))>abs(box.inf(ind))
            % [... mid ...] < 0=> [mid ...] & [... mid], reverse the
            % order of the parts
            bparts=bparts(end:-1:1);
        end
        boxes2=[boxes2;bparts];pn=part(2,i);
    end
    boxes=boxes2;
end

if ~isempty(best)
    % reset the best point in the correct part.
    for i=1:length(boxes)
        if all(boxes{i}.inf<=best.x) && all(best.x <= boxes{i}.sup)
            boxes{i}.best=best;
            break;
        end
    end
end

% TESTING:
% x.inf=[-1;2;-5];x.sup=[5;44;-2];x.best.x=[1;2;-3];
% boxes=glboxpart(x, [2 1;3 4]);
% for i=1:length(boxes);
% [boxes{i}.inf boxes{i}.sup], if isfield(boxes{i},'best'), disp('**');end,
% end
