%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcchol.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [perm,R,ptrace]=glcchol(A,opt);
%
% Cholesky factorization with diagonal pivoting, A(perm,perm) = R^T R
%
% A		symmetric matrix (assumed positive semidefinite)
% opt=0:	choose pivot by max divisor (default)
% opt=1:	choose pivot by max partial trace (doubled work)
%
% perm		permutation vector
% R		upper triangular square Cholesky factor
% ptrace	partial traces if opt=1; pivot entries if opt=0
% 		ptrace(k)=R(k,k:p)*R(k,k:p)'; trace(A)=sum(ptrace)
% 

function [perm,R,ptrace]=glcchol(A,opt)

% argcheck
if nargin<1, error('glcchol: Not enough arguments!');end
if nargin<2, opt=0; end;

% initialize data
n=size(A,1);
if any(~(A'==A)), 
    error('glcchol requires symmetric matrix as input'); 
end; 
perm=1:n;
R=A;
ptrace=zeros(1,n);
d=zeros(1,n);
ind=1:n;
if opt, minpiv=sqrt(eps)*max(abs(A(:))); end;

% main loop
for i=1:n,

  % find next pivot 
  if opt,
    for p=ind,
      d(p)=R(p,ind)*R(ind,p)/max(minpiv,R(p,p));
    end;
  else 
    d=diag(R);
  end;
  [piv,p]=max(d(ind));q=p+(i-1);
  if R(q,q)>0,
    ptrace(i)=piv;
  else 
    % rest of matrix negligible
    R(ind,:)=0*R(ind,:);
    return;
  end;

  if p>1,
    % permute matrix
    p=i-1+p;
    pi=perm(i);perm(i)=perm(p);perm(p)=pi;
    Ri=R(:,i);R(:,i)=R(:,p);R(:,p)=Ri;
    Ri=R(i,:);R(i,:)=R(p,:);R(p,:)=Ri;
  end;
  
  R(i,i)=sqrt(R(i,i));
  if i==n, return; end;

  ind=i+1:n;
  r=R(i,ind)/R(i,i);	% a row vector of length n-i
  R(i,ind)=r;
  R(ind,i)=zeros(n-i,1);
  R(ind,ind)=R(ind,ind)-r'*r;

end;
