%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gldchol.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [perm, R, ier]=gldchol(An, Ap, method, sigma);
%
% directed Cholesky factorization with diagonal pivoting
% for every A in [-An,Ap], A(perm,perm) - R^T R is guaranteed
% to be positive definite and tiny
%
% An,Ap		symmetric matrices (assumed positive semidefinite)
% method    factorization type: using 1:pivoting 2:H-test 3: 1 without alpha
% sigma     1 x np vector of diagonal perturbation factors (method=2,otional)
%
% perm		permutation vector, if method==1 else empty
% R         upper triangular square Cholesky factor
% ier       0 = ok; inf = not positive definite, >0 rank of the incomplete
%           factorization
% 
function [perm, R, ier]=gldchol(An, Ap, method, sigma)

% argcheck
if nargin<2, error('gldchol: Not enough arguments!');end
if nargin<3, method=1;end
if nargin<4,
    n=size(An,2);
    nz=nnz(abs(Ap)+abs(An));
    sigma=[(0.015*nz+0.5*n)*eps (0.015*nz+n)*eps];
end

% initialize data
perm=1:n;R=zeros(n,n);ier=0;
if any(~(An'==An & Ap'==Ap)), 
    error('gldchol requires symmetric matrix as input'); 
end; 
if any(diag(An)>=0),
    % the matrix is not PSD
    R=[]; ier=inf; return;
end
n=size(An,1);setround(1);                    % round up


if method==1
    % Directed Cholesky with H-test
    indD=n*(0:n-1)+(1:n);           % index of the diagonal
    ind1=Ap>=An;ind1(indD)=0;       % Ap>=An and k~=j
    ind2=Ap<An;ind2(indD)=1;        % Ap<An oder k==j

    % create \tilde A
    A2=zeros(n);
    A2(ind1)=Ap(ind1);              % A2_ij=Ap_ij for Ap>=An and k=~j
    A2(ind2)=-An(ind2);             % A2_ij=-An_ij else
    % perurbe the diagonal with the weigted sum of the width of the non
    % diagonal entries
    u=1./diag(A2);
    S=sum((Ap+An).*(ones(length(u),1)*u'),2);S=S./u;
    A2(indD)=A2(indD)'-S;

    for k=1:length(sigma)
        % perturbe the diagonal entries by -sigma(k)*diag
        A2(indD)=(1-sigma(k))*A2(indD);

        % approximate cholesky factorization
        [R,ier] = chol(A2);
        if ier>0, continue; end

        % compute the error matrix
        E.inf=-(An+R'*R);
        E.sup=Ap+(-R')*R;

        % H-test the error matrix
        ier=inf;
        if all(diag(E.inf)>0)
            % diagonal positive,
            A3=glcompmat(E);
            %max_width=max(max(E.sup-E.inf))
            u=1./diag(A2);
            if all(A3*u>=0)
                % H-test with u=1/diag(A2) success
                ier=0;break;
            else
                %t1_min=min(A3*u),t1_max=max(A3*u)
                u=A2\ones(n,1); % u=A2^-1*(1...1)'
                u=u./max(u); % scale the maximum entry of u to one
                %t2_min=min(A3*u),t2_max=max(A3*u)
                if all(u>=0) && all(A3*u>=0)
                    % H-test with u=A2^-1*(1...1)' success
                    ier=0;break;
                end
            end
        end
    end
elseif method==2 ||  method==3 
    % Directed Cholesky with diagonal pivoting
    perm=1:n;ind=1:n;R=zeros(n,n);ier=0;
    sqrtn=sqrt(n)+1;
    for k=1:n,
        % (1) find next pivot
        dAn=-diag(An);  % diagonal of -An
        if method==2,
            % pivoting on
            [alpha,p]=max(dAn(ind));
        else
            % pivoting off
            alpha=dAn(ind(1));
            p=1;
        end
        % now alpha is the maximal diagonal entry of -An
        if alpha<=0,
            ier=k;
            % disp([num2str(k) ': pivot=' num2str(alpha)]);
            return;
        end;

        % (2) permutate An,Ap, and R (if needed)
        if p>1,
            p=k-1+p;
            pi=perm(k);perm(k)=perm(p);perm(p)=pi;
            swap=An(:,k);An(:,k)=An(:,p);An(:,p)=swap;
            swap=An(k,:);An(k,:)=An(p,:);An(p,:)=swap;
            swap=Ap(:,k);Ap(:,k)=Ap(:,p);Ap(:,p)=swap;
            swap=Ap(k,:);Ap(k,:)=Ap(p,:);Ap(p,:)=swap;
            swap=R(:,k);R(:,k)=R(:,p);R(:,p)=swap;
            swap=R(k,:);R(k,:)=R(p,:);R(p,:)=swap;
        end;

        % (3) last entry
        if k==n,
            % last entry reached, compute it and return
            R(k,k)=(1-eps)*sqrt(alpha);
            return;
        end;

        % (4) factor the non factorized part of [An Ap], 
        % into [alphan,alphap],[an,ap],[Bn,Bp]
        ind=k+1:n;  % index non factorized part
        alphan=An(k,k);alphap=Ap(k,k);
        an=An(k,ind);ap=Ap(k,ind);
        Bn=An(ind,ind);Bp=Ap(ind,ind);
        
        % (5) compute the regularization factor
        q=alphan*diag(Bn);
        if any(q<=0),
            % the diagonal of the remaning part of An contains negative
            % entries, return with error
            ier=k;
            return;
        end;
        gamma=eps+max((ap+an)./sqrt(q'));
        gamma=min(sqrtn*gamma,0.01);
        
        % (6) compute the k-th row of the Cholesky factor R 
        rho=(1-gamma)*sqrt(alpha);
        R(k,k)=rho;         % set the diagonal element of R
        r=(ap-an)/(2*rho);	% a row vector of length n-k
        R(k,ind)=r;         % set upper diagonal part of R
        
        % (7) update the An and Ap
        rhn=-rho;
        delta=max(alphap+rhn*rho,alphan+rho^2);   % residual pivot
        if delta<=0,
            % the residual pivot is negative return with error
            ier=k;
            return;
        end;
        s=-r;d=max(ap+rho*s,an+rho*r);              % residual row
        D=d'*(d/delta);
        An(ind,ind)=Bn+r'*r+D;
        Ap(ind,ind)=Bp+s'*r+D;
    end;
else
    error('gldchol: unknown method found!');
end