%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glderiv.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function dA=glderiv(A, xp, h)
%
% Calculate the Jacobian dA=Aq(xp)' or the gradient dA=(Aq(xp)')^T*h.
%
% A             nc x nx^2+nx constraints without constant terms
% xp            nx x 1 (interval) vector to evaluate the derivate at
% h             nc x 1 (interval) vector, if this argument is missing
%               the nc x nx Jacobian; if given the nc x 1 gradient is
%               returned.
%
% dA            the nc x nx Jacobian (if h is missing) or nx x 1 gradient 
%               vector (if h is given). 

function dA=glderiv(A, xp, h)

% argcheck
if nargin <2, error('glderiv: Not enough arguments!');end
if nargin <3, h=[]; end

% init
[nc, nC]=size(A);       % number of constraints
nx=size(xp,1);          % number of variables
indx=1:nx;
nxindx=nx*indx;
if nC ~= nx^2+nx
    gldisp('nx',nx);
    error('glderiv: A has wrong number of rows!');
end
if isempty(h)
    % compute the Jacobian
    if isintval(xp),
        D=intval(sparse(indx,indx,ones(1,nx),nx,nx*(nx+1),nx*(nx+3)));
    else
        D=sparse(indx,indx,ones(1,nx),nx,nx*(nx+1),nx*(nx+3));
    end

    % variable multiplier matrix (example for nx=3) (nx x nx(nx+1))
    % 1 0 0, 2x1 x2 x3, x2 0   0 , x3 0  0
    % 0 1 0, 0   x1 0 , x1 2x2 x3, 0  x3 0
    % 0 0 1, 0   0  x1, 0  0   x2, x1 x2 2x3
    xps=xp';
    for i=indx
        % create the diagonal parts
        D(i,nxindx+i)=xp;
        % add the vertical parts
        ind=nx*i+indx;
        D(i,ind)=D(i,ind)+xps;
    end
    % compute the jacobian
    dA=A*D'; % nc x nx sparse
else
    % compute the gradient
    if nc ~= length(h)
        gldisp('nx',nx,'h',h);
        error('glderiv: The vector h has wrong dimension!');
    end
    % set the nx(nx+1) x 1 intval vector Ah=h'*A
    if isintval(h) || ~isintval(xp), Ah=h'*A; 
    else Ah=infsup(h,h)'*A;
    end
    dA=Ah(indx)';
    % multiplication (example for nx=3)
    % 1 0 0, 2x1 x2 x3, x2 0   0 , x3 0  0
    % 0 1 0, 0   x1 0 , x1 2x2 x3, 0  x3 0    * Ah
    % 0 0 1, 0   0  x1, 0  0   x2, x1 x2 2x3
    for i=indx
        % create the sum of the diagonal parts and add the sum of the 
        % vertical parts
        dA(i)=dA(i)+(Ah(nxindx+i)+Ah(nx*i+indx))*xp;
    end
end

% old code
% D=sparse(indx,indx,ones(1,nx),nx,nx*(nx+1),nx*(nx+3));
% for i=1:nx,
%     ind=nx*i;
%     for j=1:nx,
%         D(j,ind+j)=xp(i);
%     end
%     ind=ind+1:ind+nx;
%     D(i,ind)=D(i,ind)+xp';
% end
% for i=1:nc,
%     dA2(i,:)=A(i,:)*D';
% end
% dA,dA2
