%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gleval.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [fx, fd, fdv]=gleval(A, xp, F, xb);
%
% Evaluate the constraint Aq(x) at xp, and the feasibility distance.
%
% A             nc x nx^2+nx constraints without constant terms
% xp            nx x 1 vector to evaluate the constraint at 
% F             constraint bounds (optional)
%   .inf .sup   nc x 1  outer enclosures
% xb            bound constraints
%   .inf .sup   nx x 1  outer enclosures
%
% fx            nc x 1 or (2) nc+nx x 1 result vector
% fd            feasibility distance 0=feasible, computed if F supplied
% fdv           nc x 1 or (2) nc+nx x 1 feasibility distance vector: 
%               fdv_i=f(x)-Fu if f(x)>Fu, f(x)-Fl if f(x)< Fl, 0 otherwise
%               fdv_j=xp-xu if xp>xu, xp-xl if xp<xl, 0 otherwise

function [fx, fd, fdv]=gleval(A, xp, F, xb)

% argcheck
if nargin<2, error('gleval: Not enough arguments!');end

% init
[nc, nC]=size(A);       %number of constraints
nx=size(xp,1);          %number of variables

if nC ~= nx^2+nx
    gldisp('nx',nx,'nC',nC);
    error('gleval: A has wrong number of rows!');
end

% compute fx
if isintval(A)
    fx=infsup(zeros(nc,1),zeros(nc,1));
else
    fx=zeros(nc,1);
end
for i=1:nc,
    fx(i)=A(i,1:nx)*xp + xp'*reshape(A(i,nx+1:end),nx,nx)*xp;
end
% compute the feasibility distance if needed
if nargin>3
    % signed nc+nx x 1 feasibility distance vector: for i=1:nc, j=1:nx
    % fdv_i=f(x)_i-Fu_i if f(x)_i>Fu_i, 
    % fdv_i=f(x)_i-Fl_i if f(x)_i<Fl_i, 
    % fdvi=0 otherwise,
    % fdv_(j+nc)=x_j-xu_j if x_j>xu_j, 
    % fdv_(j+nc)=x_j-xl_j if x_j<xl_j, 
    % fdvi=0 otherwise,
    f1=[fx-F.sup; (xp-xb.sup)];f2=[fx-F.inf; (xp-xb.inf)];
    fdv=zeros(size(f1));
    if isintval(f1),
        fdv(f1>0)=f1.sup(f1.inf>0);
        fdv(f2<0)=f2.inf(f2.sup<0);
    else
        fdv(f1>0)=f1(f1>0);
        fdv(f2<0)=f2(f2<0);
    end
    % feasibility distance
    fd=norm(abs(fdv),inf);
end