%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glgain.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [xgain, Fgain, out]=glgain(P, x, xnew, Fnew)
%
% Computes the gain (P.Q.F, x) -> (Fnew, xnew) and returns a string output
% cell describing the gain.
% 
% x             original bound constraints
%   .inf,.sup   outer enclosures
% xnew          new bound constraints
%   .inf,.sup   outer enclosures
% P             original problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% Fnew          new general constraint bounds
%   .inf,.sup   outer enclosures
%
% xgain         nx x 1 gain vector
% Fgain         nc x 1 gain vector
% out           gain output string cell

function [xgain, Fgain, out]=glgain(x, xnew, P, Fnew)

% argcheck
if nargin<2 || (nargin<4 && nargout>1), 
    error('glgain: Not enough arguments!');
end

% compute the xgain
xgain=glreduc(x, xnew);

% compute the Fgain
if nargout > 1, Fgain=glreduc(P.Q.F, Fnew);end

% create text output
if nargout == 3
    out=[]; xred=find(xgain' ~= 0); Fred=find(Fgain' ~= 0);
    % xgain output
    if ~isempty(xred)
        nr=length(xred);xg=xgain(xred);
        txt='Reduced x component';
        if nr > 1, txt=[txt 's']; end
        txt=[txt '(reduction): '];
        for i=1:nr
            txt=[txt 'x_' num2str(xred(i)) '(' num2str(xg(i)) '), '];
        end
        out=[out ; {[txt 'new box:']} ; glbox2str(xnew)];
    elseif all(isinf(xnew.inf)) && all(isinf(xnew.sup))
        out = [out ; ...
            {'No better bounds gained, the variables are still unbounded'}];
    elseif ~isempty(Fred)
        out = [out ; {'No better x gained.'}];
    end
    % Fgain output
    if ~isempty(Fred)
        P.Q.A.inf=P.Q.A.inf(Fred,:);P.Q.A.sup=P.Q.A.sup(Fred,:); 
        P.Q.F.inf=Fnew.inf(Fred);P.Q.F.sup=Fnew.sup(Fred);  
        nr=length(Fred);Fg=Fgain(Fred);
        txt='Reduced F component';
        if nr > 1, txt=[txt 's']; end
        txt=[txt '(reduction): '];
        for i=1:nr
            txt=[txt 'F_' num2str(Fred(i)) '(' num2str(Fg(i)) '), '];
        end
        if nr > 1, txt=[txt 'the changed constraints:'];
        else txt=[txt 'the changed constraint:'];
        end
        out=[out ; {txt} ; glprob2str(P)];
    elseif ~isempty(xred)
        out = [out ; {'No better F gained.'}];
    end
    if isempty(xred) && isempty(Fred), 
        out = [out ; {'No bound improvements.'}];
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glcreduc %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function reduc=glreduc(oldbox, box)
%
% Compute the reduction factor between the two given boxes. For the ith
% component reduc_i \in [0,1] where reduc_i=1 means infinite reduction and
% reduc_i=0 means no reduction.
%
% oldbox    old box
% box       new box
%
% reduc     reduction vector

function reduc=glreduc(oldbox, box)

% argcechk
if nargin<2, error('glcreduc: Not enough arguments!');end

% dimension check
if any(size(oldbox.inf)~=size(box.inf))
    error('glcreduc: dimension mismatch in gain calculation');
end

% calcualte reduc; avoid inf/inf=NaN
% vector of ratios of diameters 1-after/before reduction
warning off MATLAB:divideByZero
xd=oldbox.sup-oldbox.inf; % diameter of x for gain calculation
xn=box.sup-box.inf;
reduc=xn./xd;
reduc(isnan(reduc))=1;  % infinity bounds without improvement
% infinity bounds improved
reduc(isinf(oldbox.sup) & ~isinf(box.sup))=0;
reduc(isinf(oldbox.inf) & ~isinf(box.inf))=0;
reduc=1-reduc;
warning on MATLAB:divideByZero
