%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glgaussjord.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [C, cperm, rind, exc]=glgaussjord(A, b, x);
%
% Gauss-Jordan algorithm for partial inversion of A, 
% if exc==0, then C*A(:,cperm)~I
%
% Permuting: if B=A(K,J) then A(K,J)=B. 
%
% A             n x m interval matrix
%   .inf.sup    outer enclosure
% b             m x 1 interval vector
%   .inf.sup    outer enclosure
% x             n x 1 bounds on x
%   .inf.sup    outer enclosure
% prt           output level 0=no output 1=output
%
% C             (pseudo)inverse of mid(A)
% cperm         indices of the columns made to an identity column
% rind          sorted row indices forming an approx basis of the row space
% exc           number of exceptions

function [C, cperm, rind, exc]=glgaussjord(A, b, x, prt)

global glconf;      %gloptlab configuration file
INF=1e5;            %positive and negative infinity

%argcheck
if nargin <3, error('glgaussjord: Not enough arguments!');end
if nargin <4, prt=0;end

%set configuration
if isfield(glconf,'infty'),INF=glconf.infty;end 

%xINF=INF*xscal
%error('dsd');
%init
[m,n]=size(A.inf);
if m~=size(b.inf,1), error('glgaussjord: b has wrong number of entries');end
if n~=size(x.inf,1), error('glgaussjord: x has wrong number of entries');end

%(1) SCALING
delta=eps^(1/2);
f=max([abs(x.inf(x.inf>-INF))' abs(x.sup(x.sup<INF))'])/delta;
if isempty(f) || f==0, f=INF/delta; end 
% intersect x with [-f, f]
z.inf=max(x.inf, -f.*ones(n,1));
z.sup=min(x.sup, f.*ones(n,1));
if prt, gldisp('m',m,'n',n,'f',f,'z',[z.inf,z.sup]); end
% convert to interval
Ai=infsup(A.inf,A.sup);
zi=infsup(z.inf,z.sup);
bi=infsup(b.inf,b.sup);
%u=mag(Ai*zi-bi); %old
u=b.sup-b.inf+delta*mag(Ai*zi-bi);
maxu=max(u);
if maxu==0, u=u+1;
else u(u==0)=maxu;u=min(u)./u;
end
v=z.sup-z.inf;
ind=v==0;v(ind)=abs(z.sup(ind));
mv=max(v);
if mv==0, v=v+1;
else v=v/mv;
end
v(v==0)=1;
if prt, gldisp('u',u,'v',v); end
%u=ones(size(u));
%v=ones(size(v));
%delta=1;
%u=[1.2;2.1];v=[0.2;1.1;0.1];
%u, v
 
[C, cperm, rind, rank]=glgaussjord2(mid(Ai), v, u, delta);
exc=m-rank;

if prt, gldisp('v_perm',1./v(cperm)); end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glgaussjord.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [C, cperm, rind, rank]=glgaussjord(A, v, u, delta);
%
% Gauss-Jordan algorithm for partial inversion of A: C*A(:,cperm)~I_rank
%
% A             n x m real matrix
% v             row scaling vector (optional)
% u             column scaling vector (optional)
% delta         sclaing factor of the indetity matrix used
%
% C             (pseudo)inverse of A
% cperm         indices of the columns made to an identity column
% rind          sorted row indices forming an approx basis of the row space
% rank          found numerical rank of A

function [C, cperm, rind, rank]=glgaussjord2(A, v, u, delta)

%argcheck
if nargin <1, error('glgaussjord: Not enough arguments!');end
if ~isnumeric(A),  
    error('glgaussjord: for interval matrices use the function "glinv"!');
end
if nargin <4, 
    % generate scaling factors
    [m,n]=size(A);v=ones(1,m);u=ones(1,n);delta=eps^(1/2);
end

[m,n]=size(A);  				% dimension of the matrix A
As=(u*ones(1,n)).*A.*(ones(m,1)*v');% scaled matrix
B=[As delta*eye(m,m)];          % initial B
cperm=1:(n+m);					% column permutation vector
rind=false(1,m);                % vector, has ones for the columns, for
                                % which the pivot was selected from the
                                % first m columns
for k=1:m
    % (1) pivot search: find the pivot row and column
    [m1,i1]=max(abs(B(k:end,k:end)),[],1);
    [m2,i2]=max(abs(m1));
    piv=[i1(i2)+k-1,i2+k-1]; % piv=[pivot row, pivot col]
    if piv(2)<=n,
        % pivot selected from the first n rows
        rind(k)=1;
        % else a column of the identity matrix was selected as the pivot,
        % the scaling factor delta may be wrong or the rank of A
        % is less than m
    end
    % update the column permutation vector
    swap=colperm(k);cperm(k)=cperm(piv(2));cperm(piv(2))=swap;
    % swap rows & cols, bring pivot to B(k,k)
    if piv(1)~=k,swap=B(k,:);B(k,:)=B(piv(1),:);B(piv(1),:)=swap;end
    if piv(2)~=k,swap=B(:,k);B(:,k)=B(:,piv(2));B(:,piv(2))=swap;end
    % (2) update the new matrix B.
    p=B(k,k);% pivot element
    % find the indices of all nonzero entries in the kth row and
    % exclude the kth
    indnz=(B(:,k)~=0)';indnz(k)=0;
    % find multipliers for nonpivot rows with nonzero entries in the
    % kth col
    B(indnz,k)=B(indnz,k)/p;
    % adjust all nonpivot rows
    B(indnz,k+1:end)=B(indnz,k+1:end)-B(indnz,k)*B(k,k+1:end);
    B(:,k)=zeros(m,1);
    % adjust pivot row
    B(k,k)=1;B(k,k+1:end)=B(k,k+1:end)/p;
end
% (3) unpermute B
B(:,cperm)=B;cperm=cperm(1:m);
% (4) check exception occured
rank=sum(rind);	% rank of A
if rank<m,
    % exceptions occured, shorten the vector cperm
    cperm=cperm(rind);
end
rind=find(rind);	% indices of the colums, without exception
% (5) generate C (rank of A=size(C,1))
C=zeros(rank,m);
for j=1:m, C(:,j)=B(rind,n+j)*(u(j)/delta); end
for i=1:rank, C(i,:)=v(cperm(i))*C(i,:);end



%%%OLD
% B=[zeros(m,n) delta*eye(m,m)];A2=mid(Ai);
%     for j=1:n, B(:,j)=u.*A2(:,j)*v(j); end
%     if prt,xb=[x.inf x.sup],A2, format long;,B,format short;end
% 
%     %(2) GAUSS-JORDAN INVERSION
%     rind=logical(zeros(m,1));
%     colperm=1:(n+m);exc=0;
%     for k=1:m
%         %find pivot
%         [m1,i1]=max(abs(B(k:end,k:end)),[],1);
%         [m2,i2]=max(abs(m1));
%         %pivot row and col
%         perm=[i1(i2)+k-1,i2+k-1];
%         %check if an identity column was moved to the pivot column or not
%         if perm(2)>n, exc=exc+1;
%         else rind(k)=1;
%         end
%         %swap permutation vectors
%         swap=colperm(k);colperm(k)=colperm(perm(2));colperm(perm(2))=swap;
%         if prt,perm,end
%         %swap rows & cols, bring pivot to (k,k)
%         if perm(1)~=k,swap=B(k,:);B(k,:)=B(perm(1),:);B(perm(1),:)=swap;end
%         if perm(2)~=k,swap=B(:,k);B(:,k)=B(:,perm(2));B(:,perm(2))=swap;end
%         piv=B(k,k);
%         %find index of nonzero entries, exlude the kth entry
%         indnz=(B(:,k)~=0)';indnz(k)=0;
%         %find multiplyers for nonpivot row swith nonzero entries in the kth col
%         B(indnz,k)=B(indnz,k)/piv;
%         %adjust all nonpivot rows with nonzero entries in the kth col
%         B(indnz,k+1:end)=B(indnz,k+1:end)-B(indnz,k)*B(k,k+1:end);
%         B(:,k)=zeros(m,1);
%         %adjust pivot row
%         B(k,k)=1;B(k,k+1:end)=B(k,k+1:end)/piv;
%         if prt, format long;,B,format short;end
%     end
%     rind=find(rind)';
%     %(3) BACKPERMUTATION & BACKSCALING
%     B(:,colperm)=B;cperm=colperm(1:m);C=[];
%     if exc~=0, ind=cperm<=n; cperm=cperm(ind); rind=rind(ind); end
%     for j=1:m, C(:,j)=B(:,n+j)*(u(j)/delta); end
%     for i=rind, C(i,:)=v(cperm(i))*C(i,:);end
%     C=C(rind,:);