%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glintround.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [p,q,err] = glintround(x, sigma, inorm, qma, maxiterx)
%
% optimal rounding of the vector x with standard deviation sigma 
% by minimizing (norm(max(round(q*x),q*x),inorm)^2/q
%
% x	        vector to be rounded (row vector)
% sigma	    vector of standard deviations of x (row vector)
% inorm     choice of norm (1,2,inf)
% qmax	    limit on the q-interval to be searched (optional)
% maxiter   maximum number of iterations
%
% p	        the row vector of the optimal rounded multiples of x
% q	        the factor used for roundin i.e. p = round(q*x)
% err       error of the approximation
%

function [p,q,err] = glintround(x, sigma, inorm, qmax, maxiter)

if nargin < 4, qmax = inf; end
if nargin < 5, maxiter = inf; end

fbest=inf;
x2=x(x~=0);%remove zero entries
if isempty(x2)
    p=x;q=1;err=0;
    return;
elseif length(x2)==1
    q=1/abs(x2);p=round(q*x);err=abs(x2-sign(x2));
    return;
end
faclist=sort(1./abs(x2));
qmin=faclist(2)/2;it=1;qmax=max(qmax,2*qmin);
for fac = faclist
    k = 1+fix(qmin/fac);q1 = k*fac;
    while it<maxiter 
        if q1 > min(fbest/norm(sigma,inorm)^2,qmax) && it>1, break;end;
        f = norm(max(abs(round(q1*x2)-q1*x2),q1*sigma),inorm)^2/q1;
        if f < fbest,
            fbest = f; q=q1; p=round(q*x);err=f;
        end
        it=it+1;
        k = k + 1;
        q1 = k*fac;
    end
end