%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glpdeftest.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [R, ier]=glpdeftest(An, Ap, sigma);
%
% Test if all the matrices A in [-An,Ap], are positive definite.
% If yes return R with A - R^T R is guaranteed to be positive semidefinite 
% and tiny.
%
% An,Ap		symmetric matrices (assumed positive semidefinite)
% sigma     1 x np vector of diagonal perturbation factors (optional)
%
% R         upper triangular square Cholesky factor
% ier       0 = ok; -1 = numerically indefinite, >0 rank of the incomplete
%           factorization

function [R, ier]=glpdeftest(An, Ap, sigma);

% argcheck
if nargin<2, error('glpdeftest: Not enough arguments!');end
if nargin<3,
    n=size(An,2);
    nz=nnz(abs(Ap)+abs(An));
    sigma=[(0.015*nz+0.5*n)*eps (0.015*nz+n)*eps];
end

% initialize
if any(~(An'==An & Ap'==Ap)) 
    gldisp('An',An,'Ap',Ap);
    error('glpdeftest requires symmetric matrix as input'); 
end; 
if any(diag(An)>0)
    % A in not PSD
    ier=inf; 
    R=[]; 
    return;
end
n=size(An,1);R=zeros(n,n);ier=0;
ind=Ap>=An;ind2=Ap<An;

% create \tilde A
A2=zeros(n);
A2(ind)=Ap(ind);A2(ind2)=-An(ind2);
for i=1:length(sigma)
    % perturbe the diagonal entries trough sigma(i)
    A2(n*(0:n-1)+(1:n))=(sigma(i)-1)*diag(An);

    % approximate cholesky factorization
    [R,ier] = chol(A2);
    if ier>0, continue; end

    % compute the error matrix
    setround(1); % round up
    E.inf=-(An+R'*R);
    E.sup=Ap+(-R')*R;

    % H-test the error matrix
    ier=inf;
    if all(diag(E.inf)>0)
        % diagonal positive,
        A3=glcompmat(E);
        u=1./diag(A2);
        if all(A3*u>=0)
            % H-test with u=1/diag(A2) success
            ier=0;break;
        else
            %t1_min=min(A3*u),t1_max=max(A3*u)
            u=A2\ones(n,1); % u=A2^-1*(1...1)'
            u=u./max(u); % scale the maximum entry of u to one
            %t2_min=min(A3*u),t2_max=max(A3*u)
            if all(u>=0) && all(A3*u>=0)
                % H-test with u=A2^-1*(1...1)' success
                ier=0;break;
            end
        end
    end
end