%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glscaling.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [xs,cs,As,meas]=glscaling(A, E, x, method, basis, delta, prt)
% 
% find a scaling vector for the variables and the constraints.
%
% A             nc x np matrix of inequality constraint coeffs
%  .inf,.sup    bounds for coefficients
% E             np x nx matrix representing the monomial vector 
%               q(x)_k=\sum_{j=1...nx} x_j^(E_{k,j})
% x             nx x 1 bound constraints
%   .inf.sup    bounds
% method        method number 1:Iter, 2:LP, 3:Hompack, 4:Morgan,
%               or the corresponding string
% lsolver       linear solver name which is used for solving LP
% basis         basis of the log used in the scaling
% delta         scaling step must be > 0
% prt           text output 0=no 1=yes
%
% xs            nx x 1 variable scaling vector
% cs            nc x 1 constraints scaling vector
% As            nc x (nx+nx^2) scaled magnitude matrix of A
% meas          constant measuring the scaling quality, better scalings
%               of the same problem have higher meas values:
%               meas=d'u+q'E*v.

function [xs,cs,As,meas]=glscaling(A, E, x, method, lsolver, basis, delta, prt)

global glconf;      % global configuration variable
TINY=1e-6;          % tiny enties

% argcheck
if nargin <3, error('glscaling: Not enough arguments!');end
if nargin <4, method=1;end
if nargin <5, lsolver='LpSolve';end
if nargin <6, basis=8;end
if nargin <7, delta=1;end
if nargin <8, prt=0;end
if ~exist('basis','var') || basis <= 0 || ~exist('delta','var') || delta <= 0 
    error('glscaling: Wrong input arguments!!');
end
if ~isnumeric(method)
    if strcmp(method,'Iter'), method=1;
    elseif strcmp(method,'LP'), method=2;
    elseif strcmp(method,'Hompack'), method=3;
    elseif strcmp(method,'Morgan'), method=4;
    else error(['glscaling: Unknown method:' method]);
    end
end
% set constants from config
if isfield(glconf,'tiny'),TINY=glconf.tiny;end

% starting values
if isempty(x), x.sup=inf+zeros(size(E,2),1);x.inf=x.sup;
elseif size(x.inf,1)<size(x.inf,2), x.inf=x.inf'; x.sup=x.sup'; end

mi=glmig(x);ma=glmag(x);                    % mig and mag of x
mmi=max(mi);mma=min(ma);
if mmi~=0, mmir=1/mmi; 
else mmir=inf;
end
epsilon=eps*min([1, mmir, mma]);        % scaling constant
u=max(mi,epsilon);                      % scale factor 1
r=1./min(ma,1/epsilon);                 % scale factor 2
vb=[round(log(u)/log(basis)) round(-log(r)/log(basis))];%tolerance box of v
v=round((vb(:,1)+vb(:,2))/2);           % initial value of v

% remove vars with tiny bounds
Eo=E;Ao=A;[nco,npo]=size(A.inf);vo=v;nxo=length(x.inf);
txI=find((vb(:,2)-vb(:,1))<1)';         % v_i with tiny bounds
ntxI=1:length(x.inf);ntxI(txI)=[];      % v_i with not tiny bounds
np=size(E,1);
[A, x, E, nJ]=glremvars(A, x, E, txI);  % remove vars with tiny width
J=1:np;J(nJ)=[];                        % removed monomials

if ~isempty(txI),
    vb(txI,:)=[]; v(txI)=[];
    if prt, disp(['Variable(s) removed: x(' mat2str(txI) ')']);end
end
if prt,
    disp(['Monomials(s) removed: E([' mat2str(J) '],:)'])
end
if ~isempty(ntxI)
    % not all variable ranges are tiny
    % remove zero constraints
    nzcI=find(sum(glmag(A),2)>TINY);        % non zero constraints indices
    A.inf=A.inf(nzcI,:);A.sup=A.sup(nzcI,:);% remove zero constraints
    %now nc=length(nzI)=size(A,1), nx=length(nI)=size(x)=size(E,2),
    %    np=length(nJ)=size(A,2)=size(E,1)
    [nc,np]=size(A.inf);nx=length(x.inf);   % number of constraints & vars
    magA=(glmag(A));                        % magnitude of A
    if any([nc nx np np nx]~=[length(nzcI) length(ntxI) length(nJ) size(E)])
        gldisp('nc nx np np nx',[nc nx np np nx],'nzcI ntxI nJ E_1 E_2',...
            [length(nzcI) length(ntxI) length(nJ) size(E)]);
        error('glscaling: Dimension mismatch!');
    end

    if prt,
        gldisp('nc nx np',[nc nx np],'old nc nx np',[nco nxo npo],...
            'magA',magA,'E',full(E),'x',[x.inf,x.sup],'mi',mi,'ma',ma,...
            'epsilon',epsilon,'u',u,'r',r,'vb',vb,'v',v);
    end

    warning('off', 'MATLAB:log:logOfZero');
    L=-log(magA)/log(basis);               % log_b of A, nc x np
    warning('on', 'MATLAB:log:logOfZero');
    nz=(~isinf(L));                        % finite entries of L
    maxv=sum(sum(L(nz)));                  % maximum objective value(info)
    d=sum(nz,2)';q=sum(nz,1);
    e=q*E;                                 % weigthed col sum of E, 1 x nx
    I=cell(1,nc);nI=zeros(1,nc);Es=cell(1,nc);% for hompack, and morgan
    for i=1:nc
        I{i}=find(nz(i,:));
        nI(i)=length(I{i});
        Es{i}=sum(E(I{i},:));
    end

    if prt
        gldisp('L',full(L),'nz',nz,'maxv',maxv,'d',d,'q',q,'e',full(e));
    end

    if method==1
        %%%%%%%%
        % ITER %
        %%%%%%%%
        C=L-ones(nc,1)*(E*v)';              % C(i,k)=L(i,k)-E*v(k) nc x np
        m=min(C,[],2);                      % min of the rows in C, nc x 1
        delta=-delta;                       % initial delta value
        j=1;                                % coordinate component counter
        noch=0;                             % # of unchanged components
        % since the last change in z
        while 1,
            vs=v(j);vg=vs+delta;jo=j;
            if vg>=vb(j,1) && vg<=vb(j,2)
                % compute
                % Delta=d'*(min_k(C(ik)-delta*E(kj))-min_k(C(ik)))-delta*q'E
                %
                C_hat=C-delta*ones(nc,1)*E(:,j)';
                m_hat=min(C_hat,[],2);
                Delta=d*(m_hat-m)+delta*e(j);
                vv=v;vv(j)=vg;
                if prt,
                    obj=d*m+e*v; % objective value (info)
                    if(obj+Delta) ~= (d*m_hat+e*vv),error('wrong Delta!');end
                    if any(m_hat~=min(L-ones(nc,1)*(E*vv)',[],2)),error('wrong C!');end
                end
                if Delta>0
                    % new v accepted set v, C, m and the no-change counter to 0.
                    v(j)=vg; C=C_hat; m=m_hat; noch=0;
                elseif delta<0,
                    % new value not accepted, delta<0, change delta
                    delta=-delta;
                else
                    % new value not accepted, next coordinate
                    delta=-delta;
                    % next v component
                    if j==nx, j=1;
                    else j=j+1;
                    end
                    % if the last nx coordinates did not change break the while
                    if noch>nx, break;
                    else noch=noch+1;
                    end
                end
                if prt,
                    obj=d*m+e*v;
                    out=['v_' num2str(jo)];
                    if vs == v(jo), out=[out ' not changed by ' num2str(-delta)];
                    else out=[out ':' num2str(vs) '=>' num2str(v(jo))];
                    end
                    out=[out ', Delta:' num2str(Delta) ', obj: ' num2str(obj)];
                    disp(out);
                    if obj>maxv,
                        EE=[];
                        for i=1:nx
                            for k=1:np
                                EE(i,k)=u(i)+Ev(k);
                            end
                        end
                        gldisp('obj',obj,'maxv',maxv,'Ev',E*v,'u',...
                            min(L-ones(nc,1)*Ev',[],2),'err',EE>L);
                        error('Objective greater than the maxv!!');
                    end
                end
            else
                if prt,disp(['v_' num2str(j) '+delta = ' num2str(vg)...
                        ' not in ' mat2str(vb(j,:),3) '!']);
                end
                if delta<0,
                    % v(j)+delta not in vb(j) change dir
                    delta=-delta;
                else
                    % v(j)+delta not in vb(j), next coordinate
                    % invert delta
                    delta=1-delta;
                    % next v component
                    if j==nx, j=1;
                    else j=j+1;
                    end
                    % if the last nx coordinates not change break while
                    if noch>nx, break;
                    else noch=noch+1;
                    end
                end
            end
        end
        if prt, gldisp('obj',-d*m-e*v); end
    elseif method==2,
        %%%%%%%%%%%%%%%%
        % LINEAR SOLVE %
        %%%%%%%%%%%%%%%%
        %Variables x(1) ... x(nc):u , x(nc+1) ... x(nc+nx):v
        nfv=length(find(~isinf(L)));                    % num finite L entries
        nnzv=length(find(~isinf(L) & L~=0));            % num finite nonzero L
        bind=find(~isinf(vb(:,1)) | ~isinf(vb(:,2)));   % finite bounds on v
        nb=length(bind);                                % number of fin /v
        n2=nx+nc;
        cin.obj.b=[-d -e]';                             % objective
        A=sparse([],[],[],nfv,n2,nfv+nnz(E)*nx);
        b=sparse([],[],[],nfv,2,nfv+nnzv);
        j=1;
        for i=1:nc
            for k=1:np
                if ~isinf(L(i,k))
                    A(j,i)=1;                  % u_i-s
                    A(j,nc+1:nc+nx)=E(k,:);    % E-s
                    b(j,1)=-inf;
                    if L(i,k)~=0, b(j,2)=L(i,k); end
                    j=j+1;
                end
            end
        end
        % bounds on v
        Ab=sparse([],[],[],nb,n2,nb);
        bb=sparse([],[],[],nb,2,nb*2);
        for i=1:nb
            Ab(i,nc+bind(i))=1;              
            bb(i,:)=vb(bind(i),:);
        end
        cin.lin.A=[A ; Ab];cin.lin.b=[b ; bb];
        if prt, out=glcprogr2str(cin);
            for i=1:length(out),disp(out{i}); end
        end
        v=zeros(nx,1);
        [y, xb, info]=gloptsolve(lsolver, cin);
        if ~isempty(y)
            %remove degeneracies
            cin.obj.b=y;
            bb=sparse([],[],[],n2,2,nnz(y));
            bb(y<0,1)=-inf; bb(y>0,2)=inf;
            cin.lin.A=[A ; sparse([-d -e]) ; sparse(eye(n2))];

            cin.lin.b=[b ; sparse([-inf, [-d -e]*y]) ; bb];
            if prt, out=glcprogr2str(cin);
                for i=1:length(out),disp(out{i}); end
            end
            y=gloptsolve(lsolver, cin);
            if ~isempty(y)
                %create solution
                v=round(y(nc+1:end));
                if prt,
                    gldisp('res1',(cin.lin.b(:,2)-cin.lin.A*y)','obj',cin.obj.b'*y);
                    %onu=ones(nx,1);onv=ones(nc,1);
                    %res2=L-u(:,onu)-v(:,onv)'
                end
            end
        end
        if prt,
            if info.numerr==1,disp('Inaccurate Resuts');end
            if info.numerr==2,
                disp('Serious numeriacal errors!');
            elseif info.pinf<1 && cert.info.dinf<1,
                disp('Enclosure found: Problem is feasible.');
            elseif info.pinf==1 && cert.info.dinf<1
                disp(['Enclosure failed: Primal problem is infeasible, '...
                    'infeasiblity check needed!']);
            else
                disp('Dual Problem infesible!');
            end
        end
        
    elseif method==3
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %HOMPACK METHOD SOLVED WITH LEAST SQ%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        nzind=[];
        for i=1:nc
            nzs=find(nz(i,:));
            for j=1:length(nzs)
                nzind=[nzind; i nzs(j)];
            end
        end
        nzz=size(nzind,1);
        G=sparse([], [], [], nzz, nc+nx, 1000);
        a=sparse([], [], [], nzz, 1, nzz);
        for k=1:nzz
            i=nzind(k,1);j=nzind(k,2);
            G(k,i)=1;
            G(k,nc+1:nc+nx)=E(j,:);
            a(k)=L(i,j);
        end
        B=full(G'*G);
        res=(B\(G'*a))';
        %u=res(1:nc)';
        v=res(nc+(1:nx))';
        if prt, gldisp('G',full(G),'a',full(a'),'res',res);end
    elseif method==4
        %%%%%%%
        %SCGEN%
        %%%%%%%
        nzind1=[];nzind2=[];
        for i=1:nc
            nzs=find(nz(i,:));
            for j2=1:length(nzs)
                nzind1=[nzind1; i nzs(j2)];
                for j1=1:j2-1;
                    nzind2=[nzind2; i nzs(j1) nzs(j2)];
                end
            end
        end
        nzz1=size(nzind1,1);
        nzz2=size(nzind2,1);
        G=sparse([], [], [], nzz1+nzz2, nc+nx, 1000);
        a=sparse([], [], [], nzz1+nzz2,1, nzz1+nzz2);
        for k=1:nzz1
            i=nzind1(k,1);j=nzind1(k,2);
            G(k,i)=1;
            G(k,nc+1:nc+nx)=E(j,:);
            a(k)=L(i,j);
        end
        for k=1:nzz2
            i=nzind2(k,1);j1=nzind2(k,2);j2=nzind2(k,3);
            G(k+nzz1,nc+1:nc+nx)=E(j1,:)-E(j2,:);
            a(k+nzz1)=L(i,j1)-L(i,j2);
        end
        B=full(G'*G);
        res=(B\(G'*a))';
        %u=res(1:nc)';
        v=res(nc+(1:nx))';
        if prt, gldisp('G',full(G),'a',full(a'),'res',res);end
    else
        error(['glscaling: Unknown method number: ' ...
            num2str(method) ' found!']);
    end
end

% compute xs = b^v
vv=zeros(nxo,1);
vv(ntxI)=v;
vv(txI)=vo(txI);
xs=basis.^vv;

% compute cs = b^min_k(L(:,k)-(EV)_k) without the constant monomials
warning('off', 'MATLAB:log:logOfZero');
C=-log(glmag(Ao))/log(basis)-ones(nco,1)*(Eo*vv)';
warning('on', 'MATLAB:log:logOfZero');
C(:,J)=inf;             % do not include the removed entries
uu=min(C,[],2);
cs = basis.^uu;

% scaled magnitude matrix
if nargout>2,
    % compute q(x)
    qx=zeros(npo,1);
    for k=1:npo,
        qx(k)=prod(xs'.^Eo(k,:));
    end
    if prt, gldisp('qx',full(log(qx)/log(basis))'); end
    % compute As
    Am=(Ao.inf+Ao.sup)/2;
    As=(cs*ones(1,npo)).*Am.*(ones(nco,1)*qx');
end
if nargout>3, 
    warning('off', 'MATLAB:log:logOfZero');
    nzo=~isinf(-log(glmag(Ao))/log(basis));
    warning('on', 'MATLAB:log:logOfZero');
    qo=sum(nzo,1);
    if ~isempty(ntxI),
        meas=d*uu+qo*Eo*vv;
    else
        meas=inf;
    end
end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glremvars.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [A, x, E, nJ, c]=glremvars(A, x, E, I)
%
% removes the selected variables from A and xbox, where A can be the
% coefficient matrix in inequality or in equality form
%
% A              nc x np matrix of constr coeffs
%   .inf,.sup    bounds for coefficients
% x              nx x 1 bounds on x 
%   .inf,.sup    outer enclosure
% E              np x nx matrix representing the monomial vector 
%                q(x)_k=\sum_{j=1...nx} x_j^(E_{k,j})
% I              index set of the variables to remove
%
% A              nc x np_new sparse matrix of const coeffs
%   .inf,.sup    bounds for coefficients
% x              nx_new dim structure with bounds on x 
%   .inf,.sup    outer enclosure
% E              np_new x nx_new matrix representing the monomial vector 
%                q(x)_k=\sum_{j=1...nx_new} x_j^(E_{k,j})
% nJ             index set of the remaining entries of q(x) (non constant
%                monomials
% c              nc x 1 struct of derived constants, this should be added
%                to the lhs of the csp.
%   .inf,.sup    outer enclosure

function [A, x, E, nJ, c, perm]=glremvars(A, x, E, I)

%argcheck
if nargin<4, error('glremvars: Not enough arguments!'); end

% init
[np,nx]=size(E);
[nc,np2]=size(A.inf);
if (nx~=length(x.inf) || np~=np2)
    gldisp('nx',nx,'nx2',length(x.inf),'np',np,'np2',np2);
    error('glremvars: Dimension mismatch!');
end
xi=infsup(x.inf,x.sup);
nI=1:nx;nI(I)=[];
Ai=infsup(A.inf,A.sup);

% remove variables
for i=1:length(I)
    j=I(i);             % index of the var to be removed
    pw=E(:,j);          % the col in E correspondig the var
    C=ones(nc,1)*(xi(j).^pw)';% constants matrix
    Ai=Ai.*C;           % new A
end
E(:,I)=[];              % remove enries in E 
[np,nx]=size(E);        % new nx

% remove entries matching the constant and duplicate monomials
[E2,perm]=sortrows(E);
e=zeros(1,nx);j=0;J=[];
for i=perm'
    if all(E(i,:)==0),
        % found a zero row (constant monomial)
        J=[J i]; 
    elseif all(e==E(i,:)), 
        % found a duplicate monomial
        J=[J i]; Ai(:,j)=Ai(:,j)+Ai(:,i);
    else e=E(i,:);j=i;
    end
end

nJ=1:np;nJ(J)=[];       % remaining monomials
ci=sum(Ai(:,nJ),2);     % constant factor vector
Ai(:,J)=[];             % remove entries matching the constant monomials
E(J,:)=[];              % do the same in E 

%generate output
A.inf=Ai.inf;A.sup=Ai.sup;
x.inf=x.inf(nI);x.sup=x.sup(nI);
c.inf=ci.inf;c.sup=ci.sup;