%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glvariation.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [ind, var, type]=glvariation(P, x, method, deb)
%
% Sort the components of x by the criteria which has the most effect
% on the constraints. 
% If method==1 the indices of x are sorted after the width.
% If the method input argument is not set or 0 the proceed as follows:
%   --If some componets of x is onbounded, sort after the
%   maximal width. 
%   --Otherwise compute epsilon:=(xu-xl)B(xu-xl), B=sum(|G|), 
%   where G is the quadratic part of the matrices.
%       --If epsilon > MINVAR compute the quadratic variation 
%       --else the smear.
% 
%
% P             problem definition
%   .Q.A        nc x nx^2+nx constraint coeff matrix
%     .inf,.sup outer enclosures
%   .Q.F        nc x 1 general constraint bounds
%     .inf,.sup outer enclosures
% x             nx x 1 bound constraints
%   .inf,.sup   outer enclosures 
% method        method select from 'intelligent','width' or 'variation'
% scount        nx x 1 counter of the selected components, needed for the
%               'intelligent' method
% deb           debug flag 0=no output, 1=output 
%
% ind           nx x 1 sorted vector of the variable indices, sorted after
%               the variation, beginning with the greatest. If no input
%               argument is given a cell of the possible methods is
%               returned.
% var           nx x 1 sorted vector of the variation in the constraints,
%               caused by the change of x: var(i) is the variation of the 
%               constraints caused by x_(ind(i))
% type          type of the computed variation:
%               'width', 'quadratic variation' or 'smear'

function [ind, var, type]=glvariation(P, x, method, scount, deb)

% global variables
global glconf;      % global configuration variable
INF=1e5;            % positive and negative infinity
MINVAR=1e-3;        % constant to decide if the quadratic variaton or the 
                    % smear will be computed
% argcheck
if nargin ==0, 
    ind=[{'intelligent'};{'width'};{'variation'}];
    return
end
if nargin < 2, error('glvariation: Not enough arguments!'); end
nx=length(x.inf);
if nargin < 3, method='intelligent'; end
if nargin < 4, scount=zeros(nx,1); end
if nargin < 5, deb=0; end

% set constants from config
if isfield(glconf,'infty'),INF=glconf.infty;end 
if isfield(glconf,'minvar'),MINVAR=glconf.minvar;end 

% init
[nc,nA]=size(P.Q.A.inf);
if nA~=(nx+1)*nx, error('glvariation: dimesion mismatch in A and x!'); end
if length(P.Q.F.inf)~=nc, 
    error('glvariation: dimesion mismatch in A and F!'); 
end
linf=x.inf<=-INF;uinf=x.sup>=INF;
cscal=ones(nc,1);

if  (method(1)=='i' || method(1)=='v') && ~any(linf) && ~any(uinf)
    % find the index of the component which reduces the variation of
    % the constraints the most. (xu-xl)B(xu-xl), B=sum(|G|), G is the
    % quadratic part of the matrices
    % use constraint propagation to get the range of the constraints
    r=glrange(P, x, 0);
    % intersect F with r so get new bounds
    u.inf=max(r.inf,P.Q.F.inf);u.sup=min(P.Q.F.sup,r.sup);
%     z1=r.sup-u.inf;z2=u.sup-r.inf;
%     q=inf+zeros(nc,1);zz=z1+z2;
%     q(zz>0)=zz(zz>0);
%     ind=z1>0&z2>0;
%     m=min(z1,z2);
%     q(ind)=m(ind);
    q=max(u.sup-u.inf,cscal);
    if any(q<=0)
        gldisp('q',q);
        error('glvariation: negative constraint weight vector!')
    end
    % sum the quadratic part of all constraints
    A=(P.Q.A.inf+P.Q.A.sup)/2;B=zeros(nx,nx);
    G=cell(nc);
    for i=1:nc
        G{i}=glqvec2qmat([0 A(i,:)]);
        G{i}=G{i}(2:end,2:end);
        B=B+(1/q(i))*G{i};
    end
    d=x.sup-x.inf;% x diameter
    Bd=B*d;epsilon=d'*Bd;
    diB=diag(B);
    if epsilon > MINVAR
        % The quadratic components are dominant, compute the quadratic
        % variation
        if deb, 
            disp(['epsilon=' num2str(epsilon) ...
                ', quadratic variation computed!']);
        end
        v=d.*(Bd-(d/4).*diB);
        %v=v+rand(size(v))*max(v);
        type='quadratic variation';
    else
        % Compute the smear
        if deb, 
            disp(['epsilon=' num2str(epsilon) ', smear computed!']);
        end
        q=max(r.sup-r.inf, cscal);
        xb=infsup(x.inf, x.sup);
        w=0;
        for k=1:nc
            % compute sum F'(x)_k := c_k + x'(G{k}+G'{k}) with
            % c:=P.Q.A(:,1:nx) and G_k:=G{k}
            D=infsup(P.Q.A.inf(k,1:nx),P.Q.A.sup(k,1:nx))+xb'*(G{k}+G{k}');
            w=w+D/q(k);
        end
        v=d.*glmag(w)';
        %!!!!
        %v=mid(v);
        %!!!!!
        %v=glmag(v);
        type='smear';
    end
    [var,ind]=sort(v,1,'descend');
else
    if deb,
        disp('infinite components, witdth computed!');
    end
    
    % sort after the width of the x
    wid=x.sup-x.inf;
    
    % infinite indices
    bothinf=find(linf & uinf);
    oneinf=[find(~linf&uinf) ; find(linf&~uinf)];
    [var, ind]=sort([-x.inf(~linf&uinf) ; x.sup(linf&~uinf)],...
        1, 'descend');
    oneinf=oneinf(ind);
    fin=find(~linf&~uinf);
    [var, ind]=sort(wid(fin), 1, 'descend');
    fin=fin(ind);
    ind=[bothinf ; oneinf ; fin];
    var=[inf+zeros(nx-length(fin),1) ; var];
    type='width';
    % debug
    if deb, gldisp('bothinf',bothinf,'oneinf',oneinf,'fin',fin); end
end

if method(1)=='i'
    % merge the occurance and the variation order; the number of occurance is
    % weighted by the constant VF and in the second by OF.
    OF=1; % occurance number weight factor
    VF=5; % variation order weight factor
    scount=scount*OF;
    scount(ind)=scount(ind)+(1:nx)'*VF;
    [s,ind]=sort(scount);
end
