%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsolstatus.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function code=glsolstatus(statusstr)
%
% Either:
% (1) Get all possible statues (all stored in gsolstatus.txt)
% (2) Get the status code corresponding to the given status id.
% (3) Get the status string(s) corresponding to the given code(s)
% (4) Check if the vector/problem contains the given status (id) or not
% (5) Add the given status (id) to the given vector/problem
% (6) Remove the given status (id) to the given vector/problem
%
% arg1   (1) missing, (2) string encoding the status,
%        (4)-(6) status code vector or problem P with field P.solstatus
% arg2   (1)-(3) missing, (4)-(6) status id
% setf   (1)-(4) missing, (5) = true(1), (6) = false(0)
% ret    (1) cell containing all possible status strings, (2) status code, 
%        (3),(5)-(6) a cell of status strings or the problem P, 
%        (4) true(1) if the status has been found else false(0)

function ret=glsolstatus(arg1,arg2,setf)

global glsolst;

if isempty(glsolst)
    % load the different statuses
    glsolst.ids=[];
    glsolst.codes=[];
    glsolst.strings=[];
    filen='glsolstatus.cfg';
    if exist(filen,'file')==2,
        fid=fopen(filen);
        while 1
            tline = fgetl(fid);
            if ~ischar(tline), break, end
            if ~isempty(tline) && tline(1)~='%';
                ind=find(tline==',');
                if isempty(ind) || length(ind)~=2 || ind(1)<2 || ...
                        ind(2)>length(tline)-3
                    error(['glsolstatus: error parsing the file ' ...
                        filen ' !']);
                end
                id=tline(1:ind(1)-1);
                str=tline(ind(1):ind(2)-1);
                code=str2num(str);
                if isempty(code)
                    error(['glsolstatus: error parsing the number ' ...
                        str ' !']);
                end
                str=tline(ind(2)+2:end);
                glsolst.ids=[glsolst.ids {id}];
                glsolst.codes=[glsolst.codes code];
                glsolst.strings=[glsolst.strings {str}];
            end
        end
        fclose(fid);
    else
        error(['glsolstatus: error opening ' filen ' !']);
    end
end

%argcechk
if nargin < 1, 
    % (1) Get all possible statues (all stored in gsolstatus.txt)
    ret=glsolst;
    return;
elseif ischar(arg1), 
    ret=glsolst.codes(strcmp(glsolst.ids,arg1));
    % (2) Get the status code corresponding to the given status id.
    return;
elseif isnumeric(arg1),
    % any of (3)-(6) with input vector
    stat=arg1;
elseif isstruct(arg1)
    % any of (3)-(6) with input P
    if isfield(arg1,'solstatus'), stat=arg1.solstatus;
    else error('glsolstatus: solstatus field is missing in P!');
    end
else error('glsolstatus: wrong input argument!');
end

% add or remove the status 0
if all(stat>0), stat=[0 stat];
elseif any(stat<0) && any(stat==0), stat(stat==0)=[];
end
    
if nargin < 2
    % (3) Get the status string(s) corresponding to the given code(s)
    ret=[];
    for i=1:length(stat)
        ind=find(glsolst.codes==stat(i));
        if any(ind~=0), ret=[ret glsolst.strings(ind)]; end
    end
elseif ~ischar(arg2)
    error('glsolstatus: second argument must be a string!');
elseif nargin < 3
    % (4) Check if the vector/problem contains the given status (id) or not
    ind=glsolstatus(arg2);
    if ~isempty(ind), ret=any(stat==ind);
    else ret=false;
    end
else
    if setf==true
        % (5) Add the given status (id) to the status vector
        ind=glsolstatus(arg2);
        if ~isempty(ind) && ~any(stat==ind)
            stat=sort([stat ind]);
        end
    else
        % (6) Remove the given status (id) to the status vector
        ind=glsolstatus(arg2);
        if ~isempty(ind) && any(stat==ind)
            stat(stat==ind)=[];
        end
    end
    % add or remove the status 0
    if all(stat>0), stat=[0 stat];
    elseif any(stat<0) && any(stat==0), stat(stat==0)=[];
    end
    if isstruct(arg1)
        % set the resulting vector into P.solstatus
        arg1.solstatus=stat; ret=arg1;
    else ret=stat;
    end
end



