%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsystem.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [status,result] = glsystem(command, prt)
%
% executes a unix or a windows command
%
% command       a normal unix or a windows command or
%               'open <filename>' tries to open a file using an editor or a 
%                viewer:
%                   unix: ps,pdf,html
%                   windows: various
%               '<convertcommand> <filename>' converts a file into another
%                   unix: dag2def,mod2def,dag2dot,dot2ps
%                   windows: none
% prt           display the valid command 1:on 0:off
%
% status        =0: command sucessfully executed, else error code
% result        the output resulting the execution of the command

function [status,result] = glsystem(command, prt)

global glconf;      % global configuration structure

% argcechk
if nargin <1, error('glsystem: Not enough arguments!');end
if nargin <2, prt=0;end

if length(command)>8 && ...
      (strcmp(command(1:8),'glb2mod ') || strcmp(command(1:8),'def2mod ') ||...
       strcmp(command(1:8),'glb2sml ') || strcmp(command(1:8),'def2sml ')),
    % system independent conversions
    file=command(9:end);
    if length(file)>4
        outf=[file(1:end-4) '.' command(5:7)];
        [P, readerr]=glread(file, 1);
        if readerr==1,
            result=['Problem file ' file ' could not be read!'];
            status=1;
        elseif readerr==2,
            result='File type not supported, use a converter!';
            status=1;
        elseif readerr==3,
            result=['Problem file ' file ' is missing!'];
            status=1;
        else
            % convert to ampl format
            txt=glprob2str(P, P.xbox, 2);
            % save .mod or .sml (currently the same format)
            fid=fopen(outf, 'w');
            for i=1:length(txt)
                fprintf(fid,'%c',txt{i});
                fprintf(fid,'\n');
            end
            fclose(fid);
            result='File converted!';
            status=0;
        end
    end
elseif isunix,
    % check coconut path
    if length(command)>5 && strcmp(command(1:5),'open ')
        % open a file with its default editor or viewer
        file=command(6:end);
        if strcmp(file(end-2:end),'.ps') 
            if isfield(glconf,'psview'),
                command=[glconf.psview ' ' file ' &'];
                [status,result]=glunix(command, prt);
            else
                status=-1;
                result='No postscript viewer found!';
            end
        elseif strcmp(file(end-3:end),'.pdf')
            if isfield(glconf,'pdfview'),
                command=[glconf.pdfview ' ' file ' &'];
                [status,result]=glunix(command, prt);
            else
                status=-1;
                result='No pdf viewer found!';
            end
        elseif strcmp(file(end-4:end),'.html')
            if isfield(glconf,'htmlview'),
                command=[glconf.htmlview ' ' file ' &'];
                [status,result]=glunix(command);
            else
                status=-1;
                result='No html viewer found!';
            end
        end
    elseif length(command)>8 && strcmp(command(1:8),'sml2glb ')
        % convert sml2glb or mod2glb
        file=command(9:end);
        [status,result]=system([which('glsmlparse_unix.') ' -g ' file]);
        logfile=[file(1:end-4) '.glsmlparse.log'];
        if exist(logfile,'file')==2
            if status,
                fid=fopen(logfile);
                result=fgetl(fid);
                fclose(fid);
            else
                delete(logfile);
            end
        end
    elseif length(command)>8 && strcmp(command(1:8),'sml2def ')
        % convert sml2def or mod2def
        file=command(9:end);
        [status,result]=system([which('glsmlparse_unix.') ' -d ' file]);
        logfile=[file(1:end-4) '.glsmlparse.log'];
        if exist(logfile,'file')==2
            if status,
                fid=fopen(logfile);
                result=fgetl(fid);
                fclose(fid);
            else
                delete(logfile);
            end
        end
    elseif length(command)>8 && strcmp(command(1:8),'dag2def ')
        % convert dag2def
        if isfield(glconf,'convpath') && exist(glconf.convpath,'dir')==7
            file=command(9:end);
            command=[which('dag2def.') ' ' file];
            [status,result]=unix(command); %glunix() doesnt work here, as LD_LIBRARY_PATH is not passed on to execute.sh
        else
            status=-1;
            result='Conversion environment not found!';
        end
    elseif length(command)>8 && strcmp(command(1:8),'mod2def ')
        % convert mod2def
        if isfield(glconf,'convpath') && exist(glconf.convpath,'dir')==7
            file=command(9:end);
            command=[which('mod2def.') ' ' file];
            [status,result]=unix(command); %glunix() doesnt work here, as LD_LIBRARY_PATH is not passed on to execute.sh
%             command=[glconf.cocopath 'ampl2dag ' file];
%             [status,result]=glunix(command, prt);
%             if ~status,
%                 file=[file(1:end-3) 'dag'];
%                 command=[glconf.cocopath 'dag2gloptlab ' file];
%                 [status2,result2]=glunix(command, prt);
%                 status=status+status2;
%                 result=[result result2];
%             end
        else
            status=-1;
            result='Conversion environment not found!';
        end
    elseif length(command)>8 && strcmp(command(1:8),'dag2dot ')
        % convert dag2dot
        if isfield(glconf,'cocopath') && exist(glconf.cocopath,'dir')==7
            file=command(9:end);
            command=[glconf.cocopath 'dag2dot -n ' file];
            [status,result]=glunix(command, prt);
        else
            status=-1;
            result='Coconut entvironment not found!';
        end
    elseif length(command)>7 && strcmp(command(1:7),'dot2ps ')
        % convert dot2ps
        file=command(8:end-4);
        command=['dot -Tps ' file '.dot >' file '.ps'];
        [status,result]=glunix(command, prt);
    elseif length(command)>7 && strcmp(command(1:7),'ps2pdf ')
        % convert ps2pdf
        file=command(8:end-3);
        command=['ps2pdf ' file '.ps ' file '.pdf'];
        [status,result]=glunix(command, prt);
    elseif strcmp(command,'cpuinfo')
        % cpu information string
        result='';
        [status, cpu]=glunix('cat /proc/cpuinfo');
        if ~status,
            cpu=glstr2scell(cpu, char(10)); % parse string with unix '\n'
            for i=1:length(cpu)
                if strfind(cpu{i}, 'model name')
                    result=cpu{i}(find(cpu{i}==':')+1:end);
                    break;
                end
            end
        end
    else
        % execute a standard unix command
        [status,result] = glunix(command, prt);
    end
elseif ispc,
    if length(command)>5 && strcmp(command(1:5),'open '),
        % open a file with its default editor or viewer
        file=command(6:end);
        if prt, disp(['windows: open ' file]), end;
        winopen(file);
        status=0;result='';
    elseif length(command)>8 && strcmp(command(1:8),'sml2glb ')
        % convert sml2glb or mod2glb
        file=command(9:end);
        [status,result]=system(['"' which('glsmlparse_win.exe') '" -g "' file '"' ]);
        logfile=[file(1:end-4) '.glsmlparse.log'];
        if exist(logfile,'file')==2
            if status,
                fid=fopen(logfile);
                result=fgetl(fid);
                fclose(fid);
            else
                delete(logfile);
            end
        end
    elseif length(command)>8 && strcmp(command(1:8),'sml2def ')
        % convert sml2def or mod2def
        file=command(9:end);
        [status,result]=system(['"' which('glsmlparse_win.exe') '" -d "' file '"' ]);
        logfile=[file(1:end-4) '.glsmlparse.log'];
        if exist(logfile,'file')==2
            if status,
                fid=fopen(logfile);
                result=fgetl(fid);
                fclose(fid);
            else
                delete(logfile);
            end
        end
    elseif strcmp(command,'cpuinfo')
        % cpu information string
        % get number of processors from the environment variable
        nproc = str2double(getenv('NUMBER_OF_PROCESSORS'));
        status=1; result='';
        if ~isempty(nproc), 
            % query type and frequency of processors 0,...,nproc-1
            keybase = 'HARDWARE\DESCRIPTION\System\CentralProcessor\';
            result = '';
            for i=1:nproc
                key = strcat(keybase,int2str(i-1));  
                % e.g. 'HARDWARE\DESCRIPTION\System\CentralProcessor\1'
                procname = winqueryreg('HKEY_LOCAL_MACHINE', key , ...
                    'ProcessorNameString');
                ind=find(procname~=' ');
                procname=procname(ind(1):ind(end));
                mhz = winqueryreg('HKEY_LOCAL_MACHINE', key, '~MHz');
                if nproc > 1, result = [result 'CPU ' int2str(i) ': '];end
                result =[result procname ', speed: ' int2str(mhz) 'MHz, '];
            end
            result=result(1:end-2);status=0;
        end
    else
        % execute a windows command
        if prt, disp(['windows: ' command]), end;
        [status, result] = system(command);
    end
else error('Unsupported operating system!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glunix.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [status,result] = glunix(command, prt)
%
% Execute a unix command by using a shell script called 'execute.sh'
% which unsets the LD_LIBRARY_PATH and then executes the command.
%
% command       unix command to execute
% prt           display the command 1:yes 2:no
%
% status        =0: command sucessfully executed, else error code
% result        the output resulting the execution of the command

function [status,result] = glunix(command, prt)
% argcechk
if nargin <1, error('glunix: Not enough arguments!');end
if nargin <2, prt=0;end

if prt, disp(['unix: ' command]), end;
[status,result] = unix([which('execute.sh') ' ' command]);
