%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gltimer.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function status=gltimer(timer, command, P, xbox)
%
% start, reset, pause or check the timer, write outputs, logs, autosaves.
%
% timer             timer structure
%        .status    0 = off, 1 = on
%        .timeout   seconds until timeout is reaced
%        .outptime  seconds between two text and/or log outputs
%        .lastoutp  time of the last text and/or log output
%        .prt       =0 no outp, =1 brief output 2= full output, 3= both
%        .log       =1 indicates log saving enabled
%        .savetime  seconds between two automatic saves
%        .lastsave  time of the last save
%        .savedir   the directory to store the saves, if empty use config
%        .logdir    the directory to store the logs, if empty use config
%        .started   the ms at which the timer has been started
%        .ellaped   the total time ellapsed = .ellapsed+(toc-started)
% command           = 'start' or 's' start timer
%                   = 'reset' or 'r' reset timer
%                   = 'pause' or 'p' pause timer
%                   = 'check' or 'c' check timer
%                   = 'force' or 'f' force all possible outputs
%
% P                 problem definition or problem name (must be supplied for
%                   check and force)
% xbox              bound constraints
%
% status        =-1 timeout
%               = 0 tasklist ok
%               = 1 textoutput retuned

function [timer,status]=gltimer(timer, command, P, xbox)

global glconf;      % global configuration file
global gloutput;    % global output structure
% argcheck
if nargin < 2, error('gltimer: Not enough arguments!'); end
command=command(1);
if (command=='c' || command == 'f') && nargin<3,
    error('gltimer: Not enough arguments!');
end
% init
status=0;
if command=='r'
    % reset timer
    timer.lastoutp=0;timer.lastsave=0;
    timer.ellapsed=0;timer.started=toc;
elseif timer.status==0,
    % timer is off
    if command=='s'
        % start the timer
        timer.status=1;
        timer.started=toc;
    else
        msg=['The timer is off at:' glshowtime(timer.ellapsed)];
        if ~isfield(gloutput, 'outfhandle') || isempty(gloutput.outfhandle)
            % no output function handle supplied
            disp(msg);
        else
            % send output to the output function
            feval(gloutput.outfhandle, msg);
        end
    end
elseif timer.status==1 && command=='p'
    % pause timer
    timer.ellapsed=timer.ellapsed+(toc-timer.started);
    timer.status=0;
elseif timer.status==1,
    % timer is on
    %set configuration
    if ~isfield(timer,'logdir') || isempty(timer.logdir)
        if isfield(glconf,'logdir'),timer.logdir=glconf.logdir;
        else timer.logdir=[fileparts(which('Gloptlab.m')) '/Logs/'];
        end
    end
    if ~isfield(timer,'autosavedir') || isempty(timer.logdir)
        if isfield(glconf,'autosavedir'),
            timer.autosavedir=glconf.autosavedir;
        else
            timer.autosavedir=[fileparts(which('Gloptlab.m')) ...
                '/Autosaves/'];
        end
    end
    %get ellapsed time
    ellapsed=timer.ellapsed+(toc-timer.started);
    %check timer
    if ~isfield(timer,'lastoutp'),timer.lastoutp=0;end
    if ~isfield(timer,'lastsave'),timer.lastsave=0; end
    %timeout flag
    timeout=isfield(timer,'timeout') && ~isinf(timer.timeout) ...
        && (ellapsed >= timer.timeout);
    %full textoutput flag
    textout=isfield(timer,'prt')&&timer.prt>1&&~isempty(gloutput.out)...
        && ((ellapsed-timer.lastoutp>=timer.outptime) || command=='f');
    %logoutput flag
    logout=isfield(timer,'log')&&timer.log&&~isempty(gloutput.out)...
        && ((ellapsed-timer.lastoutp>=timer.outptime) || command=='f');
    %autosave flag
    autosave=isfield(timer,'savetime') && ~isinf(timer.savetime) &&...
        nargin >= 2 &&...
        ((ellapsed-timer.lastsave>=timer.savetime) || command=='f');
    %checking
    if autosave && (isstruct(P) || ischar(P))
        %autosave problem
        if exist(timer.autosavedir, 'dir')
            if isstruct(P), pfile=[timer.autosavedir P.name '_auto'];
            else pfile=[timer.autosavedir P '_auto'];
            end
            %save problem file
            save([pfile '.glb'],'P');
            %save info file
            info=glprobinfo(P);
            glsave(info,[pfile '.inf']);
            gloutput.out=[gloutput.out;...
                {['Problem with current box saved in ' pfile '.glb']}];
            timer.lastsave=ellapsed;
        else
            %save dir missing
            gloutput.out=[gloutput.out;...
                {['Automatic problem saving failed, check '...
                'the autosavedir']}];
            gloutput.out=[gloutput.out;...
                {['Autosaving disabled, saving intervals in '...
                'timer.savetime set to infinity!']}];
            timer.savetime=inf;
        end
    end
    if isfield(timer,'prt') && (timer.prt==1 || timer.prt==3) &&...
            ~isempty(gloutput.brief)
        %brief output
        if ~isfield(gloutput, 'briefhandle') || isempty(gloutput.briefhandle)
            %no brief output function handle supplied
            for i=1:length(gloutput.brief),disp(gloutput.brief{i}); end
        else
            %send output to the brief output function
            feval(gloutput.briefhandle, gloutput.brief);
        end
        gloutput.brief=[];
    end
    if textout || logout
        if logout && (isstruct(P) || ischar(P)),
            if exist(timer.logdir, 'dir')
                %set filename
                if isstruct(P), fname=[timer.logdir P.name '.log'];
                else fname=[timer.logdir P '.log'];
                end
                %append log in logfile
                fid=fopen(fname, 'a');
                %save log
                fprintf(fid,['* ' datestr(now) ' *\n']);
                for i=1:length(gloutput.out)
                    fprintf(fid,'%c',gloutput.out{i});
                    fprintf(fid,'\n');
                end
                fclose(fid);
                gloutput.out=[gloutput.out;...
                    {['Log automatically appended to ' fname]}];
            else
                %log dir missing
                gloutput.out=[gloutput.out;...
                    {['Automatic log saving failed, check '...
                    'the logdir']}];
                gloutput.out=[gloutput.out;{'Log saving disabled!'}];
                timer.log=0;
            end
        end
        if textout
            if timeout || textout || logout || autosave
                str=['Time elapsed:' glshowtime(ellapsed)];
                if ~isinf(timer.timeout),
                    str=[str '    Max time:' glshowtime(timer.timeout)];
                end
                gloutput.out=[gloutput.out;{str}];
            end
            %text output
            if ~isfield(gloutput, 'outfhandle') || isempty(gloutput.outfhandle)
                %no output function handle supplied
                for i=1:length(gloutput.out),disp(gloutput.out{i}); end
            else
                %send output to the output function
                feval(gloutput.outfhandle, gloutput.out);
            end
        end
        gloutput.out=[];
        timer.lastoutp=ellapsed;
    end
    if timeout
        gloutput.out=[gloutput.out;{'TIMEOUT!'}];
        %timeout
        status=-1;
        return;
    end
end
