%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% glsolve.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function glsolve(probs, startp, sfile, soldir, testenv, config,
%       timeout, outptime, savetime, autosavedir, prt, log, logdir, pro,
%       proofdir, stat, statfile, errhand)
%
% solve the selected problem or all the problems in a problem list or
% in a directory.
%
% probs         single .def or .glb or problem list (.prl) file with 
%               full path, or a directory, where all .def problems should
%               be solved.
% startp        file name of the problem where the problem list
%               is strating to be solved
% sfile         startegy filename with full path
% soldir        directory to store the solutions
% testenv       write solution for the testenvironment format:.res and .tim
% config        configuration filename with full path
% timeout       timeout in seconds
% outptime      text and log output time in seconds
% savetime      time after an autosave is made (in 'autosavedir')
% autosavedir   the directory to store the autosaves, if empty use config
% prt           =0 no outp, =1 brief output 2= full output, 3= both
% log           write logdir 0=no 1=yes (in logdir)
% logdir        the directory to store the logsfiles, if empty use config
% pro           proof output flag      0 = no output 1 = output
% proofdir      directory to store the proofs
% stat          statistics output flag 0 = no output 1 = output
% statfile      file to which the statistic will be appended
% errhand       error handling flag   0 = off 1 = on
%
% sols          solution cell vector, for sol fields see glsol2str
% P             last problem definition
% xbox          last bound constraint

function [sols, P, xbox]=glsolve(probs, startp, sfile, soldir, testenv, config,...
    timeout, outptime, savetime, autosavedir, prt, log, logdir, pro, ...
    proofdir, stat, statfile, errhand)

global glconf;      % global configuration file
global glconfstat;  % status of the loaded configuration
global gloutput;    % global output structure
global glbreak;     % gloptlab break flag

if nargin<1, error('solve: no problems supplied');end
cdir=pwd;
gldir=[fileparts(which('glsolve.m')) filesep];
if nargin<2, startp='';end
if nargin<3, sfile=[gldir 'Strategies/solve.str'];end
if nargin<4, soldir=[gldir 'Solutions/'];end
if nargin<5, testenv=1;end
if nargin<6, config=[gldir 'Gloptlab.cfg'];end
if nargin<7, timeout=180;end
if nargin<8, outptime=15;end
if nargin<9, savetime=30;end
if nargin<10,autosavedir=[gldir 'Autosaves/'];end
if nargin<11,prt=1;end
if nargin<12,log=1;end
if nargin<13,logdir=[gldir 'Logs/'];end
if nargin<14,pro=0;end
if nargin<15,proofdir='';end
if nargin<16,stat=0;end
if nargin<17,statfile='';end
if nargin<18,errhand=0;end

% init
sols=[];P=[];xbox=[];
% set path
cd(gldir);
path(gloptpath(prt));
cd(cdir);
if ~iscell(probs)
    % check problem file or create problem list.
    switch exist(probs) %#ok<EXIST>
        case 2
            problist=probs;
        case 7
            if probs(end)~='/' && probs(end)~='\', probs=[probs filesep]; end
            problist=[probs 'problist.prl'];
            direct=dir(probs);
            files={direct.name};
            isd=[direct.isdir];
            files=files(isd==0);
            fid=fopen(problist,'w');
            if fid == -1, error(['glsolvetest: Filename wrong:' listfile]); end
            for i=1:length(files)
                n=files{i};
                if length(n)>4 && strcmp(n(end-3:end),'.def'),
                    fprintf(fid,'%c',[probs n]);
                    fprintf(fid,'\n');
                end
            end
            fclose(fid);
        otherwise
            error(['solve: ' probs ' not found!']);
    end
else
    problist=probs;
end
% set the config filename
if isempty(config) || exist(config,'file')~=2
    basedir=[fileparts(which('Gloptlab')) '/'];
    if exist([basedir config],'file')==2, config=[basedir config];
    elseif isunix, config=[basedir 'Gloptlab_unix.cfg'];
    elseif ispc, config=[basedir 'Gloptlab_win.cfg'];
    end
    if isempty(config) || exist(config,'file')~=2,
        error('No config file found!');
    end
end

% initialize the configuration, no extra checks
if prt, disp(['Used configuration file: ' config ]); end
glinitconf(config,0,prt);

% check intlab
if glconfstat.intlab_running==0, 
    error('Intlab could not be initialized, check Gloptlab.cfg!');
end

%display modes
if prt,
    disp(' ');disp('*OUTPUT SETTINGS*');
    disp(['timeout:' num2str(timeout) ', prt:' num2str(prt) ', autosavetime:'...
        num2str(savetime) ', log:' num2str(log) ', pro:' num2str(pro) ...
        ', stat:' num2str(stat)]);
end


%init and create directories
if isempty(soldir),soldir=glconf.soldir;end
if soldir(end)~=filesep, soldir=[soldir filesep];end
if exist(soldir,'dir')~=7,mkdir(soldir);end
if prt, disp(['Solutions directory: ' soldir]);end
if ~isinf(savetime),
    if isempty(autosavedir),autosavedir=glconf.autosavedir;end
    if exist(autosavedir,'dir')~=7,mkdir(autosavedir);end
    if prt, disp(['Autosaves directory: ' autosavedir]);end
end
if log, 
    if isempty(logdir),logdir=glconf.logdir;end
    if logdir(end)~=filesep, logdir=[logdir filesep];end
    if exist(logdir,'dir')~=7,mkdir(logdir);end
    if prt, disp(['Logs directory: ' logdir]);end
end
if pro, 
    if isempty(proofdir),proofdir=glconf.proofdir;end
    if proofdir(end)~=filesep, proofdir=[proofdir filesep];end
    if exist(proofdir,'dir')~=7,mkdir(proofdir);end
    if prt, disp(['Proofs directory: ' proofdir]);end
end
if stat,
    if isempty(statfile),statfile=glconf.statfile;end
    statdir=[fileparts(statfile) filesep];
    if exist(statdir,'dir')~=7,mkdir(statdir);end
    if prt, disp(['Statistics file: ' statfile]);end
end

%set output to the disp function
gloutput.out=[];gloutput.outfhandle=[]; %full output
gloutput.brief=[];gloutput.briefhandle=[]; %brief output

%set timer
timer=glcreatetimer(timeout, outptime, prt, log, savetime, logdir, ...
    autosavedir);
%set break flag
glbreak=0;

%PROBLIST
if prt, disp('*Problems: ');end
if isempty(problist)
    probs=  [ {'/eusers/dferi/Projects/Dags/cluster.def'};...
           {'/eusers/dferi/Projects/Dags/ellprob.glb'};...
           {'/eusers/dferi/Projects/Dags/c-rump1.def'};...
           {'/eusers/dferi/Projects/Dags/rand009.glb'};...
           {glrandomprob(2, [{'='};{'='};], 'ellipsoid', [1; 1])};...
           {glrandomprob(5, ...
             [{'='};{'='};{'='};{'='};{'='};{'='};], 'ellipsoid', [1; 1; 1; 1; 1])};...
           {glrandomprob(13,...
             [{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='};{'='}],...
             'ellipsoid', [1; 1; 1; 1; 1; 1; 1; 1; 1 ;1; 1; 1 ;1 ])};...
           {glrandomprob(5,...
             [{'='};{'='};{'='};{'='};{'='};{'<'};{'<'}],...
             'ellipsoid', [1; 1; 1; 1; 1])};...
           {glrandomprob(2,...
             [{'='};{'='};{'<'};{'<'}],...
             'ellipsoid', [1; 1])};...
           {'/eusers/dferi/Projects/Dags/test2a.def'}...  
         ];
    problist= probs(4:4);
elseif ~iscell(problist)
    %filename supplied
    if exist(problist,'file')==2
        [fpath,fname,fext] =fileparts(problist);
        if strcmp(fext,'.prl') || strcmp(fext,'.lst')
            %problem list file
            fid = fopen(problist,'r');
            probs=cell(0);comm=cell(0);
            while 1
                tline = fgetl(fid);
                if ~ischar(tline), 
                    break;
                elseif ~isempty(tline) && ~strcmp(tline(1),'%')
                    if tline(end-3) ~= '.', tline=[tline '.def'];end 
                    probs=[probs; {tline}];
                else
                    comm=[comm; {tline}];
                end
            end
            problist=probs;
            if prt,
                for i=1:length(comm),
                    %print comments
                    disp(comm{i});
                end
            end
        elseif strcmp(fext,'.def') || strcmp(fext,'.glb')
            %single problem file
            problist={problist};
        end
    else
        disp(['Problem file "' problist '" is missing!']);
        return;
    end
else
    %cell of problems supplied
end

if prt,
    % display the problems
    disp(' ');
    for i=1:length(problist),
        if ischar(problist{i}), disp([num2str(i) ': ' problist{i}]);
        else disp([num2str(i) ': ' problist{i}.name])
        end
    end
end

% find the index of the start problem in the problem list
if ~isempty(startp)
    start=0;
    [spa,spn,sext]=fileparts(startp);
    for i=1:length(problist)
        [pa,pn,ext]=fileparts(problist{i});
        if isempty(sext),
            % startp contains no extension, compare the names without 
            % extensions
            if strcmp(pn,spn)
                start=i;
                break;
            end
        else
            % startp contains an extension, compare names and extensions
            if strcmp([pn ext],[spn sext])
                start=i;
                break;
            end
        end
    end
    if start==0
        disp(['Start problem: "' startp '" not found in the problem list!']);
        return
    end
else start=1;
end

if prt, disp(['Starting at index ' num2str(start) ' !']);disp(' '); end

%STRATEGY
% determine if a mat or a txt file
fid = fopen(sfile,'r');
if fid ~= -1
    fline = fgetl(fid);fclose(fid);
else
    sfile2=[glconf.stratdir filesep sfile];
    fid = fopen(sfile2,'r');
    if fid ~= -1
        fline = fgetl(fid);fclose(fid);
        sfile=sfile2;
    else
        disp(['Strategy file ' sfile ' is missing!']);
        return;
    end
end
% load the strategy
if fline(1)=='%', strategy=glload(sfile);   %text file format
else load(sfile, '-mat');                   %mat file format
end
if prt,
    % display the strategy
    disp(['*Default strategy: ' strategy.name])
    spac='';tab='   ';
    for i=1:length(strategy.tasks),
        tskn=strategy.tasks{i}.name;
        if ~isempty(findstr(tskn,'End ')), spac=spac(1:end-length(tab));end
        disp([num2str(i) ': ' spac tskn]);
        if ~isempty(findstr(tskn,'Begin ')), spac=[spac tab];end
    end
    disp(' ');
end
% check the tasklist
[status, msg] = glchktskl(strategy.tasks);
if prt && ~isempty(msg)
    for i=1:length(msg), disp(msg{i}); end
end
if status ~= 0, return; end

%PROCESS TASKLIST (nopause, begin first problem first task, textout, 
% statistics)
timer=gltimer(timer, 's');
[sols, P, xbox]=glprocplist(...
    problist, strategy, soldir, 1, start, 1, [], [], timer, strategy, prt>0,...
    testenv, 0, [], [], pro, proofdir, stat, statfile, [], errhand);
%DISPLAY REMAINIG OUTPUT (addition to the by 'glchktimer' already displayed)
for i=1:length(gloutput.out), disp(gloutput.out{i}); end
gloutput.out=[];
if prt,
    %display the results
    disp(' ');
    disp(char(zeros(1,75)+45));
    disp('*Results Summary: ')
    disp(char(zeros(1,75)+45));
    for i=1:length(sols),
        str=glsol2str(sols{i});
        for j=1:length(str), disp(str{j}); end
        disp(char(zeros(1,75)+45));
    end
end

% TESTS
% glsolve ('c:/Matlab/Gloptlab/Testproblems/c-rump.def')
% glsolve ('c:/Matlab/Gloptlab/Testproblems/lib3/');
% glsolve('~/Projects/Dags/test4.def','~/Projects/Strategy/solve.str',...
%     '~/Projects/Gloptlab/Solutions/', 1,...
%     '~/Projects/Gloptlab/Gloptlab.cfg',...
%     100, 15, 30, '~/Projects/Gloptlab/Autosaves/',...
%     1, 1, '~/Projects/Gloptlab/Logs/')